/* aktdata.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef AKT_DATA_H
#define AKT_DATA_H

#include <sys/types.h>

/*
	Erklrung einzelner Variablen in akt_anruf:

	anruftyp:    Wer oder was rief an/geschah?
	free_string: Protokollintrag fr den Fall, da es kein User war. 
	hard_limit:  Gibt an, wann der User unbedingt 'raus sein mu, z.B.
	             weil der nchste Event ausgefhrt werden mu.
	soft_limit:  Dann wre seine Onlinezeit eigentlich zuende. Sie kann
	             aber eventuell berzogen werden, fr zu lange Downloads, 
	             oder wg. Chat mit dem Sysop. In keinen Fall darf hard_limit
	             berschritten werden.
	             Soft_limit wird fr alle Abfragen, ob der User 'rausgeworfen
	             werden mu, verwendet!
*/

typedef enum 
{
	TRISTATE_NOT_TRIED, /* MUSS 0. Element sein! */
	TRISTATE_FAILURE,
	TRISTATE_OK
} TRISTATE;

typedef enum
{
	ANRUF_NIL,
	ANRUF_EVENT,  /* Event */
	ANRUF_INTERN, /* interner Event, z.B. Mitternachtsupdate */
	ANRUF_CTAUSCH, /* Consolentausch */
	ANRUF_CCOMMAND, /* Kommandozeilenkommando */
	ANRUF_CBATCH, /* Kommandozeilenbatch */
	ANRUF_SPEC, /* Speziallogin */
	ANRUF_FAX,	  /* Fax entgegengenommen */
	ANRUF_USER   /* User hat angerufen */
} ANRUFTYP;


/* 
 * Definition der Struktur fr den aktuellen Anruf.
 * Idealerweise bezieht sich dies auf einen User.
 */
struct anrufdaten
{
	ANRUFTYP anruftyp;

	ULONG	anrufnummer;         /* Nummer des Anrufes */
	
	char 	name[UDAT_NAME_LEN];
	time_t 	startzeit;           /* als Connect zustande kam */
	time_t 	userstartzeit;       /* als dieser User sich einloggte begann */
	time_t 	endzeit;             /* und endete (Logoff) */

	char 	free_string[17];	 /* Fr non-user's */

	time_t  soft_limit;          /* dann sollte Schlu sein */
	time_t  hard_limit;          /* dann *ist* Schlu. Siehe oben */

	BOOLEAN ist_zahler;          /* Hat er Zahlerstatus? */

	TRISTATE chat_status;        /* Chat? */ 
	TRISTATE tausch_status;		 /* Tausch? */
	TRISTATE batch_status;       /* Batch? */

	BOOLEAN neuuser;             /* neu eingetragen */
	BOOLEAN ist_relogin;         /* Nach einen Relogin gesetzt */

	BOOLEAN hat_neue_pms;        /* Sie/Er hat seine neuen PMs nicht gelesen */


	/* 0: kein Reset. 1: nicht logoutdatum speichern. 2: Userdaten nicht 
	 * speichern */
	int     did_reset;

	char    zuletzt[4];          /* Carrier-Lost-Eintrag (BOX,EXT,...)*/

		/* maximale Transferzeit fr *jeden* einzelnen Download festsetzen. 
		 * 0 disabled das Feature.
		 */
	time_t  max_transferzeit;

	ULONG   total_msg_zahl;
	ULONG   total_msg_bytes;

	BOOLEAN mitten_im_tausch; /* gelegentlich braucht man das ... */
	/* Interaktiver Login, kein Tauschlogin und kein Kommandozeilen-
	 * -batch oder -kommando
	 */
	BOOLEAN interaktiv;

};

/*
 * Diese Struktur wird durch inquire_remote() initialisiert, aber 
 * an einigen Stellen nachbearbeitet. Das tut der interne getty sowie
 * in eingeschrnktem Umfang die Kommandozeilenparameter.
 */
struct connectiondaten 
{
	BOOLEAN is_tty;               /* ein echtes tty */
	char *ttyname;                /* /dev/xxxx oder NULL (!is_tty) */
	time_t 	startzeit;            /* Start der Box oder Aktivitt des getty*/

	BOOLEAN peer;                 /* ber Netz */
	char *rusername;              /* Name des Remoteusers oder NULL (!peer) */
	char *rhostname;              /* Remotehostname oder NULL (!peer) */
	char *rhostaddr;              /* 1.23.45.67 oder NULL (!peer) */
	unsigned short localport;     /* Auf diesem Port ist die Verbindung lokal */
	unsigned short remoteport;    /* und remote */

	BOOLEAN auth_ok;              /* Authentisierung OK */
	BOOLEAN error_free;           /* Verbindung mit Fehlerkorrektur */

	/*
	 * Achtung: is_modem + is_console <=1.
	 * Soll heien: Mglicherweise trifft keines von Beiden zu.
	 * "is_modem" kann auch gesetzt sein, wenn das Modem *nicht* an diesem
	 * Rechner hngt (Terminalserver). In dem Fall ist "has_modem_lines"
	 * aber FALSE.
	 * 
	 */
	BOOLEAN is_modem;             /* Eine Modemverbindung */
	BOOLEAN is_console;           /* Consolenlogin */

	BOOLEAN has_modem_lines;      /* Carrier kann abgefragt werden */
	char *carrierstring;    /* 14k4, FAX */
	char *carriercmd;       /* /usr/sbin/faxin */
	char *carrierstringraw; /* +FCON */
	/*char 	connect[5];	 Connectstring, nun carrierstring */
	ULONG       carrier_baud;    /* 2400, 19200, usw */
	ULONG       carrier_cps;     /* 230, 2100, usw */

	/* dieser User hat sich ber eine Shell eingeloggt. */
	BOOLEAN shelluser;
	/* kann auch dann 0 sein, wenn shelluser TRUE ist - so eine Art
	 * Gastlogin z.B.. In die Initialisierungsphase der Box ist 
	 * dieser Wert immer 0 
	 */
	USERNR  shellusernummer;
	/* Reale und effektive uid */
	uid_t ruid;
	uid_t euid;
	gid_t rgid;
	uid_t egid;
	/* Environment des Users */
	char **user_environ;
	/* Environment der Box */
	char **box_environ;

	/* was sendet der Benutzer als Zeilenende? */
	BOOLEAN sends_cr;
	BOOLEAN sends_lf;

	/* Caller ID */
	char *clid_from;
	char *clid_to;
};

struct status_dat
{
	long postertrag;
	time_t last_call;
	char last_user[128];
	long anz_calls;
	WORD tagesanrufe;
};

extern struct connectiondaten akt_connection;
extern struct anrufdaten akt_anruf;

	/* Initialisiert die Datenstruktur, zu finden in log.c */
void anrufdaten_init(ANRUFTYP);

	/* Initialisiert die Verbindungsdaten, zu finden in lib/service.c */
BOOLEAN inquire_remote(int fd);
	/* Nach Auswertung der quark.cnf zu starten */
BOOLEAN inquire_remote2(int fd);

void write_close_status_dat(FILE *f, struct status_dat *st);
FILE *open_read_status_dat(BOOLEAN read_only, struct status_dat *st);

#endif
