/* bb_lock.c: lock/unlock a brett.
    Copyright (C) 19yy  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/
#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <syslog.h>
#include <assert.h>
#include <errno.h>

#include "config.h"
#include "libquark.h"

#include "brett_db.h"
#include "brett_int.h"
#include "filelock.h"


/*
BB_FUNCTION(Interne Funktion,BOOLEAN,bb_lock_brett,
[[(BRETTNR @var{nummer},enum locktyp @var{locktyp}, BOOLEAN @var{block})]],[[
Lockt oder entlockt das Brett mit der Nummer @var{nr}. Ist @var{block}
TRUE, wird auf den Erhalt eines Locks gewartet. @var{locktyp} kann
folgende Werte annehmen:

@table @bullet
@item BB_UNLOCK
Lock soll freigegeben werden.
@item BB_READLOCK
Das Brett soll nur zum Lesen gelockt werden. Andere Prozesse 
drfen lesend zugreifen.
@item BB_WRITELOCK
Das Brett soll mglicherweise beschrieben werden, andere
Prozesse drfen nicht zugreifen.
@end table

Liefert TRUE, falls der Lock erhalten oder freigegeben werden konnte, sonst
FALSE.

Die Funktion liefert TRUE auch dann, wenn das System (oder die Library)
kein Filelocking untersttzt.
]])
*/

BOOLEAN bb_lock_brett(BRETTNR nr,enum locktyp locktyp, BOOLEAN block)
{
#ifdef CAN_LOCK_FILES
	struct flock lck;

	/* Dateibereich lock'en */
	if (locktyp==BB_READLOCK)
			lck.l_type = F_RDLCK;
	else if (locktyp==BB_WRITELOCK)
		lck.l_type = F_WRLCK;
	else
		lck.l_type = F_UNLCK;
	lck.l_whence = SEEK_SET;
	lck.l_start  = BB_OFFSET_FIRST_BRETT + br_ind[nr].brett->index * sizeof(BRETT);
	lck.l_len    = sizeof(BRETT);
	if (fcntl(bb_fh,(locktyp!=BB_UNLOCK && block) ? F_SETLKW : F_SETLK,&lck) 
		== -1)
	{
		if (errno == EDEADLK)
		{
			fputs("Deadlock (bb_lock_brett)\r\n",stderr);
			return FALSE;
		}
		if (errno == EINTR)
		{
			fputs("Signal while waiting for lock (bb_lock_brett)\r\n",stderr);
			return FALSE;
		}
		fprintf(stderr,"Locking error (bb_lock_brett): %s\r\n",strerror(errno));
		return FALSE;
	}

	if (locktyp!=BB_UNLOCK)
	{
		/* fputs("lock erhalten\r\n",stderr); */
		br_ind[nr].brett->flags |= B_LOCKED;
	}
	else
	{
		br_ind[nr].brett->flags &= ~(B_LOCKED);
		/* fputs("lock freigegeben\r\n",stderr); */
	}
#endif
	return TRUE;

} /* bb_lock_brett */


/*
BB_FUNCTION(Funktion,BOOLEAN,bb_unlock_brett,[[(BRETTNR @var{bnr})]],[[
    gibt den Lock auf das Brett frei. Liefert TRUE bei Erfolg, sonst FALSE.
]])
*/
BOOLEAN bb_unlock_brett(BRETTNR bnr)
{
	if (bnr<0 || bnr > max_brettnr || !(br_ind[bnr].brett->flags & B_LOCKED))
		return FALSE;
	return bb_lock_brett(bnr,BB_UNLOCK,FALSE);
} /* bb_unlock_brett */
