/* bb_sortierung.c: Routinen zur Bereitstellung einer sortierten Liste
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <syslog.h>

#include "config.h"
#include "libquark.h"

#include "brett_db.h"
#include "brett_int.h"

static int compare_brettnr (const void *a, const void *b);

static BRETTNR *sort_anker;

static int
compare_brettnr (const void *a, const void *b)
{
	const BRETTNR *za = a;
	const BRETTNR *zb = b;
	const char *ca;
	const char *cb;
	int r;
	ca=br_ind[*za].canon;
	cb=br_ind[*zb].canon;

	if (!ca && !cb)
		return 0;
	if (!ca)
		return 1;
	if (!cb)
		return -1;

	r = *ca - *cb;
	if (r != 0)	
		return r;
	return strcmp (ca, cb);
}

/*
BB_FUNCTION(Funktion,BRETTNR *,bb_sortierung,[[(void)]],[[
    Liefert einen Zeiger auf ein Array von Brettnummern zurck, die
    nach Alphabet sortiert sind. Die Funktion sortiert nur, wenn
    dies notwendig ist, sonst liefert sie ein vorheriger Sortier-
    ergebnis zurck.

    Bei Speichermangel wird NULL zurckgeliefert.

    Diese Funktion existiert in sehr hnlicher Form in der Quark selbst
    nocheinmal (da mu sie beachten, da der User das Brett mglicherweise
    nicht lesen kann und es bergehen).
]])
*/
BRETTNR *
bb_sortierung(void)
{
	BRETTNR i;
	BOOLEAN found=FALSE;
	BRETTNR j;

	/* Wenn schon mal sortiert wurde, dann haben wir es einfach.
	 * Das setzt aber voraus, da b_free_sortierung beim Logout des
	 * Users, bei Anlage neuer Bretter, beim Lschen vorhandener
	 * Bretter und beim Umbenennen aufgerufen wird.
	 * Und da b_sortierung immer dann aufgerufen wird, wenn auf
	 * das Array zugegriffen wird.
	 */
	if (sort_anker)
		return sort_anker;

	sort_anker = malloc (sizeof (BRETTNR) * max_brettnr);

	if (!sort_anker)
	{
		syslog(LOG_ERR,"out of memory in b_sortierung()");
		fprintf(stderr,"out of memory in b_sortierung()");
		return NULL;
	}

	for (i = 1, j=0; i < max_brettnr; i++)
	{
		if (br_ind[i].brett)
		{
			sort_anker[j++] = i;
			found = TRUE;
		}
	}
	/* Damit ist sichergestellt, da immer ein Feldelement mit Nummer NULL
	 * als Endesignal vorhanden ist. Das Postfach soll ja nicht in der
	 * Liste auftauchen, daher entsteht hier kein Problem.
	 */
	sort_anker[j] = 0;

	if (!found)
	{
		fprintf (stderr,"Keine Bretter vorhanden!\r\n");
		bb_free_sortierung();
		return NULL;
	}

	qsort (sort_anker, j, sizeof (BRETTNR), compare_brettnr);

	return sort_anker;
}

/*
BB_FUNCTION(Funktion,void,bb_free_sortierung,(void),[[
    Diese Funktion ist bei jedem Userwechsel, bei Anlage neuer Bretter,
    und beim beim Lschen und Umbenennen der Bretter aufzurufen. Sie
    sorgt dafr, da beim nchsten Aufruf von bb_sortierung die
    Bretterliste neu sortiert wird.
]])
*/
void
bb_free_sortierung(void)
{

	if (!sort_anker)
		return;

	free(sort_anker);
	sort_anker=NULL;
}
