/* boxstatistik.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
 * TOM Mailboxsystem
 * Copyright (C) 1993 1994 1995 Frank Baumgart, Uwe Ohse
 *
 * #include <all_rights_reserved>
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * befehle/boxstat.c: diverse Statistikfunktionen.
 */

#include "config.h"

#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "aktdata.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "befehle.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/message.h"
#else
#include "offline/version.h"
#include "offline/message.h"
#endif
#include "box_conf.h"


extern WORD vtmodeW;

extern const char *comp_type[];
extern UWORD comp_type_zahl;

extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;
extern long max_brettnr;

static BOOLEAN make_comp_stat(void);
static BOOLEAN make_user_stat(void);
static BOOLEAN make_anruf_stat(void);
static BOOLEAN make_gruppen_stat(void);


/*
	15.12.92 Frank
	07.10.93 Frank
	17.01.97 Frank, GRUPPEN ab MDB 1.99 deaktiviert
*/
BOOLEAN bef_boxstatistik(const char *para)
{
	char name[PATH_MAX];

	if (!stricmp(para,"COMP") || !stricmp(para,"COMPUTER"))
		return make_comp_stat();
	if (!stricmp(para,"BENUTZER") || !stricmp(para,"USER"))
		return make_user_stat();
	if (!stricmp(para,"ANRUF"))
		return make_anruf_stat();
#ifndef OFFLINE_VERSION
	if (!stricmp(para,"GRUPPEN"))
		return make_gruppen_stat();
#endif

	sprintf(name,"%s%s",box_conf.var_dir,"statist.inf");
	if (!fileexist(name))
		mboutput(" Es liegt noch keine Statistik vor\r\n\n");
	else
	{
		int old_mode;
		set_status_line("Statistik der Box:");
		mboutput(vt_pos_S(4,1));
		mbprintf(" %sDatum        Anrufe    MR     MW     DL    UL    n.U.   Postertr.    Auslast.%s\r\n",
					vt_S(ATTR_FETT),vt_S(ATTR_OFF));
		mbprintf(" %s\r\n",trennzeile);
		old_mode=set_scroll_area(2);
		zeige_datei(name);
		mboutput("\r\n");
		set_scroll_area(old_mode);
	}
	return TRUE;
}



/*
	25.03.93 Frank, auf Userdatenbank umgestellt, fast konverterfrei
*/
static BOOLEAN make_comp_stat(void)
{
	UWORD i;
	long anzL = 0;
	WORD maxW = 0;
	WORD max_compW = 0;
	char balken1;
	char balken2;
	int *compW;
	USER u;

	set_status_line("Computerverteilung der User:");

	if (vtmodeW == T_VT100)
	{
		balken1 = '\261';
		balken2 = '\262';
	}
	else
	{
		balken1 = '=';
		balken2 = '+';
	}
	compW=calloc(comp_type_zahl,sizeof(int));

	u.nummer = 0;
	while (ub_walk_user(&u))
	{
		if (u.computertyp < comp_type_zahl)
			compW[u.computertyp]++;
		else
			compW[0]++;
		anzL++;
	}

	for (i = 0; i < comp_type_zahl; i++)
	{
		if (compW[i] > maxW)
		{
			maxW=compW[i];
			max_compW=i;
		}
	}

	if (!maxW || !anzL)
	{
		free(compW);
		return FALSE;
	}

	for (i = 1; i < comp_type_zahl; i ++)
	{
		mbprintf(" %-18s-> %s %s%d%s\r\n",
					comp_type[i],
					string_make((50*compW[i])/maxW,(i&1)? balken1 : balken2),
					vt_S(ATTR_INVERS),
					compW[i],
					vt_S(ATTR_OFF));
	}
	mbprintf(" %-18s-> %s %s%d%s\r\n",
				comp_type[0],
				string_make((50*compW[0])/maxW,(comp_type_zahl&1)? balken2:balken1),
				vt_S(ATTR_INVERS),
				compW[0],
				vt_S(ATTR_OFF));
	mbprintf(" Diese Mailbox wird berwiegend von: %s%s%s Usern benutzt (%d%%)\r\n\n",
				vt_S(ATTR_INVERS),
				comp_type[max_compW],
				vt_S(ATTR_OFF),
				(int)(((100*compW[max_compW]/anzL)*100)/100));
	return TRUE;
}

static BOOLEAN make_anruf_stat(void)
{
	ULONG anz_4 = 0;
	ULONG anz_9 = 0;
	ULONG anz_49 = 0;
	ULONG anz_99 = 0;
	ULONG anz_999 = 0;
	ULONG anz_more = 0;
	ULONG maxanrufe = 0;
	time_t now;
	ULONG tage_2=0;
	ULONG tage_7=0;
	ULONG tage_14=0;
	ULONG tage_28=0;
	ULONG tage_longer=0;
	
	USER u;

	set_status_line("Anrufstatistik:");

	now=time(NULL);
	u.nummer = 0;
	while (ub_walk_user(&u))
	{
		if (u.anrufe <= 4) anz_4++;
		else if (u.anrufe <= 9) anz_9++;
		else if (u.anrufe <= 49) anz_49++;
		else if (u.anrufe <= 99) anz_99++;
		else if (u.anrufe <= 999) anz_999++;
		else
			anz_more++;
		if (u.anrufe>maxanrufe)
			maxanrufe=u.anrufe;
		if (now-u.letzter_login<2*86400L)
			tage_2++;
		else if (now-u.letzter_login<7*86400L)
			tage_7++;
		else if (now-u.letzter_login<14*86400L)
			tage_14++;
		else if (now-u.letzter_login<28*86400L)
			tage_28++;
		else if (u.letzter_login>(time_t) 0)
			tage_longer++;
	}

	mbprintf("Anrufe:    Benutzer\r\n");
	mbprintf("1-4:       %5lu\r\n",anz_4);
	mbprintf("5-9:       %5lu\r\n",anz_9);
	mbprintf("10-49:     %5lu\r\n",anz_49);
	mbprintf("50-99:     %5lu\r\n",anz_99);
	mbprintf("100-999:   %5lu\r\n",anz_999);
	mbprintf("> 1000:    %5lu\r\n",anz_more);
	mboutput("\r\n");
	mbprintf("Der fleiigste Anrufer hat %lu Anrufe gemacht.\r\n",maxanrufe);
	mboutput("\r\n");
	mbprintf("Letzter Anruf:               Benutzer\r\n");
	mbprintf("gestern und heute:           %5lu\r\n",tage_2);
	mbprintf("in der letzten Woche:        %5lu\r\n",tage_7);
	mbprintf("in den letzten 14 Tagen:     %5lu\r\n",tage_14);
	mbprintf("in den letzten 4 Wochen:     %5lu\r\n",tage_28);
	mbprintf("Lnger als 4 Wochen nicht:   %5lu\r\n",tage_longer);
	mboutput("\r\n");
	mbprintf("Bis jetzt gab es insgesamt %lu Anrufe.\r\n",
		akt_anruf.anrufnummer);
	mboutput("\r\n");
	return TRUE;
}

#ifndef OFFLINE_VERSION
static BOOLEAN make_gruppen_stat(void)
{
	BRETTNR i;
	long anzahl,bytes;

	anzahl = bytes = 0;
	mbprintf(" %-60.60s %-6s   %-5s\r\n","Gruppe","Anzahl","KB");
	for (i = 1; i < max_brettnr; i++)
	{
		long anz,gr;
		if (!br_ind[i].brett)
			continue;
		if (!(br_ind[i].brett->flags & B_READ))
			continue;
		if (!group_info_msgs(br_ind[i].brett->name,FALSE,&anz,&gr))
			continue;
		anzahl += anz;
		bytes  += gr;
		if (mbprintf("%-60.60s  %6ld  %5ld\r\n",
			br_ind[i].brett->name,anz,gr/1024)<0)
			break;
	}
	mboutput("========================================"
	         "=======================================\r\n");
	mbprintf(" %-60.60s %6ld %6ld\r\n","",anzahl,bytes / 1024);

	return TRUE;
}
#endif

static BOOLEAN make_user_stat(void)
{
	struct tm now;
	time_t t;
	ULONG zeit_1m=0;
	ULONG zeit_3m=0;
	ULONG zeit_6m=0;
	ULONG zeit_12m=0;
	ULONG zeit_2j=0;
	ULONG zeit_longer=0;
	ULONG sumalter=0;
	ULONG maenner=0;
	ULONG frauen=0;
	ULONG zahler=0;
	ULONG programme=0;
	ULONG alter_ok_zahl=0;
	USER u;

	t=time(NULL);
	now=*localtime(&t);	

	set_status_line("Benutzerstatistik:");

	u.nummer = 0;
	while (ub_walk_user(&u))
	{
		UWORD vglmon;
		UWORD vgljahr;
		UWORD thisalter;
		if (u.typ!=UTYP_USER)	
		{
			programme++;
			continue;
		}
		if (u.geschlecht==NEUTRUM)	
		{
			programme++;
			continue;
		}

		if (u.erster_login != (time_t) 0)
		{
			time_t dif;
			dif=t-u.erster_login;
			if (dif<30*86400L) zeit_1m++;
			else if (dif<3*30*86400L) zeit_3m++;
			else if (dif<6*30*86400L) zeit_6m++;
			else if (dif<365*30*86400L) zeit_12m++;
			else if (dif<730*30*86400L) zeit_2j++;
			else 
				zeit_longer++;
		}
		if (u.geschlecht==MAENNLICH) 
			maenner++;
		else 
			frauen++;
		vglmon=u.geburtsmonat-1;
		if (u.geburtsjahr>1900)
			vgljahr=u.geburtsjahr-1900;
		else
			vgljahr=u.geburtsjahr;
		if (vgljahr)
		{
			alter_ok_zahl++;
			if (vglmon>now.tm_mon 
				|| (vglmon==now.tm_mon && u.geburtstag>now.tm_mday))
				thisalter=now.tm_year-vgljahr-1;
			else
				thisalter=now.tm_year-vgljahr;
			sumalter+=thisalter;
		}

		if (u.bezahlt)
			zahler++;
	}

	mbprintf(
	"Zur Zeit sind %lu Benutzer eintragen, davon haben %lu Zahlerstatus.\r\n",
		maenner+frauen,zahler);
	if (alter_ok_zahl)
	{
		ULONG anz=alter_ok_zahl;
		mbprintf(
		"%lu sind weiblich (%d %%), %lu mnnlich.\r\n",
		frauen, (int) ((100*frauen)/anz),maenner);
		mboutput("\r\n");
		mbprintf("Das Durchschnittsalter betrgt %ld.%1ld Jahre.\r\n",
			sumalter/anz,((sumalter%anz)*10)/anz);
		mboutput("\r\n");
		mbprintf("%5lu %s keinen ganzen Monat dabei (%ld %%).\r\n",
			zeit_1m, zeit_1m==1 ? "ist" : "sind",
			zeit_1m*100/anz);
		mbprintf("%5lu %s keine drei Monate dabei (%ld %%).\r\n",
			zeit_3m, zeit_3m==1 ? "ist" : "sind",
			zeit_3m*100/anz);
		mbprintf("%5lu %s kein halbes Jahr dabei (%ld %%).\r\n",
			zeit_6m, zeit_6m==1 ? "ist" : "sind",
			zeit_6m*100/anz);
		mbprintf("%5lu %s kein ganzes Jahr dabei (%ld %%).\r\n",
			zeit_12m, zeit_12m==1 ? "ist" : "sind",
			zeit_12m*100/anz);
		mbprintf("%5lu %s keine zwei Jahre dabei (%ld %%).\r\n",
			zeit_2j, zeit_2j==1 ? "ist" : "sind",
			zeit_2j*100/anz);
		mbprintf("%5lu %s zwei Jahre dabei (%ld %%).\r\n",
			zeit_longer, zeit_longer==1 ? "ist" : "sind",
			zeit_longer*100/anz);
	}
	return TRUE;
}
