/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
/* bretteditor.c: bretteditor und Hilfsroutinen. */

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#include <syslog.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "fnmatch.h"

#include "tomsysdep.h"
#include "config.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/mdb_global.h"
#include "offline/inc/message.h"
#else
#include "offline/version.h"
#include "offline/mdb_global.h"
#include "offline/message.h"
#endif
#include "prototyp.h"
#include "brett.h"
#include "tausch.h"
#include "user.h"
#include "bin_msg.h"
#include "modem.h"
#include "tompaths.h"
#include "terminal.h"
#include "box_conf.h"

#define UMASK			(S_IRUSR | S_IWUSR | S_IXUSR)

extern long max_brettnr;
extern int logout_modus;
extern BRETTINFO *br_ind;
extern BRETTNR brettnrW;

extern USER user;

extern int col_ix[];

#ifdef HAVE_DOSDRIVES
extern char known_drivesS[];
char installed_drivesS[30];
static WORD drive_zahl(char *name);
static BOOLEAN copy_brett(BRETTNR bnr,char *drive);
static long byte_summe;
#endif

extern WORD count_crW;

extern BOOLEAN bef_brett_sortierung_maybe_wrong;
static BOOLEAN bedt_common_edit(BRETTNR bnr, char *name);
static BOOLEAN brettedt_edit(BOOLEAN neu, BRETT *b);
static BOOLEAN net_itg_info(const char *name,char *u_info,char *s_info);
static int bedt_select_from_list(const char *titel, UWORD *wert, const char * (*fn) (UWORD));
static void be_bestellen(BRETTNR bnr, const char *name);

static const char *on=NULL;
static const char *off=NULL;
static const char *clr=NULL;

void bretteditor(const char *para)
{
	BOOLEAN changes = FALSE;
	char name[80];
	char buf[160];
	BRETTNR nr;
	long i;
	struct mbselect_t auswahl[] =
	{
		{'A', "Brett anlegen\r\n\n"},
		{'E', "Brett ndern\r\n\n"},
		{'L', "Brett lschen\r\n\n"},
		{'Z', "Bretterliste anzeigen\r\n\n"},
		{'\033', "Ende\r\n\n"},
		{'\r', "Ende\r\n\n"},
		{'\n', "Ende\r\n\n"},
		{0, NULL}
	};

	clr=vt_S(DEL_SCR);
	if (clr && *clr)
		mboutput(clr);
	on = vt_S(col_ix[C_ROT]);
	off=vt_S(ATTR_OFF);
	if (!on)
		on=vt_S(ATTR_FETT);
	if (!on || !off)
	{
		on="\'";
		off="\'";
	}

	while (TRUE)
	{
		char sel;

		if (para)
		{
			if (0==strnicmp(para,"liste",5))
			{
				sel='Z';
				para+=5;
			}
			else if (0==strnicmp(para,"delete",6))
			{
				sel='L';
				para+=6;
			}
			else if (0==strnicmp(para,"lschen",7)
					 || 0==strnicmp(para,"lschen",7))
			{
				sel='L';
				para+=7;
			}
			else
			{
				/* alles andere drfte der Brettname sein */
				sel='E';
			}
		}
		else
		{
			set_status_line("Bretteditor");
			mboutput("\r\n\n\n\n");
			mbprintf("(%sA%s)nlegen\r\n",on,off);
			mbprintf("(%sE%s)di%seren\r\n",on,off,(box_conf.ti) ? "ti" : "");
			mbprintf("(%sL%s)schen\r\n",on,off);
			mbprintf("(%sZ%s)eigen der Liste\r\n",on,off);
			mboutput("\r\n");
			mbprintf("(%sEscape%s) Abbruch\r\n",on,off);
			mbprintf("(%sCR%s) Ende\r\n",on,off);
			mboutput("\r\n->");

			sel=mbselect(auswahl);

			if (!sel || sel=='\033' || sel=='\n' || sel=='\r')
				break;
		}
	
		switch(sel)
		{
		case 'A': /* anlegen */
			if (para) {
				strcpy(name,para);
				strtrim(name);
			} else
				*name=0;
			if (!*name)
				mbfrage(64,MBF_LINE | MBF_TRIM," Brett > ",name);
			if (!*name)
				break;
			nr=brettnr(name);
			if (nr)
			{
				mboutput(" Ein Brett dieses Namens ist bereits vorhanden\r\n");
				mbprintf(" %lu, %s\r\n",nr,br_ind[nr].brett->name);
				break;
			}
			/* dies ist so eine Art fall-through */
		case_A: /* GOTOs: i sometimes like to use them! */
			if (bedt_common_edit(0,name))
			{
				nr=brettnr(name);
				if (nr 
					&& br_ind[nr].brett->netzstatus
					&& mboutput(" Brett gleich bestellen? (J/N) ") > 0
					&& 1==mbdecide("JN","Ja\r\n|Nein\r\n"))
				{
					if (nr!=0)
						be_bestellen(nr,name);
				}
			}
			break;
		case 'E': /* Editieren */
			if (para) {
				strcpy(name,para);
				strtrim(name);
			} else
				*name=0;
			if (!*name) {
				if (!mbfrage(64,MBF_LINE | MBF_TRIM," Brett > ",name)) {
					break;
				}
				if (!*name)
					break;
			}
			nr=brettnr(name);
			if (!nr)
			{
				nr=strtol(name,NULL,10);
				if (nr <=0 || nr>=max_brettnr)
					nr=0;
			}
			if (!nr) {
				nr=find_brett(name);
				if (nr <=0 || nr>=max_brettnr)
					nr=0;
			}
			if (!nr)
			{
				if (strtol(name,NULL,10))
				{
					mboutput( "Dieses Brett gibt es nicht\r\n");
				}
				else
				{
					mboutput(
						"Dieses Brett gibt es nicht. Wollen Sie es anlegen? (J/N)");
					if (1==mbdecide("JN","Ja\r\n|Nein\r\n"))
						goto case_A;
				}
				break;
			}
			if (!bb_read_brett(nr,TRUE))
			{
				mbfehler("Brett wird gerade bearbeitet");
				break;
			}
			bedt_common_edit(nr,NULL);
			break;
		case 'L': /* lschen */
			if (para) {
				strcpy(name,para);
				strtrim(name);
			}
			else
				*name=0;
			if (!*name) {
				if (!mbfrage(64,MBF_LINE | MBF_TRIM," Brett > ",name)) {
					break;
				}
				if (!*name)
					break;
			}
			nr=brettnr(name);
			if (nr == 0)
				nr=strtol(name,NULL,10);

			if (nr <= 0 || nr >= max_brettnr || !br_ind[nr].brett)
			{
				mbfehler("Dieses Brett gibt es nicht");
				break;
			}

			if (nr==brettnrW)
			{
				mbfehler("Lschen des aktuellen Brettes nicht mglich");
				break;
			}

			mbprintf("%s%s",vt_pos_S(4,1),vt_S(DEL_SCR_REST));
			mbprintf(" Brettname: %s\r\n\n",br_ind[nr].brett->name);
			mboutput(" >> Dieses Brett wirklich lschen?  (J/N): ");
			if (mbdecide("JN","Ja\r\n\n|Nein...\r\n\n") == 2)
				break;
			if (!bb_read_brett(nr,TRUE))
			{
				mboutput(" Keine Zugriffsberechtigung erhalten\r\n");
				break;
			}
			changes = TRUE;

			b_remove_brett(nr);
			break;

		case 'Z': /* Liste zeigen */
			if (!brettanzahl)
			{
				mboutput(" Es sind noch keine Bretter installiert\r\n");
				break;
			}
			if (para) {
				strcpy(name,para);
				strtrim(name);
			} else
				*name=0;
			if (!*name) {
				if (!mbfrage(64,MBF_LINE | MBF_TRIM," Wildcard > ",name)) {
					break;
				}
				if (!*name)
					break;
			}
			entumlauten(name,buf); /* b_canonical wandelt `*' und `?'! */
			mbprintf("%s%s",vt_pos_S(1,1),vt_S(DEL_SCR_REST));
			mboutput("\r\n");
#ifdef HAVE_DOSDRIVES
			mboutput("  Nr.!-!T!Brettname                      !LSP!Verwalter      !Lim.!Anz. !kByte\r\n");
			mboutput(" ----!-!-!-------------------------------!---!---------------!----!-----!-----\r\n");
#else
			mboutput("  Nr.!T!Brettname                      !LSP!Verwalter      !Lim.!Anz. !kByte\r\n");
			mboutput(" ----!-!-------------------------------!---!---------------!----!-----!-----\r\n");
#endif
			set_status_line(name ? name : "Brettliste");
			set_scroll_area(1);
#ifndef FNM_CASEFOLD
			strlwr(buf);
#endif
			for (i=1; i < max_brettnr; i++)
			{
				long anzahl;
				long groesse;
				if (!br_ind[i].brett)
					continue;
#ifdef FNM_CASEFOLD
				if (0!=fnmatch(buf,br_ind[i].canon,FNM_CASEFOLD))
#else
				if (0!=fnmatch(buf,br_ind[i].canon,0))
#endif
					continue;

#ifdef HAVE_DOSDRIVES
				mbprintf(" %4ld!%c!%c!%-31.31s!",
#else
				mbprintf(" %4ld!%c!%-31.31s!",
#endif
						(long)br_ind[i].brett->nr,
#ifdef HAVE_DOSDRIVES
						br_ind[i].brett->partition + 'A',
#endif
						(br_ind[i].brett->art == TEXT_BRETT) ? 'T' : 'B',
						br_ind[i].brett->name);
				mbprintf("%d%d%c!%-15.15s",
						br_ind[i].brett->lesen_level,
						br_ind[i].brett->schreib_level,
						(*br_ind[i].brett->passwort) ? 'x' : '0',
						b_verwalter_S(i));

				/* zwischen Binr- und Textbrettern unterscheiden */
				anzahl = groesse = 0;
				if (br_ind[i].brett->art == TEXT_BRETT)
#ifndef OFFLINE_VERSION
					group_info_msgs(br_ind[i].brett->name,FALSE,&anzahl,&groesse);
#else
					/* ### allenfalls die Anzahl ist via LIST ermittelbar */
					anzahl = groesse = 0;
#endif
				else
					binaer_dat_anzahl(i,0,&anzahl,NULL,&groesse,NULL);
				mbprintf("!%5ld!%5ld",anzahl,groesse/1000);
				if (mboutput("\r\n") < 0)
					break;

				if (userbreak())
					break;
			}
			if (count_crW)
				wait_mehr();
			set_scroll_area(0);
			mboutput("\r\n");
			break;
		}	

		check_timeout();
		if (logout_modus)
			break;
		if (para)
			break;
	}

	if (changes)
		b_set_flags(&user);

} /* bretteditor */



static BOOLEAN 
brettedt_edit(BOOLEAN neu, BRETT *b)
{
	char str[256];
	char str2[256];
	USERNR old_chef;
	UWORD changes=0;
	char oldname[BDAT_GRP_NAME_LEN];
	struct mbselect_t auswahl[30];
	int wahlzahl=0;
	LISTUSER *lu;

	while (!logout_modus)
	{	
		int c;
		char status_str[256];
		wahlzahl=0;
		strcpy(oldname,b->name);

		mboutput(vt_pos_S(1,1));
		mboutput(vt_S(DEL_SCR));
		strcpy(status_str,"Bretteditor: ");
		mboutput(vt_pos_S(5,1));
		if (b->nr)
			sprintf(status_str+strlen(status_str),"Nr. %5ld  ",b->nr);
		else
			sprintf(status_str+strlen(status_str),"Neuanlage  ");
		if (changes)
			sprintf(status_str+strlen(status_str),"(verndert)");
		else
			sprintf(status_str+strlen(status_str),"(noch unverndert)");
		set_status_line(status_str);

		mbprintf("(%sN%s)ame:     %s\r\n",on,off,b->name);
		auswahl[wahlzahl].key='N';
		auswahl[wahlzahl++].output="Name";

		mbprintf("Al(%sI%s)as:    %s\r\n",on,off,
				 b->aliasname ? b->aliasname : "");
		auswahl[wahlzahl].key='I';
		auswahl[wahlzahl++].output="Aliasname";

		if (b->followup_to_default && br_ind[b->followup_to_default].brett)
			strcpy(str2,br_ind[b->followup_to_default].brett->name);
		else
			*str2=0;
		mbprintf("F(%sO%s)llowup: %s\r\n",
			on,off,str2);
		auswahl[wahlzahl].key='O';
		auswahl[wahlzahl++].output="Followup-To";

		mbprintf("(%sC%s)hef:     ",on,off);
		lu=NULL;
		if (b->grp_chef)
			lu=ub_find_usernummer(b->grp_chef);
		if (lu)
			mbprintf("%s\r\n",lu->name);
		else
			mboutput("keiner/Sysop\r\n");
		auswahl[wahlzahl].key='C';
		auswahl[wahlzahl++].output="Chef";

		mbprintf("(%sV%s)ernetzt: %s    ", on,off,
			b->netzstatus ? "ja   " : "nein ");
		auswahl[wahlzahl].key='V';
		auswahl[wahlzahl++].output="Vernetzung";

		/* Brettart lt sich nur bei Neuanlage ndern */
		if (neu)
		{
			mbprintf("(%sA%s)rt:        %s  ", on,off,
				b->art==TEXT_BRETT ? "text " : 
					 b->art==BINAER_BRETT ? "binr" : "NNTP");
			auswahl[wahlzahl].key='A';
			auswahl[wahlzahl++].output="Art";
		}
		else
			mbprintf("Art:          %s  ", 
				b->art==TEXT_BRETT ? "text " : 
					 b->art==BINAER_BRETT ? "binr" : "NNTP");

	#ifdef HAVE_DOSDRIVES
		mbprintf("Par(%sT%s)ition: %c\r\n", on,off,
			'A'+b->partition);
		auswahl[wahlzahl].key='T';
		auswahl[wahlzahl++].output="Partition";
	#else
		mboutput("\r\n");
	#endif

		mbprintf("(%sL%s)esen:    %d        ",on,off,
			b->lesen_level);
		auswahl[wahlzahl].key='L';
		auswahl[wahlzahl++].output="Leselevel";

		mbprintf("(%sS%s)chreiben: %d       ",on,off,
			b->schreib_level);
		auswahl[wahlzahl].key='S';
		auswahl[wahlzahl++].output="Schreiblevel";

		if (b->flags & B_PASSWORT)
		{
			mbprintf("Pass(%sW%s)ort: %s\n",on,off,
				(*b->passwort) ? "ja" : "sollte");
			auswahl[wahlzahl].key='W';
			auswahl[wahlzahl++].output="Passwort";
		}
		else
			mboutput("\r\n");


		mbprintf("(%sF%s)lags:    ", on,off);
		if (b->flags & B_ANONYM)
			mboutput("anonym ");
		if (b->flags & B_GEHEIM)
			mboutput("geheim ");
		if (b->flags & B_MITGLIED)
			mboutput("Mitgliedschaft ");
		if (b->flags & B_ZAHLER)
			mboutput("Zahler ");
		if (b->flags & B_PASSWORT)
			mboutput("Pawort ");
		if (b->flags & B_ZWANG)
			mboutput("Zwang ");
		if (b->flags & B_DEFAULT)
			mboutput("Default ");
		if (b->flags & B_MODERIERT)
			mboutput("Moderiert ");
		if (b->flags & B_ACL)
			mboutput("ACL ");
		if (b->flags & B_LESMITGLIED)
			mboutput("Lesen ");
		if (b->flags & B_PAYWRITE)
			mboutput("Paywrite ");
		mboutput("\r\n");
		auswahl[wahlzahl].key='F';
		auswahl[wahlzahl++].output="Flags";


		mbprintf("(%sK%s)urzinfo: %s\r\n",
			on,off,
			(b->userinfo) ? b->userinfo : "");
		auswahl[wahlzahl].key='K';
		auswahl[wahlzahl++].output="Kurze Information";

		mbprintf("Lan(%sG%s)info: %s\r\n",
			on,off,
			(b->lokalinfo) ? "ja" : "nein");
		auswahl[wahlzahl].key='G';
		auswahl[wahlzahl++].output="Lange Information";
		mbprintf("S(%sY%s)sopin.: %s\r\n",
			on,off,
			(b->sysopinfo) ? b->sysopinfo: "");
		auswahl[wahlzahl].key='Y';
		auswahl[wahlzahl++].output="Sysopinformation";

		mbprintf("(%sH%s)erkunft: %s\r\n",
			on,off,bb_ursprungsnetz(b->ursprungsnetz));
		auswahl[wahlzahl].key='H';
		auswahl[wahlzahl++].output="Herkunft";

		mbprintf("S(%sP%s)rache:  %s\r\n",
			on,off,bb_sprache(b->sprache));
		auswahl[wahlzahl].key='P';
		auswahl[wahlzahl++].output="Sprache";


		mboutput("\r\n");
		mbprintf("Bitte auswhlen, (%sEscape%s) fr Ende:\r\n",
			on,off);
		/* Escape */
		auswahl[wahlzahl].key='\033';
		auswahl[wahlzahl++].output="Ende";

		auswahl[wahlzahl].key='\0';
		auswahl[wahlzahl++].output=NULL;
		c=mbselect(auswahl); 
		mboutput("\r\n");
		if (c==0 || logout_modus)
			return FALSE;
		if (c=='\033')
			break;
		switch(c)
		{
		case 'N': /* Name ndern */
			mboutput(" Neuer Brettname:\r\n");
			strcpy(str,b->name);
			if (!mbfrage(78,MBF_LINE|MBF_TRIM,"",str)) {
				continue;
			}
			/* Leereingabe ist Abbruch */
			if (!*str)
				continue;
			if (!check_brettname_syntax(str))
			{
				mboutput(" nur Buchstaben, Umlaute, Zahlen sowie ._-+&/ erlaubt\r\n\n");
				continue;
			}
			/* wenn alter und neuer Name sich nicht unterscheiden */
			if (strcmp(b->name,str)!=0)
			{
				BRETTNR bnr;

				bnr=brettnr(str);
				/* wenn kein Brett dieses Namens existiert oder die 
				 * Schreibweise des bearbeiteten Brettes verndert wird */
				if (bnr==0 || bnr==b->nr)
				{
					changes++;
					strcpy(b->name,str);
					continue;
				}
				mbprintf(" Brett dieses Namens bereits vorhanden (Nr. %lu)\r\n",
					bnr);
			}
			break;
		case 'I':
		{
			/* Aliasname ndern. ble Sache, mu durch die ganze Liste zur 
			 * Erkennung von Namenskollisionen
			 */
			BRETTNR j;
			char b1[BDAT_GRP_NAME_LEN*2+1];
			char b2[BDAT_GRP_NAME_LEN*2+1];
			int i;
			mboutput(" Neuer Aliasname:\r\n");
			strcpy(str,b->aliasname ? b->aliasname : b->name);
			if (!mbfrage(78,MBF_LINE|MBF_TRIM,"",str)) {
				continue;
			}
			/* Leereingabe ist Abbruch */
			if (!*str)
				continue;
			if (!check_brettname_syntax(str))
			{
				mboutput(" nur Buchstaben, Umlaute, Zahlen sowie ._-+&/ erlaubt\r\n\n");
				continue;
			}
			/* wenn alter und neuer Name sich nicht unterscheiden */
			if (b->aliasname && !strcmp(b->aliasname,str)) 
				continue;
			j=brettnr(str);
			if (j!=0) {
				mbprintf("\r\n Namenskollision mit %s\r\n",
						 br_ind[j].brett->name);
				continue;
			}
			bb_canonical(str,b1);
			for (i=1; i < max_brettnr; i++) {
				if (!br_ind[j].brett)
					continue;
				if (!br_ind[j].brett->aliasname)
					continue;
				bb_canonical(br_ind[j].brett->aliasname,b2);
				if (!strcasecmp(b1,b2))
					break;
			}
			if (i < max_brettnr) {
				mbprintf("\r\n Namenskollision mit %s,\r\n dem Aliasnamen von %s\r\n",
						 br_ind[i].brett->aliasname,br_ind[i].brett->name);
				continue;
			}
			if (!*str) {
				free(b->aliasname);
				b->aliasname=NULL;
				changes++;
			} else {
				free(b->aliasname);
				b->aliasname=XSTRDUP(str);
				changes++;
			}
			break;
		}
#ifdef HAVE_DOSDRIVES
		case 'T': /* Partition */
			/*
			 * Brettpartition
			 */
			sprintf(str2,"%c",'A'+b->partition);
			while (TRUE)
			{
				mbquestion(1,2,TRUE," Partition   : ",str2,str);
				/*                    123456789012:4 */
				strupr(str);
				strtrim(str);
				if (!*str)
					return FALSE;
				if (!strstr(known_drivesS,str) && !neu)
				{
					mboutput(" Diese Partition gibt es nicht\r\n");
					continue;
				}
				if (str[0]==b->partition+'A')
					break;
				changes++;
				break;
			} /* while */
			break;
#endif
		case 'A': /* Brettart */
			if (b->art==TEXT_BRETT)
				b->art=BINAER_BRETT;
			else
				b->art=TEXT_BRETT;
			changes++;
			break;
		case 'V': /* Netzstatus */
			b->netzstatus=!b->netzstatus;
			changes++;
			break;
		case 'L': /* Leselevel */
		case 'S': /* Schreiblevel */
			{
				WORD oldlv;
				WORD neulv;
				if (c=='L')
					oldlv=b->lesen_level;
				else
					oldlv=b->schreib_level;
				neulv=oldlv;
				mboutput("Abbruch mit ESCAPE, Ziffer whlt Level: ");
				do
				{
					int x;
					x=mbchar();
					if (x=='\033')
						break;
					else if (isdigit(x))
					{
						neulv=x-'0';
						break;
					}
					if (logout_modus)
						break;
				}
				while (TRUE);
				if (logout_modus)
					break;
				if (c=='L')
				{
					if (neulv!=oldlv)
					{
						b->lesen_level=neulv;
						changes++;
					}
				}
				else
				{
					if (neulv!=oldlv)
					{
						b->schreib_level=neulv;
						changes++;
					}
				}
			}
			break;
		case 'F': /* Flags */
			{
				ULONG flags=b->flags;
				while (1)
				{
					/* aktuell eingestellten Zustand zeigen */
					int x;
					set_scroll_area(0);
					mboutput(vt_pos_S(1,1));
					mboutput(vt_S(DEL_SCR));
#define FLAGOUT(x,pre,text,post) mbprintf(" %c %s%s%s%s%s\r\n", \
						(flags & x) ? '+' : '-', pre,on,text,off,post)
					mboutput("Flags einstellen:\r\n\n");
					FLAGOUT(B_ANONYM,"","A","nonym");
					FLAGOUT(B_GEHEIM,"","G","eheim");
					FLAGOUT(B_MITGLIED,"","S","chreibermitgliedschaft");
					FLAGOUT(B_LESMITGLIED,"Leser","M","itgliedschaft");
					FLAGOUT(B_ZAHLER,"","$"," Zahler drfen lesen");
					FLAGOUT(B_PASSWORT,"","P","asswort");
					FLAGOUT(B_ZWANG,"","Z","wang");
					FLAGOUT(B_DEFAULT,"","D","efault");
					FLAGOUT(B_MODERIERT,"M","O","deriert");
					FLAGOUT(B_ACL,"","A","CL");
					FLAGOUT(B_PAYWRITE,"Pa","Y","write");
					mboutput("\r\n");
					mbprintf("   %sEscape%s beendet\r\n",on,off);
					mboutput("\r\n");
#undef FLAGOUT
					while (TRUE)
					{
						BOOLEAN done=FALSE;
						x=mbchar();
						if (x=='E' ||x=='e' || x=='\033' || logout_modus)
							break;
						x=toupper(x);
						switch(x)
						{
#define FLAGIN(x,y) case x: done=TRUE; \
	if(flags&y) {flags&=~(y);} else {flags|=(y);}; break
						FLAGIN('A',B_ANONYM);
						FLAGIN('G',B_GEHEIM);
						FLAGIN('S',B_MITGLIED);
						FLAGIN('M',B_LESMITGLIED);
						FLAGIN('$',B_ZAHLER);
						FLAGIN('P',B_PASSWORT);
						FLAGIN('Z',B_ZWANG);
						FLAGIN('D',B_DEFAULT);
						FLAGIN('O',B_MODERIERT);
						FLAGIN('C',B_ACL);
						FLAGIN('Y',B_PAYWRITE);
#undef FLAGIN
						}
						if (done)
							break;
					};
					if (logout_modus)
						flags=b->flags;
					if (x=='E' || x=='e' || x=='\033')
						break;
				};
				if (flags!=b->flags)
				{
					changes++;
					b->flags=flags;
				}
			}
			break;
		case 'W': /* Passwort */
			strcpy(str,b->passwort ? b->passwort : "");
			if (!mbfrage(10,MBF_LINE|MBF_TRIM," Passwort: ",str)) {
				continue;
			}
			if (*str)
				strcpy(b->passwort,str);
			break;
		case 'C': /* Chef */
			old_chef=b->grp_chef;
			if (!neu)
				strcpy(str,b_verwalter_S(b->nr));
			else
				*str=0;

			while (TRUE)
			{
				USERNR n;
				if (!mbfrage(70,MBF_LINE|MBF_TRIM,"> ",str)) {
					break;
				}
				if (!*str)
					break;
				if (ub_find_username(&n,str))
				{
					b->grp_chef=n;
					break;
				}
				else
				{
					mbfehler(" unbekannter Benutzer\r\n");
				}
			} /* while */
			if (!logout_modus && b->grp_chef != old_chef)
				changes++;
			break;
		case 'O': /* Followup-To-Default */
			*str2=0;
			if (b->followup_to_default)
			{
				if (b->followup_to_default>0 
					&& b->followup_to_default <max_brettnr)
				{
					strcpy(str2,br_ind[b->followup_to_default].brett->name);
				}
			}
			strcpy(str,str2);
			while (TRUE)
			{
				BRETTNR fb;
				if (!mbfrage(70,MBF_LINE|MBF_TRIM,"> ",str)) {
					break;
				}
				if (strcmp(str2,str)==0)
					break;
				if (!*str)
				{
					changes++;
					b->followup_to_default=0;
					break;
				}
				fb=brettnr(str);
				if (fb==0)
				{
					mboutput(" Unbekannte Gruppe!\r\n");
					continue;
				}
				changes++;
				b->followup_to_default=fb;
				break;
			}
			break;
		case 'K': /* Userinfo, einzeilig */

			if (!b->userinfo)		
			{
				char netbuf[1024]="";
				(void) net_itg_info(b->name,netbuf,NULL);
				if (*netbuf)
					b->userinfo=XSTRDUP(netbuf);
			}
			strcpy(str,b->userinfo ? b->userinfo : "");
			if (!mbfrage(70,MBF_LINE|MBF_TRIM,": ",str)) {
				break;
			}
			if (b->userinfo)
			{
				if (strcmp(b->userinfo,str)!=0)
				{
					free(b->userinfo);
					b->userinfo=XSTRDUP(str);
					changes++;
				}
			}
			else if (!b->userinfo && *str)
			{
				b->userinfo=XSTRDUP(str);
				changes++;
			}
			break;
		case 'Y': /* Sysopinfo, einzeilig */
			if (!b->sysopinfo)		
			{
				char netbuf[1024]="";
				(void) net_itg_info(b->name,NULL,netbuf);
				if (*netbuf)
					b->sysopinfo=XSTRDUP(netbuf);
			}
			strcpy(str,b->sysopinfo ? b->sysopinfo : "");
			if (!mbfrage(70,MBF_LINE|MBF_TRIM,": ",str)) {
				break;
			}
			if (b->sysopinfo)
			{
				if (strcmp(b->sysopinfo,str)!=0)
				{
					free(b->sysopinfo);
					b->sysopinfo=XSTRDUP(str);
					changes++;
				}
			}
			else if (!b->sysopinfo && *str)
			{
				b->sysopinfo=XSTRDUP(str);
				changes++;
			}
			break;
		case 'G': /* Lange Gruppenbeschreibung */
			if (1)
			{
				FILE *f;
				char datei[PATH_MAX];
				char *s;
				sprintf(datei,"%s%s",tom_tmp_dir,"b_info.txt");

				f=xfopen(datei,"w");
				s=b->lokalinfo;
				/* b->lokalinfo darf verndert werden, es wird spter auf
				 * jeden Fall aus der Datei gelesen */

				while (s && *s)
				{
					char *p=strpbrk(s,"\r\n");
					if (p)
					{
						*p++='\0';
						if (*p=='\n')
							p++;
					}
					fprintf(f,"%s\n",s);
					s=p;
				}
				fclose(f);
				call_editor(datei, FALSE);
				f=fopen(datei,"r");
				if (!f)
				{
					if (b->lokalinfo)
					{
						free(b->lokalinfo);
						b->lokalinfo=NULL;
						changes++;
					}
				}
				else
				{
					char buf[1024];
					free(b->lokalinfo);
					b->lokalinfo=NULL;
					while (fgets(buf,sizeof(buf),f))
					{
						char *old;
						char *p;
						p=strpbrk(buf,"\r\n");
						if (p)
							*p='\0';
						old=b->lokalinfo;
						if (b->lokalinfo)
							b->lokalinfo=concat(b->lokalinfo,XSTRDUP(buf),"\n",NULL);
						else
							b->lokalinfo=concat(buf,"\n",NULL);
						XNULLCHECK(b->lokalinfo);
						free(old);
					} /* while */
					fclose(f);
				}
			}
		case 'H': /* Herkunftsnetz */
			mboutput(" Ursprungsnetz des Brettes:\r\n");
			changes+=bedt_select_from_list("Netz : ", &b->ursprungsnetz,
				bb_ursprungsnetz);
			break;
		case 'P':
			mboutput(" Sprache des Brettes:\r\n");
			changes+=bedt_select_from_list("Sprache : ", &b->sprache,
				bb_sprache);
			break;
		}
	};

	if (!neu && !changes)
	{
		mboutput(" Keine nderung!\r\n\r\n");
		return FALSE;
	}


	if (neu)
		mboutput ("Neues Brett speichern? (J/N):");
	else
		mboutput ("nderungen speichern? (J/N):");
	if (1!=mbdecide("JN","Ja\r\n|Abbruch\r\n"))
		return FALSE;

	return TRUE;
}


static void
be_bestellen(BRETTNR bnr, const char *name)
{
	/* Trick 17 mit Gefahr der Selbstberlistung! */
	char buf[128];
	char fname[PATH_MAX];
	USERNR unr;
	FILE *f;

	if (!mbfrage(64,MBF_LINE|MBF_TRIM," Bei+Fr > ",buf)) {
		return;
	}
	if (!*name)
		return;

	if (!ub_find_username(&unr,buf))
	{
		mbfehler("Unbekannter Benutzer");
		return;
	}

	/*
	 * Zuerst mal bei dem Server bestellen.
	 */
	/* ### das mu gelockt werden! */
	sprintf(fname,"%s%lu.once",box_conf.tausch_dir,(ULONG) unr);
	f=fopen(fname,BINAPPEND);
	if (!f)
	{
		syslog(LOG_ERR,"cannot write to %s: %s",fname,strerror(errno));
		mbfehler("kann Bestellung nicht speichern");
		return;
	}
	fprintf(f,"G+%s\r\n", name);
	fclose(f);

	/*
	 * Nun fr den Server bestellen
	 */
	sprintf(fname,"%s%lu.tau",box_conf.tausch_dir,(ULONG) unr);
	if (!fileexist(fname))
	{
		mbfehler("Zuerst mu TAUSCH INST fr den Server aufgerufen werden");
		return;
	}
	f=fopen(fname,BINAPPEND);
	if (!f)
	{
		syslog(LOG_ERR,"cannot write to %s: %s",fname,strerror(errno));
		mbfehler("kann Bestellung nicht speichern");
		return;
	}
	fprintf(f,"+ %lu %s\r\n",
		(ULONG) bnr,
		name);
	fclose(f);
}

/* gemeinsame Routine von Brett anlegen und editieren  */
static BOOLEAN
bedt_common_edit(BRETTNR bnr, char *name)
{
	BOOLEAN angelegt=(bnr!=0);
	char *oldname;
	BOOLEAN neu=!bnr;
	BRETT b;
	BOOLEAN retcode=FALSE;

	if (!bnr)
	{
		/* Brett mu neu angelegt werden */
		b_default_brett(&b);
		strcpy(b.name,name);
	}
	else
	{
		/* wir arbeiten hier auf einer Kopie! */

		b=*br_ind[bnr].brett;
		if (b.lokalinfo)
			b.lokalinfo=XSTRDUP(b.lokalinfo);
		if (b.sysopinfo)
			b.sysopinfo=XSTRDUP(b.sysopinfo);
		if (b.userinfo)
			b.userinfo=XSTRDUP(b.userinfo);
		if (b.aliasname)
			b.aliasname=XSTRDUP(b.aliasname);
	}
	oldname=XSTRDUP(b.name);

nochmal:
	if (!brettedt_edit(!angelegt,&b))
	{
		bb_unlock_brett(bnr);
		goto cleanup;
	}

	/* anlegen */
	if (!angelegt)
	{
		if (!b_create_brett(&b))
		{
			mboutput("Brett kann nicht angelegt werden!\r\n");
			mboutput("Wollen Sie es nochmal versuchen? (J/N)");
			if (1==mbdecide("JN","Ja\r\n|Abbruch\r\n"))
				goto nochmal;
			goto cleanup;
		}
		angelegt=TRUE;
		bnr=b.nr;
	}

#ifdef HAVE_DOSDRIVES
	if (!neu)
	{
		mboutput("kopiere Dateien im Brett ...");
		ordner_daten(tprintf("%s*.*",brettpfad(bnr)),FALSE,NULL,&byte_summe);
		if (byte_summe < bytes_free(drive_zahl(name))+100000L)
		{
			if (!copy_brett(bnr,name))
			{
				/* Keine Ahnung, wie man das wieder rckgngig machen sollte
				 * von hier aus */
				mbfehler("Ich kann die Daten nicht kopieren! Datenverlust!");
				goto cleanup;
			}
			mboutput(" OK\r\n\n");
			b.partition = name[0] - 'A';
		}
		else
		{
			mbfehler("Auf der Ziel-Partition ist zuwenig Speicherplatz vorhanden");
			goto cleanup;
		}
	}
#endif

	if (!neu)
	{
		b.configaenderung=time(NULL);

		bef_brett_sortierung_maybe_wrong=TRUE;
		if (br_ind[bnr].brett->lokalinfo)
			free(br_ind[bnr].brett->lokalinfo);
		if (br_ind[bnr].brett->userinfo)
			free(br_ind[bnr].brett->userinfo);
		if (br_ind[bnr].brett->sysopinfo)
			free(br_ind[bnr].brett->sysopinfo);
		if (br_ind[bnr].brett->aliasname)
			free(br_ind[bnr].brett->aliasname);

		*br_ind[bnr].brett=b;
		if (b.sysopinfo)
			br_ind[bnr].brett->sysopinfo=XSTRDUP(b.sysopinfo);
		else
			br_ind[bnr].brett->sysopinfo=0;
		if (b.aliasname)
			br_ind[bnr].brett->aliasname=XSTRDUP(b.aliasname);
		else
			br_ind[bnr].brett->aliasname=0;
		if (b.lokalinfo)
			br_ind[bnr].brett->lokalinfo=XSTRDUP(b.lokalinfo);
		else
			br_ind[bnr].brett->userinfo=0;
		if (b.userinfo)
			br_ind[bnr].brett->userinfo=XSTRDUP(b.userinfo);
		else
			br_ind[bnr].brett->userinfo=0;
		/* unlocken */
		if (!bb_write_brett(bnr,TRUE,TRUE))
		{
			mboutput("Brett kann nicht gespeichert werden!\r\n");
			mboutput("Wollen Sie es nochmal versuchen? (J/N)");
			if (1==mbdecide("JN","Ja\r\n|Abbruch\r\n"))
				goto nochmal;
			goto cleanup;
		}
		b_set_flags_brett(&user,bnr);
		if (B_MITLISTE(b.flags))
			b_add_access(bnr,b.grp_chef);
		else
			unlink(tprintf("%s%s",brettpfad(bnr),"zugriff.dat"));
	}
	mboutput(" OK\r\n\n");
	mbprintf(" %sOK%s\r\n\n",on,off);

	if (neu || strcmp(oldname,b.name) != 0)
	{
		char buf[sizeof(b.name)*2+1];
		if (!neu)
		{
			/* irgendwie umbenannt */
#ifdef OFFLINE_VERSION
			mdb_rename_group(oldname,b.name);
#else
			mdb_rename_group(oldname,b.name,FALSE);
#endif
			ta_add_rename_info(time(NULL),oldname,b.name);
		}
		free(br_ind[bnr].canon);
		bb_canonical(b.name, buf);
		br_ind[bnr].canon=XSTRDUP(buf);
		b_free_sortierung();
	}
	b_set_flags_brett(&user,bnr);
	retcode=TRUE;

cleanup:
	/* die drei Strings sind Kopien */
	if (b.lokalinfo)
		free(b.lokalinfo);
	if (b.userinfo)
		free(b.userinfo);
	if (b.sysopinfo)
		free(b.sysopinfo);
	if (b.aliasname)
		free(b.aliasname);
	free(oldname);
	return retcode;
}




#ifdef HAVE_DOSDRIVES
static WORD drive_zahl(char *name)
{
	return *name-64;
}



/*
	BUG:
	Behandlung von Textbrettern fehlt noch
*/
static BOOLEAN copy_brett(BRETTNR bnr,char *drive)
{
	WORD nW;
	long file_anzL = 0L;
	char folderS[PATH_MAX];
	BOOLEAN ok = TRUE;

	if (br_ind[bnr].brett->art != BINAER_BRETT)
	{
		mboutput(" Derzeit knnen nur Binrbretter automatisch verschoben werden\r\n");
		return FALSE;
	}

	if (!strstr(installed_drivesS,drive))
		strcat(installed_drivesS,drive);

	mboutput(" -> lege neues Brettverzeichnis an");
	mkdir(tprintf("%s:%s",drive,box_conf.binbrett_dir),UMASK);
	mkdir(tprintf("%s:%s%03ld",drive,box_conf.binbrett_dir,bnr),UMASK);
	mboutput(".");
	mboutput(" OK\r\n\n");

	file_anzL = anzahl_dateien(brettpfad(bnr),"*.*");
	if (file_anzL>0)
	{
		const char *mv=getenv("MV");
		if (system(NULL))
		{
			mboutput(" kopiere ...");
			system(tprintf("%s %s%s %s:/tom/",
				mv ? mv : "mv", brettpfad(bnr),"*.*",drive));
		}
		else
		{
#if 0 /* funktionsunfhig */
			mboutput(" kopiere (intern) ... ");
			for (nW = 0; nW < file_anzL; nW++)
			{
				char rad[] = "-\\|/";
				char src[PATH_MAX],
					dst[PATH_MAX];

				sprintf(src,"%s%s",brettpfad(bnr),file_nameS_f.p.S2[nW]);
				sprintf(dst,"%s/%03ld/%s",box_conf.binbrett_dir,bnr,
						file_nameS_f.p.S2[nW]);
				file_move(src,dst);*/
				mbprintf("%c\b",rad[nW % 4]);
			}
#else
			mbfehler("keine Shell fr Kopierfunktion verfgbar");
			ok=FALSE;
#endif
		}
		mboutput("\r\n");
	}
	else
		mboutput(" Keine Dateien umzukopieren");

	mboutput("\r\n -> lsche alten Brett-Ordner");
	strcpy(folderS,brettpfad(bnr));
	folderS[strlen(folderS)-1] = 0;

	rmdir(folderS);

	mboutput(".");
	mboutput(". OK\r\n");

	return ok;

} /* copy_brett */
#endif

static BOOLEAN
net_itg_info(const char *name,char *u_info,char *s_info)
{
    BOOLEAN ok = FALSE;
    char line[256];
    FILE *itg_f;
    char *p;

    if ((itg_f = fopen(tprintf("%sITG",tom_info_net_dir),"r")) == NULL)
        return FALSE;
    while (fgets(line,sizeof(line),itg_f))
    {
        if (*line != 'G')
            continue;

        if ((p = strpbrk(line+1,"\r\n")) != NULL)
            *p = 0;

        if (stricmp(line+1,name))
            continue;

        if (u_info)
            *u_info = 0;
        if (s_info)
            *s_info = 0;
        while (fgets(line,sizeof(line),itg_f) && *line!='G')
        {
            if ((p = strpbrk(line,"\r\n")) != NULL)
                *p = 0;

            if (*line=='U' && u_info)
                strcpy(u_info,line+1);
            if (*line=='S' && s_info)
                strcpy(s_info,line+1);
        }
        ok = TRUE;
        break;
    }
    fclose(itg_f);
    return ok;

} /* brett_itg_info */

static int
bedt_select_from_list(const char *titel, UWORD *wert, const char * (*fn) (UWORD))
{
	int zahl,i;
	char str[40]="";
	long nr;

	while (!logout_modus)
	{
		mboutput("\r\n");
		for (zahl=0;;zahl++)
		{
			const char *s;
			s=fn(zahl);
			if (!s)
				break;
			mbprintf(" %d %s\r\n",zahl+1,s);
		}
		mboutput("Bitte whlen Sie\r\n");
		if (!mbfrage(sizeof(str)-1,MBF_LINE|MBF_TRIM,titel,str)) {
			return 0;
		}
		if (!*str)
			return 0;
		nr=strtol(str,NULL,10);
		if (nr >0 && nr<=zahl)
		{
			if (*wert!=nr-1)
			{
				mbprintf(" %s eingestellt\r\n",fn(nr-1));
				*wert=nr-1;
				return 1;
			}
			else
				return 0;
		}
		for (i=0;;i++)
		{
			const char *s;
			s=fn(i);
			if (!s)
				break;
			if (!stricmp(s,str))
			{
				if (*wert!=i)
				{
					*wert=i;
					mbprintf(" %s eingestellt\r\n",fn(i));
					return 1;
				}
				else
					return 0;
			}
		}
		mbprintf(" Nicht bekannt!\r\n");
	}

	return 0;
}
