/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <errno.h>

#include "befehle.h"
#include "prototyp.h"
#include "modem.h"
#include "terminal.h"
#include "user.h"
#include "brett.h"
#include "bin_msg.h"
#include "bin_out.h"
#include "business.h"
#include "box_conf.h"
#include "tompaths.h"

extern USER user;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;

extern BYTE
batchB,
offlineB,
pm_schutzB;

extern long
actual_downloadL;

struct dl_liste {
	ULONG nr;
	struct dl_liste *next;
} *anker;


static int download(BIN_MSG *bin_info,BOOLEAN auto_download);



BOOLEAN bef_download(const char *para)
{
	set_status_line("Dateien downloaden");

	return download_intern(para);
}



/*
	diese Funktion wird bentigt, um auch aus anderen Programmteilen (im 
	Source) Downloads vornehmen und auf das Ergebnis reagieren zu knnen.
	z.B.:
	- TAUSCH-Download
	- globales SUCHEN

	->			para			:	normaler Befehlsparameter

	<-			DOWN_OK		:	Download wurde erfolgreich durchgefhrt
				DOWN_WEITER	:	kein Download, Aufrufer kann aber weitermachen
				DOWN_ABBRUCH:	Abbruch weiterer Downloads

	<-			TRUE	:	ggf. weitere Downloads erwnscht/durchfhrbar
				FALSE	:	Abbruch
*/
BOOLEAN download_intern(const char *para)
{
	char par[80];
	BOOLEAN paraB = FALSE;
	BOOLEAN auto_download = FALSE;
	BOOLEAN old_schutz;
	WORD parW = 0;
	int status = 0;
	long all,neu;
	long next;
	FILE *bin_f;
	int ok;
	long nr;

	strncpy(par,para,80);

	if (brettart(brettnrW) != BINAER_BRETT)
	{
		mbfehler("Download nur in Binrbrettern mglich");
		return TRUE;
	}

	if (!(br_ind[brettnrW].brett->flags & B_READ))
	{
		mbfehler("Download hier nicht erlaubt");
		return TRUE;
	}

	if (!binaer_dat_anzahl(brettnrW,0,&all,&neu,NULL,NULL))
		return TRUE;
	if (!all)
	{
		mboutput(" Keine Eintrge vorhanden\r\n\n");
		return TRUE;
	}

	if (!check_business('d',&next,NULL))
	{
		if (user.level == 0)
			mboutput(" Gast-");
		else
			mboutput(" ");
		if (next)
			mbprintf("Downloads erst wieder ab %ld:%02ld Uhr",next / 60,next % 60);
		else
			mboutput("Downloads sind zur Zeit leider nicht");
		mboutput(" erlaubt (s. Infodatei IIL)\r\n\n");

		return FALSE;
	}

	if (strlen(par) > 2 && !stricmp(par+strlen(par)-2," Z"))
	{
		par[strlen(par)-2] = '\0';
		strtrim(par);
		auto_download = TRUE;
	}

	/* Gste bekommen alles als neu angezeigt */
	if (!*par && !neu && level())
	{
		mboutput(" Keine neuen Eintrge vorhanden\r\n\n");
		return TRUE;
	}

/*lesen*/
/*	if (!*par)
	{
		mbprintf(" Kein Eintrag mit '%s' gefunden\r\n",para_altS);
		return TRUE;
	}*/
/*ende*/

	if (!*par)
	{
		strcpy(par,"*");
		paraB = TRUE;
		parW = 1;
	}

	old_schutz=pm_schutzB;
	if (brettnrW==0)
		pm_schutzB = TRUE;

	/* Schreibzugriff wegen Zugriffszhlung */
	if ((bin_f = binaer_dat_oeffnen(brettnrW,TRUE)) == NULL)
	{
		pm_schutzB = old_schutz;
		return TRUE;
	}

	for (ok = lrange(par,&nr); ok && nr > 0; ok = lrange(NULL,&nr))
	{
		BIN_MSG bin_info;

		if (nr <= 0)
			continue;
		if (nr > all)
			break;

		paraB = TRUE;
		fseek(bin_f,(nr-1)*sizeof(BIN_MSG),SEEK_SET);
		fread(&bin_info,sizeof(BIN_MSG),1,bin_f);

		if (bin_info.deleted)
			continue;

		if (bin_read_access(&bin_info)
		&& (!parW || bin_info.datime > user.letzter_login))
		{
			bin_msg_header(&bin_info);
			if (!fileexist(tprintf("%s%ld.bin",brettpfad(brettnrW),bin_info.nr)))
				mbfehler("Datei nicht vorhanden");
			else
			{
				char auswahl;
				struct mbselect_t sel[]=
				{ 
					{ 'A' , "Abbruch" },
					{ 27  , "Abbruch" },
					{ 'S' , "Spter" },
					{ 'D' , "Download" },
					{ 'W' , "Weiter" },
					{ 13  , "Weiter" },
					{ 'N' , "Nicht" },
					{ 0   , NULL }
				};
				mboutput("(D)ownload, (S)pter, (A)bbruch, CR/(W)eiter:");
				auswahl=mbselect(sel);
				mboutput("\r\n");
				if (auswahl=='A' || auswahl==27)
					break;
				if (strchr("NW\r\n",auswahl))
					continue;
				if (auswahl=='S')
				{
					struct dl_liste *n=XMALLOC(sizeof(struct dl_liste));
					n->nr=bin_info.lokal;
					n->next=anker;
					anker=n;
					continue;
				}
					
				status = download(&bin_info,auto_download);
				if (status == 1)
					continue;
				else if (status == 2)
					break;

				/* Downloadzhler erhhen */
				bin_info.zugriffe++;
				/* letzten Downloadzeitpunkt merken */
				bin_info.last_download = time(NULL);
				fseek(bin_f,(nr-1)*sizeof(BIN_MSG),SEEK_SET);
				fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
			}
		}
	}
	
	/* fr (S)pter markierte Files downloaden */
	if (anker)
	{
		char name[PATH_MAX];
		char art='\0';
		char quelle[PATH_MAX];
		char *namebuf=NULL;
		size_t namebuflen=0;
		size_t namebufsize=0;
		time_t need_time=0;
		ULONG files_to_send=0;
		ULONG bytes_to_send=0;
		struct dl_liste *next_ele;

		rm_dir_content(tom_utmp_dir);

		while (anker)
		{
			BIN_MSG bin_info;
			time_t zeit_dieses;
			char dl_time[30];

			fseek(bin_f,(anker->nr-1)*sizeof(BIN_MSG),SEEK_SET);
			fread(&bin_info,sizeof(BIN_MSG),1,bin_f);

			sprintf(name,"%s%ld.bin",brettpfad(brettnrW),bin_info.nr);

			strcpy(dl_time,download_time_S(brettnrW,bin_info.nr));
			zeit_dieses=download_time(bin_info.bytes);

			mbprintf("  %s (%lu Bytes",bin_info.name,bin_info.bytes);
			if (zeit_dieses)
				mbprintf(", ca. %s Min:Sec\r\n",dl_time);
			mboutput(")\r\n");

			if (zeit_dieses + need_time > restzeit() 
				&& (!user.bezahlt || !box_conf.paid_download_unlimited))
				mboutput(" Zum Download ist leider zuwenig Onlinezeit vorhanden\r\n\n");
			else
			{
				BOOLEAN this_ok=FALSE;
				char tmp_file[PATH_MAX];
				const char *s;

				sprintf(tmp_file,"%s%s",tom_utmp_dir,bin_info.name);

				/* absoluter Pfad wegen symlink! */
				s=brettpfad(brettnrW);
				if (*s=='/')
					sprintf(quelle,"%s%ld.bin", s,bin_info.nr);
				else
					sprintf(quelle,"%s%s%ld.bin",
							box_conf.quark_dir,s,bin_info.nr);
#if defined(HAVE_SYMLINK)
				if (symlink(quelle,tmp_file)==0)
						this_ok=TRUE;
#endif
				if (!this_ok)
					this_ok = file_copy(quelle,tmp_file);
				if (this_ok)
				{
					size_t l=strlen(tom_utmp_dir)+strlen(bin_info.name);
					if (l+namebuflen+1 >= namebufsize)
					{
						namebufsize += PATH_MAX*2;
						namebuf=XREALLOC(namebuf,namebufsize);
					}
					sprintf(namebuf+namebuflen," %s%s",tom_utmp_dir,bin_info.name);
					namebuflen += l+1;
					need_time += zeit_dieses;
					files_to_send++;
					bytes_to_send += bin_info.bytes;
				}
			} /* pat die Datei noch? */
			next_ele = anker->next;
			free(anker);
			anker=next_ele;
		} /* while (anker) */

		if (!files_to_send)
			mboutput("Keine Dateien zu versenden\r\n");
		else
		{
			/* Wenn der User ein Defaultprotokoll eingestellt hat *und* 
			 * das fr die aktuelle Schnittstelle auch erlaubt ist
			 */
			if (user.default_protokoll 
				&& transfer_protokollcheck(user.default_protokoll))
				art=user.default_protokoll;
			status=transfer_send(&art,namebuf);
			rm_dir_content(tom_utmp_dir);
			if (status==0)
			{
				user.dl_bytes    += bytes_to_send;
				actual_downloadL += bytes_to_send;
			}
			else
				status=2;
		}
	} /* if (anker) */

	binaer_dat_schliessen(bin_f);

	if (!paraB)  parameter_falsch("DOWNLOAD",para);

	pm_schutzB = old_schutz;

	if (status == 2)
		return FALSE;
	else
		return TRUE;
}



/*
	Datei downloaden

	->			bin_info	:	Binrindex-Datensatz

	<-			0			:	Download ok
				1			:	Weiter
				2			:	Abbruch
*/
static int download(BIN_MSG *bin_info,BOOLEAN auto_download)
{
	char dl_time[30];
	char name[PATH_MAX];
	char tmp_file[PATH_MAX];
	char art='\0';
	char quelle[PATH_MAX];
	BOOLEAN done=FALSE;
	int ret;
	const char *bf;

	if (offlineB)
		return 2;

	sprintf(name,"%s%ld.bin",brettpfad(brettnrW),bin_info->nr);

	sprintf(tmp_file,"%s%s",tom_utmp_dir,bin_info->name);
	strcpy(dl_time,download_time_S(brettnrW,bin_info->nr));

	mbprintf("\r\n Download-Zeit, ca. %s Min:Sec\r\n",dl_time);

	if (download_time(bin_info->bytes) > restzeit()
			&& (!user.bezahlt || !box_conf.paid_download_unlimited))
	{
		mboutput(" Zum Download ist leider zuwenig Onlinezeit vorhanden\r\n\n");
		return 1;
	}

	/* absoluter Pfad wegen symlink */
	bf=brettpfad(brettnrW);
	if (*bf=='/')
		sprintf(quelle, "%s%ld.bin",bf,bin_info->nr);
	else
		sprintf(quelle, "%s%s%ld.bin",box_conf.quark_dir,bf,bin_info->nr);
#if defined(HAVE_SYMLINK)
	if (symlink(quelle,tmp_file))
		strerror(errno);
	else
		done=TRUE;
#endif
#ifdef HAVE_LINK
	if (!done)
	{
		if (link(quelle,tmp_file))
			strerror(errno);
		else
			done=TRUE;
	}
#endif
	if (!done)
		file_copy(quelle,tmp_file);

	/* Wenn der User ein Defaultprotokoll eingestellt hat *und* 
	 * das fr die aktuelle Schnittstelle auch erlaubt ist
	 */
	if (user.default_protokoll 
	&& transfer_protokollcheck(user.default_protokoll))
		art=user.default_protokoll;
	ret=transfer_send(&art,tmp_file);
	unlink(tmp_file);

	if (ret==0)
	{
		user.dl_bytes    += bin_info->bytes;
		actual_downloadL += bin_info->bytes;
		return 0;
	}
	return 1;

} /* download */
