/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
 * befehle/edit.c:
 * ed text: texte editieren
 * ed brett: bretteditor aufrufen
 * ed user: Usereditor aufrufen
 * ed access: Mitgliederlisten editieren
 * ed include: Massenupload von Files - hier vllig deplaziert.
 * ed info: Brettinfotext editieren
 */

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h> /* mkdir */
#include <dirent.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "befehle.h"
#include "brett.h"
#include "user.h"
#include "modem.h"
#include "msg.h"
#include "bin_msg.h"
#include "tompaths.h"
#include "box_conf.h"

static void edit_access(void);
static void edit_acl(void);
static void edit_include(void);
static void edit_brettinfo(void);
static void edit_text(void);
static void edit_dateiinfo(const char *para);

extern BRETT akt_brett;
extern BRETTNR brettnrW;

extern USER user;

extern long
actual_uploadL;



BOOLEAN bef_editieren(const char *para)
{
	if (!strnicmp(para,"BRETT",5))
	{
		const char *p=para+5;
		set_status_line("Bretteditor");
		while (isspace(*p))
			p++;
		bretteditor(*p ? p : NULL);
	}
	else if (!strnicmp(para,"USER",4))
	{
		const char *p=para+4;
		set_status_line("Usereditor");
		while (isspace(*p))
			p++;
		usereditor(*p ? p : NULL);
	}
	else if (!stricmp(para,"ACCESS"))
	{
		edit_access();
	}
	else if (!stricmp(para,"ACL"))
	{
		edit_acl();
	}
	else if (!stricmp(para,"INCLUDE"))
	{
		edit_include();
	}
	else if (!stricmp(para,"INFO"))
	{
		edit_brettinfo();
	}
	else if (!stricmp(para,"TEXT"))
	{
		edit_text();
	}
	else if (*para)
	{
		edit_dateiinfo(para);
	}
	else
	{
		parameter_falsch("EDITIEREN",para);
		return FALSE;
	}

	return TRUE;
}



/*
	Brett-Zugriffsliste editieren

	19.11.92 Frank

	Vorgehensweise:
	1) Export der binren Zugriffsliste als Textdatei
	2) Aufruf des Editors mit Mglichkeit zur nderung
	3) Umwandlung der Textdate in binre Zugriffsliste
*/
static void edit_access(void)
{
	char *p;
	char datei[PATH_MAX];

	if (!brettnrW)
	{
		mbfehler("Post- und Programmfcher besitzen keine explizite Zugriffsliste");
		return;
	}

	if (!B_MITLISTE(akt_brett.flags))
	{
		mbfehler("Brett ist nicht durch Userliste geschtzt");
		return;
	}

	set_status_line("Zugriffsliste, (bitte Usernamen korrekt eintragen)");

	strcpy(datei,brettpfad(brettnrW));

	p=strlen(datei)+datei-1;
	if (p && *p=='/')
		*p=0;

	mkdir(datei,S_IRWXU | S_IRGRP | S_IROTH);

	b_export_access(brettnrW);

	sprintf(datei,"%s%s",brettpfad(brettnrW),"zugriff.lst");
	call_editor(datei, TRUE);

	b_import_access(brettnrW);
} /* edit_access */


static void edit_acl(void)
{
	char *p;
	char datei[PATH_MAX];

	if (!brettnrW)
	{
		mbfehler("Post- und Programmfcher besitzen keine explizite Zugriffsliste");
		return;
	}

	if (!B_MITLISTE(akt_brett.flags))
	{
		mbfehler("Brett ist nicht durch Userliste geschtzt");
		return;
	}

	set_status_line("Zugriffsliste, (bitte Usernamen korrekt eintragen)");

	strcpy(datei,brettpfad(brettnrW));
	p=strlen(datei)+datei-1;
	if (p && *p=='/')
		*p=0;
	mkdir(datei,S_IRWXU | S_IRGRP | S_IROTH);

	sprintf(datei,"%s%s",brettpfad(brettnrW),"acl.lst");

	call_editor(datei, TRUE);
	acl_invalidate_cachefile(datei);

} /* edit_acl */

/*
	Einfgen von Dateien in Binrbretter

	21.01.93 Frank, ausschlielich in Binrbrettern erlaubt
*/
static void edit_include(void)
{
	BOOLEAN abbruchB = FALSE;
	WORD decideW;
	long bytes;
	long anzahl;
	char betreff[80];
	char empfaenger[80];
	char file[20];
	char pfad[PATH_MAX];
	char pfad_file[PATH_MAX];
	char tmp[PATH_MAX];
	DIR *dir;
	struct dirent *dp;

	if (brettart(brettnrW) != BINAER_BRETT)
	{
		mbfehler("Nur in Binrbrettern erlaubt");
		return;
	}

	set_status_line("Dateien einfgen");
	mbquestion(60,0,TRUE," Dateinamen: ","",pfad);
	mboutput("\r\n");

	if (!strchr(pfad,'*') && !fileexist(pfad))
	{
		mbfehler("Pfad bzw. Dateiname existiert nicht");
		return;
	}

	ordner_daten(pfad,FALSE,&anzahl,&bytes);
	mbprintf(" Gesamt[kB]: %ld\r\n",bytes / 1000);

	if (anzahl > 1)
	{
		mbprintf(" -> Es wurden %ld Dateien gefunden\r\n",anzahl);
		mboutput(" >> Einzelabfrage? (J/N) : ");
		decideW = mbdecide("JN"," Ja ...\r\n\n|Nein\r\n\n");
		if (decideW==2)
		{
			ordner_daten(pfad,TRUE,&anzahl,NULL);
			mboutput("\r\n");
			/* NOPLENK-START */
			if (brettnrW==0)
				mbquestion(60,2,TRUE," Empfnger : ","",empfaenger);
			else
				*empfaenger = 0;
			mbquestion(55,0,TRUE," Betreff    : ","",betreff);
			/* NOPLENK-END */
			mboutput("\r\n");
		}
		else
			*betreff = 0;
	}
	else
	if (anzahl == 1)
		decideW = 1;
	else
	{
		mboutput(" Es wurden keine Dateien gefunden\r\n");
		return;
	}

	if (!*betreff && decideW==2)
		return;

	strcpy(tmp,pfad);

	dir = opendir(pfad);
	while (!abbruchB && (dp = readdir(dir)) != NULL)
	{
		BIN_MSG bin_info;
		long zahl;

		/*
			feststellen, wiviele Uploads bereits in dem Brett liegen (iL),
			damit die Brett-lokale Nummerierung (s.u. bin_info.lokal) die
			aufsteigenden Nummern verwenden kann.
			Im Moment wird diese Funktion vor jeder einzelnen Datei aufge-
			rufen, denn zwischenzeitlich knnte an einem anderen Port jemand
			ins gleiche Brett uploaden.
			Das ist natrlich keine zuverlssige Methode, um doppelt verge-
			bene, lokale Nummern zu vermeiden, aber besser, als die nur ein-
			malige Ermittlung der Anzahl
		*/
		binaer_dat_anzahl(brettnrW,0,&zahl,NULL,NULL,NULL);

		if (dp->d_name[0] == '.')
			continue;

		strcpy(file,dp->d_name);
		sprintf(pfad_file,"%s/%s",tmp,file);

		memset(&bin_info,0,sizeof(bin_info));
		bin_info.lokal = zahl + 1;										/* ??? */
		bin_info.brettnr = brettnrW;
		bin_info.usernr = user.nummer;
		bin_info.zugriffe = 0;
		strncpy(bin_info.name,file,60);
		bin_info.name[59] = 0;
		strncpy(bin_info.absender,user.name,64);
		bin_info.absender[63] = 0;
		bin_info.datime = time(NULL);

		mbprintf(" -> %s\r\n\n",pfad_file);
		if (decideW==1)
		{
			if (brettnrW==0)
				mbquestion(60,2,TRUE," Empfnger : ","",empfaenger);
			else
				*empfaenger = 0;
			*betreff = 0;
			mbquestion(55,0,TRUE," Betreff    : ","",betreff);
			mboutput("\r\n");
		}
		if (*betreff)
		{
			bytes = len_datei(pfad_file);
			bin_info.bytes = bytes;
#ifdef HAVE_DOSDRIVES
			if (bytes+100000L > bytes_free(akt_brett.partition))
			{
				mbfehler("Speicherplatz-Mangel");
				break;
			}
			else
#endif
			{
				char ziel[PATH_MAX];

				strncpy(bin_info.betreff,betreff,60);
				bin_info.betreff[59] = 0;
				bin_info.nr = get_new_id_nr(TRUE);
				sprintf(ziel,"%s%ld.bin",brettpfad(brettnrW),bin_info.nr);
				mbprintf(" kopiere %s nach %s\r\n",pfad_file,ziel);
				if (!file_copy(pfad_file,ziel))
					continue;
				mboutput("einsortieren ... ");
				binaer_dat_anhaengen(brettnrW,&bin_info);
				bb_update_brett(brettnrW);
				mboutput("ok\r\n\n");
				actual_uploadL += bytes;
			}
		}
	}
	closedir(dir);
}



/*
	Brett-Infodatei edieren

	19.11.92 Frank
*/
static void edit_brettinfo(void)
{
	char datei[PATH_MAX];
	FILE *f;
	char *s, *p;
	BRETT *b;

	set_status_line("Brett Infodatei (bitte max. 10 Zeilen):");

	if (!bb_read_brett(brettnrW,TRUE))
	{
		mbfehler("Brettdaten knnen nicht gelockt/gelesen werden");
		return;
	}
	b=br_ind[brettnrW].brett;
	sprintf(datei,"%s%s",tom_tmp_dir,"b_info.txt");

	f=xfopen(datei,"w");
	if (b->userinfo)
		fprintf(f,"%s\n",b->userinfo);
	else
		fprintf(f,"Keine Kurzbeschreibung vorhanden\n");
	s=b->lokalinfo;
	while (s && *s)
	{
		p=strpbrk(s,"\r\n");
		if (p)
		{
			*p++='\0';
			if (*p=='\n')
				p++;
		}
		fprintf(f,"%s\n",s);
		s=p;
	}
	fclose(f);

	call_editor(datei, FALSE);
	f=fopen(datei,"r");

	free(b->userinfo);
	free(b->lokalinfo);
	b->userinfo=NULL;
	b->lokalinfo=NULL;

	if (f)
	{
		int line=0;
		char buf[16384];
		free(b->userinfo);
		free(b->lokalinfo);
		b->userinfo=NULL;
		b->lokalinfo=NULL;
		while (fgets(buf,sizeof(buf),f))
		{
			p=strpbrk(buf,"\r\n");
			if (p)
				*p='\0';
			line++;
			if (line==1)
				b->userinfo=XSTRDUP(buf);
			else
			{
				char *old=b->lokalinfo;
				if (b->lokalinfo)
					b->lokalinfo=concat(b->lokalinfo,XSTRDUP(buf),"\n",NULL);
				else
					b->lokalinfo=concat(buf,"\n",NULL);
				XNULLCHECK(b->lokalinfo);
				free(old);
			}
		} /* while */
		fclose(f);
	}
	/* else: Die Infotexte wurden gelscht. */
	b->configaenderung=time(NULL);
	if (!bb_write_brett(brettnrW,TRUE,TRUE))
		mbfehler("konnte neue Beschreibung nicht speichern");
}



/*
	21.01.93 Frank, beliebige Dateien edier- und erzeugbar
*/
static void edit_text(void)
{
	char file[PATH_MAX];

	mbquestion(50,1,TRUE," Dateiname: ","",file);

	call_editor(file, TRUE);
}



static void edit_dateiinfo(const char *para)
{
	char datei[PATH_MAX];
	WORD iW;
	long all;
	FILE *bin_f;
	BIN_MSG bin_info;

	if (brettart(brettnrW) != BINAER_BRETT)
	{
		mbfehler("Nachrichten knnen nicht nachtrglich verndert werden");
		return;
	}

	if (box_conf.ti)
		set_status_line("Infotext editieren");
	else
		set_status_line("Infotext edieren");

	if (!binaer_dat_anzahl(brettnrW,0,&all,NULL,NULL,NULL))
		return;
	/* iW ist nur die externe, in der Box angezeigte Nummer */
	iW = atoi(para);
	if (iW <= 0 || iW > all)
	{
		mbfehler("Angegebene Nummer existiert nicht");
		return;
	}

	if ((bin_f = binaer_dat_oeffnen(brettnrW,TRUE)) == NULL)
		return;
	fseek(bin_f,(iW-1)*sizeof(BIN_MSG),SEEK_SET);
	fread(&bin_info,sizeof(BIN_MSG),1,bin_f);

	if (!bin_change_access(&bin_info))
	{
		mbfehler("Zugriff nicht erlaubt");
		binaer_dat_schliessen(bin_f);
		return;
	}

	mbquestion(50,0,TRUE," Betreff: ",bin_info.betreff,bin_info.betreff);
	fseek(bin_f,(iW-1)*sizeof(BIN_MSG),SEEK_SET);
	fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
	binaer_dat_schliessen(bin_f);

	sprintf(datei,"%s%ld.txt",brettpfad(brettnrW),bin_info.nr);
	call_editor(datei, FALSE);

	/* ### Beschrnkung auf 5 bzw. 12 Zeilen fehlt */
	/* auerdem berprfung, ob Datei vorhanden bzw. leer */

} /* edit_dateiinfo */
