/* befehle/extern_p.c: EXTERN-befehl zum aufruf externer Programme */
/*
    Copyright (C) 1994-1999 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
*/


#include "config.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <dirent.h>
#include <sys/time.h>
#include <sys/stat.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "befehle.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "tompaths.h"
#include "box_conf.h"
#include "conflib.h"

#ifdef HAVE_TERMIOS_H
#  include <termios.h>
#elif defined(HAVE_TERMIO_H)
#  include <termio.h>
#elif defined(HAVE_SYS_TERMIOS_H)
#  include <sys/termios.h>
#elif defined(HAVE_SYS_TERMIO_H)
#  include <sys/termio.h>
#endif

extern USER user;

extern WORD
boxsexte_anzW,
vtmodeW;
extern long actual_downloadL;


const char *external_parameter;
static BOOLEAN do_extern(const char *name, const char *para);

static BOOLEAN
download_tmpfiles(void)
{
	char fn[PATH_MAX];
	DIR *d;
	struct dirent *e;
	d=opendir(tom_utmp_dir);
	if (!d) return FALSE; /* XXX logging fehlt */;
	while ((e=readdir(d))!=0) {
		struct stat st;
		char art=0;
		int ret;
		sprintf(fn,"%s%s",tom_utmp_dir,e->d_name);
		if (-1==stat(fn,&st)) continue; /* ??? */
		if (S_ISDIR(st.st_mode)) continue; /* ??? */
		/* Wenn der User ein Defaultprotokoll eingestellt hat *und*
		 * das fr die aktuelle Schnittstelle auch erlaubt ist
		 */
		if (user.default_protokoll && transfer_protokollcheck(user.default_protokoll))
			art=user.default_protokoll;
		ret=transfer_send(&art,fn);
		unlink(fn);
		if (ret==0) {  /* what a bloated software */
			user.dl_bytes+=st.st_size;
			actual_downloadL+=st.st_size;
		}
		if (ret==1) break; /* abbruch */
	}
	closedir(d); /* XXX logging fehlt */
	return TRUE;
}

static BOOLEAN 
do_extern(const char *name, const char *para)
{
	char *cmd=NULL;
	cl_list_t *portlist=NULL;
	char *set_path=NULL;
	char *para_file=NULL;
	char *paras_in_file=NULL;
	char *infilename=NULL;
	int cls=0;
	int pagen=0;
	int download_files=0;

	char fname[PATH_MAX];
	BOOLEAN ok;
	char new_path[PATH_MAX];
	char call_cmd[PATH_MAX+2048];

	cl_var_t ex[]=
	{
		{ "cmd",           NULL, CL_STRING,  &cmd,      0, NULL },
		{ "port",          NULL, CL_LIST,    &portlist, 0, NULL },
		{ "set-path",      NULL, CL_STRING,  &set_path, 0, NULL },
		{ "para-file",     NULL, CL_BOOLEAN, &para_file,0, NULL },
		{ "para-in-file",  NULL, CL_BOOLEAN, &paras_in_file, 0, NULL },
		{ "cls",           NULL, CL_BOOLEAN, &cls,      0, NULL },
		{ "pagen",         NULL, CL_BOOLEAN, &pagen,    0, NULL },
		{ "import-infile", NULL, CL_STRING,  &infilename,    0, NULL },
		{ "download-files",NULL, CL_BOOLEAN, &download_files,0, NULL },
		{ NULL,            NULL, 0,          NULL,      0, NULL }
	};
	if (tom_port_config_dir)
		sprintf(fname,"%sextern.cnf",tom_port_config_dir);
	if (!tom_port_config_dir|| !fileexist(fname))
		sprintf(fname,"%sextern.cnf",box_conf.config_dir);

	if (cl_getconfstanza (fname, ex, name))
	{
		mbfehler("Externes Programm nicht konfiguriert");
		return FALSE;
	}

	*new_path=0;
	ok=TRUE;

	external_parameter=para;
	if (!cmd || (para_file && !paras_in_file))
	{
		mbfehler("Externes Programm nicht korrekt konfiguriert");
		ok=FALSE;
	}
	if (ok && portlist)
	{
		cl_list_t *akt;
		akt=portlist;
		ok=FALSE;
		while (!ok && akt)
		{
			if (akt_connection.is_console && (!stricmp(akt->inhalt,"CONS")
			|| !stricmp(akt->inhalt,"CONSOLE")))
				ok=TRUE;
			else if (akt_connection.is_modem && (!stricmp(akt->inhalt,"MODE")
			|| !stricmp(akt->inhalt,"MODEM")))
				ok=TRUE;
			akt=akt->next;
		}
		if (!ok)
		{
			if (akt_connection.is_modem)
				mboutput(" Externes Programm nicht fr Modemlogin freigegeben\r\n\n");
			else
				mboutput(" Externes Programm nicht fr Consolenlogin freigegeben\r\n\n");
		}
	}
	if (ok && para_file)
	{
		FILE *f=fopen(para_file,"w");
		if (f)
		{
			char buf[4096];
			if (!do_interpret(paras_in_file,buf,sizeof(buf)))
			{
				mbfehler("kann Dateiparameter nicht interpretieren");
				ok=FALSE;
			}
			else
				fprintf(f,"%s",buf);
			fclose(f);
		}
		else
		{
			mbfehler("kann Parameterdatei nicht ffnen");
			ok=FALSE;
		}
	}
	if (ok && !do_interpret(cmd,call_cmd,sizeof(call_cmd)))
	{
		mbfehler("kann Kommando nicht interpretieren");
		ok=FALSE;
	} 
	if (ok && set_path && *set_path)
	{
		if (!do_interpret(set_path,new_path,sizeof(new_path)))
		{
			mbfehler("kann Pfad nicht interpretieren");
			ok=FALSE;
		}
	}
	if (ok)
	{
		void *quit;
		void *inter;
		inter=signal(SIGINT,SIG_IGN);
		quit=signal(SIGQUIT,SIG_IGN);
		if (pagen)
			call_piped(call_cmd,FALSE);
		else
			(void) call_extern(call_cmd,
				*new_path ? new_path : NULL, /* spezielles Arbeitsverzeichnis */
				akt_anruf.soft_limit, /* killzeitpunkt */
				FALSE,                /* ignore_exitcode */
				TRUE);                /* unter der uid des Benutzers aufrufen */
		/* SIG_ERR is not defined under go32 */
#ifdef SIG_ERR
		if (quit!=(void *)SIG_ERR)
#endif
			signal(SIGQUIT,quit);
#ifdef SIG_ERR
		if (inter!=(void *)SIG_ERR)
#endif
			signal(SIGQUIT,inter);
	}
	if (cls)
		bef_clearscreen("");

	if (ok && infilename && *infilename)
	{
		if (!do_interpret(infilename,call_cmd,sizeof(call_cmd)))
		{
			mbfehler("kann Infilename nicht interpretieren");
			ok=FALSE;
		} else if (fileexist(call_cmd)) {
			ta_infile_einsortieren(call_cmd);
			unlink(call_cmd); /* muss sein */
		}
	}
	if (ok && download_files) ok=download_tmpfiles();

	if (set_path) /* why this? */
	{
		if (vtmodeW == T_VT52 || vtmodeW == T_VT100)
		{
			mbprintf("%s%s",vt_pos_S(2,1),vt_S(DEL_SCR_REST));
			redraw_status();
		}
	}
	
	external_parameter=NULL;
	cl_clearconf(ex); /* Speicher freigeben */

	return ok;
}


BOOLEAN 
bef_extern_prg(const char *paraT)
{
	char *p;
	char *para=XSTRDUP(paraT);
	BOOLEAN ret;

	p=strchr(para,' ');
	if (p)
		*p++='\0';
	ret = do_extern(para,p ? p : "");
	free(para);
	return ret;
} /* bef_extern_prg */

