/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/


/*
 * befehle/hitliste.c: Hitlisten aller Art.
 * Siehe auch befehle/boxstat.c
 */

#include "tomsysdep.h"

#include <portab.h>
#include <stdlib.h>
#include <string.h>

#include "prototyp.h"
#include "befehle.h"
#include "terminal.h"
#include "modem.h"
#include "user.h"
#include "brett.h"
#include "msg.h"
#include "bin_msg.h"
#include "bin_out.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/message.h"
#else
#include "offline/version.h"
#include "offline/message.h"
#endif


extern USER user;
extern BRETT akt_brett;

extern int col_ix[];


typedef enum { TOP_15, TOP_ALL, TOP_PM, TOP_PP, TOP_CALL, TOP_BRETT } HITTYP;

typedef struct
{
	ULONG unr;					/* User- bzw. Upload-Nummer, union wre korrekt */
	long wert;					/* sortierrelevanter Wert */
	char listen;				/* Mit Name auflisten? */

} VGL;


static int srt_up(const void *a,const void *b);

static int srt_up(const void *a,const void *b)
{
	return (((const VGL *)b)->wert - ((const VGL *)a)->wert);
}



/*
	Hitliste
	Parameter: ([nix] | * | CALL[S] | PM | PP | BRETT)

	16.12.94 Frank, kpl. neu
	26.01.95 Frank, HIT BRETT
	17.01.97 Frank, HIT PM ab MDB 1.99 auf unbestimmte Zeit deaktiviert

	Funktionsweise:
	- es werden alle User/Uploadnummern mit dem jeweiligen sortierrelevanten 
	  Vergleichswert ermittelt (also mssen alle n Userdatenstze gelesen bzw. 
	  angefordert werden)
	- die Sortierung des Vergleichsarrays erfolgt je nach Funktion auf- oder 
	  absteigend anhand der Vergleichswerte
	- auszugebende (i.A. 15 bis n) Userdatenstze werden kpl. gelesen, um alle 
	  interessanten Daten, insb. den Usernamen, darstellen zu knnen

	Auf einige Optimierungen wurde wg. Lesbarkeit verzichtet:
	- einige Varianten (PM und PP) kmen mit den LISTUSER-Daten aus, so da man 
	  auf die Anforderung der kpl. Userdaten verzichten knnte
	- auer HIT * kme man mit Sortierung der ersten max. 15 Elemente aus, wo-
	  bei ein insertion sort effizienter als als ein quicksort ber alle Daten 
	  wre.
*/
BOOLEAN bef_hitliste(const char *para)
{
	ULONG anzahl = ub_userzahl;
	USER u;
	HITTYP typ;
	VGL *p_vgl;
	long mails;
	long i,n;

	if (!strcmp(para,"*"))
	{
		set_status_line(tprintf("Hitliste der %lu User",anzahl));
		typ = TOP_ALL;
	}
	else if (!stricmp(para,"PM"))
	{
		set_status_line("Hitliste der PM-Sammler (Postfach)");
		typ = TOP_PM;
	}
	else if (!stricmp(para,"PP"))
	{
		set_status_line("Hitliste der PP-Sammler (Programmfach)");
		typ = TOP_PP;
	}
	else if (!stricmp(para,"CALL") || !stricmp(para,"CALLS"))
	{
		set_status_line("Hitliste der Anrufer");
		typ = TOP_CALL;
	}
	else if (!stricmp(para,"BRETT"))
	{
		if (brettart(akt_brett.nr) != BINAER_BRETT)
		{
			mbfehler("Hitliste nur in Binrbrettern mglich");
			return FALSE;
		}
		if (akt_brett.nr == 0)
		{
			mbfehler("Hitliste im Programmfach nur mit HITLISTE PP");
			return FALSE;
		}
		if (!binaer_dat_anzahl(akt_brett.nr,0,&anzahl,NULL,NULL,NULL) || !anzahl)
		{
			mbfehler("Keine Dateien im aktuellen Brett");
			return FALSE;
		}
		set_status_line(tprintf("Hitliste der Downloads in <%s>",akt_brett.name));
		typ = TOP_BRETT;
	}
	else if (!*para)
	{
		set_status_line(tprintf("Top Fnfzehn der %lu User",anzahl));
		typ = TOP_15;
	}
	else
	{
		parameter_falsch("HITLISTE",para);
		return FALSE;
	}

	/* Wertepaare (User-/Uploadnr + Sortier-Vergleichswert) initialisieren */
	if ((p_vgl = (VGL *) calloc(anzahl,sizeof(VGL))) == NULL)
	{
		speicherproblem(FALSE,"bef_hitliste");
		return FALSE;
	}
	/* Usernummern mit jeweiligem Vergleichswert merken */
	i = 0;
	if (typ != TOP_BRETT)
	{
		u.nummer = 0;
		while (ub_walk_user(&u))
		{
			/* gelschte und nicht aufzulistende User verwerfen */
			if (u.geloescht)
				continue;
			if (!u.auflisten)
			{
				if (u.typ!=UTYP_USER)
					continue;
				p_vgl[i].listen=FALSE;
			}
			else
				p_vgl[i].listen=TRUE;

			p_vgl[i].unr = u.nummer;

			switch (typ)
			{
				case TOP_15:
				case TOP_ALL:
					p_vgl[i].wert = 0;		/* separat zu ermitteln */
					break;

				case TOP_CALL:
					p_vgl[i].wert = u.anrufe;
					break;

#ifndef OFFLINE_VERSION
				case TOP_PM:
					if (group_info_msgs(u.name,TRUE,&mails,NULL))
						p_vgl[i].wert = mails;
					else
						p_vgl[i].wert = 0;
					break;
#endif

				case TOP_PP:
					p_vgl[i].wert = 0;		/* separat zu ermitteln */
					break;

				default:
					mbfehler("falscher Parameter");
					free(p_vgl);
					return FALSE;
			}
			i++;
		}
		/* Anzahl der aufzulistenden, ungelschten User */
		anzahl = i;
	}

	if ((typ == TOP_PM) || (typ == TOP_PP) || (typ == TOP_CALL) || (typ == TOP_BRETT))
	{
		long max = (typ == TOP_ALL) ? anzahl : 15;
		long last = -1;
		const char *titel = "";
		const char *einheit = "";
		BIN_MSG bin_info;
		FILE *bin_f = NULL;

		if (typ == TOP_PM)
			titel = "Sortiert nach Anzahl:" , einheit = "Anzahl";
		else
		if (typ == TOP_PP)
			titel = "Sortiert nach Files:" , einheit = "Bytes";
		else
		if (typ == TOP_CALL)
			titel = "Sortiert nach Anrufen:" , einheit = "Anzahl";

		if (typ == TOP_BRETT)
		{
			bin_heading();
		}
		else
		{
			mbprintf(" %s%-29.29s %s %s\r\n",
			         vt_S(ATTR_FETT),titel,einheit,vt_S(ATTR_OFF));
			mboutput(" ------------------------------------\r\n");
		}
		set_scroll_area(2);

		if (typ == TOP_PP)
		{
			i = 0;
			if ((bin_f = binaer_dat_oeffnen(0,FALSE)) == NULL)
			{
				mboutput(" Kann PP-bersicht nicht ffnen\r\n");
				free(p_vgl);
				return FALSE;
			}
			while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
			{
				if (bin_info.deleted || bin_info.status == 2)
					continue;

				while (i < anzahl && p_vgl[i].unr != bin_info.empfaenger)
					i++;
				if (p_vgl[i].unr == bin_info.empfaenger)
					p_vgl[i].wert += bin_info.bytes;
			}
			binaer_dat_schliessen(bin_f);
		}
		else
		if (typ == TOP_BRETT)
		{
			if ((bin_f = binaer_dat_oeffnen(akt_brett.nr,FALSE)) == NULL)
			{
				mboutput(" Kann Brett-bersicht nicht ffnen\r\n");
				free(p_vgl);
				return FALSE;
			}
			for (n = 0; n < anzahl; n++)
			{
				if (!fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
					continue;
				if (bin_info.deleted || bin_info.status == 2)
					continue;

				p_vgl[n].unr  = n; /*bin_info.lokal;*/
				p_vgl[n].wert = bin_info.zugriffe;
			}
		}

		qsort(p_vgl,anzahl,sizeof(VGL),srt_up);

		if (typ == TOP_BRETT)
		{
			for (n = 0; n < 15 && n < anzahl; n++)
			{
				if (!p_vgl[n].wert)
					break;
				fseek(bin_f,p_vgl[n].unr * sizeof(BIN_MSG),SEEK_SET);
				fread(&bin_info,sizeof(BIN_MSG),1,bin_f);
				if (bin_msg_inhalt(&bin_info) < 0)
					break;
			}
			binaer_dat_schliessen(bin_f);
		}
		else
		for (n = 0; n < max && n < anzahl; n++)
		{
			if (!ub_read_user(p_vgl[n].unr,&u,FALSE))
				continue;

			if (!p_vgl[n].wert)
				break;

			if ((last != -1) && (last == p_vgl[n].wert))
				mboutput("     ");
			else
				mbprintf(" %s%2ld%s. ",vt_S(ATTR_FETT),n+1,vt_S(ATTR_OFF));
			last = p_vgl[n].wert;

			/* aktuellen User hervorheben */
			if (p_vgl[n].unr == user.nummer)
				mbprintf("%s%s",vt_S(ATTR_FETT),vt_S(col_ix[C_ROT]));
			if (p_vgl[i].listen
			|| p_vgl[i].unr==u.nummer 
			|| level()==ULV_MAX)
				mbprintf("%-22.22s  ",u.name);
			else
				mbprintf("%-22.22s  ","anonym");
			if (p_vgl[n].unr == user.nummer)
				mbprintf("%s",vt_S(ATTR_OFF));

			mbprintf("%8ld\r\n",p_vgl[n].wert);

			if (userbreak())
				break;
		}
		mboutput("\r\n");
	}
	else if ((typ == TOP_15) || (typ == TOP_ALL))
	{
		USERNR *utab;		/* Download/Upload/Write fr jeden User */
		int k;						/* akt. Kategorie (s.o.) von 0 bis 2 */
		long wert;
		int realmax;

		if (typ==TOP_15)
			realmax=15;
		else
			realmax=anzahl;
		if (realmax > anzahl)
			realmax=anzahl;

		utab=calloc(3*realmax,sizeof(USERNR));
		if (!utab)
		{
			mbfehler("Abbruch wegen Speichermangel");
			return FALSE;
		}

		mbprintf(" %sTop-Downloader:(KB)          Top-Uploader:(KB)       Top-Schreiber:(KB)    %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF));
		mboutput(" -----------------------------------------------------------------------------\r\n");
		set_scroll_area(2);

		for (k = 0; k < 3; k++)
		{
			i = 0;
			u.nummer = 0;
			while (ub_walk_user(&u))
			{
				if (u.geloescht)
					continue;
				if (!u.auflisten)
				{
					if (u.typ!=UTYP_USER)
						continue;
					p_vgl[i].listen=FALSE;
				}
				else
					p_vgl[i].listen=TRUE;

				p_vgl[i].unr = u.nummer;
				if (!k)
					p_vgl[i].wert = u.dl_bytes;
				else
				if (k == 1)
					p_vgl[i].wert = u.ul_bytes;
				else
					p_vgl[i].wert = u.txt_bytes;
				i++;
			}

			qsort(p_vgl,anzahl,sizeof(VGL),srt_up);

			for (n = 0; n < realmax; n++)
				utab[k*realmax + n] = p_vgl[n].unr;
		}

		for (n = 0; n < realmax; n++)
		{
			BOOLEAN leer = TRUE;

			for (k = 0; k < 3; k++)
			{
				if (!ub_read_user(utab[k*realmax + n],&u,FALSE))
					continue;

				if (!k)
					mbprintf(" %s%3ld%s.",vt_S(col_ix[C_ROT]),n+1,vt_S(ATTR_OFF));

				if (!k)
					wert = u.dl_bytes;
				else
				if (k == 1)
					wert = u.ul_bytes;
				else
					wert = u.txt_bytes;

				if (!wert)
					mboutput("                        ");
				else
				{
					if (u.nummer == user.nummer)
						mbprintf("%s",vt_S(col_ix[C_ROT]));
					if (p_vgl[i].listen
					|| p_vgl[i].unr==u.nummer 
					|| level()==ULV_MAX)
						mbprintf(" %-16.16s",u.name);
					else
						mbprintf(" %-16.16s","anonym");
					mbprintf("%7ld",wert / 1024);
					if (u.nummer == user.nummer)
						mbprintf("%s",vt_S(ATTR_OFF));
					leer = FALSE;
				}
			}
			if (mboutput("\r\n") < 0)
				break;
			if (leer)
				break;
		}
		mboutput("\r\n");
		free(utab);
	}

	free(p_vgl);
	set_scroll_area(1);

	return TRUE;
}
