/*
    Copyright (C) 1994 1995 1996 1997 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
/*
 * befehle/kill.c: Nachrichten lschen bzw. restaurieren.
 */

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>

#include "prototyp.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/message.h"
#include "offline/inc/mdb_global.h"
#else
#include "offline/version.h"
#include "offline/message.h"
#include "offline/mdb_global.h"
#endif
#include "routing.h"
#include "befehle.h"
#include "brett.h"
#include "user.h"
#include "modem.h"
#include "bin_msg.h"
#include "msg.h"

extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;


static BOOLEAN un_kill_text(const char *para,BOOLEAN del);
static BOOLEAN un_kill_binaer(const char *para,BOOLEAN del);



BOOLEAN bef_kill(const char *para)
{
	if (brettart(brettnrW) == BINAER_BRETT)
		return un_kill_binaer(para,TRUE);
	else
		return un_kill_text(para,TRUE);

} /* bef_kill */



BOOLEAN bef_unkill(const char *para)
{
	if (brettart(brettnrW) == BINAER_BRETT)
		return un_kill_binaer(para,FALSE);
	else
		return un_kill_text(para,FALSE);

} /* bef_unkill */



/*
	Nachrichten zum Lschen markieren bzw. restaurieren

	15.12.92 Frank
	23.01.95 Frank, jeder darf eigene Mails lschen/restaurieren
	20.01.97 Frank, o.g. Feature erstmal disabled
*/
static BOOLEAN un_kill_text(const char *para,BOOLEAN del)
{
	char zeile[200];
	BOOLEAN pm = (brettnrW == 0);
	BOOLEAN erlaubt;			/* generelle Lscherlaubnis */
	const char *name;
	char maske[80];
	int gh;
	int ok;
#ifdef OFFLINE_VERSION
	long von,bis;				/* aktueller Messagebereich */
#else
	long nr;						/* aktuelle Messagenummer */
#endif
	long anzahl = 0;
	long max;

	erlaubt = (akt_brett.grp_chef==user.nummer || level()==9 || pm);

	*maske=0;
	if (!erlaubt)
	{
		mbfehler("Keine Erlaubnis zum Lschen von Daten");
		return FALSE;
	}
	else if (!*para)
	{
		mbfehler("Keine Eintrge angegeben");
		return FALSE;
	}
	else if (isalpha(*para))
	{
		size_t len = strcspn(para," \r\n");
		strncpy(maske,para,len);
		maske[len] = 0;
		strlwr(maske);
		para += len;
		while (isspace(*para))
			para++;
	}

	if (del)
		set_status_line("Nachrichten lschen");
	else
		set_status_line("Nachrichten restaurieren");

	name = (pm) ? user.name : akt_brett.name;

#ifdef OFFLINE_VERSION
	for (ok = larea(para,&von,&bis); ok && von > 0; ok = larea(NULL,&von,&bis))
	{
		long markiert;

		if (von <= 0 || bis <= 0)
			continue;

		if (userbreak())
			break;

		markiert = 0;
		mdb_mark(name,pm,&markiert,del,FALSE,von,bis);
		anzahl += markiert;
	}
#else
	if (!group_info_msgs(name,pm,&max,NULL) || !max
	|| (gh = group_open(getpid(),name,pm,TRUE)) < 0)
	{
		mbfehler("Keine Eintrge vorhanden");
		return TRUE;
	}

	for (ok = lrange(para,&nr); ok && nr > 0; ok = lrange(NULL,&nr))
	{
		if (nr <= 0)
			continue;
		if (nr > max)
			break;

		if (userbreak())
			break;

		if (!index_goto(gh,(MESSAGE) (nr-1)) || !index_read(gh))
			break;

#if 0
		/* falls keine generelle Erlaubnis, Absender vergleichen */
		if (!pm && !erlaubt)
		{
			USERNR unr;
			char *p;
			p=strchr(zeile+20,'(');
			if (!p)
				continue;
			p[-1]=0;
			p=strchr(zeile+20,'@');
			if (!p)
				continue;
			*p++=0;
			strtrim(p);
			strtrim(zeile+20);
			if (!r_is_me(p))
				continue;
			if (!ub_find_username(&unr,zeile+20))
				continue;
			if (unr!=user.nummer)
				continue;
		}
#endif

		if ((del && msg_set_flag(gh,TOTAL))
		|| (!del && msg_clr_flag(gh,TOTAL)))
			anzahl++;
	}
	group_close(gh,getpid());
#endif

	if (brettnrW)
		bb_update_brett(brettnrW);

	if (anzahl)
	{
		mbprintf("\r\n %ld Nachricht%s %s\r\n",
		         anzahl,(anzahl > 1) ? "en" : "",
		         (del) ? "zum lschen markiert" : "restauriert");
	}
	else
		mboutput(" Keine passenden Eintrge vorhanden\r\n");

	return TRUE;

} /* un_kill_text */



/*
	Binrdatensatz zum Lschen markieren bzw. restaurieren
*/
static BOOLEAN un_kill_binaer(const char *para,BOOLEAN del)
{
	BOOLEAN paraB = FALSE;
	FILE *bin_f;
	BIN_MSG bin_info;
	long nr;
	int ok;
	long all;

	if (del)
		set_status_line("Dateien lschen");
	else
		set_status_line("Dateien restaurieren");

	if (!binaer_dat_anzahl(brettnrW,0,&all,NULL,NULL,NULL))
		return FALSE;
	if (!all)
	{
		mbfehler("Keine Dateien in diesem Brett vorhanden");
		return FALSE;
	}

	if ((bin_f = binaer_dat_oeffnen(brettnrW,TRUE)) == NULL)
		return FALSE;

	for (ok = lrange(para,&nr); ok && nr > 0; ok = lrange(NULL,&nr))
	{
		if (nr <= 0)
			continue;
		if (nr > all)
			break;

		if (userbreak())
			break;

		paraB = TRUE;

		fseek(bin_f,(nr-1)*sizeof(BIN_MSG),SEEK_SET);
		fread(&bin_info,sizeof(BIN_MSG),1,bin_f);

		if (del && bin_info.deleted && bin_info.status == 2)
			continue;
		if (!del && !bin_info.deleted && bin_info.status != 2)
			continue;

		if (!bin_change_access(&bin_info))
		{
			mbprintf(" Zugriff auf %ld nicht erlaubt\r\n",nr);
			continue;
		}

		if (del)
		{
			bin_info.deleted = TRUE;
			bin_info.status = 2;
			fseek(bin_f,(nr-1)*sizeof(BIN_MSG),SEEK_SET);
			fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
			mbprintf(" Nr.%ld >> Eintrag gelscht\r\n",nr);
		}
		else
		{
			bin_info.deleted = FALSE;
			bin_info.status = 0;
			fseek(bin_f,(nr-1)*sizeof(BIN_MSG),SEEK_SET);
			fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
			mbprintf(" Nr.%ld >> Eintrag restauriert\r\n",nr);
		}
	}
	binaer_dat_schliessen(bin_f);

	/* neues Brettvergleichsdatum setzen */
	if (brettnrW)
		bb_update_brett(brettnrW);

	mboutput("\r\n");

	if (!paraB)
	{
		parameter_falsch((del) ? "KILL" : "UNKILL",para);
		return FALSE;
	}
	else
		return TRUE;
}
