/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>
#include "fnmatch.h"

#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/message.h"
#include "offline/inc/mdb_global.h"
#ifdef OFFLINE_VERSION
#define mdb_markiere	mdb_mark
#endif
#else
#include "offline/version.h"
#include "offline/message.h"
#include "offline/mdb_global.h"
#endif
#include "prototyp.h"
#include "befehle.h"
#include "brett.h"
#include "user.h"
#include "routing.h"
#include "box_conf.h"
#include "modem.h"
#include "bin_msg.h"

#ifndef OFFLINE_VERSION
extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;
extern long max_brettnr;

struct putzlist {
	char *expression;
	struct putzlist *next;
};

struct putzentry {
	struct putzlist *gruppen;
	time_t mindays;
	time_t normaldays;
	time_t maxdays;
	ULONG maxnr;
	ULONG maxkb;
	ULONG status;
	struct putzentry *next;
};
#endif

static BOOLEAN bef_putz_cnf(void);
static BOOLEAN bef_putz_user(void);

/*
	Nachrichten gem Brettkonfiguration zum Lschen markieren und putzen
	diese Fn. durchluft smtliche Gruppen (und nur diese, keine PMs)
*/
BOOLEAN bef_putz(const char *para)
{
	if (!stricmp(para,"CNF"))
	{
		return bef_putz_cnf();
	}
	else if (!stricmp(para,"USER"))
	{
		return bef_putz_user();
	}
	else if (!stricmp(para,"STATUS"))
	{
		return r_st_expire();
	}
	else if (!stricmp(para,"*"))
	{
		return (bef_putz_cnf() && bef_putz_user());
	}
	else
	{
		mbfehler("optionale Parameter: CNF, USER, *");
		return FALSE;
	}

} /* bef_putz */




/*
	Postfcher putzen

	im Gegensatz zu den Gruppen wird nicht nach MsgAnzahl sondern nach Zeit 
	geputzt, so da nur alte Nachrichten gelscht werden.
	mdb_markiere() von Offline kmmert sich dabei selbst darum, da nur bereits 
	seit (derzeit 21) Tagen getauschte PMs zum lschen markiert werden
*/
static BOOLEAN bef_putz_user(void)
{
	USER u;
	time_t alt;
	long max;

	memset(&u,0,sizeof(u));

	/* Referenzzeitpunkt: lter als 3 Wochen */
	alt = time(NULL) - 21 * 86400L;

	while (ub_walk_user(&u))
	{
		if (userbreak())
			break;

#ifndef OFFLINE_VERSION
		/* berhaupt Mails fr den User vorhanden? */
		if (!group_info_msgs(u.name,TRUE,&max,NULL) || !max)
			continue;

		/* nach Zeit markieren, alle Mails lter als 3 Wochen */
		mdb_markiere(u.name,TRUE,TRUE,TRUE,0,alt);
#else
		mdb_markiere(u.name,TRUE,NULL,TRUE,TRUE,0,alt);
#endif
	}

#ifdef OFFLINE_VERSION
	return mdb_putz("*",TRUE);
#else
	return mdb_putze_alles(TRUE,FALSE);
#endif
} /* bef_putz_user */


#ifndef OFFLINE_VERSION
static struct putzentry *anker;

#define GET_PART(str,delim,ptr) \
		if ((ptr=strchr(str,delim))==NULL) \
		{ \
			mbfehler("unparsbare Zeile in putz.cnf:"); \
			mbfehler(tprintf(buf,"%ld",zeile)); \
			continue; \
		} \
		*ptr++='\0';

/* 
 * Dateiformat:
 * gruppe,gruppen:Flags:Mindays:Normaldays:Maxdays:Maxmessages:Maxkb
 */
static BOOLEAN load_putz_config(void)
{
	char fname[PATH_MAX];
	char buf[1024];
	char *p;
	char *doppel;
	struct putzlist *lnew;
	struct putzlist *lanker;
	struct putzentry *enew;
	long zeile;

	time_t mindays;
	time_t normaldays;
	time_t maxdays;
	ULONG maxnr;
	ULONG maxkb;
	ULONG status;

	FILE *f;
	sprintf(fname,"%sputz.cnf",box_conf.config_dir);
	f=xfopen(fname,"r");
	zeile=0;
	while (fgets(buf,sizeof(buf),f))
	{
		zeile++;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		strtrim(buf);
		if (*buf=='#' || *buf==';' || !*buf)
			continue;

		mindays=0;
		normaldays=0;
		maxdays=0;
		maxnr=0;
		maxkb=0;
		status=0;

		/* Gruppen isolieren */
		GET_PART(buf,':',doppel);

		/* Flags holen */
		p=doppel;
		GET_PART(p,':',doppel);
		string_to_brettflags(p,&status);

		/* Mindays holen */
		p=doppel;
		GET_PART(p,':',doppel);
		mindays=strtol(p,NULL,10);

		/* Normaldays holen */
		p=doppel;
		GET_PART(p,':',doppel);
		normaldays=strtol(p,NULL,10);

		/* Maxdays holen */
		p=doppel;
		GET_PART(p,':',doppel);
		maxdays=strtol(p,NULL,10);

		/* Maxmessages holen */
		p=doppel;
		GET_PART(p,':',doppel);
		maxnr=strtol(p,NULL,10);

		/* Maxkb holen */
		maxkb=strtol(doppel,NULL,10);
		
		lanker=NULL;
		while ((p=strtok((lanker) ? NULL : buf," \t,"))!=NULL)
		{
			lnew=XMALLOC(sizeof(struct putzlist));
			lnew->expression=XSTRDUP(p);
			strlwr(lnew->expression);
			lnew->next=lanker;
			lanker=lnew;
		}
		enew=XMALLOC(sizeof(struct putzentry));
		enew->gruppen=lanker;
		lanker=NULL;
		enew->mindays=mindays;
		enew->normaldays=normaldays;
		enew->maxdays=maxdays;
		enew->maxnr=maxnr;
		enew->maxkb=maxkb;
		enew->status=status;
		enew->next=anker;
		anker=enew;
		/* nchsten Datensatz lesen */
	}
	fclose(f);
	return TRUE;
}
#endif
	
static BOOLEAN bef_putz_cnf(void)
{
#ifdef OFFLINE_VERSION
	return mdb_putz("*",FALSE);
#else
	int i;
	long total_zahl;
	long total_bytes;
	char name[BDAT_GRP_NAME_LEN];
	struct putzentry *e;
	struct putzentry *enext;
	struct putzlist *l;
	struct putzlist *lnext;

	if (!load_putz_config())
		return FALSE;

	for (i = 0; i < max_brettnr; i++)
	{
		if (!br_ind[i].brett)
			continue;

		strcpy(name,br_ind[i].brett->name);

		strlwr(name);
		mbprintf(" %s\r\n",name);

		if (!group_info_msgs(name,FALSE,&total_zahl,&total_bytes))
			continue;
		if (!total_bytes && !total_zahl)
			continue;

		if (userbreak())
			break;
		
		e=anker;
		while (e)
		{
			l=e->gruppen;
			while (l)
			{
				if (0==fnmatch(l->expression,name,0))
					break;
				l=l->next;
			}
			if (l)
				break;
			e=e->next;
		}

		if (!e)
		{
			mbprintf("    unmatched\r\n");
			continue;
		}

		if (e->status)
		{
			if ((br_ind[i].brett->flags & e->status)!=e->status)
			{
				mbprintf("    Flags passen nicht\r\n");
				continue;
			}
		}

		if (e->maxnr!=0 && e->maxnr < (ULONG) total_zahl)
		{
			/* nach Msg.Nummern markieren */
			mbprintf("    ");
			mdb_markiere(name,FALSE,TRUE,FALSE,0,(total_zahl - e->maxnr));
		}
		if (e->normaldays != 0 && e->normaldays)
		{
			time_t ref=time(NULL)-e->normaldays * (60*60*24L);
			mbprintf("    ");
			mdb_markiere(name,FALSE,TRUE,TRUE,0,ref);
		}
		if (e->maxkb!=0 && e->maxkb< (ULONG) total_bytes)
		{
			mdb_markiere_size(name,FALSE,e->maxkb*1024);
		}

		/* Mindays und Maxdays bleiben derzeit unbeachtet. */

	}

	/* alles ist markiert, jetzt Speicher wieder freigeben */
	e=anker;
	while (e)
	{
		l=e->gruppen;
		while (l)
		{
			free(l->expression);
			lnext=l->next;
			free(l);
			l=lnext;
		}
		enext=e->next;
		free(e);
		e=enext;
	}
	anker=NULL;

	/*
		jetzt ist alles markiert und es kann global geputzt werden.
		Wg. Linux' Buffercache wre es gnstiger, jede Gruppe sofort nach dem 
		Markieren zu putzen, da sich die Indexdaten i.A. noch im Cache befinden,
		doch die globale Putzfunktion von Offline hat den Vorteil, eben auch 
		ein globales Putz-Log zu erstellen, fr die Box daher die bessere Wahl
		### globales Putzlog? Wo?
	*/
	return mdb_putze_alles(FALSE,FALSE);
#endif
} /* bef_putz_cnf */
