/* show.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "befehle.h"
#include "box_conf.h"
#include "modem.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/mdb_conf.h"	/* wg. log_path von offline */
#else
#include "offline/version.h"
#include "offline/mdb_conf.h"	/* wg. log_path von offline */
#endif
#include "tompaths.h"



static struct known_log
{
	const char *cmd;
	const char *cmd2;
	const char *format;
	const char *name;
}
logs[]=
{
	{ "POLL",    NULL,             "%02d_%02d_%02d.pro",    "Pollprotokoll" },
	{ "CAPTURE", NULL,             "%02d_%02d_%02d.cpt",    "Capturefile" },
#ifdef OFFLINE_VERSION
	{ "PUTZ",    NULL,             "%02d_%02d_%02d.cln",    "Putzinfo" },
	{ "IMPORT",  NULL,             "%02d_%02d_%02d.imp",    "Importinfo" },
#else
	{ "PUTZ",    NULL,             "db/pt%02d%02d%02d.log", "Putzinfo" },
	{ "IMPORT",  NULL,             "db/tr%02d%02d%02d.log", "Importinfo" },
#endif
	{ "CHAT",    NULL,             "%02d_%02d_%02d.cht",    "Chat-Log" },
	{ "USER",    "INF",            "%02d_%02d_%02d.inf",    "Userprotokoll" },
	{ "QO",      "QUARKOUTFILE",   "%02D_%02d_%02d.qo",     "Quarkoutfile" },
	{ "LOG",     "TAGESPROTOKOLL", "%02d_%02d_%02d.log",    "Tagesprotokoll"},
	{ "PANIC",   "PANIK",          "%02d_%02d_%02d.panic",  "Paniklogfile"},
	{ "MSG",     "MESSAGE",        "%02d_%02d_%02d.msg", "Nachrichtenlogfile"},
	{ NULL, NULL, NULL, NULL}
};

/*
	16.02.94 Frank, SHOW PUTZ
	26.02.95 Frank, SHOW CHAT

	Diese Fn. bietet sich zur Auslagerung in ein externes Programm an.
*/
BOOLEAN bef_show(const char *para)
{
	BOOLEAN paraB = FALSE;
	char tmp[PATH_MAX];
	char datei[PATH_MAX];
	char datum[12];
	const char *space;
	int found=0;
	int passend=0;
	int t,m,j;
	unsigned int i;
	size_t l;

	if (!*para)
		para = "USER";

	space=strchr(para,' ');
	if (space)
		l=space-para;
	else
		l=strlen(para);

	for (i = 0; logs[i].cmd!=NULL; i++)
	{
		if (0==strnicmp(para,logs[i].cmd,l))
		{
			found=i;
			passend++;
		}  
		else if (logs[i].cmd2 && 0==strnicmp(para,logs[i].cmd2,l))
		{
			found=i;
			passend++;
		}  
	}
	if (passend!=1)
	{
		parameter_falsch("SHOW",para);
		return FALSE;
	}

	if (space)
	{
		while (isspace(*space))
			space++;
		if (!*space)
			space=NULL;
	}
	if (space)
		paraB=zerlege_datum(space,&t,&m,&j);
	else
		paraB=zerlege_datum(dd_mm_yy(),&t,&m,&j);

	if (!paraB)
	{
		parameter_falsch("SHOW",para);
		return FALSE;
	}

	sprintf(tmp,logs[found].format,j,m,t);
	sprintf(datei,"%s%s",box_conf.log_dir,tmp);
	sprintf(datum,"%02d.%02d.%d",t,m,j);
	if (!fileexist(datei))
		strcat(datei,".gz");
	if (fileexist(datei))
	{
		set_status_line(tprintf("%s vom: %s",logs[found].name,datum));
		zeige_datei(datei);
	}
	else
	{
		mbprintf(" Es ist kein %s fr den %s vorhanden\r\n\n",
				 logs[found].name,datum);
	}

	return TRUE;
}


/*
 * Local variables:
 *  version-control: t
 *  kept-new-versions: 5
 *  tab-width: 4
 *  c-indent-level: 4
 *  c-tab-always-indent: xya
 * End:
 */
