/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdlib.h>
#include <string.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "befehle.h"
#include "cslib.h"
#include "aktdata.h"
#include "routing.h"


extern USER user;

static BOOLEAN 
bef_status_anruf(void)
{
	mbprintf(" peer %s\r\n", akt_connection.peer ? "TRUE" : "FALSE");
	mbprintf(" console %s\r\n", akt_connection.is_console ? "TRUE" : "FALSE");
	mbprintf(" modem %s\r\n", akt_connection.is_modem ? "TRUE" : "FALSE");
	mbprintf(" modemlines %s\r\n", 
			 akt_connection.has_modem_lines ? "TRUE" : "FALSE");
	mbprintf(" tty %s\r\n", 
			 akt_connection.is_tty ? akt_connection.ttyname : "none");
	if (akt_connection.rusername)
		mbprintf(" remoteuser %s\r\n",akt_connection.rusername);
	if (akt_connection.rhostname)
		mbprintf(" remotehost %s\r\n",akt_connection.rhostname);
	if (akt_connection.rhostaddr)
		mbprintf(" remoteaddr %s\r\n",akt_connection.rhostaddr);
	mbprintf(" authentication %s\r\n", akt_connection.auth_ok ? "true" : "false");
	mbprintf(" errorfree %s\r\n", akt_connection.error_free ? "true" : "false");
	if (akt_connection.localport)
		mbprintf(" localportnr %u",(unsigned) akt_connection.localport);
	if (akt_connection.remoteport)
		mbprintf(" remoteportnr %u",(unsigned) akt_connection.remoteport);
	mboutput("\r\n");
	if (akt_connection.carrierstring)
		mbprintf(" carrier: %s\r\n",akt_connection.carrierstringraw); 
	if (akt_connection.shelluser)
		mbprintf(" uid: %ld, gid %ld\r\n",
				 (long) akt_connection.ruid, (long) akt_connection.rgid);
	mboutput(" Zeilenende: ");
	if (akt_connection.sends_cr && akt_connection.sends_lf)
		mboutput("CR/LF\r\n");
	else if (akt_connection.sends_cr)
		mboutput("CR\r\n");
	else
		mboutput("LF\r\n");
	if (akt_connection.clid_from && akt_connection.clid_to)
		mbprintf(" CLID from: %s to: %s\r\n",
			akt_connection.clid_from,akt_connection.clid_to);
	else if (akt_connection.clid_from)
		mbprintf(" CLID from: %s\r\n", akt_connection.clid_from);
	else if (akt_connection.clid_to)
		mbprintf(" CLID to: %s\r\n", akt_connection.clid_to);
	return TRUE;
}


/*
	25.02.93 Frank, auf Userdatenbank umgestellt und konverterfrei
	24.6.1994, Uwe, den Datenschutz deutlich verschrft. "Adresse gesperrt"
	           sperrt die allermeisten Daten.
*/
BOOLEAN bef_status(const char *para)
{
	USER status;
	USERNR usernr;
	cstl_t *cs;

	status = user;
	usernr = user.nummer;

	if (*para)
	{
		if (0==stricmp(para,"-anruf"))
			return bef_status_anruf();
		if (!ub_find_username(&usernr,para))
		{
			mbfehler("Dieser Benutzer existiert nicht");
			return FALSE;
		}
		ub_read_user(usernr,&status,FALSE);
	}

	set_status_line(tprintf("Benutzerstatus von: %s",user.name));

	if (*status.ansage)
	{
		mbprintf(" %s Ansage      :%s %-50s\r\n\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.ansage);
	}
	mboutput(" Daten ");
	if (status.adresse_frei)
		mboutput("freigegeben\r\n");
	else
	{
		mboutput("gesperrt\r\n");
		if (usernr!=user.nummer && level()<8)
			return FALSE;
	}

	mbprintf(" %s Name        :%s %-24s %s Letzter Anruf:%s %s\r\n",
		vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.name,
		vt_S(ATTR_FETT),vt_S(ATTR_OFF),datum_zeit_string(&status.letzter_login));

	mbprintf(" %s Userlevel   :%s %d%24s%s Erster Anruf :%s %s\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.level,"",
		vt_S(ATTR_FETT),vt_S(ATTR_OFF),datum_zeit_string(&status.erster_login));

	mbprintf(" %s Benutzernr. :%s %ld\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),(long)status.nummer);
	mbprintf(" %s Anrufe ges. :%s %-25ld%s Terminaltyp  :%s ",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),
			status.anrufe,
			vt_S(ATTR_FETT),vt_S(ATTR_OFF));
	switch (status.terminal_typ)
	{
		case T_ASCII :
			mboutput("ASCII");
			break;

		case T_VT100 :
			mboutput("VT100");
			break;

		case T_VT52 :
			mboutput("VT52");
			break;

		case T_ANSI :
			mboutput("ANSI");
			break;

		default:
			mboutput("unbekannt");
			break;
	}
	mboutput("\r\n");

	cs=get_charset_from_nr(status.zeichensatz);
	mbprintf(" %s Rechner     :%s %-24s %s Zeichensatz  :%s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.computer,
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				(cs) ? cs->name : "unbekannt");

	/* Zahlerverhalten geht wirklich nicht jeden etwas an */
	if (user.level>7 || user.nummer==usernr) 
	{
		if (status.bezahlt)
		{
			char buf[20];
			struct tm *tm;
			tm=localtime(&status.bezahlt_bis);
			strftime(buf,20,"%d.%m.%Y",tm);
			mbprintf(" %s Bezahlt bis :%s %-24s\r\n",
					vt_S(ATTR_FETT),vt_S(ATTR_OFF),
					buf);
		}
		else
		{
			mbprintf(" %s Zahler      :%s %-24s\r\n",
					vt_S(ATTR_FETT),vt_S(ATTR_OFF),"nein");
		}
	}

	mbprintf(" %s Uploads     :%s %-25ld%s Downloads    :%s %ld\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.ul_bytes,
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.dl_bytes);

	mbprintf(" %s Gastupload  :%s %-24s %s Oberflche   :%s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.gast_upload ? "ja" : "nein",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				(status.box_stil == QUARK_STYLE) ? "Quark" : "Maus");

	/* Ab Level 8 Listenauftauchen/Protokoll anzeigen */
	if (level() >= ULV_MAX-1)
	{
		mbprintf(" %s In Listen   :%s %-24s %s Protokoll    :%s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.auflisten ? "ja" : "nein",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.protokoll ? "ja" : "nein");
	}

	{
		const char *t="?";
		const char *g="?";
		const char *desc="Path";
		switch(status.typ)
		{
			case UTYP_USER: 
				t="Benutzer"; 
				desc="Geschlecht"; 
				switch(status.geschlecht)
				{
					case NEUTRUM: g="Programm"; break;
					case MAENNLICH: g="mnnlich"; break;
					case WEIBLICH: g="weiblich"; break;
				}
				break;
			case UTYP_GATE: 
				t="Gateway"; 
				g=get_path_entry(status.nummer);
				break;
			case UTYP_BOX: 
				t="Box"; 
				g=get_path_entry(status.nummer);
				break;
		}
		if (!g)
			g = "unbekannt";
		mbprintf(" %s Eintragtyp  :%s %-24s %s %-13s:%s %s\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),
			t,
			vt_S(ATTR_FETT),desc,vt_S(ATTR_OFF),
			g);
	}

	mbprintf(" %s PLZ/Ort     :%s %s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.plz,status.ort);
	mbprintf(" %s Strasse     :%s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.strasse);
	mbprintf(" %s Telefon     :%s %s\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),status.telefon);
	mbprintf(" %s Geb.Datum   :%s %d.%d.%d\r\n",
				vt_S(ATTR_FETT),vt_S(ATTR_OFF),
				status.geburtstag,status.geburtsmonat,status.geburtsjahr);

	return TRUE;
}
