/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <pwd.h>

#include "prototyp.h"
#include "modem.h"
#include "io.h"
#include "user.h"
#include "routing.h"
#include "cslib.h"
#include "aktdata.h"
#include "terminal.h"
#include "box_conf.h" /* MYDOMAINPART */
#include "tausch.h"
#include "xstrtoul.h"
#include "libquark.h"
#include "address_hash.h"
#include "str_ulong.h"

extern USER user;

static void ue_show (USER * work, struct mbselect_t *auswahl);
static BOOLEAN ue_edit (USER * w);
static void ue_show_andere (USER * work, struct mbselect_t *auswahl);
static BOOLEAN ue_edit_andere (USER * w);
static char *mkkeyword (const char *in, UWORD level, char *buf,
						struct mbselect_t **next);
static void ue_hilfe(const char *prefix, struct mbselect_t *mbs);

extern WORD count_crW,
 anz_lineW,
 vtmodeW;
extern int logout_modus;

extern WORD temp_levelW;
extern WORD anz_spalten;
extern UWORD comp_type_zahl;
extern const char *comp_type[];

extern int col_ix[];

static const char *on=NULL;
static const char *off=NULL;

static void
ue_init_attr(void)
{
	on = vt_S(col_ix[C_ROT]);
	off=vt_S(ATTR_OFF);
	if (!on)
		on=vt_S(ATTR_FETT);
	if (!on || !off)
	{
		on="\'";
		off="\'";
	}
}

void
usereditor (const char *para)
{
	USER antrag;
	USERNR nr;
	BOOLEAN ist_neu=FALSE;
	struct mbselect_t auswahl[] =
	{
		{'J', "Ja\r\n\n"},
		{'N', "Nein\r\n\n"},
		{0, NULL}
	};

	ue_init_attr();
                                                        
	if (para)
	{
		USERNR *nummern;
		nummern=ub_find_username_multi(para,FALSE);
		if (!nummern && isdigit(*para)) {
			nummern=XMALLOC(2*sizeof(USERNR));
			*nummern=strtol(para,0,10);
			nummern[1]=0;
		}
		if (!nummern)
		{
			mbfehler("unbekannter Benutzer");
			return;
		}
		if (nummern[1])
		{
			int i;
			mboutput("nicht eindeutig. Ausdruck pat auf:\r\n");
			for (i=0;nummern[i];i++)
			{
				LISTUSER *lu;
				lu=ub_find_usernummer(nummern[i]);
				if (lu) /* paranoia ... */
				{
					mbprintf("  %lu %s\r\n",(unsigned long) lu->nummer,lu->name);
				}
				else
					mbprintf("  %lu (nicht gefunden? TSNH)\r\n",
						(ULONG) nummern[i]);
			}
			free(nummern);
			return;
		}
		nr=nummern[0];
		free(nummern);
		if (nr==user.nummer)
		{
			nr = user.nummer;
			antrag = user;
		}
		else
		{
			if (!ub_read_user (nr, &antrag, TRUE))
			{
				mbfehler ("Kann Userdaten nicht lesen");
				return;
			}
			/* Level < 8 drfen nur sich selbst editieren (wozu gibt der 
			 * User dann einen Parameter an? Geht doch auch ohne :-).
			 * Level 8 darf nicht Level 9 editieren.
			 */
			if (antrag.level > level() || level() < 8)
			{
				mbfehler ("Das drfen Sie nicht!");
				ub_write_user (&antrag, TRUE);
				return;
			}
		}
	}
	else if (user.level >= 8)
	{
		/* "static" weil gcc das sonst falsch compiliert (crash). 
		 * Komischerweise seit Umstieg auf glibc-2.0.3. Hmmm ...
		 * -Osuper_optimize,-O2,-O1, alles dasselbe. -O0 compiliert
		 * richtig, -O1 mit etlichen "-fno-xxx" compiliert wieder
		 * falsch.
		 */
		static char opfer[80];
		USERNR *nummern;
		mboutput("\r\n");
		if (!mbquestion (60, 0, TRUE, " Username     > ", "", opfer))
			return;
		strtrim(opfer);
		if (!*opfer)
			return;
		if (logout_modus)
			return;
		nummern=ub_find_username_multi(opfer,FALSE);
		if (nummern && nummern[1])
		{
			int i;
			mboutput("nicht eindeutig. Ausdruck pat auf:\r\n");
			for (i=0;nummern[i];i++)
			{
				LISTUSER *lu;
				lu=ub_find_usernummer(nummern[i]);
				if (lu) /* paranoia ... */
				{
					mbprintf("  %s\r\n",lu->name);
				}
				else
					mbprintf("  %lu (nicht gefunden? TSNH)\r\n",
						(ULONG) nummern[i]);
			}
			free(nummern);
			return;
		}
		if (nummern)
		{
			nr=nummern[0];
			free(nummern);
		}
		else
			nr=0;

		if (!nr)
		{
        	char buf[256];
			mboutput(
	"Benutzer unbekannt. Wollen Sie einen neuen Benutzer anlegen? (J/N)");
			if (mbselect(auswahl)!='J')
				return;
			if (!check_username_syntax (opfer, TRUE))
				return;
			memset(&antrag,0,sizeof(antrag));
			if (!ub_find_username(&nr,"gast"))
			{
				mboutput("Oops: Kein Gast vorhanden?\r\n");
				strcpy(antrag.name,opfer);
			}
			else
			{
				USER gast;
				if (!ub_read_user (nr, &gast, FALSE))
				{
					mbfehler ("Kann Userdaten des Gasts nicht lesen");
					return;
				}
				strcpy(antrag.name,opfer);
				antrag.mehr=gast.mehr;
				antrag.bs_zeilen=gast.bs_zeilen;
				antrag.bs_spalten=gast.bs_spalten;
				antrag.terminal_typ=gast.terminal_typ;
				antrag.uhrmodus=gast.uhrmodus;
				antrag.box_stil=gast.box_stil;
				antrag.menue_stil=gast.menue_stil;
				antrag.glocke=gast.glocke;
				antrag.autohilfe=gast.autohilfe;
				antrag.autobefehl=gast.autobefehl;
				antrag.zeichensatz=gast.zeichensatz;
				antrag.protokoll=gast.protokoll;
				antrag.letzter_login=time(NULL); /* yessir, wegen Usercrunch */

			}
			/* Zuflliges Passwort whlen. 
			 * srand darf nicht mit time(0) initialisiert werden, da das
			 * Rckschlsse zult.
			 */
			srand (akt_anruf.startzeit);
			sprintf(buf,"%lu%lu-%d@%s",
				(ULONG) user.nummer,(ULONG)akt_anruf.anrufnummer,
					rand(),MYDOMAINPART);
			passwort_crypt(antrag.passwort,buf,0,PW_MD5);
			antrag.pw_encoding=PW_MD5;
			antrag.passwortdatum=time(NULL);

			ist_neu=TRUE;
		}
		else if (nr == user.nummer)
			antrag = user;
		else if (!ub_read_user (nr, &antrag, TRUE))
		{
			mbfehler ("Kann Userdaten nicht lesen");
			return;
		}
		if (antrag.level > user.level)
		{
			mbfehler ("Das drfen Sie nicht!");
			ub_write_user (&antrag, TRUE);
			return;
		}
	}
	else
	{
		nr = user.nummer;
		antrag = user;
	}
retry: 
	if (!ue_edit (&antrag) || logout_modus)
	{
		if (user.nummer != antrag.nummer)
			ub_unlock_user(antrag.nummer);
		return;
	}
	mboutput ("nderungen speichern? (J/N):");
	if (mbselect (auswahl) != 'J') /* != J wegen \0 -> logout_modus gesetzt */
	{
		if (antrag.nummer != user.nummer)
		{
			ub_read_user (antrag.nummer, &antrag, FALSE);
			ub_write_user (&antrag, TRUE);
		}
		return;
	}
	if (ist_neu)
	{
		/* neu angelegter Benutzer */
		if (!ub_create_user(&antrag))
		{
			mboutput(
"Der Benutzerdatensatz konnte nicht angelegt werden. Mglicherweise ist das\r\n"
"Problem temporr, sie sollten es daher vielleicht erneut versuchen.\r\n"
"Bitte drcken Sie eine Taste\r\n");
			wait_mehr();
			goto retry;
		}
		if (antrag.nummer != user.nummer)
			ub_write_user(&antrag,TRUE); /* unlocken */
		else
			ub_write_user(&antrag,FALSE); /* nicht unlocken */
	}
	else if (antrag.nummer != user.nummer)
		ub_write_user (&antrag, TRUE);
	else if (antrag.geloescht)
	{
		/* User will sich selbst loeschen, klappt im Moment nur fr
		 * Sysops >= LV 8  
		 */
		user = antrag;
		ub_write_user (&user, TRUE);
	}
	else
	{
		if (stricmp (user.name, "gast"))
			/* Gast nie abspeichern */
			ub_write_user (&antrag, FALSE);
		/*
		 * Jetzt allen Mll setzen:
		 * - Brettflags bei Levelnderung
		 */
		if (user.level != antrag.level)
			b_set_flags (&antrag);
		if (user.zeichensatz != antrag.zeichensatz)
		{
			cstl_t *cs;

			cs = get_charset_from_nr (antrag.zeichensatz);
			if (cs)
				set_charset (cs);
		}
		anz_lineW = antrag.bs_zeilen;
		anz_spalten=antrag.bs_spalten;
		vtmodeW=antrag.terminal_typ;
		sync_terminal();
		user = antrag;
	}
	if (!antrag.geloescht)
		return;
	delete_user(antrag.nummer);
}

#define APPEND(x) { next->key=x; next->output=NULL; next++; }
static char *
mkkeyword (const char *in, UWORD level, char *buf, struct mbselect_t **next)
{
	const char *p;
	char *q;
	size_t zahl = 0;
	BOOLEAN fill = FALSE;

	p = in;
	q = buf;
	while (*p)
	{
		if (*p == '(')
		{
			if (level <= user.level)
			{
				q += sprintf (q, "(%s", on);
				(*next)->key = p[1];
				(*next)->output = NULL;
				(*next)++;
				zahl++;
			}
			else
				fill = TRUE;
		}
		else if (*p == ')')
		{
			if (level <= user.level)
			{
				q += sprintf (q, "%s)", off);
				zahl++;
			}
		}
		else
		{
			*q++ = *p;
			zahl++;
		}
		p++;
	}
	*q++ = ':';
	zahl = p - in;
	if (fill)
		zahl -= 2;				/* Klammern ausgleichen */
	while (zahl++ < 13)
		*q++ = ' ';
	*q = '\0';
	return buf;
}

#define KEY(xxx,erlaubt) mkkeyword(xxx,erlaubt,keybuf,&next)

static void
ue_show (USER * work, struct mbselect_t *auswahl)
{
	char buf[128];
	char keybuf[256];
	struct mbselect_t *next = auswahl;
	struct tm *tm;

	mboutput ("\r\n");
	count_crW = 0;

	/* 
	 * Name
	 */
	mbprintf ("%s %s\r\n", KEY ("(N)ame", 1), work->name);

	/*
	 * Vertreter
	 */
	mbprintf ("%s %s\r\n", KEY ("(V)ertreter", 1), work->vertreter);

	/* 
	 * Level und Benutzernummer
	 */
	sprintf (buf, "%u", work->level);
	mbprintf ("%s %-25.25s", KEY ("(L)evel", 8), buf);
	sprintf (buf, "%lu", work->nummer);
	mbprintf ("%s %s\r\n", KEY ("Nummer", 10), buf);

	/* 
	 * Zahlerstatus und Typ
	 */
	if (work->bezahlt_bis) {
		tm = localtime (&work->bezahlt_bis);
		strftime (buf, 12, "%d.%m.%Y", tm);
		if (work->bezahlt_bis < time(0))
			strcat(buf," (abgelaufen)");
	}
	else 
		strcpy (buf, "nein");
	mbprintf ("%s %-25.25s", KEY ("($)-Bezahlt", 8), buf);
	if (work->typ == UTYP_USER)
		strcpy (buf, "User");
	else if (work->typ == UTYP_GATE)
		strcpy (buf, "Gate");
	else if (work->typ == UTYP_BOX)
		strcpy (buf, "Box");
	else
		strcpy (buf, "???");
	mbprintf ("%s %s\r\n", KEY ("(T)yp", 8), buf);

	/*  
	 * Erster Login + Anzahl Anrufe
	 */
	tm = localtime (&work->erster_login);
	strftime (buf, 12, "%d.%m.%Y", tm);
	mbprintf ("%s %-25.25s", KEY ("Erster Login", 10), buf);
	mbprintf ("%s %lu\r\n", KEY ("Anrufzahl", 10), work->anrufe);

	/*
	 * Letzter Login 
	 */
	tm = localtime (&work->letzter_login);
	strftime (buf, 12, "%d.%m.%Y", tm);
	mbprintf ("%s %s\r\n", KEY ("Letzter Login", 10), buf);

	/*
	 * Passwort + Restzeit
	 */
	mbprintf ("%s %-25.25s", KEY ("(P)asswort", 1),
			  (work->pw_encoding != PW_UNENCODED || work->passwort[0]) ?
			  "ja" : "nein");
	mbprintf ("%s %lu\r\n", KEY ("(R)estzeit", 8), work->restzeit/60);

	/*
	 * Zeichensatz + Uhrmodus
	 */
	{
		cstl_t *cs;

		cs = get_charset_from_nr (work->zeichensatz);
		if (!cs)
			cs = get_charset_from_name ("usascii");
		mbprintf ("%s %-25.25s", KEY ("(Z)eichensatz", 0), cs->name);
	}
	switch (work->uhrmodus)
	{
	case UHR_REST:
		mbprintf ("%s Restzeit\r\n", KEY ("(U)hrmodus", 0));
		break;
	case UHR_ZEIT:
		mbprintf ("%s Tageszeit\r\n", KEY ("(U)hrmodus", 0));
		break;
	case UHR_NORMAL:
		mbprintf ("%s Verbindung\r\n", KEY ("(U)hrmodus", 0));
		break;
	case UHR_AUS:
		mbprintf ("%s Aus\r\n", KEY ("(U)hrmodus", 0));
		break;
	}

	/*
	 * Gastupload + Glocke
	 */
	mbprintf ("%s %-25.25s", KEY ("(G)astupload", 1),
			  work->gast_upload ? "ja" : "nein");
	mbprintf ("%s %s\r\n", KEY ("Gl(O)cke", 1),
			  work->glocke ? "ja" : "nein");

	/*
	 * Auflisten und Protokoll
	 */
	mbprintf ("%s %-25.25s", KEY ("(I)n Listen", 2),
			  work->auflisten ? "ja" : "nein");
	mbprintf ("%s %s\r\n", KEY ("Proto(K)oll", 8),
			  work->protokoll ? "ja" : "nein");


	/*
	 * Terminaltyp und Pflichtbefreiung
	 */
	switch (work->terminal_typ)
	{
	case T_VT100:
		strcpy (buf, "VT100");
		break;
	case T_VT52:
		strcpy (buf, "VT52");
		break;
	case T_ANSI:
		strcpy (buf, "ANSI");
		break;
	default:
		strcpy (buf, "Ascii");
		break;
	}
	mbprintf ("%s %-25.25s\r\n", KEY ("Terminalt(Y)p", 0), buf);
	mbprintf ("%s %s\r\n", KEY ("P(F)lichtfrei", 8),
			  work->pflicht_befreit ? "ja" : "nein");


	/* 
	 * Zeilen und Boxstil 
	 */
	sprintf (buf, "%u", work->bs_zeilen);
	mbprintf ("%s %-25.25s", KEY ("Z(E)ilen", 0), buf);
	if (work->box_stil == QUARK_STYLE)
		strcpy (buf, "Geonetkompatibel");
	else
		strcpy (buf, "Maushnlich");
	mbprintf ("%s %s\r\n", KEY ("Bo(X)stil", 0), buf);

	/* 
	 * Spalten und Autohilfe 
	 */
	sprintf (buf, "%u", work->bs_spalten);
	mbprintf ("%s %-25.25s", KEY ("(S)palten", 0), buf);
	mbprintf ("%s %s\r\n", KEY ("Auto(H)ilfe", 0),
			  work->autohilfe ? "ja" : "nein");


	/*
	 * Automatische Terminalgrenerkennung und Menlnge
	 */
	mbprintf ("%s %-25.25s", KEY ("(A)uto-Resize", 0),
			  work->flags & UB_FLAG_AUTO_RESIZE ? "ja" : "nein");
	mbprintf ("%s %s\r\n", KEY ("Menhilfe (J)", 0),
			  work->menue_stil ? "lang" : "kurz");

	/*
	 * Mehr-Flag + Autobefehl
	 */
	mbprintf ("%s %-25.25s", KEY ("(M)ehr", 0),
			  work->mehr ? "ja" : "nein");
	mbprintf ("%s %s\r\n", KEY ("Auto(B)efehl", 0),
			  work->autobefehl ? "ja" : "nein");

	/* 
	 * Weitere Daten Submen + Tauschkonfiguration Submen
	 */
	mbprintf ("%s %-25.25s", KEY ("(W)eiteres", 1), "");
	if (work->nummer == user.nummer)
	{
		/* Fr sich selbst kann man die Tauschinstallation aufrufen */
		UWORD lv = 1;

		if (!befehl_check_befehl (NULL, "TAUSCH", "INST"))
			lv = 10;
		mbprintf ("%s\r\n", KEY ("Taus(C)h konf.", lv));
	}
	else
		mbprintf ("\r\n");

	APPEND ('?');
	APPEND (27);
	APPEND (0);
}

static BOOLEAN
ue_edit (USER * work)
{
	struct mbselect_t *mbs;
	BOOLEAN changed = FALSE;
	BOOLEAN ende = FALSE;
	char buf[Max (UDAT_NAME_LEN, 128)];
	mbs = malloc (128 * sizeof (struct mbselect_t));

	if (!mbs)
	{
		mbfehler ("Abbruch wegen Speichermangel");
		return TRUE;
	}

	while (!ende)
	{
		char c;

		ue_show (work, mbs);
		mbprintf (
			"\r\nBitte auswhlen, Zurck mit (%sESCAPE%s), Hilfe mit '?':\r\n",
			on,off);
		c = mbselect (mbs);
		if (logout_modus)
			return TRUE;
		switch (c)
		{
		case '?':
			ue_hilfe("main",mbs);
			break;
		case 27:
			ende = TRUE;
			break;
		case 'N':				/* Name */
			{
				USERNR *nummern;

				if (user.level < 8) {
					mboutput(
"Sie knnen hier die Schreibweise ihres Namens korrigieren (speziell Gro/-");
					mboutput(
"kleinschreibung, Umlaute oder expandierte Umlaute), oder Ihre Daten in der");
					mboutput(
"Box lschen, indem Sie ihren Namen lschen\r\n");
				}
				mboutput ("\r\n");
				mbquestion (UDAT_NAME_LEN - 1, 0, TRUE, " Name: ",
							work->name, buf);
				if (!*buf)
				{
					mboutput ("\r\n >> Benutzerdaten lschen? (J/N): ");
					if (mbdecide ("JN", "Ja\r\n\n|Nein...\r\n\n") == 1)
					{
						/* Beim Selbstlschen noch mal nachfragen */
						if (user.nummer==work->nummer) {
							char rand_char;
							srand(time(NULL));
							rand_char= '0' + (rand() % 10);
mboutput( "Wenn Sie sich wirklich lschen wollen (dieser Schritt ist nicht mehr\r\n");
mbprintf( "rckgngig zu machen), dann drcken Sie bitte jetzt die Taste '%c'\r\n",rand_char);
mboutput( "Alle anderen Tasten brechen die Lschung ab.\r\n");
							if (mbchar()==rand_char) {
								ende = TRUE;
								work->geloescht = TRUE;
								changed=TRUE;
								mbprintf("ok, Daten werden gelscht statt gespeichert.\r\n");
							} else  {
								mbprintf("Sehr gut.\r\n");
							}
						} else {
							ende = TRUE;
							work->geloescht = TRUE;
							mbprintf("ok, Daten werden gelscht statt gespeichert.\r\n");
							changed=TRUE;
						}
					}
				}
				else if (!check_username_syntax (buf, TRUE))
					;			/* leer */
				else
				{
					nummern=ub_find_username_multi(buf,FALSE);
					if (user.level < 8) {
						if (nummern[1] || nummern[0]!=work->nummer) {
							mbfehler ("nicht erlaubt");
						} else {
							strcpy (work->name, buf);
						}
					} else if (!nummern || (!nummern[1] && nummern[0]==work->nummer)) {
						/* bewut kein capitalize_words - irgendwie mu man
						 * falsche Groschreibung ja korrigieren knnen.
						 */
						strcpy (work->name, buf);
						changed = TRUE;
					}
					else
						mbfehler ("Benutzername schon vorhanden");
					if (nummern)
						free(nummern);
				}
			}
			break;
		case 'T':				/* Typ */
			{
				USERTYP t;
				struct mbselect_t auswahl[] =
				{
					{'B', "Box\r\n"},
					{'U', "User\r\n"},
					{'G', "Gate\r\n"},
					{27, "Abbruch\r\n"},
					{0, NULL}
				};

				mboutput ("Neuer Typ: (U)ser, (G)ateway, (B)ox?");
				switch (mbselect (auswahl))
				{
				case 'B':
					t = UTYP_BOX;
					break;
				case 'U':
					t = UTYP_USER;
					break;
				case 'G':
					t = UTYP_GATE;
					break;
				default:
					/* kann bei Carrier Lost passieren */
					t = work->typ; 
					break;
				}
				if (t != work->typ)
				{
					changed = TRUE;
					work->typ = t;
				}
			}
			break;
		case 'L':				/* Level */
			sprintf (buf, "%d", (int) work->level);
			mbquestion (1, 2, TRUE, " Level: ", buf, buf);
			if (strtol (buf, NULL, 10) < 0 || strtol (buf, NULL, 10) > 10)
				mbfehler ("Userlevel mu >=0 und <=9 sein");
			else if (strtol (buf, NULL, 10) > user.level)
				mbfehler ("Vergabe hherer Userlevel nicht erlaubt");
			else
			{
				work->level = strtol (buf, NULL, 10);
				changed = TRUE;
			}
			break;
		case 'V':				/* Vertreter */
			mbquestion (UDAT_NAME_LEN - 1, 0, TRUE, " Vertreter: ",
						work->vertreter, buf);
			if (!*buf)
			{
				if (*work->vertreter)
				{
					*work->vertreter = '\0';
					changed = TRUE;
				}
			}
			else
			{
				BOOLEAN eintragen=FALSE;
				/* vrfy mit Level 9 und Zahlerstatus! */
				if (!r_vrfy(buf,NULL,9,TRUE))
					mbfehler ("Adresse nicht zustellbar");
				else
				{
					char *pat;
					eintragen=TRUE;
					pat=strchr(buf,'@');
					if (!pat || r_is_me(pat+1))
					{
						USERNR unr;
						BOOLEAN found;

						eintragen=TRUE;

						if (pat)
							*pat=0;
						found=ub_find_username(&unr,buf);
						if (pat)
							*pat='@';
						if (found && unr==work->nummer)
						{
							*buf=0;
							mboutput ("Vertreter identisch, daher gelscht\r\n");
						}
					}
				}
				if (eintragen)
				{
					strcpy (work->vertreter, buf);
					changed = TRUE;
				}
			}
			break;
		case 'Z':				/* Zeichensatz */
			{
				cstl_t *alt,
				*neu;
				BOOLEAN ende_cs = FALSE;

				alt = get_charset_from_nr (work->zeichensatz);
				cst_list (0);
				do
				{
					mboutput (" \"liste\" fr eine Liste der Zeichenstze,"
							  " \"alias\" fr eine lange Liste.\r\n");
					mbquestion (40, 0, TRUE, " Zeichensatz: ",
								(alt) ? alt->name : "", buf);
					neu = NULL;
					if (strtol (buf, NULL, 10) > 0)
					{
						/* dann Nr. eingegeben */
						neu = get_charset_from_nr (strtol (buf, NULL, 10));
					}
					if (!neu)
						neu = get_charset_from_name (buf);
					if (!neu)
					{
						if (!strnicmp (buf, "list", 4))
							cst_list (0);
						if (!strnicmp (buf, "alias", 5))
							cst_list (1);
						else if (*buf == '\0')
							ende_cs = TRUE;
						else
							mbfehler ("unbekannter Zeichensatz");
					}
					else if (work->zeichensatz != neu->csid)
					{
						work->zeichensatz = neu->csid;
						mbprintf (" Zeichensatz %s eingestellt.\r\n", neu->name);
						if (user.nummer==work->nummer)
							set_charset (neu);
						changed = TRUE;
						ende_cs = TRUE;
					}
					else
						ende_cs = TRUE;		/* keine nderung */
				}
				while (!ende_cs);
			}
			break;
		case '$':				/* Bezahlt bis */
			{
				struct tm *tm;
				time_t t;

				tm = localtime (&work->bezahlt_bis);
				strftime (buf, 12, "%d.%m.%Y", tm);
				mbquestion (10, 2, TRUE, " Bezahlt bis (oder +ZAHL[tdmjy]): ", buf, buf);

				if (*buf && zerlege_datum_2_time_t (buf, &t))
				{
					work->bezahlt_bis = t;
					if (t > time (NULL))
						work->bezahlt = TRUE;
					else
						work->bezahlt = FALSE;
					mbprintf (" ist %sZahler\r\n",
							  (work->bezahlt) ? "" : "kein ");
					changed = TRUE;
				}
				else if (*buf)
				{
					char *end;
					long zahl;
					zahl=strtol(buf,&end,10);
					if (end==buf)
						mbfehler ("unparsbares Datum");
					else
					{
						BOOLEAN ok=TRUE;
						switch(tolower(*end))
						{
						case 't': 
						case '\0': 
						case 'd': 
							zahl *= 86400L; 
							break;
						case 'm': 
							zahl *= 31*86400L; 
							break;
						case 'j': 
						case 'y': 
							zahl*=365 * 86400L; 
							break;
						default:
							mbfehler ("unparsbarer Modifizierer");
							ok=FALSE;
							break;
						}
						if (ok)
						{
							if (work->bezahlt_bis<time(NULL))
								work->bezahlt_bis = time(NULL) + zahl;
							else
								work->bezahlt_bis += zahl;
							if (work->bezahlt_bis > time (NULL))
								work->bezahlt = TRUE;
							else
								work->bezahlt = FALSE;
							changed=TRUE;
						}
					}
				}
			}
			break;
		case 'P':
			{
				BOOLEAN leer;

				if (passwort_aenderung (work, &leer))
					changed = TRUE;
			}
			break;
		case 'R':				/* Restzeit */
			sprintf (buf, "%u", (unsigned) work->restzeit / 60);
			mbquestion (10, 2, TRUE, " Restzeit: ", buf, buf);
			if (strtol (buf, NULL, 10) < 0 || strtol (buf, NULL, 10) > 999)
				mbfehler ("ungltige Restzeit");
			else
			{
				work->restzeit = strtol (buf, NULL, 10) * 60;
				changed = TRUE;
			}
			break;
		case 'U':				/* Uhrmodus */
			{
				struct mbselect_t feld[] =
				{
					{'R', NULL},
					{'T', NULL},
					{'V', NULL},
					{'A', NULL},
					{27, NULL},
					{0, NULL}
				};

				mbprintf (
							 "Neuer Uhrmodus: (%sR%s)est, (%sT%s)ageszeit, (%sV%s)erbindung, (%sA%s)us:",
							 vt_S (ATTR_FETT), vt_S (ATTR_OFF),
							 vt_S (ATTR_FETT), vt_S (ATTR_OFF),
							 vt_S (ATTR_FETT), vt_S (ATTR_OFF),
							 vt_S (ATTR_FETT), vt_S (ATTR_OFF));
				switch (mbselect (feld))
				{
				case 'R':
					work->uhrmodus = UHR_REST;
					break;
				case 'T':
					work->uhrmodus = UHR_ZEIT;
					break;
				case 'V':
					work->uhrmodus = UHR_NORMAL;
					break;
				case 27:
					break;
				default:
					work->uhrmodus = UHR_AUS;
					break;
				}
				changed = TRUE;
			}
			break;
		case 'O':				/* Glocke */
			changed = TRUE;
			work->glocke = !work->glocke;
			break;
		case 'G':				/* Gastupload */
			changed = TRUE;
			work->gast_upload = !work->gast_upload;
			break;
		case 'I':				/* In Listen? */
			changed = TRUE;
			work->auflisten = !work->auflisten;
			break;
		case 'H':				/* Autohilfe */
			changed = TRUE;
			work->autohilfe = !work->autohilfe;
			break;
		case 'J':				/* Mens */
			changed = TRUE;
			work->menue_stil = !work->menue_stil;
		case 'B':				/* Autobefehl */
			changed = TRUE;
			work->autobefehl = !work->autobefehl;
			break;
		case 'K':				/* Protokoll? */
			changed = TRUE;
			work->protokoll = !work->protokoll;
			break;
		case 'F':				/* Pflicht */
			changed = TRUE;
			work->pflicht_befreit = !work->pflicht_befreit;
			break;
		case 'M':				/* Mehr */
			changed = TRUE;
			work->mehr = !work->mehr;
			break;
		case 'E':				/* zEilen */
			if (work->bs_zeilen)
				sprintf (buf, "%u", work->bs_zeilen);
			else
				sprintf (buf, "25");
			mbquestion (2, 2, TRUE, " Anzahl der Zeilen: ", buf, buf);
			if (strtol (buf, NULL, 10) < 10)
				mbfehler ("Ihr Bildschirm wird doch mindestens 10 Zeilen haben");
			else if (strtol (buf, NULL, 10) != work->bs_zeilen)
			{
				changed = TRUE;
				work->bs_zeilen = strtol (buf, NULL, 10);
			}
			break;
		case 'S':				/* Spalten */
			if (work->bs_spalten)
				sprintf (buf, "%u", work->bs_spalten);
			else
				sprintf (buf, "80");
			mbquestion (2, 2, TRUE, " Anzahl der Spalten: ", buf, buf);
			if (strtol (buf, NULL, 10) < 10)
				mbfehler ("Ihr Bildschirm wird doch mindestens 10 Spalten haben");
			else if (strtol (buf, NULL, 10) != work->bs_spalten)
			{
				changed = TRUE;
				work->bs_spalten = strtol (buf, NULL, 10);
			}
			break;
		case 'Y':				/* Terminaltyp */
			{
				struct mbselect_t feld[] =
				{
					{'S', NULL},
					{'1', NULL},
					{'5', NULL},
					{'N', NULL},
					{27, NULL},
					{0, NULL}
				};

				mbprintf (
							 "Terminaltyp: A(%sS%s)cii, VT(%s1%s)00, VT(%s5%s)2, A(%sN%s)si:",
							 on,off,
							 on,off,
							 on,off,
							 on,off);
				switch (mbselect (feld))
				{
				case 'S':
					work->terminal_typ = T_ASCII;
					break;
				case '1':
					work->terminal_typ = T_VT100;
					break;
				case '5':
					work->terminal_typ = T_VT52;
					break;
				case 'N':
					work->terminal_typ = T_ANSI;
					break;
				}
				changed = TRUE;
			}
			break;
		case 'X':				/* Boxstil */
			if (work->box_stil == QUARK_STYLE)
				work->box_stil = MAUS_STYLE;
			else
				work->box_stil = QUARK_STYLE;
			changed = TRUE;
			break;
		case 'C':
			ta_do_inst ();
			break;
		case 'W':
			changed |= ue_edit_andere (work);
			break;
		case 'A':
			if (work->flags & UB_FLAG_AUTO_RESIZE)
				work->flags &= ~(UB_FLAG_AUTO_RESIZE);
			else
				work->flags |= UB_FLAG_AUTO_RESIZE;
			changed=TRUE;
			break;
		}
	}
	free (mbs);
	return changed;
}

/*
 * weniger wichtige Daten
 */
static void
ue_show_andere (USER * work, struct mbselect_t *auswahl)
{
	char buf[128];
	char keybuf[256];
	struct mbselect_t *next = auswahl;
	UWORD lv;

	mboutput ("\r\n");
	count_crW = 0;

	/* 
	 * Name zur Info
	 */
	mbprintf ("%s %s\r\n", KEY ("Name", 10), work->name);

	/* 
	 * Geschlecht und Geburtsdatum
	 */
	switch (work->geschlecht)
	{
	case WEIBLICH:
		strcpy (buf, "weiblich");
		break;
	case MAENNLICH:
		strcpy (buf, "mnnlich");
		break;
	default:
		strcpy (buf, "Programm");
		break;
	}
	mbprintf ("%s %-25.25s", KEY ("(G)eschlecht", 8), buf);
	mbprintf ("%s %u.%u.%u\r\n", KEY ("G(E)burtsdatum", 1),
			  work->geburtstag, work->geburtsmonat, work->geburtsjahr);

	/*
	 * Strasse
	 */
	mbprintf ("%s %s\r\n", KEY ("(S)trae", 1), work->strasse);
	/*
	 * PLZ + Freigabe
	 */
	mbprintf ("%s %-25.25s", KEY ("(P)LZ", 1), work->plz);
	mbprintf ("%s %s\r\n", KEY ("(F)reigabe", 1),
			  work->adresse_frei ? "ja" : "nein");
	/*
	 * Ort
	 */
	mbprintf ("%s %s\r\n", KEY ("(O)rt", 1), work->ort);
	/*
	 * Telefon.
	 */
	mbprintf ("%s %-25.25s\r\n", KEY ("(T)elefon", 1), work->telefon);
	
	/*
	 * Gltigkeitsbeschrnkung der Adresse.
	 */
	{
		const char *s;
		char xbuf[STR_ULONG+7];
		if (work->flags & UB_FLAG_IMMER_AEXPIRE) {
			s="Immer";
			str_ulong(xbuf,work->address_hash_expire);
		} else if (work->address_hash_expire) {
			s="Fallweise";
			str_ulong(xbuf,work->address_hash_expire);
		} else {
			s="Nie";
			*xbuf=0;
		}
		mbprintf ("%s %s", KEY ("Adress-Exp(I)re", 1), s);
		if (*xbuf)
			mbprintf (" (%s Tage)\r\n",xbuf);
		else
			mboutput ("\r\n");
	}

	/* 
	 * Computer und Typ 
	 */
	mbprintf ("%s %-25.25s", KEY ("(C)omputer", 1), work->computer);
	if (work->computertyp < comp_type_zahl)
		mbprintf ("%s %s\r\n", KEY ("T(Y)p", 1), comp_type[work->computertyp]);
	else
		mbprintf ("%s %s\r\n", KEY ("T(Y)p", 1), comp_type[0]);

	/* 
	 * Defaultprotokoll( braindead), Automatisches Anlegen von Gruppen  
	 */
	mbprintf ("%s %c%-18.18s", KEY ("(D)efault-Protokoll", 1),
			  work->default_protokoll ? work->default_protokoll : ' ', "");
	mbprintf ("%s %s\r\n", KEY ("(A)utomatik-Gruppen", 8),
			  work->auto_grp_create ? "ja" : "nein");

	/*
	 * Shelluserid
	 */
	if (work->assigned_uid)
	{
		struct passwd *pwd;
		pwd=getpwuid(work->assigned_uid);
		if (pwd)
			sprintf (buf, "%s", pwd->pw_name);
		else
			sprintf (buf, "%lu (nicht in passwd)", work->assigned_uid);
		lv=1;
	}
	else
	{
		if (akt_connection.shelluser)
			lv=1;
		else
			lv=9;
		strcpy(buf,"keine");
	}
	mbprintf ("%s %-25.25s", KEY ("(U)serid", lv), buf);
	mbprintf ("%s %s\r\n", KEY ("(R)eadline", 8),
			  work->flags & UB_FLAG_READLINE ? "ja" : "nein");

	mbprintf ("%s %-25.25s", KEY ("Pro(M)i", 8),
			  work->flags & UB_FLAG_PROMI ? "ja" : "nein");
	if (work->async_max_spool)
		mbprintf ("%s %lu\r\n", KEY ("Async-Ma(X)", 8),
				  work->async_max_spool);
	else
		mbprintf ("%s default\r\n", KEY ("Async-Ma(X)", 8));

	/*
	 * Ansage
	 */
	mbprintf ("%s %s\r\n", KEY ("A(N)sage", 1), work->ansage);

	APPEND (27);
	APPEND ('?');
	APPEND (0);
}

static void
ue_hilfe(const char *prefix, struct mbselect_t *mbs)
{
	char c;
	FILE *f;
	char buf[1024];
	BOOLEAN show;
	BOOLEAN found;
	mboutput("\r\nKurzbeschreibung zu welchem Punkt?\r\n\n\n");
	c = mbselect (mbs);
	if (logout_modus)
		return;
	if (c=='\033') {
		mboutput("Abbruch\r\n");
		return;
	} else if (c=='?') {
		mboutput(
"Mit ? knnen Sie eine Kurzbeschreibung der Menpunkte anfordern\r\n\n\n");
		return;
	}
	f=fopen(tprintf("%susereditor.men",box_conf.hilfe_dir),"r");
	if (!f)
	{
		mboutput("Hilfetext nicht verfgbar!\r\n\n\n");
		return;
	}
	show=FALSE;
	found=FALSE;
	while (fgets(buf,sizeof(buf),f))
	{
		char *p;
		if (*buf=='#')
		{
			size_t l;
			p=strpbrk(buf,"\r\n");
			if (p)
				*p=0;
			strtrim(buf+1);
			p=strpbrk(buf," \t");
			if (!p)
				continue;
			*p++=0;
			while (isspace(*p))
				p++;
			if (0!=stricmp("#entry",buf))
				continue;
			show=FALSE;
			l=strlen(prefix);
			if (0!=strnicmp(p,prefix,l))
				continue;
			if (p[l]!='-')
				continue;
			if (tolower(p[l+1])==tolower(c))
			{
				show=TRUE;
				found=TRUE;
			}
			continue;
		}
		if (!show)
			continue;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p=0;
		mbprintf("%s\r\n",buf);
	}
	fclose(f);
	if (!found)
	{
		mbprintf("Hilfeeintrag %s-%c nicht gefunden!\r\n",
				 prefix,c);
	}
	mboutput("\r\n\n");
}

static BOOLEAN
ue_edit_andere (USER * work)
{
	struct mbselect_t *mbs;
	BOOLEAN changed = FALSE;
	BOOLEAN ende = FALSE;
	char buf[128];
	ULONG ultmp;
	enum strtol_error s_err;

	mbs = malloc (128 * sizeof (struct mbselect_t));

	if (!mbs)
	{
		mbfehler ("Abbruch wegen Speichermangel");
		return FALSE;
	}

	while (!ende)
	{
		char c;

		ue_show_andere (work, mbs);
		mbprintf (
			"\r\nBitte auswhlen, Zurck mit (%sESCAPE%s), Hilfe mit '?':\r\n",
			on,off);
		c = mbselect (mbs);
		if (logout_modus)
			return FALSE;
		switch (c)
		{
		case '?':
			ue_hilfe("andere",mbs);
			break;
		case 27:
			ende = TRUE;
			break;
		case 'X': /* async-max-spool */
			sprintf (buf, "%lu", work->async_max_spool);
			mbquestion (20, 1, TRUE, " Maximale Groesse aller vorgespoolten Outfiles (nnn[km]): ", buf, buf);
			s_err=xstrtoul (buf, NULL, 0, &ultmp, "km");
			if (s_err != LONGINT_OK) {
				STRTOL_FAIL_WARN(buf,"Maximalgre",s_err);
			} else if (ultmp < 1000000)
				mbfehler ("Mindestens eine Millionen Bytes!");
			else if (ultmp != work->async_max_spool)
			{
				changed = TRUE;
				work->async_max_spool = ultmp;
			}
			break;
		case 'M':
			if (work->flags & UB_FLAG_PROMI)
				work->flags &= ~(UB_FLAG_PROMI);
			else
				work->flags |= UB_FLAG_PROMI;
			changed = TRUE;
			break;
		case 'R':
			if (work->flags & UB_FLAG_READLINE)
				work->flags &= ~(UB_FLAG_READLINE);
			else
				work->flags |= UB_FLAG_READLINE;
			changed = TRUE;
			break;
		case 'A':
			work->auto_grp_create = !work->auto_grp_create;
			changed = TRUE;
			break;
		case 'F':
			work->adresse_frei = !work->adresse_frei;
			changed = TRUE;
			break;
		case 'I':
			if (work->flags & UB_FLAG_IMMER_AEXPIRE) {
				work->flags-=UB_FLAG_IMMER_AEXPIRE;
				mboutput(" --> fallweise\r\n");
			} else if (work->address_hash_expire) {
				work->address_hash_expire=0,
				mboutput(" --> nie\r\n");
			} else {
				work->flags+=UB_FLAG_IMMER_AEXPIRE;
				mboutput(" --> immer\r\n");
			}
			if (work->flags & UB_FLAG_IMMER_AEXPIRE 
				|| work->address_hash_expire) {
				sprintf (buf, "%lu", 
					work->address_hash_expire ? work->address_hash_expire : 14);
				mbquestion (20, 1, TRUE, " Gltigkeit von Adressen in Tagen: (minimal 14, 0 endlos) ", buf, buf);
				s_err=xstrtoul (buf, NULL, 0, &ultmp, NULL);
				if (s_err != LONGINT_OK) {
					STRTOL_FAIL_WARN(buf,"Gltigkeit",s_err);
				} else if (ultmp < 14 && ultmp!=0)
					mbfehler ("Mindestens 14 Tage!");
				else if (ultmp != work->address_hash_expire)
				{
					changed = TRUE;
					work->address_hash_expire = ultmp;
					if (!work->address_hash_secret && ultmp)
						work->address_hash_secret=address_hash_gen_key();
					if (!ultmp)
						work->flags&=~(UB_FLAG_IMMER_AEXPIRE);
				}
			}
			break;
		case 'Y':				/* Computertyp */
			{
				char help[8];
				UWORD n;

				do
				{
					int i;

					for (i = 0; i < comp_type_zahl; i++)
						mbprintf ("  %s %d%s -> %s\r\n",
								  on, i + 1, off, comp_type[i]);

					sprintf (help, "%d", work->computertyp + 1);
					mbquestion (2, 0, TRUE, "    Computertyp (fr die Statistik): ",
								help, buf);
					n = strtol (buf, NULL, 10);
				}
				while (n > comp_type_zahl || n == 0);
				n--; /* 1 offset ab */
				if (n != work->computertyp) {
					work->computertyp = n;
					changed++;
				}
			}
			break;
#define CHANGESTR(SEL,LEN,TEXT,VAR) \
case SEL: \
mbquestion (LEN - 1, 0, TRUE, TEXT, work->VAR, buf); \
if (strcmp(work->VAR,buf)) { strcpy(work->VAR,buf); changed=TRUE; } \
break
			CHANGESTR ('S', UDAT_STRA_LEN, " Strae: ", strasse);
			CHANGESTR ('O', UDAT_ORT_LEN, " Ort: ", ort);
			CHANGESTR ('P', UDAT_PLZ_LEN, " PLZ: ", plz);
			CHANGESTR ('T', UDAT_TEL_LEN, " Telefon: ", telefon);
			CHANGESTR ('C', UDAT_COMP_LEN, " Computer: ", computer);
			break;
		case 'G':				/* Geschlecht */
			{
				GESCHLECHT t=work->geschlecht;
				struct mbselect_t auswahl[] =
				{
					{'M', "mnnlich\r\n"},
					{'W', "weiblich\r\n"},
					{'P', "Programm\r\n"},
					{0, NULL}
				};

				mbprintf (
					"Geschlecht: (%sm%s)nnlich, (%sw%s)eiblich, (%sP%s)rogramm?",
					on,off,on,off,on,off);
				switch (mbselect (auswahl))
				{
				case 'W':
					t = WEIBLICH;
					break;
				case 'M':
					t = MAENNLICH;
					break;
				default:		/* Bei Carrier Lost mglich */
					break;
				case 'P':
					t = NEUTRUM;
					break;
				}
				if (t != work->geschlecht)
				{
					changed = TRUE;
					work->geschlecht = t;
				}
			}
			break;
		case 'E':				/* Geburtsdatum */
			{
				int t,
				 m,
				 j;

				sprintf (buf, "%u.%u.%u", work->geburtstag, work->geburtsmonat,
						 work->geburtsjahr);
				mbquestion (20, 0, TRUE, " Geburtsdatum: ", buf, buf);
				if (zerlege_datum (buf, &t, &m, &j))
				{
					work->geburtsjahr = j;
					work->geburtsmonat = m;
					work->geburtstag = t;
					changed = TRUE;
				}
				else
					mbfehler ("unverstndliches Datum");
			}
			break;
		case 'D':				/* Defaultprotokoll */
			/* Braindead idea, aber weil es gewnscht wurde ... */
			{
				char wahl_prot;

				/* Eines von allen Protokollen auswhlen */
				wahl_prot = transfer_protokollauswahl (
									   level () == 9 ? TRUE : FALSE, TRUE);
				if (wahl_prot && wahl_prot != work->default_protokoll)
				{
					if (wahl_prot==' ')
						work->default_protokoll = 0;
					else
						work->default_protokoll = wahl_prot;
					changed = TRUE;
				}
			}
			break;
		case 'U':				/* Unix-Userid */
			{
				struct passwd *pwd;
				pwd=getpwuid(work->assigned_uid);
				if (pwd)
					strcpy (buf, pwd->pw_name);
				else
					sprintf (buf, "%lu (nicht in passwd)", work->assigned_uid);
				mbquestion (32, 0, TRUE, " Zugeordnete Systembenutzer-ID: ", 
					buf, buf);
				strtrim(buf);
				if (!*buf)
				{
					/* Jeder darf das Flag fr sich lschen */
					changed = TRUE;
					work->assigned_uid = 0;
				}
				else
				{
					char *n;
					pwd=getpwnam(buf);
					if (!pwd)
					{
						if (strtol(buf,&n,10)>0 && *n==0)
							pwd=getpwuid(strtol(buf,NULL,10));
					}
					if (pwd)
					{
						if (level()<9 && akt_connection.ruid!=pwd->pw_uid)
						{
							mbfehler ("nicht erlaubt");
						}
						else
						{
							changed = TRUE;
							work->assigned_uid = pwd->pw_uid;
						}
					}
					else
						mbfehler ("nicht bekannt");
				}
			}
			break;
		case 'N':				/* Ansage */
			mbquestion (UDAT_ANS_LEN - 1, 0, TRUE, " Ansage: ", work->ansage,
						work->ansage);
			if (!strnicmp(work->ansage,"PMs an",6))
			{
				mboutput(
					"`PMs an' ist falsch, benutzen Sie 'V'ertreter\r\n");
			}
			changed = TRUE;
			break;
		}
	}
	free (mbs);
	return changed;
}
