/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Binrteil-Verwaltung
	- brettweise
	- global

	von Frank Baumgart

	begonnen am 26.01.1993
	Version vom 11.08.1994
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <unistd.h>
#include <regex.h>
#include <syslog.h>

#include "bin_msg.h"
#include "prototyp.h"
#include "tompaths.h"
#include "box_conf.h"

#include "filelock.h" /* libquark */


extern long len_datei(const char *datei);
extern BOOLEAN fileexist(const char *name);
extern void mboutput(const char *daten);

extern long max_brettnr;


extern BRETTINFO *br_ind;	/* Feld mit Brettnummern, -namen, -flags */


static long bin_global_loeschen(int art,...);


/*
	Binrindexdatei ffnen

	->			brettnr	:	interne Brettnummer
				write		:	Schreibzugriff (TRUE / FALSE)

	<-			stream oder NULL fr (Schreib-)Zugriff nicht mglich

	### fr MultiPort-Tauglichkeit fehlen noch zwei wesentliche Dinge:
	- write mu die Datei lock-en
	- es mu ein Mechanismus (z.B. handle) existieren, um locks freizugeben
	  (in binaer_dat_schliessen)
*/
FILE *binaer_dat_oeffnen(BRETTNR bnr,BOOLEAN for_write)
{
	char name[PATH_MAX];

	sprintf(name,"%s%s",brettpfad(bnr),"inhalt.dat");

	if (for_write)
		return fopen(name,BIN_RW_OPEN);
	else
		return fopen(name,BINREAD);

} /* binaer_dat_oeffnen */



/*
	Binrindexdatei schlieen
*/
void binaer_dat_schliessen(FILE *bin_f)
{
	fclose(bin_f);

} /* binaer_dat_schliessen */



/*
	Binrdatensatz anhngen
*/
BOOLEAN binaer_dat_anhaengen(BRETTNR bnr,BIN_MSG *bin_info)
{
	char name[PATH_MAX];
	struct stat statbuf;
	FILE *bin_f;
	long msgnr;

	sprintf(name,"%s%s",brettpfad(bnr),"inhalt.dat");

	if (stat(name,&statbuf))
		msgnr = 1;
	else
		msgnr = statbuf.st_size / sizeof(BIN_MSG) +1;

	if ((bin_f = fopen(name,BINAPPEND)) == NULL)
		return FALSE;

/*	bin_info->nr = mnk_bin;*/
	bin_info->lokal=msgnr;
	bin_info->brettnr=bnr;
	fwrite(bin_info,sizeof(BIN_MSG),1,bin_f);
	fclose(bin_f);

	bin_global_append(bin_info);

	return TRUE;

} /* binaer_dat_anhaengen */



/*
	Binrdatensatz entfernen

	->			bnr	:	interne Brettnummer
				nummer	:	lokale Dateinummer

	<-			FALSE		:	Eintrag nicht gefunden
				TRUE		:	Binr-ID Nummer

	es wird nur der Eintrag als gelscht markiert
	physikalisch gelscht wird erst durch die Putz-Funktion

	Da auf dieser Ebene nichts ber Benutzer und Zugriffsrechte bekannt ist, 
	darf diese Funktion nur fr interne Zwecke verwendet werden.
*/
BOOLEAN binaer_dat_loeschen(BRETTNR bnr,long nummer)
{
	FILE *bin_f;
	BIN_MSG bin_info;
	BOOLEAN found = FALSE;

	if ((bin_f = binaer_dat_oeffnen(bnr,TRUE)) == NULL)
		return FALSE;

	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (bin_info.lokal == nummer)
		{
			bin_info.deleted = TRUE;
			fseek(bin_f,(long) - sizeof(BIN_MSG),SEEK_CUR);
			fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
			fclose(bin_f);

			bin_global_loeschen('I',bin_info.nr);

			found = TRUE;
			break;
		}
	}
	binaer_dat_schliessen(bin_f);

	return found;

} /* binaer_dat_loeschen */



/*
	Binrdatensatz suchen

	->			bnr	:	interne Brettnummer
				bin_nummer:	lokale Dateinummer
			<-	bin_info	:	Speicher fr Binrstruktur bzw. NULL

	<-			FALSE		:	Eintrag nicht gefunden
				TRUE		:	Binr-ID Nummer
*/
BOOLEAN binaer_dat_suchen(BRETTNR bnr,long bin_nummer,BIN_MSG *bin_info)
{
	FILE *bin_f;
	BIN_MSG tmp_info;
	BOOLEAN found = FALSE;

	if ((bin_f = binaer_dat_oeffnen(bnr,FALSE)) == NULL)
		return FALSE;

	while (fread(&tmp_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (tmp_info.lokal == bin_nummer)
		{
			if (bin_info)
				*bin_info = tmp_info;
			found = TRUE;
			break;
		}
	}
	binaer_dat_schliessen(bin_f);

	return found;

} /* binaer_dat_suchen */



/*
	Binrdatensatz kopieren

	->			bnr	:	interne Brettnummer
				zielbrett:	interne Brettnummer fr Ziel
				nummer	:	lokale Dateinummer

	<-			FALSE		:	Fehler, beim Kopieren, Eintrag/Zielbrett nicht vorhanden
				TRUE		:	Binr-ID Nummer

	Plausibilittsprfungen bzgl. Userstatus und Brettart haben beim Aufrufer 
	zu erfolgen.
*/
BOOLEAN binaer_dat_kopieren(BRETTNR bnr,BRETTNR zielbrett,long nummer)
{
	BIN_MSG bin_info;
	char src[PATH_MAX],
	     dst[PATH_MAX];
	long src_nr,
	     dst_nr;

	if (!binaer_dat_suchen(bnr,nummer,&bin_info))
		return FALSE;

	src_nr = bin_info.nr;
	dst_nr = get_new_id_nr(TRUE);
	bin_info.nr = dst_nr;

	sprintf(src,"%s%ld.bin",brettpfad(bnr),src_nr);
	sprintf(dst,"%s%ld.bin",brettpfad(zielbrett),dst_nr);
	if (!file_copy(src,dst))
		return FALSE;

	sprintf(src,"%s%ld.txt",brettpfad(bnr),src_nr);
	sprintf(dst,"%s%ld.txt",brettpfad(zielbrett),dst_nr);
	if (fileexist(src))
		file_copy(src,dst);

	if (!binaer_dat_anhaengen(zielbrett,&bin_info))
		return FALSE;

	return TRUE;

} /* binaer_dat_kopieren */


#if 0
/*
	Binrdatensatz verschieben

	->			bnr	:	interne Brettnummer
				zielbrett:	interne Brettnummer fr Ziel
				nummer	:	lokale Dateinummer

	<-			FALSE		:	Fehler, beim Kopieren, Eintrag/Zielbrett nicht vorhanden
				TRUE		:	Binr-ID Nummer

	Plausibilittsprfungen bzgl. Userstatus und Brettart haben beim Aufrufer 
	zu erfolgen.
	### Im Quellbrett mte der Eintrag eigentlich sofort physikalisch aus 
	der inhalt.dat entfernt werden -> einfach den crunch aufrufen?
*/
BOOLEAN binaer_dat_verschieben(BRETTNR bnr,BRETTNR zielbrett,long nummer)
{
	BIN_MSG bin_info;
	char src[PATH_MAX],
	     dst[PATH_MAX];
	FILE *src_f,
	     *dst_f;
	long src_nr;

	if (!binaer_dat_suchen(bnr,nummer,&bin_info))
		return FALSE;

	src_nr = bin_info.nr;

	if ((dst_f = binaer_dat_oeffnen(zielbrett,TRUE)) == NULL)
		return FALSE;

/*	strncpy(bin_info.absender,user.name,sizeof(bin_info.absender));
	bin_info.usernr = user.nummer;*/
	bin_info.datime = time(NULL);

	/* ... */

	bin_info.nr = get_new_id_nr(TRUE);
	if (!binaer_dat_anhaengen(zielbrett,&bin_info))
		return FALSE;

	sprintf(src,"%s%ld.bin",brettpfad(bnr),src_nr);
	sprintf(dst,"%s%ld.bin",brettpfad(zielbrett),bin_info.nr);
	if (!file_move(src,dst))
		return FALSE;

	sprintf(src,"%s%ld.txt",brettpfad(bnr),src_nr);
	sprintf(dst,"%s%ld.txt",brettpfad(zielbrett),bin_info.nr);
	if (fileexist(src))
		file_move(src,dst);

	if (!binaer_dat_loeschen(bnr,nummer))
		return FALSE;

	return TRUE;

} /* binaer_dat_verschieben */
#endif



/*
	lokale Binrindexdatei putzen

	->			bnr	:	interne Nummer des Brettes

	<-			FALSE		:	Fehler
				TRUE		:	ok, auch wenn nichts gelscht

	Dateien in ffentlichen Brettern, die seit mehr als 6 Monaten nicht 
	downgeloadet wurden, werden automatisch gelscht.
	Ebenso Dateien in Programmfchern, die bereits vor mehr als 7 Tagen 
	bertragen wurden oder dort seit mehr als 3 Monaten liegen.

	Aufgaben:
	Lschen der Binrdateien
	Umnummerierung der lokalen, brettinternen Nummern.

	BUG:
	derzeit wird immer alles radikal weggeputzt
*/
BOOLEAN binaer_dat_putzen(BRETTNR bnr)
{
	char src[PATH_MAX],
	     dst[PATH_MAX];
	FILE *src_f,
	     *dst_f;
	BIN_MSG bin_info;
	long ret = 0;
	time_t ref_time = time(NULL);
	const char *pfad;

	pfad = brettpfad(bnr);
	sprintf(src,"%s%s",pfad,"inhalt.dat");
	sprintf(dst,"%s%s",pfad,"inhalt.tmp");

	if ((src_f = fopen(src,BIN_RW_OPEN)) == NULL)
		return FALSE;
	if ((dst_f = fopen(dst,BIN_RW_CREATE)) == NULL)
	{
		fclose(src_f);
		return FALSE;
	}

#define EIN_TAG		24*60*60L

	while (fread(&bin_info,sizeof(BIN_MSG),1,src_f))
	{
		/* Vergleichsdatum entweder Zeitpunkt des letzten Downloads, 
			oder, falls noch kein Download erfolgte, des Uploads */
		time_t haltezeit = bin_info.last_download
		                 ? bin_info.last_download
		                 : bin_info.datime;

		/*
			Bedingungen fr Lschung:
			- zum Lschen markiert
			- ffentlich, lnger als 6 Monate nicht downgeloadet
			- persnlich, lnger als 3 Monate nicht abgerufen
			- persnlich, vor mehr als 7 Tagen abgerufen
		*/
		if (bnr)
			haltezeit += 6*30 * EIN_TAG;
		else
			if (bin_info.zugriffe)
				haltezeit += 7 * EIN_TAG;
			else
				haltezeit += 3*30 * EIN_TAG;

		if (bin_info.deleted || haltezeit < ref_time)
		{
			char old[PATH_MAX];

			/* Binr- und Infodatei lschen */
			sprintf(old,"%s%ld.bin",pfad,bin_info.nr);
			unlink(old);
			sprintf(old,"%s%ld.txt",pfad,bin_info.nr);
			unlink(old);

			mboutput("Datei gelscht\r\n");
			ret++;
		}
		else
		{
			bin_info.lokal -= ret;
			fwrite(&bin_info,sizeof(BIN_MSG),1,dst_f);
		}
	}

	fclose(src_f);
	fclose(dst_f);

	if (ret)
	{
		unlink(src);
		rename(dst,src);
	}
	else
		unlink(dst);

	return TRUE;

} /* binaer_dat_putzen */



/*
	alle Binrbretter bzw. Programmfcher putzen
	07.10.93 Frank
*/
BOOLEAN binaer_dat_putze_alle(BOOLEAN pm)
{
	BRETTNR bnr = 0;
	BOOLEAN ok = TRUE;

	if (pm)
		ok = binaer_dat_putzen(0);
	else
	{
		for (bnr = 0; bnr < max_brettnr; bnr++)
			if (br_ind[bnr].brett && br_ind[bnr].brett->art==BINAER_BRETT 
				&& !binaer_dat_putzen(bnr))
				ok = FALSE;
	}

	return ok;

} /* binaer_dat_putze_alle */



/*
	Anzahl Binrdatenstze liefern

	<-			bnr	:	interne Brettnummer
				ref_time	:	Referenzzeit, normalisiert
			<-	all		:	Anzahl aller Dateien, inkl. lschmarkierter
			<-	neu		:	Anzahl neuer Dateien
			<-	bytes		:	Summe der Gren neuer Dateien
			<-	geloescht:	davon lschmarkiert

	<-			FALSE		:	Fehler, Brett nicht vorhanden
				TRUE		:	ok

	all, neu, bytes und geloescht knnen als Eingabewerte auch NULL sein, 
	dann wird der entsprechende Wert nicht zurckgeliefert

	Bug:
	lschmarkierte sollten bei Sysop und Brettverwalter mitgezhlt werden
	dito nicht freigegebene
*/
BOOLEAN binaer_dat_anzahl(BRETTNR bnr,time_t ref_time,long *all,long *neu,long *bytes,long *geloescht)
{
	FILE *bin_f;
	BIN_MSG bin_info;

	if (all)  *all = 0;
	if (neu)  *neu = 0;
	if (bytes)  *bytes = 0;
	if (geloescht)  *geloescht = 0;

	/* nur Gesamtanzahl feststellen ? ist wesentlich schneller */
	if (!ref_time && !neu && !bytes && !geloescht && all)
	{
		char datei[PATH_MAX];
		sprintf(datei,"%s%s",brettpfad(bnr),"inhalt.dat");
		*all = len_datei(datei) / sizeof(BIN_MSG);
		return TRUE;
	}

	if ((bin_f = binaer_dat_oeffnen(bnr,FALSE)) == NULL)
		return FALSE;

	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (all)  (*all)++;

		if (bin_info.datime >= ref_time && bin_read_access(&bin_info))
		{
			if (bytes)
				*bytes += bin_info.bytes;

			if (geloescht && bin_info.deleted)
				*geloescht += bin_info.bytes;

			/* Status-defines fehlen ### */
			if (bin_info.status == 0  &&  neu)
				(*neu)++;
		}
	}
	binaer_dat_schliessen(bin_f);

	return TRUE;

} /* binaer_dat_anzahl */



/*
	Programmfach eines Users entfernen

	->			usernr	:	interne Usernummer

	erst werden alle Eintrge als gelscht markiert,
	dann physikalisch gelscht durch Aufruf der Putz-Funktion
*/
void loesche_programmfach(USERNR usernr)
{
	FILE *bin_f;
	BIN_MSG bin_info;

	if ((bin_f = binaer_dat_oeffnen(0,TRUE)) == NULL)
		return;

	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (bin_info.empfaenger == usernr)
		{
			bin_info.deleted = TRUE;
			fseek(bin_f,(long) - sizeof(BIN_MSG),SEEK_CUR);
			fwrite(&bin_info,sizeof(BIN_MSG),1,bin_f);
			fseek(bin_f,0,SEEK_CUR);
		}
	}
	binaer_dat_schliessen(bin_f);

	binaer_dat_putzen(0);

} /* binaer_dat_loeschen */



/*
	globale Binrverwaltung
*/



/*
	globale Binrindexdatei ffnen

	->			write		:	Schreibzugriff (TRUE / FALSE)

	<-			stream oder NULL fr (Schreib-)Zugriff nicht mglich
*/
FILE *bin_global_oeffnen(BOOLEAN for_write)
{
	char name[PATH_MAX];

	sprintf(name,"%s%s",box_conf.var_dir,"bin_glob.dat");

	if (for_write)
		return fopen(name,BIN_RW_OPEN);
	else
		return fopen(name,BINREAD);

} /* bin_global_oeffnen */



/*
	globalen Binrindex erstellen

	07.07.94 Frank
	### diese Funktion mu spter in den globalen Server
*/
BOOLEAN bin_global_create(void)
{

	char name[PATH_MAX];
	FILE *glob_f;
	FILE *bin_f;
	BIN_MSG bin_info;
	BRETTNR i;

	sprintf(name,"%s%s",box_conf.var_dir,"bin_glob.dat");
	if ((glob_f = fopen(name,BINWRITE)) == NULL)
		return FALSE;

	for (i = 1; i < max_brettnr; i++)
	{
		if (!br_ind[i].brett)
			continue;

		if (br_ind[i].brett->art != BINAER_BRETT)
			continue;

		if ((bin_f = binaer_dat_oeffnen(i,FALSE)) == NULL)
			continue;

		while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
		{
			if (bin_info.deleted)
				continue;

			bin_info.brettnr = i;
			fwrite(&bin_info,sizeof(BIN_MSG),1,glob_f);
		}
		binaer_dat_schliessen(bin_f);
	}
	fclose(glob_f);

	return TRUE;

} /* bin_global_create */



/*
	Binrdatensatz anhngen

	->			bin_info	:	Binrdatensatz

	<-			TRUE		:	ok
				FALSE		:	Fehler
*/
BOOLEAN bin_global_append(BIN_MSG *bin_info)
{
	FILE *glob_f;

	if ((glob_f = bin_global_oeffnen(TRUE)) == NULL)
		return FALSE;
	fseek(glob_f,0,SEEK_END);
	fwrite(bin_info,sizeof(BIN_MSG),1,glob_f);
	fclose(glob_f);

	return TRUE;

} /* bin_global_append */



/*
	Binrdatensatz suchen

	->			art		:	Suchkriterium,
								I = ID
								N = Name
								W = Name mit Wildcard
								A = Absender
								B = Betreff
								S = System
								R = Regulrer Ausdruck in Name und Betreff
				suche		:	Suchtext bzw. Zahl in ASCII
				start		:	Startpunkt, globale Binrdatensatznummer
			<-	bin_info	:	Puffer fr Binrdatensatz bzw. NULL

	<-			< 0		:	Fehler
				>= 0		:	Index in globaler Binrdatendatei
*/
long bin_global_suchen(int art,const char *suche,long start,BIN_MSG *bin_info)
{
	FILE *glob_f;
	BIN_MSG bin_tmp,ref;
	long nr = start;
	BOOLEAN regular=FALSE;
	regex_t expression;
	


	switch (art)
	{
		case	'I'	:
			ref.nr = atol(suche);
			break;

		case	'N'	:
		case	'W'	:
		case	'A'	:
		case	'B'	:
			break;

		case	'S'	:
			ref.system = atol(suche);
			break;

		case    'R' :
			regular = TRUE;
			if (regcomp(&expression,suche,REG_ICASE|REG_NOSUB))
				return -1;
			break;

		default	:
			return -1;
	}

	if ((glob_f = bin_global_oeffnen(FALSE)) == NULL)
		return -1;

	if (start)
		fseek(glob_f,start * sizeof(BIN_MSG),SEEK_SET);

	while (!feof(glob_f))
	{
		if (!fread(&bin_tmp,sizeof(BIN_MSG),1,glob_f))
		{
			nr = -1;
			break;
		}

		if (regular)
		{
			if (!regexec(&expression,bin_tmp.name,0,NULL,0)
				|| !regexec(&expression,bin_tmp.betreff,0,NULL,0))
			{
				*bin_info = bin_tmp;
				break;
			}
		}
		if((art == 'I' && bin_tmp.nr == ref.nr)
		|| (art == 'W' && wildcard(bin_tmp.name,suche))
		|| (art == 'N' && !stricmp(suche,bin_tmp.name))
		|| (art == 'A' && !stricmp(suche,bin_tmp.absender))
		|| (art == 'B' && !stricmp(suche,bin_tmp.betreff))
		|| (art == 'S' && bin_tmp.system == ref.system))
		{
			*bin_info = bin_tmp;
			break;
		}

		nr++;
	}

	fclose(glob_f);
	if (regular)
		regfree(&expression);

	return nr;

} /* bin_global_suchen */



/*
	Binrdatensatz lschen

	->			art	:	Suchkriterium,
							I = ID       (long)
							N = Name     (char *)
							A = Absender (char *)
							B = Betreff  (char *)
							S = System   (long)

	<-			<0		:	Fehler
				 0		:	Eintrag nicht gefunden
				>0		:	Binr-ID Nummer
*/
static long bin_global_loeschen(int art,...)
{
	FILE *glob_f;
	BIN_MSG bin_info,ref;
	va_list param;
	long nr = 0;

	if ((glob_f = bin_global_oeffnen(TRUE)) == NULL)
		return -1;

	va_start(param,art);

	switch (art)
	{
		case	'I'	:
			ref.nr = va_arg(param,long);
			break;

		case	'N'	:
			strncpy(ref.name,va_arg(param,char *),sizeof(ref.name));
			break;

		case	'A'	:
			strncpy(ref.absender,va_arg(param,char *),sizeof(ref.absender));
			break;

		case	'B'	:
			strncpy(ref.betreff,va_arg(param,char *),sizeof(ref.betreff));
			break;

		case	'S'	:
			ref.system = va_arg(param,long);
			break;

		default	:
			fclose(glob_f);
			return -1;
	}

	while (!feof(glob_f))
	{
		if (!fread(&bin_info,sizeof(BIN_MSG),1,glob_f))
		{
			nr = 0;
			break;
		}

		if((art == 'I' && bin_info.nr == ref.nr)
		|| (art == 'N' && !strcmp(bin_info.name,ref.name))
		|| (art == 'A' && !strcmp(bin_info.absender,ref.absender))
		|| (art == 'B' && !strcmp(bin_info.betreff,ref.betreff))
		|| (art == 'S' && bin_info.system == ref.system))
		{
			bin_info.deleted = TRUE;
			bin_info.status = 2;
			fseek(glob_f,(long) - sizeof(BIN_MSG),SEEK_CUR);
			fwrite(&bin_info,sizeof(BIN_MSG),1,glob_f);
			fclose(glob_f);

			nr = bin_info.nr;
			break;
		}
	}

	va_end(param);

	fclose(glob_f);

	return nr;

} /* bin_global_loeschen */



/*
	globale Binrindexdatei putzen

	<-			-1			:	Fehler
				 0			:	ok, aber nichts gelscht
				>0			:	Anzahl gelschter Eintrge
*/
long bin_global_putzen(void)
{
	char src[PATH_MAX],
	     dst[PATH_MAX];
	FILE *src_f,
	     *dst_f;
	BIN_MSG bin_info;
	long ret = 0;

	sprintf(src,"%s%s",box_conf.var_dir,"global.dat");
	sprintf(dst,"%s%s",box_conf.var_dir,"global.tmp");

	if ((src_f = bin_global_oeffnen(TRUE)) == NULL)
		return -1;
	if ((dst_f = fopen(dst,BIN_RW_CREATE)) == NULL)
	{
		fclose(src_f);
		return -1;
	}

	while (fread(&bin_info,sizeof(BIN_MSG),1,src_f))
	{
		if (bin_info.deleted)
			ret++;
		else
		{
			bin_info.lokal -= ret;
			fwrite(&bin_info,sizeof(BIN_MSG),1,dst_f);
		}
	}

	fclose(src_f);
	fclose(dst_f);

	if (ret)
	{
		unlink(src);
		rename(dst,src);
	}
	else
		unlink(dst);

	return ret;

} /* bin_global_putzen */
