/*
   Copyright (C) 1993,1994,1995,1996  Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   For questions contact:
   e-mail: (prefered)
   uwe@tirka.gun.de
   Uwe_Ohse@mausdu3.gun.de
   Uwe Ohse @ DU3 (inside MausNet)
 */

#include "tomsysdep.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <ctype.h>

#include "portab.h"
#include "quarkoutfile.h"
#include "libquark.h"
#include "dynhash.h"
#include "crc32.h"


#define MAUS 1
#define FREMD 2

typedef struct reverse_dom_t {
	const  char          *domainname;
	const char           *kuerzel;
	char                 *up_str;
	struct reverse_dom_t *up;
	struct reverse_dom_t *next;
	int type;
} reverse_dom_t;

static reverse_dom_t *anker;

static void *kuerzel_handle = NULL;
static void *domain_handle = NULL;


const char *
get_boxkuerzel(const char *dom)
{	
	DYN_ENTRY *e;
	DYN_ENTRY such;
	char buf[1024];
	reverse_dom_t *p;
	entumlauten(dom,buf);
	strlwr(buf);
	such.u.ptr = 0;
	such.crc = crc32ansi (buf, strlen (buf));
	e = hhsearch (domain_handle, &such, DYN_FIND);
	if (!e) 
		return NULL;
	p=e->u.ptr;
	return p->kuerzel;
};

BOOLEAN
mask_as_fremdbox(const char *kuerzel)
{
	DYN_ENTRY *e;
	DYN_ENTRY such;
	char buf[1024];
	reverse_dom_t *p;
	entumlauten(kuerzel,buf);
	strlwr(buf);
	such.u.ptr = 0;
	such.crc = crc32ansi (buf, strlen (buf));
	e = hhsearch (kuerzel_handle, &such, DYN_FIND);
	if (!e) 
		return FALSE; /* should not happen */
	p=e->u.ptr;
	if (p->type==MAUS)
		return FALSE;
	return TRUE;
}


void 
lade_boxlist(const char *fname, const char *me, const char *maus)
{
	FILE *f;
	struct reverse_dom_t *neu=NULL; /* keep gcc quiet */
	char kuerzel[256];
	char k[1024];
	unsigned char buf[1024];
	unsigned char *p;
	f=fopen(fname,"rb");
	XNULLCHECK(f);
	domain_handle=hhcreate(250);
	kuerzel_handle=hhcreate(250);
	while (fgets(buf,sizeof(buf),f))
	{
		DYN_ENTRY such;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		p=strtrim(buf);
		switch(*p)
		{
		case '*':
			neu=XMALLOC(sizeof(reverse_dom_t));
			entumlauten(buf+1,k);
			strlwr(k);
			neu->kuerzel=XSTRDUP(k);
			neu->up_str=NULL;
			neu->up=NULL;
			neu->next=anker;
			neu->type=0;
			neu->domainname=NULL;
			anker=neu;

			such.u.ptr = neu;
			such.crc = crc32ansi (k, strlen (k));
			hhsearch (kuerzel_handle, &such, DYN_ENTER);
			break;
		case '-':
			entumlauten(buf+1,k);
			strlwr(k);
			neu->up_str=XSTRDUP(k);
			break;
		case 'D':
		case 'Z':
			if (p[1]=='.')
			{
				char *ndn; /* neuer domain name */
				ndn=XMALLOC(strlen(p+1)+1+strlen(kuerzel)*2);
				entumlauten(neu->kuerzel,ndn);
				strcat(ndn,p+1);
				neu->domainname=ndn;
			}
			else
				neu->domainname=XSTRDUP(p+1);

			entumlauten(neu->domainname,k);
			strlwr(k);
			such.u.ptr = neu;
			such.crc = crc32ansi (k, strlen (k));
			hhsearch (domain_handle, &such, DYN_ENTER);
			break;
		}
	}
	fclose(f);

	/* jetzt den Baum aufbauen, Mask-Flag setzen */
	if (me && maus)
	{
		char me_canon[1024];
		char maus_canon[1024];
		DYN_ENTRY such;
		DYN_ENTRY *e;
		entumlauten(me,me_canon);
		entumlauten(maus,maus_canon);
		strlwr(me_canon);
		strlwr(maus_canon);
		/* Baum bauen */
		for (neu=anker;neu;neu=neu->next) {
			if (neu->up_str) {
				such.u.ptr = 0;
				such.crc = crc32ansi (neu->up_str, strlen (neu->up_str));
				free(neu->up_str);
				neu->up_str=NULL;
				e = hhsearch (kuerzel_handle, &such, DYN_FIND);
				if (e)
					neu->up=e->u.ptr;
			}
		}
		/* uns markieren */
		such.u.ptr = 0;
		such.crc = crc32ansi (me_canon, strlen (me_canon));
		e = hhsearch (kuerzel_handle, &such, DYN_FIND);
		if (e) {
			neu=e->u.ptr;
			neu->type=FREMD;
		}
		else
			error(0,0,"warning: didn't find %s",me_canon);
		/* maus markieren */
		such.u.ptr = 0;
		such.crc = crc32ansi (maus_canon, strlen (maus_canon));
		e = hhsearch (kuerzel_handle, &such, DYN_FIND);
		if (e) {
			neu=e->u.ptr;
			neu->type=MAUS;
			/* von maus aus nach oben markieren */
			neu=neu->up;
			while (neu && !neu->type) {
				neu->type=MAUS;
				neu=neu->up;
			}
		}
		else
			error(0,0,"warning: didn't find %s",maus_canon);

		/* nun durch alle noch nicht markierten Boxen gehen */
		for (neu=anker;neu;neu=neu->next) {
			reverse_dom_t *x;
			if (neu->type)
				continue;
			for (x=neu->up;x;x=x->up)
				if (x->type)
					break;
			if (!x || x->type==FREMD)
				neu->type=FREMD;
			else
				neu->type=MAUS;
		}
#if 0
		for (neu=anker;neu;neu=neu->next) {
			fprintf(stderr,"%c %s",neu->type==MAUS ? 'M' : 'F',
					neu->kuerzel);
			if (neu->up)
				fprintf(stderr," [%s]",neu->up->kuerzel);
			fprintf(stderr,"\n");
		}
#endif
	}
}

