/* Declarations for libbrett
    Copyright (C) 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/

#if !defined(BBBASE_H)
#define BBBASE_H

#include <portab.h>
#include <time.h>
#include <tomattribs.h>

typedef long BRETTNR;

typedef enum { TEXT_BRETT, BINAER_BRETT=2, BRETTART_NNTP } BRETTART;

/*
 * Brettflags.
 */
/* Maske fr die Flags, die in der Brettdatenbank gespeichert werden */
#define B_MASK_DATABASE (0xffffff00)

#define B_LOCKED   0x1   /* Prozess hlt Lock */
#define B_NEU      0x2   /* neuer Inhalt fr aktuellen User */
#define B_READ     0x4   /* Leseerlaubnis */
#define B_WRITE    0x8   /* Schreiberlaubnis */
#define B_LOKAL    0x10  /* Brett ist lokal */
#define B_UPDATE   0x20  /* Brettdatum  nach DB-Import aktualisieren */

/* Gruppenstatus */
	/* Nachrichten erhalten keinen Absender */
#define B_ANONYM		0x100
	/* Geheim: Lesen nur fr Mitglieder */
#define B_GEHEIM		0x200
	/* Schreiben nur fr Mitglieder */
#define B_MITGLIED		0x400
	/* Lesen nur fr Zahler */
#define B_ZAHLER			0x800
	/* Fr das Brett gibt es ein Passwort - es ist nur online betretbar */
#define B_PASSWORT		0x1000
	/* nicht abbestellbar */
#define B_ZWANG			0x2000
	/* Defaultmig bestellt */
#define B_DEFAULT       0x4000
	/* Moderiert? */
#define B_MODERIERT     0x8000
	/* Zugriffskontrolliste -> etwas anderes als Mitgliedschaft oder
	 * Geheim
	 */
#define B_ACL           0x10000
#define B_LESMITGLIED   0x20000
#define B_PAYWRITE      0x40000

#define B_MITLISTE(status) \
	((status & (B_MITGLIED | B_GEHEIM | B_LESMITGLIED))!=0)

/********************************************************************
 * Struktur BRETT
 ********************************************************************/

#define BDAT_GRP_NAME_LEN	128
#define BDAT_PW_LEN			20

#ifndef USERNR_TYPEDEF
#define USERNR_TYPEDEF
typedef LONG USERNR;
#endif

/* In der folgenden Struktur sind Strings als char-Arrays abgelegt.
	Ihr maximale Lnge ist BDAT_*_LEN inclusive abschlieendem '\0'
*/
typedef struct
{
	BRETTNR nr;				/* eindeutig */
	UWORD index;			/* Index des Datensatzes in der Datei */
	char name[BDAT_GRP_NAME_LEN]; /* Brettname */

	UWORD partition;		/* auf welcher Partition steht das Brett */
								/* 0 = A, 1 = B usw. */

	UWORD lesen_level;	/* Userlevel (0-9) fr Lesezugriff */
	UWORD schreib_level;	/* Userlevel (0-9) fr Schreibzugriff */

	ULONG flags;			/* Brettzugriffsflags, z.B. B_GEHEIM */
	char passwort[BDAT_PW_LEN];	/* _noch_ unverschlsselt */

	USERNR grp_chef;		/* Usernummer des Gruppenchefs (1 = Sysop) */
	/* tmpflags werden zwar abgespeichert, beim Laden aber genullt. Sind
	 * also fr den prozessinternen Gebrauch 
	 */
	ULONG tmpflags; /* process internal flags */
	time_t created;		/* Zeitpunkt der Anlage des Brettes */
	time_t datum;			/* Zeitpunkt der letzten Vernderung im Brett */

	BOOLEAN unused_dl_sperre;	/* Downloadsperre, FALSE = generell frei */

	BRETTART art;			/* Text oder Binr */
	UWORD netzstatus;		/* 0=lokal, 1=vernetzt */
	UWORD unused_max_laenge;		/* maximale Lnge in KB */
	UWORD unused_info_laenge;	/* maximale Lnge der Infotexte in Byte */
	UWORD unused_info_zeilen;	/* maximale Lnge der Infotexte in Zeilen */
	UWORD unused_aufbewahrzeit;	/* maximale Aufbewahrungszeit in Tagen */

	UWORD sprache;          /* in der Gruppe verwendete Sprache */
	UWORD ursprungsnetz;    /* daher importiert ... */
	BRETTNR followup_to_default; /* dahin sollen Followups per Default */
		/* letzte nderung an den MausNetweiten Daten der Gruppe */
	time_t netzaenderung; 
		/* Zeitpunkt der letzten nderung an den Gruppenverwaltungsdaten. */
	time_t configaenderung;
	void *unused_xinfos;
	BOOLEAN geloescht;
	char *userinfo;
	char *sysopinfo;
	char *lokalinfo;
	char *aliasname;

	char resvd[16];

	/* Das alles zusammen mu 256 Byte ergeben. */

} BRETT;

typedef struct
{
	BRETT *brett;     /* Zeiger auf Datensatz oder NULL */
	/* "brett->name", kleingeschrieben, entumlautet, Sonderzeichen nach . gewandelt */
	char *canon;
} BRETTINFO;

typedef struct bb_rename_t {
	struct bb_rename_t *next;
	char *old_name;
	char *new_name;
	time_t when;
} bb_rename_t;

extern bb_rename_t *bb_rename_anker;

bb_rename_t *bb_find_rename_entry(const char *name);


/********************************************************************
 * Globale Variablen
 ********************************************************************/

extern long brettanzahl;

/*
    Initialisiert die Brettdatenbank in @var{dir}. Dabei werden alle
	Brettdaten gelesen und in den internen Cache geschrieben. 
	@var{binaer_db_dir} ist die Basis der Verzeichnishierarchie, in 
	der die Zugriffsdaten und die Inhalte der Binrbretter untergebracht 
	werden, @var{text_db_dir} enthlt die Zugriffslisten der Textbretter.
*/
BOOLEAN bb_init(const char *dir, const char *text_db_dir,
				const char *binaer_db_dir) 
    TOM_ATTRIB_SECTION(tom_init);

/*
*/
BOOLEAN bb_exit(void) TOM_ATTRIB_SECTION(tom_exit);



/*
    Liest die Daten des Bretts @var{nr} von der Platte in den Cache.
	Falls @var{for_write} TRUE ist wird das Brett gelockt.
*/
BOOLEAN bb_read_brett(BRETTNR nr,BOOLEAN for_write);


/*
    Schreibt die Daten des Brettes mit der Nummer @var{bnr} in die
	Datenbank. Ein eventueller Lock wird freigegeben, wenn
	@var{unlock} TRUE ist.
	Ist @var{changed_xinfo} TRUE werden auch die Infotexte in 
	die xbrett.dat geschrieben.
*/
BOOLEAN bb_write_brett(BRETTNR bnr,BOOLEAN unlock,BOOLEAN changed_xinfo);



/*
    gibt den Lock auf das Brett frei.
*/
BOOLEAN bb_unlock_brett(BRETTNR bnr);




/*
    Lscht das Brett mit der Nummer @var{nr} aus den Verwaltungsdaten 
	des Prozesses und der Datenbank.
*/
BOOLEAN bb_remove_brett(BRETTNR nr) TOM_ATTRIB_SECTION(tom_rare);


/*
    Legt ein neues Brett an und gibt dem Brett seine Nummer.
*/
BOOLEAN bb_create_brett(BRETT *b) TOM_ATTRIB_SECTION(tom_rare);


/*
    Legt eine neue Brettdatenbank im Verzeichnis @var{dir} 
	an, falls die Datei noch nicht existiert.
*/
BOOLEAN bb_create_db(const char *path) TOM_ATTRIB_SECTION(tom_rare);


/*
    Wandelt den angegebenen Brettnamen mittels bb_canonical in die 
	interne Form und sucht ihn in der Liste aller Bretter (also nicht
	nur in einer der dem aktuellen User bekannten).
*/
BRETTNR brettnr(const char *name);


/*
    Liefert die Brettnummer zu einem unvollstndigen Brettnamen.
	Prziser: Wandelt den angegebenen Namen in die interne Form 
	und liefert die Nummer eines Brettes zurck, da
	- entweder genau identisch ist, oder
	- zumindest das einzige Brett ist, das auf die Wildcard "name*"
	  pat.
*/
BRETTNR find_brett(const char *name);




void bb_canonical(const char *from, char *to);

/*
    Liefert einen Zeiger auf ein Array von Brettnummern zurck, die 
	nach Alphabet sortiert sind. Die Funktion sortiert nur, wenn
	dies notwendig ist, sonst liefert sie ein vorheriger Sortier-
	ergebnis zurck. 
*/
BRETTNR *bb_sortierung(void);

/*
	Diese Funktion ist bei jedem Userwechsel, bei Anlage neuer Bretter,
	und beim beim Lschen und Umbenennen der Bretter aufzurufen. Sie
	sorgt dafr, da beim nchsten Aufruf von bb_sortierung die
	Bretterliste neu sortiert wird.
*/
void bb_free_sortierung(void);


/* 
    Prueft auf illegale Zeichen in Brettnamen. Liefert TRUE, wenn der Name
    OK ist.
*/
BOOLEAN check_brettname_syntax(const char *name);

const char *bb_ursprungsnetz(UWORD netz);
const char *bb_sprache(UWORD sprache);

/*
    Liefert den Buchstaben des Laufwerks zurck, auf dem die Daten des
	Brettes liegen. Unter Systemen, die nicht unter der Verwendung von
	Laufwerksbuchstaben leiden, wird generell Nullzeichen (ASCII-Null,
	Stringende geliefert).
*/
const char brettdrive(BRETTNR nr);
const char *brettpfad(BRETTNR nrW);

		/* Zugriffszeitpunkt aktualisieren */
BOOLEAN bb_update_brett(BRETTNR nr);

void bb_get_status_str(long status,char *zwi);
BOOLEAN brettflags_to_string(ULONG flags, char *s, size_t place);
BOOLEAN string_to_brettflags(const char *s, ULONG *flags);

BOOLEAN bb_reorg(void) TOM_ATTRIB_SECTION(tom_rare);



extern BRETTINFO *br_ind;

extern long max_brettnr;  /* hchstmgliche Brettnummer */

#define CHECK_BRETTNR(bnr,action_if_illegal) \
	if (bnr <0 || bnr>=max_brettnr) { action_if_illegal; }

#define FOR_EACH_BRETTNR(brett_nr) \
    for (brett_nr=1; brett_nr < max_brettnr;brett_nr++)

#endif
