/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	business hours - Geschftszeiten
	wer wann was darf in der Box

	(c) 1993 bei Frank Baumgart

	begonnen am 16.10.1993
	Version vom 22.10.1993
*/


#include "tomsysdep.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "prototyp.h"
#include "modem.h"
#include "user.h"
#include "business.h"
#include "box_conf.h"
#include "tompaths.h"

static BUSINESS *business;
static long anzahl = 0;

extern USER user;

BUSINESS *get_business_array(long *para_anzahl)
{
	if (para_anzahl)
		*para_anzahl=anzahl;
	return business;
}

/*
	business-Datei auswerten
	16.10.93 Frank

	Die Datei ist optional.
	Default ist: Login, Download, Gast und Tausch ab Level 4 erlaubt
*/
BOOLEAN read_business(void)
{
	char zeile[200];
	char *p;
	FILE *cnf_f=NULL;
	long znr = 1;

	anzahl = 0;
	if (tom_port_config_dir)
		cnf_f = fopen(tprintf("%s%s",tom_port_config_dir,"business.cnf"),
					  BINREAD);
	if (!cnf_f 
		&& (cnf_f = fopen(tprintf("%s%s",box_conf.config_dir,"business.cnf"),
						  BINREAD)) == NULL)
		return TRUE;

	/* erstmal ermitteln, wieviele Definitionen vorliegen */
	while (fgets(zeile,200,cnf_f))
	{
		p=zeile;
		while (isspace(*p))  /* Space Tab \r \n */
			p++;
		if (!*p || *p == ';' || *p == '#')
			continue;

		anzahl++;
	}

	if (!anzahl)
	{
		fclose(cnf_f);
		return TRUE;
	}

	/* Speicher fr Definitionen anfordern */
	business = (BUSINESS *) XCALLOC(anzahl,sizeof(BUSINESS));

	anzahl = 0;
	rewind(cnf_f);
	while (tom_fgetline(zeile,cnf_f))
	{
		long h,m;			/* Stunde und Minuten */
		char *pos;
		int i;

		znr++;

		p=zeile;
		while (isspace(*p))
			p++;
		if (!*p || *p == ';' || *p == '#')
			continue;

		/* vorher alle Flags lschen */
		memset(&business[anzahl],0,sizeof(BUSINESS));

		/* Default setzen */
		for (i=0;i<10;i++)
			business[anzahl].onlinezeit[i]=box_conf.onlinezeit[i];

		/* Auswertung des Eintrags */
		h = m = 0;
		h = strtol(p,&pos,10);
		if (h < 0 || h > 23)
		{
			mbprintf(" Falsche Stunden-Angabe in Zeile %ld\r\n",znr);
			continue;
		}
		if (*pos==':')
			pos++;
		m = strtol(pos,&pos,10);
		if (m < 0 || m > 60)
		{
			mbprintf(" Falsche Minuten-Angabe in Zeile %ld\r\n",znr);
			continue;
		}
		business[anzahl].start = (UWORD) (h*60 + m);

		/* Flags auswerten */
		p=strpbrk(pos,"#;");
		if (p)  
		{
			*p++=0;
			strtrim(p);
			business[anzahl].titel=XSTRDUP(p);
		}
		else 
			business[anzahl].titel="";
		strtrim(pos);

		while (*pos)
		{
			BOOLEAN ok;
			int typ=B_LOGIN; /* keep gcc quiet */

			ok = FALSE;
			switch (toupper(*pos++))
			{
				case 'L':
					typ = B_LOGIN; 
					ok = TRUE; 
					break;

				case 'T':
					typ = B_TAUSCH; 
					ok = TRUE; 
					break;

				case 'D':
					typ = B_DOWNLOAD; 
					ok = TRUE; 
					break;

				case 'Z':
					typ=B_ZEIT;
					ok=TRUE;
					break;

			}

			if (ok)
			{
				/* Dann den zugehrigen parameter lesen */
				if (typ==B_ZEIT)
				{
					/* Default wieder lschen */
					for (i=0;i<10;i++)
						business[anzahl].onlinezeit[i]=0;
					while (pos && (isdigit(*pos) || *pos=='*'))
					{
						if (pos[0]=='*' && pos[1]==':')
						{
							int j;
							for (j=0;j<10;j++)
							{
								business[anzahl].onlinezeit[j] = 
									strtol(pos+2,NULL,10);
							}
							(void) strtol(pos+2,&pos,10);
						}
						else if (pos[1]==':')
						{
							business[anzahl].onlinezeit[strtol(pos,NULL,10)] = 
								strtol(pos+2,&pos,10);
						}
						else if (pos[1]=='-' 
							&& isdigit(pos[2])
							&& pos[3]==':')
						{
							UWORD st,en;
							long z;
							if (*pos=='*')
								st=0;
							else
								st=strtol(pos,NULL,10);
							if (pos[2]=='*')
								en=ULV_MAX;
							else
								en=strtol(pos+2,NULL,10);
							if (en<st)
							{
								UWORD zwi=en;
								en=st;
								st=zwi;
							}
							z=strtol(pos+4,&pos,10);
							while (st<=en)
							{
								business[anzahl].onlinezeit[st] = z;
								st++;
							}
						}
						if (*pos==',')
							pos++;
						else
						{
							/* ber eventuellen Schmutz springen */
							while (*pos && !isspace(*pos))
								pos++;
						}
					} /* while isdigit */
				}
				else  /* ! B_ZEIT */
				{
					while (*pos && !isspace(*pos))
					{
						if (*pos == '$')
						{
							business[anzahl].grenze[typ].zahler = TRUE;
						}
						else if (isdigit(*pos))
						{
							business[anzahl].grenze[typ].level = atoi(pos);
						}
						business[anzahl].grenze[typ].erlaubt = TRUE;
						pos++;
					}
				}
			}
		}
		anzahl++;
	}
	fclose(cnf_f);

	return TRUE;

} /* read_business */



/*
	berprfung, ob Aktion (Login, Tausch usw.) mglich ist, ansonsten 
	optional Ausgabe des nchstmglichen Zeitpunkts

	->			user		:	aktueller User (fr Login nicht ntig ?)
				action	:	't' (Tausch), 'l' (Login), 'd' (Download)
			<-	next		:	nchstmglicher Zeitpunkt (in Minuten seit 0:00)
								Sonderfall: 0 = nie mglich (NULL erlaubt)
			<-	max		:	bis wann ist das Login max. erlaubt (NULL erlaubt)

	<-			TRUE		:	Aktion sofort mglich
				FALSE		:	Aktion gar nicht (next = 0) oder nicht sofort mgl.
*/
BOOLEAN check_business(char action,long *next,time_t *max)
{
	BUSINESS *akt_bus=NULL;	/* aktuell gltiger Eintrag */
	time_t jetzt;			/* aktueller Zeitpunkt */
	struct tm *zeit;		/* dito, fr Umwandling in Minuten */
	long jetztminuten;			/* Minuten seit Mitternacht */
	BOOLEAN ok = FALSE;
	int typ = 0;
	int nr;
	int start_nr = 0; /* keep gcc quiet */

	/* ohne explizite Konfiguration ist immer alles erlaubt */
	if (!anzahl)
		return TRUE;
	if (next)
		*next = 0;
	if (max)
		*max = 0;

	/* Minuten seit Mitternacht */
	time(&jetzt);
	zeit = localtime(&jetzt);
	jetztminuten = zeit->tm_hour * 60 + zeit->tm_min;

	/* fr aktuelle Uhrzeit gltigen Eintrag ermitteln */
	nr = 0;
	while (nr < anzahl)
	{
		if (jetztminuten >= business[nr].start)
		{
			akt_bus = &business[nr];
			start_nr = nr;
		}
		else if (nr==anzahl-1)
		{
			akt_bus = &business[nr ? nr - 1 : nr ];
			start_nr = nr ? nr - 1 : nr;
		}
		else
			break;
		nr++;
	}
	
	switch (action)
	{
		case	't'	:	/* automatischer Tausch erlaubt ? */
							typ = B_TAUSCH;
							break;

		case	'l'	:	/* Login erlaubt ? */
							typ = B_LOGIN;
							break;

		case	'd'	:	/* Download erlaubt ? */
							typ = B_DOWNLOAD;
							break;

		default		:	fputs("falsche Businessabfrage",stderr);
							return FALSE;
	}

#define Erlaubt(bus,typ)	bus->grenze[typ].erlaubt									\
							&& ((!bus->grenze[typ].zahler)							\
							||   (bus->grenze[typ].zahler & user.bezahlt))	\
							&& (bus->grenze[typ].level <= level())

	/* Aktion erlaubt ? */
	if (Erlaubt(akt_bus,typ))
		ok = TRUE;
	else
		ok = FALSE;

	/* nchstmglichen Zeitpunkt ermitteln */
	/* wird nur berechnet, falls Aktion zum jetzigen Zeitpunkt nicht erlaubt */
	if (!ok && next)
	{
		int i = nr;

		while (i < anzahl)
		{
			BUSINESS *bus=&business[i];
			if (Erlaubt(bus,typ))
			{
				*next = bus->start;
				goto calc_max;
			}
			i++;
		}
		i = 0;
		while (i < nr)
		{
			BUSINESS *bus=&business[i];
			if (Erlaubt(bus,typ))
			{
				*next = bus->start;
				goto calc_max;
			}
			i++;
		}
	}

calc_max:;
	/* maximal erlaubten Endzeitpunkt fr aktuelle Aktion errechnen */
	/* Berechnung nur, wenn Aktion derzeit erlaubt ist */
	if (ok && max)
	{
		long minuten;
		int i = start_nr;

		/* Mit "minuten" wird nun errechnet, wie lange der User
		 * in der Box bleiben darf, bis ihm kein Login erlaubt ist
		 */
		minuten = 0;

		while (i < anzahl)
		{
			BUSINESS *bus=&business[i];
			minuten = bus->start;
			if (!Erlaubt(bus,typ))
				goto calc_end;
			i++;
		}
		i = 0;
		while (i < start_nr)
		{
			BUSINESS *bus=&business[i];
			minuten = bus->start+24*60; /* offset nchster Tag */
			if (!Erlaubt(bus,typ))
				break;
			i++;
			continue;
		}

		if (minuten>0)
			minuten--; /* um exakt 1 zu hoch */
		else
			minuten=24*60-1;
		/*
		 * Wir wissen, da der User *jetzt* in der Box sein darf. Das 
		 * heit, wenn "minuten < jetztminuten", dann mssen wir
		 * einen Tag draufrechnen.
		 */
		if (minuten<jetztminuten)
			minuten+=24*60;

calc_end:;
		/* minuten gibt die Tagesendzeit in Minuten an */

		/* Jetzt das Maximum fr diesen Userlevel miteinbeziehen */
		if (business[start_nr].onlinezeit[user.level]!=(UWORD) -1)
		{
			if (minuten-jetztminuten 
				> business[start_nr].onlinezeit[user.level])
					minuten=jetztminuten 
						+ business[start_nr].onlinezeit[user.level];
		}
		/* Umrechnung auf time_t als Endzeitpunkt */
		{
			struct tm *endzeit;
			time(&jetzt);
			endzeit = localtime(&jetzt);
			endzeit->tm_hour = minuten / 60;
			endzeit->tm_min  = minuten % 60;
			endzeit->tm_sec  = 0;
			*max = mktime(endzeit);
#if 0
			if (*max < time(NULL))
				*max += 86400L;			/* SECS_PER_DAY */
#endif
		}
	}

	return ok;

} /* check_business */
