/* datei.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>


#include "config.h"
#include "tomsysdep.h"
#include "user.h"
#include "tompaths.h"
#include "box_conf.h"
#include "aktdata.h"
#include "prototyp.h"
#include "filelock.h"


void
write_close_status_dat(FILE *f, struct status_dat *st)
{
	if (!f)
	{
		/* neu anlegen */
		char datei[PATH_MAX];
		sprintf(datei,"%s%s",box_conf.var_dir,"status.dat");
		if (fileexist(datei))
		{
			syslog(LOG_WARNING,"anlegen von status.dat: existiert schon!");
			return;
		}
		f=fopen(datei,"r+");
		if (!f)
		{
			syslog(LOG_WARNING,"fopen(%s,\"r+\"): %m",datei);
			return;
		}
		if (0!=fd_write_lock(fileno(f),0,SEEK_SET,1))
		{
			fclose(f);
			syslog(LOG_WARNING,"fd_write_lock(%s): %m",datei);
			return;
		}
	}
	fflush(f);
	rewind(f);
	fprintf(f,"dummy (war: last_date)\n");
	fprintf(f,"%ld\n",(long) st->postertrag);
	fprintf(f,"%ld\n",(long) st->last_call);
	fprintf(f,"%s\n",st->last_user);
	fprintf(f,"%ld\n",(long) st->anz_calls);
	fprintf(f,"%ld\n",(long) st->tagesanrufe);
	fprintf(f,"dummy (war: Console besetzt?)\n");
	fclose(f);
}

FILE * 
open_read_status_dat(BOOLEAN read_only, struct status_dat *st)
{
	char datei[PATH_MAX];
	int res;
	char buf[1024];
	const char *open_mode;
	FILE *f;

	st->last_user[0]=0;
	st->postertrag=0;
	st->last_call=0;
	st->anz_calls=0;
	st->tagesanrufe=0;

	if (read_only)
		open_mode="r";
	else
		open_mode="r+";

	sprintf(datei,"%s%s",box_conf.var_dir,"status.dat");
	if ((f = fopen(datei,open_mode)) == NULL)
	{
		if (!f && !read_only)
			f = fopen(datei,"w");
		if (!f)
		{
			syslog(LOG_NOTICE,"cannot open %s: %m",datei);
			return NULL;
		}
	}

	if (read_only)
		res=fd_readw_lock(fileno(f),0,SEEK_SET,0);
	else
		res=fd_writew_lock(fileno(f),0,SEEK_SET,0);

	if (res==-1)
	{
		syslog(LOG_NOTICE,"cannot lock %s: %m",datei);
		fclose(f);
		return NULL;
	}

	/* jetzt lesen */
	fgets(buf,sizeof(buf),f); /* dummy last_date berspringen */
	fgets(buf,sizeof(buf),f); /* postertrag */
	st->postertrag=strtol(buf,NULL,10);

	fgets(buf,sizeof(buf),f); /* last_call (uhrzeit) */
	st->last_call=strtol(buf,NULL,10);

	if (fgets(buf,sizeof(buf),f)!=0)
	{
		char *p=strpbrk(buf,"\r\n");
		if (p) *p=0;
		strncpy(st->last_user,buf,sizeof(st->last_user));
		st->last_user[sizeof(st->last_user)-1]=0;
	}
	fgets(buf,sizeof(buf),f); /* anz_calls */
	st->anz_calls=strtol(buf,NULL,10);
	fgets(buf,sizeof(buf),f); /* tagesanrufe */
	st->tagesanrufe=strtol(buf,NULL,10);
#if 0
	fgets(buf,sizeof(buf),f); /* dummy consoleB berspringen */
#endif

	if (read_only)
	{
		fclose(f);
		return NULL;
	}
	return f;
}

/*
	Tagesauslastungs-Statistik einlesen

	17.12.92 Frank
*/
void get_auslastung(int *aus_array)
{
	FILE *auslast_f;
	char datei[PATH_MAX];
	int i;
	for (i=0; i<24; i++)
		aus_array[i]=0;
	sprintf(datei,"%s%s",box_conf.var_dir,"auslast.dat");

	auslast_f = fopen(datei,"r");
	if (!auslast_f)
	{
		syslog(LOG_WARNING,"cannot open %s: %m",datei);
		return;
	}
	if (fd_readw_lock(fileno(auslast_f),0,SEEK_SET,1)!=0)
	{
		/* Oops? */
		syslog(LOG_WARNING,"cannot lock %s: %m",datei);
		fclose(auslast_f);
		return;
	}
	else
	{
		for (i = 0; i < 24; i++)
			fscanf(auslast_f,"%d,",aus_array+i);
		fclose(auslast_f);
	}
} /* get_auslast */



/*
	Tagesauslastungs-Statistik schreiben

	17.12.92 Frank
*/
void put_auslastung(time_t start, long sekunden)
{
	FILE *ausl_f;
	int i;
	char datei[PATH_MAX];
	int ausl_array[24];
	long stunde;
	int used;
	struct tm *tm;

	sprintf(datei,"%s%s",box_conf.var_dir,"auslast.dat");
	ausl_f = fopen(datei,"r+");
	if (!ausl_f)
		ausl_f=fopen(datei,"w+");
	if (!ausl_f)
	{
		syslog(LOG_WARNING,"cannot open %s: %m",datei);
		return;
	}
	if (fd_writew_lock(fileno(ausl_f),0,SEEK_SET,1)!=0)
	{
		/* Oops? */
		syslog(LOG_WARNING,"cannot lock %s: %m",datei);
		fclose(ausl_f);
		return;
	}
	for (i=0; i<24; i++)
	{
		ausl_array[i]=0;
		fscanf(ausl_f,"%d,",&ausl_array[i]);
	}

	/*
	 * Jetzt die neue Zeit dazurechnen.
	 */
	tm=localtime(&start);
	if (!tm)
	{
		syslog(LOG_WARNING,"put_auslastung: localtime: %m");
		fclose(ausl_f);
		return;
	}
	stunde=tm->tm_hour;

	while (sekunden > 0)              /* Solange Zeit zu verteilen */
	{
		int frei;
		/* Soviele Sekunden war die Box in dieser Stunde frei */
		frei = 3600-ausl_array[stunde];

		if (sekunden <= frei)
		{
			ausl_array[stunde] += sekunden;
			sekunden=0;
			break;
		}
		/* es geht leider ber eine Stundengrenze */
		if (frei==3600)
		{
			/* wir haben die ganze Stunde fr uns */
			ausl_array[stunde] += frei;
			sekunden-=frei;
			if (++stunde==24)
				stunde=0;
			continue;
		}
		/* das ist die erste Stunde */
		used=3600 - (tm->tm_min * 60 + tm->tm_sec);
		ausl_array[stunde] += used;
		sekunden-=used;
		if (++stunde==24)
			stunde=0;
	}
	fflush(ausl_f);
	rewind(ausl_f);

	for (i=0; i<24; i++)
		fprintf(ausl_f,"%d,",ausl_array[i]);
	fputs("\n",ausl_f);
	fclose(ausl_f);

} /* put_auslast */

/*
	Tages-Statistik in Gesamt-Statistik bertragen
*/
void update_statistik(void)
{
	struct status_dat st;
	FILE *stat_f;
	FILE *ausl_f;
	char datei[PATH_MAX];
	char buf[128];
	struct tm *tm;
	time_t t;
	int i;

	open_read_status_dat(TRUE,&st);
	time(&t);
	t-=86400L; /* Ein Tag zurck */
	tm=localtime(&t);
	strftime(buf,sizeof(buf)-1,"%d.%m.%y",tm);

	/* Auslastung des vergangenen Tages an die Gesamt-Statistik anhngen */
	sprintf(datei,"%s%s",box_conf.var_dir,"statist.inf");
	stat_f = xfopen(datei,BIN_R_APPEND);
	fprintf(stat_f,"%-11s%8d%7s%7s%6s%7s%6s%9ld.%02ld DM%5d %%\r\n",
		buf,
		st.tagesanrufe,
		"",
		"",
		"",
		"",
		"",
		st.postertrag / 100,st.postertrag % 100,
		auslastung());
	fclose(stat_f);

	sprintf(datei,"%s%s",box_conf.var_dir,"auslast.dat");
	ausl_f=fopen(datei,"w+");
	if (!ausl_f)
	{
		syslog(LOG_WARNING,"cannot open %s: %m",datei);
		return;
	}
	if (fd_writew_lock(fileno(ausl_f),0,SEEK_SET,1)!=0)
	{
		/* Oops? */
		syslog(LOG_WARNING,"cannot lock %s: %m",datei);
		fclose(ausl_f);
		return;
	}
	for (i=0; i<24; i++)
		fprintf(ausl_f,"%d,",0);
	fputs("\n",ausl_f);
	fclose(ausl_f);

} /* update_statistik */


long get_new_id_nr(BOOLEAN binaer)
{
    long mnk_bin;     /* global eindeutige, fortlaufende Binrdatei-Nummer */
    long mnk_sys;     /* global eindeutige, fortlaufende ID-Nummer */
    char datei[PATH_MAX];
    FILE *f;
    int tries=20;

    sprintf(datei,"%s%s",box_conf.var_dir,"id.dat");

retry:
    if ((f = fopen(datei,"r+")) != NULL)
    {
    	if (fd_write_lock(fileno(f),0,SEEK_SET,0))
        {
            fclose(f);
            f=NULL;
            if (--tries)
            {
                msleep(100);
                goto retry;
            }
            syslog(LOG_NOTICE,"cannot lock %s",datei);
            srand((unsigned int) time(NULL));
            mnk_bin=rand() & 0xffff;
            mnk_sys=rand() & 0xffff;
			fclose(f);
			f=NULL;
        }
        else
        {
            fscanf(f,"%ld\n",&mnk_bin);
            fscanf(f,"%ld\n",&mnk_sys);
			fflush(f);
			rewind(f);
        }
    }
    else if ((f = fopen(datei,"w")) != NULL)
    {
        mnk_sys=0;
        mnk_bin=0;
    }
    else
    {
        syslog(LOG_NOTICE,"cannot open %s: %m",datei);
        mnk_sys=0;
        mnk_bin=0;
    }
    if (binaer)
		mnk_bin++;
	else
		mnk_sys++;

    if (f)
    {
        fprintf(f,"%ld\r\n",mnk_bin);
        fprintf(f,"%ld\r\n",mnk_sys);
        fclose(f);
    }

    return (binaer) ? mnk_bin : (mnk_sys & 0xffff);

} /* get_new_id_nr */

