/* dynload.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/* misc. functions, boxintern */

#include "config.h"

#ifdef MODULE_STUBS

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <dlfcn.h>

#include "tomsysdep.h"
#include "portab.h"
#include "user.h"
#include "brett.h"
#include "box_conf.h"
#include "aktdata.h"
#include "prototyp.h"
#include "dynload.h"
#include "modem.h"

#undef MOD_DEBUG

struct module_data
{
	void *handle;
	int usecount;
	char *fname;
	struct module_data *next;
};

static struct module_data *anker;


void *
module_load(const char *path)
{
    char fname[PATH_MAX];
	void *handle;
	struct module_data *akt;
#ifdef MOD_DEBUG
	mbprintf("load module %s\r\n",path);
#endif

	for (akt=anker;akt;akt=akt->next)
	{
		if (0==strcmp(akt->fname,path))
			break;
	}
	if (akt)
	{
		akt->usecount++;
		return akt;
	}

	sprintf(fname,"%s/%s/%s.so",box_conf.modules_dir,VERSION,path);
	akt=malloc(sizeof(struct module_data));
	if (akt)
		akt->fname=strdup(path); /* path, nicht fname, wegen obiger Schleife */
	if (!akt || !akt->fname)
	{
		if (akt)
			free(akt);
		mbprintf("kann %s nicht laden: out of memory\r\n", fname);
		return NULL;
	}
	
	/* RTLD_NOW ist notwendig. Mit LAZY steigt das Programm aus, wenn
	 * ein Symbol nicht gefunden wird.
	 */
	handle=dlopen(fname,RTLD_NOW);
	if (!handle)
	{
		free(akt->fname);
		free(akt);
		mbprintf("kann %s nicht laden: %s\r\n", fname,dlerror());
		return NULL;
	}
	akt->usecount=1;
	akt->handle=handle;
	akt->next=anker;
	anker=akt;
#ifdef MOD_DEBUG
	mbprintf("loaded module %s\r\n",path);
#endif
	return akt;
}

void *module_sym(void *module, const char *sym)
{
	return dlsym(((struct module_data *)module)->handle,sym);
}

void 
module_release(void *module)
{
	struct module_data *akt;
	struct module_data *last;
#ifdef MOD_DEBUG
	mbprintf("release module function\r\n");
#endif

	for (akt=anker,last=NULL;akt && akt!=module;last=akt,akt=akt->next)
		/* nichts */;

	if (!akt)
	{
#ifdef MOD_DEBUG
		mbprintf("module %p not found\r\n",module);
#endif
		return;
	}

	akt->usecount--;
#ifdef MOD_DEBUG
	mbprintf("module %s: usecount now %d\r\n",akt->fname,
			 akt->usecount);
#endif
	if (akt->usecount==0)
	{
		/* nun das Modul selbst freigeben */
		dlclose(akt->handle);
#ifdef MOD_DEBUG
		mbprintf("released module %s\r\n",akt->fname);
#endif
		free(akt->fname);
		if (last)
			last->next=akt->next;
		else
			anker=akt->next;
		return;
	}
}
#endif

