/* find_brett.c:
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/
#include <portab.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "config.h"
#include "libquark.h"

#include "brett_db.h"

/*
BB_FUNCTION(Funktion,BRETTNR,find_brett,[[(const char *@var{name})]],[[
    Liefert die Brettnummer zu einem unvollstndigen Brettnamen @var{name}.
    Prziser: Wandelt den angegebenen Namen in die interne Form
    und liefert die Nummer eines Brettes zurck, da entweder
@itemize @minus
@item genau identisch ist, oder
@item zumindest das einzige Brett ist, das auf die Wildcard "name*" pat.
@end itemize

    Wird kein oder zuviele passende Bretter gefunden, liefert die 
    Funktion 0, sonst die Nummer des Brettes.
]])
*/
BRETTNR find_brett(const char *name)
{
	size_t len;
	BRETTINFO *bi = br_ind;
	long i = 0;
	BRETTNR zahl = 0;
	char bufname[512];
	long best=0;
	size_t bestlen=0;

	zahl = (BRETTNR) strtol(name,NULL,10);
	if (zahl && zahl < max_brettnr && br_ind[zahl].brett)
		return zahl;

	bb_canonical(name,bufname);	
	len = strlen(bufname);

	/* findet Maeuse anstatt MUSE, und MAUS auch dann, wenn es nach
	 * MAUS.GRP kommt.
	 */
	while (i++ < max_brettnr)
	{
		/* bi->brett != 0 -> dass Brett gibt es. bi->canon ist beim Postfach 0. */
		if (bi->brett && bi->canon)
		{
			if (!strncmp(bi->canon,bufname,len))
			{
				size_t l=strlen(bi->canon);
				if (l==len)
					return i-1;
				else if (l>bestlen)
				{
					best=i;
					bestlen=l;
				}
			}
		}
		bi++;
	}
	if (best)
		return best-1;
	/* Seufz, nicht gefunden. Suchen wir mal nach Aliasnamen */
	for (i=1,bi=br_ind;i<=max_brettnr;i++) {
		if (br_ind[i].brett && br_ind[i].brett->aliasname) {
			char acanon[BDAT_GRP_NAME_LEN*2+1];
			bb_canonical(br_ind[i].brett->aliasname,acanon);
			if (!strcmp(acanon,bufname))
				return br_ind[i].brett->nr;
		}
	}
	/* Seufzzzz, nicht gefunden. Eventuell ein umbenannter Name? */
	{
		int tries=5;
		while (tries--) {
			bb_rename_t *r=bb_find_rename_entry(name);
			if (!r)
				break;
			i=brettnr(r->new_name);
			if (i>0 && i<=max_brettnr)
				return i;
			name=r->new_name;
		}
	}
	return 0;

} /* find_brett */
