/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <syslog.h>
#include <ctype.h>
#include <limits.h>
#include <errno.h>

#include <getopt.h>

#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif

#include "libquark.h"
#include "itbcompile.h"
#include "long-options.h"

#define stricmp strcasecmp

struct box *box_anker;

struct link *links;

struct box *top;

char *pathgate;
char *gsfilename;
char *hostname;
const char *program_name = "itbcompile";

struct box *mybox = NULL;		/* just to keep gcc happy */

int verbose;

void usage (int ec);
void print_tree (BOOLEAN mit_kosten, int level, struct box *this);
struct box *find_box (char *kuerzel);
void calc_kosten_link (long base, struct box *this);
void calc_kosten_me (struct box *me);
void calc_kosten_rest (void);
void build_tree (void);
void write_routing (const char *fname, struct box *me);
void calc_gateways (void);
void write_pathalias (struct box *me);
void write_domlist (void);

static void write_ink (const char *fname, const char *mytelprefix);
static void read_itb (FILE * f, const char *mytelprefix);
static void write_inl (const char *fname, const char *mytelprefix);
static void write_isb (const char *fname, const char *mytelprefix);
static void write_inp_sub (FILE * f, int level, struct box *this);
static void write_inp (const char *fname);
static const char *check_domain_string(const char *s);

static char *reference;

static void
complain(const char *zeile, const char *format,...) 
{
	char *tmp;
	va_list param;
	if (!verbose)
		return;

	va_start(param,format);
	vasprintf(&tmp,format,param);
	va_end(param);
	if (tmp) {
		fprintf(stderr,"%s:\r\n",reference);
		if (zeile)
			fprintf(stderr,"\t%s\r\n",zeile);
		fprintf(stderr,"\t%s\r\n",tmp);
		free(tmp);
	}
}

static const char * 
check_domain_string(const char *s) 
{
	for (;*s;s++) {
		if (*s>='A' && *s<='Z')
			continue;
		if (*s>='a' && *s<='z')
			continue;
		if (*s=='.' || *s=='-' || *s=='_')
			continue;
		if (isdigit(*s))
			continue;
		return s;
	}
	return NULL;
}

static int
phone_compare(const char *s, const char *t)
{
	while (*s && *t)
	{
		while (*s && !isdigit(*s))
			s++;
		while (*t && !isdigit(*t))
			t++;
		if (*s!=*t)
			return 1;
		s++;
		t++;
	}
	if (*s || *t)
		return 1;
	return 0;
}

static void
check_phone(char *buf, char *nr)
{
	char *p;
	if (!memcmp(nr,"+49",3)) {
		p=nr+3;
		while (*p && !isdigit(*p))
			p++;
		if (!isdigit(*p)) 
			complain(buf,"verdchtige Telefonnummer");
		else if (*p=='0')
			complain(buf,"0 direkt hinter +49?");
	}
}
	
static void
finalize_box(struct box *new, struct telefon *old_telefon,
			 ULONG len_u,ULONG len_s, ULONG len_percent)
{
	struct telefon *t;
	int found=0;
	int count;
	struct telefon *last,*next;
	for (t=new->telefon,last=NULL,count=1;t!=NULL;count++,last=t,t=next) {
		BOOLEAN loeschen=FALSE;
		if (old_telefon  && t->nummer
			&& 0==phone_compare(t->nummer,old_telefon->nummer)) {
			found=1;
			if (old_telefon->geheim && !t->geheim) {
				complain(old_telefon->nummer,
						 "... ist geheim in t-Zeile, nicht aber in [pPtT]-Zeile");
			} else if (t->geheim && !old_telefon->geheim) {
				complain(old_telefon->nummer,
						 "... ist geheim in p[tT]-Zeile, nicht aber in T-Zeile");
			}
		}
		if (!t->nummer) {
			complain(NULL, "Telefon #%d hat keine Nummer",count);
			loeschen=TRUE;
		}
		if (!t->art) {
			complain(NULL, "Telefon #%d hat keine Protokollangabe",count);
			loeschen=TRUE;
		}
		next=t->next;
		if (loeschen) {
			if (new->telefon->nummer)
				free(new->telefon->nummer);
			if (new->telefon->art)
				free(new->telefon->art);
			free(new->telefon);
			if (last)
				last->next=next;
			else
				new->telefon=next;
		} 
	}
	if (!found && old_telefon && new->telefon) {
		complain(old_telefon->nummer,
				 "Telefonnummer aus T/t-Zeile nicht in [pPtT]-Zeilen gefunden");
	}
	if (!new->telefon)
		new->telefon=old_telefon;
	else if (old_telefon) {
		free(old_telefon->nummer);
		free(old_telefon);
	}
	if (!new->telefon)
		complain(NULL, "Box hat *keine* Telefonnummer");
	if (verbose > 1) {
		if (len_u>2500)
			complain(NULL, "Box hat %lu Bytes Benutzerinformationen",len_u);
		if (len_s>2500)
			complain(NULL, "Box hat %lu Bytes Sysopinformationen",len_s);
		if (len_percent>3000)
			complain(NULL, "Box hat %lu Bytes PMK-Informationen",len_percent);
	}
}

static void
read_itb (FILE * f, const char *mytelprefix)
{
	char buf[1024];
	struct box *new = NULL;
	struct box *last = NULL;
	BOOLEAN got_domains = FALSE;
	struct telefon *last_telefon = NULL;
	struct telefon *old_telefon = NULL;
	const char *cp;
	ULONG len_u=0;
	ULONG len_s=0;
	ULONG len_percent=0;

	while (fgets (buf, sizeof (buf), f))
	{
		char *p;
		unsigned char *up;

		p = strpbrk (buf, "\r\n");
		if (p) {
			if (*p=='\r' && p[1]!='\n') {
				*p='^';
				complain(buf,"streunendes CR in der Zeile (anstelle des ^)");
			}
			*p = '\0';
		}
		for (up=buf;*up;up++) {
			if (*up>=32 && *up<=127)
				continue;
			if (*up=='\t')
				continue;
			if (strchr("",*up))
				continue;
			complain(buf,"illegale Zeichen in der Zeile (%x(%c))",*up,*up);
			break;
		}
		if (!new && *buf != '*')
			continue;
		switch (*buf)
		{
		case '*':
			if (new)
				finalize_box(new,old_telefon,len_u,len_s,len_percent);
			old_telefon=NULL;
			last_telefon = NULL;
			strtrim(buf);
			if (!buf[1]) {
				if (verbose) {
					reference=XSTRDUP("*fehlende_angabe");
					complain(buf,"Krzel fehlt, Box ignoriert");
				}
				new=NULL;
				continue;
			} else if (verbose)
				reference=XSTRDUP(buf);
			if (strlen(buf+1)>8) {
				complain(buf,"Krzel zu lang, Box ignoriert");
				new=NULL;
				continue;
			}
			new = XCALLOC (1, sizeof (struct box));

			new->kuerzel = XSTRDUP (buf + 1);
			new->next = NULL;
			new->kosten = -1;
			if (last)
				last->next = new;
			else
				box_anker = new;
			last = new;
			len_u=0;
			len_s=0;
			len_percent=0;
			break;
		case 'd':
			complain(buf,"geheime Primrdomain?");
			break;
		case 'D':
			strtrim(buf);
			if (!buf[1]) 
				complain(buf,"Leere Primrdomain nicht erlaubt");
			else if ((cp=check_domain_string(buf+1))!=NULL)
				complain(buf,"Primrdomain %s enthlt illegale Zeichen: %d(%c)",
						 buf+1,*cp,*cp);
			else if (!strchr(buf+1,'.'))
				complain(buf,"Primrdomain %s enthlt keinen Punkt",
						 buf+1);
			else if (strlen(buf+1)>40)
				complain(buf,"Primrdomain %s verdchtig lang",
						 buf+1);
			else
			{
				new->primdomain = XCALLOC (1, sizeof (struct boxdomain));
				
				if (buf[1] != '.')
				{
					new->primdomain->name = strdup (buf + 1);
					XNULLCHECK (new->primdomain->name);
				}
				else
				{
					size_t need = strlen (new->kuerzel) + strlen (buf + 1) + 1;
					
					new->primdomain->name = XMALLOC (need);
					strcpy (new->primdomain->name, new->kuerzel);
					strcat (new->primdomain->name, buf + 1);
				}
				new->primdomain->next = NULL;
				got_domains = TRUE;
				new->got_domain = TRUE;
			} 
			break;
		case 'z':
			complain(buf,"geheime Sekundrdomain?");
			break;
		case 'Z':
			strtrim(buf);
			if (!buf[1]) 
				complain(buf,"Leere Sekundrdomain nicht erlaubt");
			else if ((cp=check_domain_string(buf+1))!=NULL)
				complain(buf,"Sekundrdomain %s enthlt illegales Zeichen: %d(%c)",
						 buf+1,*cp,*cp);
			else if (!strchr(buf+1,'.'))
				complain(buf,"Sekundrdomain %s enthlt keinen Punkt",
						 buf+1);
			else if (strlen(buf+1)>40)
				complain(buf,"Sekundrdomain %s verdchtig lang",
						 buf+1);
			else
			{
				struct boxdomain *d;
				d = XCALLOC (1, sizeof (struct boxdomain));

				if (buf[1] != '.')
					d->name = XSTRDUP (buf + 1);
				else
				{
					size_t need = strlen (new->kuerzel) + strlen (buf + 1) + 1;

					d->name = XMALLOC (need);
					strcpy (d->name, new->kuerzel);
					strcat (d->name, buf + 1);
				}
				d->next = new->secdomain;
				new->secdomain = d;
				new->got_domain = TRUE;
				got_domains = TRUE;
			}
			break;
		case '#':
			new->nummer = strtol (buf + 1, NULL, 10);
			if (new->nummer < 0)
			{
				new->nummer = -new->nummer;
				new->fremdbox = TRUE;
			}
			if (new->nummer==0) 
				complain(buf,"illegale Boxnummer");
			else if (new->nummer>255)
				complain(buf,"Boxnummer %lu mit MAUS 7 unmglich",
						 (unsigned long) new->nummer);
			break;
		case 'n':
			complain(buf,"geheimer Boxname?");
			break;
		case 'N':
			strtrim(buf);
			if (!buf[1])
				complain(buf,"Leerer Boxname");
			new->name = XSTRDUP (buf + 1);
			break;
		case 'u':
			complain(buf,"geheime Benutzerinformationen?");
			break;
		case 'U':
			{
				struct list *l = XMALLOC (sizeof (struct list));

				l->text = XSTRDUP (buf + 1);
				l->next = NULL;
				if (new->userinfo_last)
					new->userinfo_last->next = l;
				else
					new->userinfo = l;
				new->userinfo_last = l;
				len_u+=strlen(buf+1);
			}
			break;
		case 'S':
			complain(buf,"ffentliche Sysopinformationen?");
			break;
		case '%':
			len_percent+=strlen(buf+1);
			if (buf[1]=='$') {
				strtrim(buf);
				if (!buf[2]) 
					complain(buf,"leere Dollarangabe");
			} else if (buf[1]=='n')
				complain(buf,"geheimer Boxname ...");
		case 's':
			if (*buf=='s')
				len_s+=strlen(buf+1);
			if (strstr(buf,"BEGIN PGP PUBLIC KEY BLOCK") && verbose > 1)
				complain(buf,
						 "unsichere und berflssige Verteilung von PGP public key");
			{
				struct list *l = malloc (sizeof (struct list));

				XNULLCHECK (l);
				l->text = strdup (buf);
				XNULLCHECK (l->text);
				l->next = NULL;
				if (new->sysopinfo_last)
					new->sysopinfo_last->next = l;
				else
					new->sysopinfo = l;
				new->sysopinfo_last = l;
			}
			break;
		case '-':
			strtrim(buf);
			if (!buf[1])
				complain(buf,"Leeres Pollboxkrzel");
			new->serverkuerzel = XSTRDUP (buf + 1);
			break;
		case 'p':
		case 'P':
			if (buf[1] == '#')
			{
				struct telefon *t;
				t = calloc (1, sizeof (struct telefon));

				XNULLCHECK (t);
				if (*buf == 'p')
					t->geheim = TRUE;
				else
					t->geheim = FALSE;
				if (last_telefon)
				{
					last_telefon->next = t;
					t->next = NULL;
				}
				else
				{
					t->next = new->telefon;
					new->telefon = t;
				}
				last_telefon = t;
			}
			else if (buf[1] == 'K' || buf[1] == 'k')
			{
				if (last_telefon)
				{
					last_telefon->art = strdup (buf + 2);
					XNULLCHECK (last_telefon->art);
				}
			}
			else if (buf[1] == 'T' || buf[1] == 't')
			{
				strtrim(buf+2);
				if (!buf[2]) {
					complain(buf,"Keine Telefonnummer angegeben");
					continue;
				}
				if (buf[2] != '+')
				{
					char mod[255];
					complain(buf,"veraltete Schreibweise fr Telefonnummer");

					if (buf[2] == '0' && buf[3] == '0')
						sprintf (mod, "+%s", buf + 4);
					else
						sprintf (mod, "%s-%s", mytelprefix, buf + 2);
					last_telefon->nummer = XSTRDUP (mod);
				}
				else {
					last_telefon->nummer =  XSTRDUP (buf + 2);
					check_phone(buf,buf+2);
				}
			}
			break;
		case 't':
		case 'T':
			strtrim(buf+1);
			if (!buf[1]) {
				complain(buf,"Keine Telefonnummer angegeben");
				continue;
			}
			old_telefon = XCALLOC (1, sizeof (struct telefon));

			if (buf[1] != '+')
			{
				char mod[255];
				complain(buf,"veraltete Schreibweise fr Telefonnummer");
				
				if (buf[1] == '0' && buf[2] == '0')
					sprintf (mod, "+%s", buf + 3);
				else
					sprintf (mod, "%s-%s", mytelprefix, buf + 2);
				old_telefon->nummer = XSTRDUP (mod);
			}
			else {
				old_telefon->nummer = XSTRDUP (buf + 1);
				check_phone(buf,buf+1);
			}

			if (*buf == 't')
				old_telefon->geheim = TRUE;
			else
				old_telefon->geheim = FALSE;
			old_telefon->next = NULL;
			break;
		case 'g':
			complain(buf,"geheime Gateways?");
			/* FALLTHROUGH */
		case 'G':
			strtrim(buf);
			if (!buf[1]) {
				complain(buf,"Leere Gatewayangabe");
				continue;
			}
			p=strchr(buf+1,',');
			if (!p) {
				complain(buf,"Gatewayzeile ohne Komma/weitere Angaben");
				continue;
			}
			if (p-buf+1>20)
				complain(buf,"verdchtig langer Gatewayname");
			{
				struct boxgate *d;
				*p++ = 0;
				p = strchr (p, ',');
				if (!p) {
					complain(buf,"Gatewayzeile ohne Angabe der Boxen");
					continue;
				}

				d = XCALLOC (1, sizeof (struct boxgate));
				d->name = XSTRDUP (buf + 1);
				d->allowed = XSTRDUP (p);
				d->next = new->gateways;
				new->gateways = d;
			}
			break;
		default:
			strtrim(buf);
			if (!buf[1])
				complain(buf,"Leere unbekannte Zeile");
			else
				complain(buf,"unbekannte Zeile");
			break;
		}
	}
	if (new)
		finalize_box(new,old_telefon,len_u,len_s,len_percent);
	if (gsfilename)
	{
		FILE *g = fopen (gsfilename, "r");
		BOOLEAN topmagic_found = FALSE;
		struct box *b;
		struct boxdomain *bdom;

		if (!g)
		{
			error(0,errno, "fopen(%s,\"r\")", gsfilename);
			return;
		}
		while (fgets (buf, sizeof (buf), g))
		{
			char *p;
			char *q;

			p = strpbrk (buf, "\r\n");
			if (p)
				*p = '\0';
			if (!stricmp (buf, "weitere Spalten: alternative Adressen"))
			{
				topmagic_found = TRUE;
				continue;
			}
			if (!topmagic_found)
				continue;
			if (!stricmp (buf,
						"Fr folgende Muse gibt es alternative Adressen:"))
				continue;
			if (!*buf)
				continue;
			if (!stricmp (buf,
						  "nderungen bitte per PM zu mir - diese Liste sollte mglichst vollstndig und"
				))
				break;
			p = buf;
			while (isupper (*p) || isdigit (*p) || *p == '-'
				   || *p == '' || *p == '' || *p == '' || *p == '')
				p++;
			if (!isspace (*p))
				continue;
			*p++ = '\0';
			while (isspace (*p))
				p++;
			b = find_box (buf);
			if (!b)
				continue;		/* Mll */
			b->got_domain = TRUE;
			if (!stricmp (p, "-keine Domain vorhanden-"))
				continue;
			bdom = malloc (sizeof (struct boxdomain));

			XNULLCHECK (bdom);
			q = strchr (p, ' ');
			if (q)
				*q++ = '\0';
			bdom->name = strdup (p);
			bdom->next = NULL;
			XNULLCHECK (bdom->name);
			b->primdomain = bdom;
			p = q;
			while (p && *p)
			{
				while (isspace (*p))
					p++;
				if (!*p)
					break;
				q = strchr (p, ' ');
				if (q)
					*q++ = '\0';
				bdom = malloc (sizeof (struct boxdomain));

				XNULLCHECK (bdom);
				bdom->name = strdup (p);
				XNULLCHECK (bdom->name);
				bdom->next = b->secdomain;
				b->secdomain = bdom;
				p = q;
			}
		}						/* while fgets */
		fclose (g);
	}
   /*
    * Wenn keine Domains in der ITB angegeben sind, dann allen Boxen, die
    * in der Liste von Gnter Stck nicht erwhnt sind, maus.de und 
    * maus.sub.org als Domains verpassen.
    */
	if (!got_domains)
	{
		char *p,
		 *q;
		struct box *akt;
		struct boxdomain *d;

		akt = box_anker;
		while (akt)
		{
			if (akt->got_domain)
			{
				akt = akt->next;
				continue;
			}
			p = buf;
			q = akt->kuerzel;
			while (*q)
			{
				switch (*q)
				{
				case '':
					*p++ = 'u';
					*p++ = 'e';
					break;
				case '':
					*p++ = 'o';
					*p++ = 'e';
					break;
				case '':
					*p++ = 'a';
					*p++ = 'e';
					break;
				case '':
					*p++ = 'U';
					*p++ = 'E';
					break;
				case '':
					*p++ = 'O';
					*p++ = 'E';
					break;
				case '':
					*p++ = 'A';
					*p++ = 'E';
					break;
				case '':
					*p++ = 'S';
					*p++ = 'S';
					break;
				default:
					*p++ = *q;
					break;
				}
				q++;
			}
			strcpy (p, ".maus.de");
			d = calloc (1, sizeof (struct boxdomain));

			XNULLCHECK (d);
			d->name = strdup (buf);
			XNULLCHECK (d->name);
			d->next = NULL;
			akt->primdomain = d;
			strcpy (p, ".maus.sub.org");
			d = calloc (1, sizeof (struct boxdomain));

			XNULLCHECK (d);
			d->name = strdup (buf);
			XNULLCHECK (d->name);
			d->next = NULL;
			akt->secdomain = d;
			akt = akt->next;
		}
	}
}

void 
build_tree (void)
{
	struct box *akt;

	for (akt = box_anker; akt; akt = akt->next)
	{
		struct box *such;

		if (!akt->serverkuerzel)
		{
			top = akt;
			continue;
		}
		for (such = box_anker; such; such = such->next)
		{
			if (such == akt)
				continue;
			if (!strcmp (such->kuerzel, akt->serverkuerzel))
			{
				akt->up = such;
				if (!such->down)
					such->down = akt;
				else
				{
					struct box *such2;

					for (such2 = such->down; such2->right; such2 = such2->right)
						;		/* do nothing */
					such2->right = akt;
					akt->left = such2;
				}
				akt->upkosten = 1;
			}
		}						/* for such */
	}							/* for akt */
}

void 
print_tree (BOOLEAN mit_kosten, int level, struct box *this)
{
	int i;
	struct box *akt;

	for (i = 0; i < level; i++)
		fputs ("    ", stdout);
	printf ("%s", this->kuerzel);
	if (mit_kosten)
		printf (" (%ld)", this->kosten);
	if (this->best_way)
		printf (" via %s", this->best_way->kuerzel);
	puts ("");
	for (akt = this->down; akt; akt = akt->right)
		print_tree (mit_kosten, level + 1, akt);
}

struct box *
find_box (char *kuerzel)
{
	struct box *akt;

	for (akt = box_anker; akt; akt = akt->next)
	{
		if (!strcmp (akt->kuerzel, kuerzel))
			return akt;
	}
	return NULL;
}

void
calc_kosten_link (long base, struct box *this)
{
	struct box *akt;

	long hops;

	this->kosten = base;
	this->best_way = this;
	hops = this->upkosten;
	for (akt = this->up; akt; akt = akt->up)
	{
		if (akt->kosten == -1 || akt->kosten > base + hops)
		{
			akt->kosten = base + hops;
			akt->best_way = this;
		}
		hops += akt->upkosten;
	}
}

void
calc_kosten_me (struct box *me)
{
	struct box *akt;

	long hops;

	me->kosten = 0;
	hops = me->upkosten;
	for (akt = me->up; akt; akt = akt->up)
	{
		if (akt->kosten == -1 || akt->kosten > hops)
		{
			akt->kosten = hops;
			akt->best_way = me->up;
		}
		hops += akt->upkosten;
	}
	hops = me->upkosten;
	for (akt = me->down; akt; akt = akt->right)
	{
		if (akt->kosten == -1 || akt->kosten > hops)
		{
			akt->kosten = hops;
			akt->best_way = akt;
		}
		hops += akt->upkosten;
	}
}

void
calc_kosten_rest (void)
{
	struct box *this,
	 *akt;
	long hops;

	for (this = box_anker; this; this = this->next)
	{
		if (this->kosten != -1)
			continue;
		hops = this->upkosten;
		for (akt = this->up; akt; akt = akt->up)
		{
			if (akt->kosten != -1)
			{
				this->kosten = hops + akt->kosten;
				this->best_way = akt->best_way;
				break;
			}
			hops += akt->upkosten;
		}
	}
}

struct gate
{
	char *domain;
	long kosten;
	struct box *best_way;
	struct gate *next;
};
struct gate *gate_anker;

void 
calc_gateways (void)
{
	struct box *b;

	for (b = box_anker; b; b = b->next)
	{
		struct boxgate *g;

		for (g = b->gateways; g; g = g->next)
		{
			struct gate *such;

			if (!g->allowed)
				continue;
			if (!strchr (g->allowed, '*') && !strstr (g->allowed, mybox->kuerzel))
				continue;
			for (such = gate_anker; such; such = such->next)
			{
				if (!strcmp (such->domain, g->name))
				{
					if (b->kosten < such->kosten)
					{
						such->kosten = b->kosten;
						such->best_way = b->best_way;
					}
					break;
				}
			}
			if (!such)
			{
			   /* nicht gefunden */
				such = calloc (1, sizeof (struct gate));

				XNULLCHECK (such);
				such->domain = strdup (g->name);
				XNULLCHECK (such->domain);
				such->best_way = b->best_way;
				such->kosten = b->kosten;
				such->next = gate_anker;
				gate_anker = such;
			}
		}
	}

}

void 
write_routing (const char *fname, struct box *me)
{
	struct box *akt;
	struct boxdomain *sec;

   /* hostalias und Boxrouting */
	FILE *f;

	if (!strcmp (fname, "-"))
		f = stdout;
	else
	{
		f = fopen (fname, "w");
		if (!f)
		{
			perror (fname);
			exit (1);
		}
	}
	for (akt = box_anker; akt; akt = akt->next)
	{
		if (akt->primdomain && akt->primdomain->name)
			fprintf (f, "hostalias %s %s\n", akt->kuerzel, akt->primdomain->name);
		else
		{
			if (akt->best_way && akt != me)
				fprintf (f, "domain %s %s\n", akt->kuerzel,
						 akt->best_way->primdomain->name);
			continue;
		}
		for (sec = akt->secdomain; sec; sec = sec->next)
		{
			/* Diese Paranioa ist fr den Fall:
			 * :D.maus.de
			 * :Z.maus.de
			 * aufgetreten im .CNF der FT.
			 */
			if (0!=strcasecmp(sec->name,akt->primdomain->name))
				fprintf (f, "hostalias %s %s\n", sec->name, 
					akt->primdomain->name);
		}
		if (akt->best_way && akt != me)
		{
			fprintf (f, "domain %s %s\n", akt->primdomain->name,
					 akt->best_way->primdomain->name);
		}
	}
   /* Gaterouting */
	{
		struct gate *g;

		for (g = gate_anker; g; g = g->next)
		{
			if (g->best_way)
				fprintf (f, "domain %s %s %ld\n",
						 g->domain, g->best_way->primdomain->name,
						 g->kosten);
		}
	}
	if (strcmp (fname, "-"))
		fclose (f);
}

#define MAIN(x) (x->primdomain ? x->primdomain->name : x->kuerzel)
void 
write_pathalias (struct box *me)
{
	struct box *akt;
	struct boxdomain *sec;
	unsigned char *p;

   /* hostalias und Boxrouting */
	for (akt = box_anker; akt; akt = akt->next)
	{
		if (!akt->primdomain)
			continue;
		p = (unsigned char *) akt->kuerzel;
		while (*p && isprint (*p))
			p++;
		if (!*p)
		{
			printf ("%s = %s", akt->primdomain->name, akt->kuerzel);
			for (sec = akt->secdomain; sec; sec = sec->next)
				printf (", %s", sec->name);
			puts ("");
		}
		if (*p && akt->secdomain)
		{
			printf ("%s = %s", akt->primdomain->name, akt->secdomain->name);
			for (sec = akt->secdomain->next; sec; sec = sec->next)
				printf (", %s", sec->name);
			puts ("");
		}

		if (akt->best_way && akt != me)
		{
			printf ("%s %s\n", MAIN (akt), MAIN (akt->best_way));
		}
		else if (akt->up)
		{
			printf ("%s %s\n", MAIN (akt), MAIN (akt->up));
		}
	}
	if (hostname)
		printf ("%s %s\n", hostname, pathgate);
	if (me || links)
	{
		BOOLEAN komma = FALSE;

		printf ("%s ", pathgate);
		if (me)
		{
			printf (" %s", MAIN (me));
			komma = TRUE;
		}
		if (links)
		{
			struct link *akt_link;
			struct box *box;

			for (akt_link = links; akt_link; akt_link = akt_link->next)
			{
				box = find_box (akt_link->kuerzel);
				if (!box)
					continue;	/* paranoia, ich wei */
				printf ("%s %s",
						komma ? "," : "",
						MAIN (box));
				komma = TRUE;
			}
		}
		puts ("");
	}
#if 0
   /* Gaterouting */
	{
		struct gate *g;

		for (g = gate_anker; g; g = g->next)
			printf ("domain %s %s %ld\n", g->domain, g->best_way->primdomain->name,
					g->kosten);
	}
#endif

	if (gate_anker)
	{
		struct gate *g;
		BOOLEAN first = TRUE;

		printf ("%s", pathgate);
		for (g = gate_anker; g; g = g->next)
		{
			if (*g->domain == '.')
			{
				printf ("%s %s", first ? "" : ",", g->domain);
				first = FALSE;
			}
		}
		puts ("");
	}
}

void 
write_domlist (void)
{
	struct box *akt;
	struct boxdomain *sec;

   /* hostalias und Boxrouting */
	for (akt = box_anker; akt; akt = akt->next)
	{
		BOOLEAN tel_done = FALSE;
		struct telefon *t;

		fputs (akt->kuerzel, stdout);
		if (akt->primdomain)
		{
			printf (" %s", akt->primdomain->name);
			for (sec = akt->secdomain; sec; sec = sec->next)
				printf (" %s", sec->name);
		}
		puts ("");
		for (t = akt->telefon; t; t = t->next)
		{
			if (!t->geheim)
			{
				printf ("\tO%s (%s)\n", akt->name, akt->telefon->nummer);
				tel_done = TRUE;
				break;
			}
		}
		if (!tel_done)
			printf ("\tO%s\n", akt->name);
	}
#if 0
   /* Gaterouting */
	{
		struct gate *g;

		for (g = gate_anker; g; g = g->next)
			printf ("domain %s %s %ld\n", g->domain, g->best_way->primdomain->name,
					g->kosten);
	}
#endif
}

void 
usage (int ec)
{
	FILE *f;

	if (ec)
		f = stdout;
	else
		f = stderr;

	fprintf (f, "usage: %s -m 'meinkuerzel'\n", program_name);
	fprintf (f,
 "  optional arguments:\n"
 "    -c             -- enter mauscon mode\n"
 "    -q filename    -- enter quark mode, write routing config to file\n"
 "    -p gatename    -- enter pathalias mode (incomplete)\n"
 "    -i itbfilename -- filename of ITB/ITS (default: stdin)\n"
 "    -l linksite[=cost] -- a way to add additional links\n"
 "    -g filename    -- filename of Gnter Stcks domain list (obsolete)\n"
 "    -h hostname    -- our hostname\n"
 "    -T telprefix   -- prefix for local phone number (default: +49)\n"
 "    generate infofiles:\n"
 "      -I dir      -- directory to write the infofiles to\n"
 "      -K filename -- file to write INK to\n"
 "      -L filename -- file to write INL to\n"
 "      -P filename -- file to write INP to\n"
 "      -S filename -- file to write ISB to\n"
		);

	exit (ec);
}

int 
main (int argc, char **argv)
{
	struct box *box;
	FILE *f = stdin;
	char *me = NULL;
	char *inkfile = NULL;
	char *inlfile = NULL;
	char *inpfile = NULL;
	char *isbfile = NULL;
	char *routingfile = NULL;
	const char *mytelprefix = "+49";

	BOOLEAN quarkmode = FALSE;
	BOOLEAN mausconmode = FALSE;
	BOOLEAN pathaliasmode = FALSE;

	struct option opts[] =
	{
		{"me", required_argument, NULL, 'm'},
		{"link", required_argument, NULL, 'l'},
		{"itb", required_argument, NULL, 'i'},
		{"infofiles", required_argument, NULL, 'I'},
		{"ink", required_argument, NULL, 'K'},
		{"inl", required_argument, NULL, 'L'},
		{"inp", required_argument, NULL, 'P'},
		{"isb", required_argument, NULL, 'S'},
		{"mauscon", no_argument, NULL, 'c'},
		{"guenter", required_argument, NULL, 'g'},
		{"pathalias", required_argument, NULL, 'p'},
		{"quark", required_argument, NULL, 'q'},
		{"hostname", required_argument, NULL, 'h'},
		{"telprefix", required_argument, NULL, 'T'},
		{"verbose", no_argument, NULL, 'v'},
		{NULL, 0, NULL, 0}
	};
	int option_index = 0;

	if (argv[0]) {
		program_name = argv[0];
		program_name = strrchr(argv[0],'/');
		if (program_name)
			program_name++;
		else
			program_name = argv[0];
	}

    parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage);

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "cg:h:i:I:K:l:L:m:p:P:q:S:T:v", opts, &option_index);

		if (c == -1)
			break;
		switch (c)
		{
		case 'I':
			{
				char buf[PATH_MAX];

				sprintf (buf, "%s/INK", optarg);
				inkfile = strdup (buf);
				XNULLCHECK (inkfile);
				sprintf (buf, "%s/INL", optarg);
				inlfile = strdup (buf);
				XNULLCHECK (inlfile);
				sprintf (buf, "%s/ISB", optarg);
				isbfile = strdup (buf);
				XNULLCHECK (isbfile);
				sprintf (buf, "%s/INP", optarg);
				inpfile = strdup (buf);
				XNULLCHECK (inpfile);
			}
			break;
		case 'K':
			inkfile = optarg;
			break;
		case 'L':
			inlfile = optarg;
			break;
		case 'P':
			inpfile = optarg;
			break;
		case 'S':
			isbfile = optarg;
			break;
		case 'c':
			mausconmode = TRUE;
			quarkmode = FALSE;
			break;
		case 'g':
			gsfilename = optarg;
			break;
		case 'h':
			hostname = optarg;
			break;
		case 'i':
			f = fopen (optarg, "rb");
			if (!f)
			{
				perror (optarg);
				exit (2);
			}
			break;
		case 'l':
			{
				struct link *new;
				char *p;
				new = calloc (1, sizeof (struct link));

				XNULLCHECK (new);
				new->kuerzel = strdup (optarg);
				XNULLCHECK (new->kuerzel);
				new->kosten = 1;
				p = strchr (new->kuerzel, '=');
				if (p)
				{
					*p++ = '\0';
					new->kosten = strtol (p, NULL, 10);
				}
				new->next = links;
				links = new;
			}
			break;
		case 'm':
			me = strdup (optarg);
			XNULLCHECK (me);
			break;
		case 'p':
			pathaliasmode = TRUE;
			pathgate = optarg;
			quarkmode = FALSE;
			break;
		case 'q':
			routingfile = optarg;
			quarkmode = TRUE;
			break;
		case 'T':
			mytelprefix = optarg;
			break;
		case 'v':
			verbose++;
			break;
		default:
			usage (1);
			break;
		}
	}
	if (quarkmode && !me)
		error(2, 0, "`--me' not given");

	read_itb (f, mytelprefix);
	build_tree ();
	if (me)
	{
		mybox = find_box (me);
		if (!mybox)
			error(1, 0, "cannot find %s",me);
		calc_kosten_me (mybox);
	}
	if (links)
	{
		struct link *akt;

		for (akt = links; akt; akt = akt->next)
		{
			box = find_box (akt->kuerzel);
			if (!box)
				error(0, 0, "cannot find %s",akt->kuerzel);
			else
				calc_kosten_link (akt->kosten, box);
		}
	}
	if (pathgate)
	{
		box = find_box (pathgate);
		if (!box)
			error(0, 0, "cannot find %s",pathgate);
		else
			calc_kosten_link (0, box);
	}
	calc_kosten_rest ();
   	/* print_tree(FALSE,0,top); */
	calc_gateways ();

	if (quarkmode)
		write_routing (routingfile, mybox);
	if (mausconmode)
		write_domlist ();
	if (pathaliasmode)
		write_pathalias (mybox);
	if (inkfile)
		write_ink (inkfile, mytelprefix);
	if (inlfile)
		write_inl (inlfile, mytelprefix);
	if (isbfile)
		write_isb (isbfile, mytelprefix);
	if (inpfile)
		write_inp (inpfile);

	exit (0);
}

static int
box_srt (const void *a, const void *b)
{
	const struct box *ab = *(struct box * const *) a;
	const struct box *bb = *(struct box * const *) b;

	if (!ab->telefon || !ab->telefon->nummer)
		return -1;
	if (!bb->telefon || !bb->telefon->nummer)
		return 1;
	return -strcmp (ab->telefon->nummer, bb->telefon->nummer);
}


static void
write_ink (const char *fname, const char *mytelprefix)
{
	FILE *f;
	struct box *b;
	int count;
	struct box **feld;

	if (!strcmp (fname, "-"))
		f = stdout;
	else
	{
		f = fopen (fname, "w");
		if (!f)
		{
			perror (fname);
			exit (1);
		}
	}
	fprintf (f, "%3.3s %-8.8s %-30.30s %-20.20s %s\n",
			 "Nr.", "Krzel", "Name", "Nummer", "Typ");
	fprintf (f, "%3.3s %-8.8s %-30.30s %20.20s %s\n",
		 "---", "--------", "--------------------", "--------------------",
		 "---------");
	b = box_anker;
	count = 0;
	while (b)
	{
		count++;
		b = b->next;
	}
	feld = XMALLOC ((count + 1) * sizeof (void *));

	count = 0;
	b = box_anker;
	while (b)
	{
		feld[count] = b;
		count++;
		b = b->next;
	}
	feld[count] = NULL;
	qsort (feld, count, sizeof (void *), box_srt);

	while (count--)
	{
		char buf[255];
		const char *what;
		const char *typ;
		struct telefon *t;
		int first=1;

		b = feld[count];
		for (t=b->telefon;t;t=t->next)
		{
			int artchar=0;
			if (t->geheim)
			{
				what = "[nicht ffentlich]";
				typ = "Unbekannt";
				artchar = -1;
			}
			else
			{
				if (!strncmp (t->nummer, mytelprefix, strlen (mytelprefix)))
				{
					what = buf;
					*buf = '0';
					strcpy (buf + 1, t->nummer + strlen (mytelprefix) + 1);
				}
				else
				{
					*buf = '0';
					buf[1] = '0';
					strcpy (buf + 2, t->nummer + 1);
					what = t->nummer;
				}
next_typ:
				if (t->art)
				{
					switch(t->art[artchar])
					{
					case 'A':
						typ="Analog";
						artchar++;
						if (!t->art[artchar])
							artchar=-1;
						break;
					case 'D':
						typ="ISDN X75";
						artchar++;
						if (!t->art[artchar])
							artchar=-1;
						break;
					default:
						typ=t->art;
						artchar++;
						if (!t->art[artchar])
							artchar=-1;
						break;
					}
				}
				else
				{
					typ="Unbekannt";
					artchar=-1;
				}
			}
			if (first)
				fprintf (f, "%3ld %-8.8s %-30.30s %-20.20s %s\n",
						 b->nummer,
						 b->kuerzel,
						 b->name,
						 what,
						 typ);
			else
				fprintf (f, "%3s %-8.8s %-30.30s %-20.20s %s\n",
						 "",
						 "",
						 "",
						 what,
						 typ);
			first=0;
			if (artchar!=-1)
				goto next_typ;
		}
	}
	free (feld);
	if (strcmp (fname, "-"))
		fclose (f);
}

static void
write_inl (const char *fname, const char *mytelprefix)
{
	FILE *f;
	struct box *b;

	if (!strcmp (fname, "-"))
		f = stdout;
	else
	{
		f = fopen (fname, "w");
		if (!f)
		{
			perror (fname);
			exit (1);
		}
	}
	b = box_anker;
	while (b)
	{
		char buf[255];
		int l1;
		const char *what;

		if (!b->telefon) 
			what = "[nicht angegeben]";
		else if (b->telefon->geheim)
			what = "[nicht ffentlich]";
		else
		{
			if (!strncmp (b->telefon->nummer, mytelprefix, strlen (mytelprefix)))
			{
				what = buf;
				*buf = '0';
				strcpy (buf + 1, b->telefon->nummer + strlen (mytelprefix) + 1);
			}
			else
			{
				*buf = '0';
				buf[1] = '0';
				strcpy (buf + 2, b->telefon->nummer + 1);
				what = b->telefon->nummer;
			}
		}

		l1 = fprintf (f, "%ld %s", b->nummer, b->name);
		fprintf (f, "   (Krzel: %s)\n", b->kuerzel);
		fprintf (f, "%*.*s\n",
				 l1, l1, "---------------------------------------");
		fprintf (f, "\n");
		fprintf (f, "Nummer: %s\n", what);
		fprintf (f, "\n");
		{
			struct list *li;

			li = b->userinfo;
			while (li)
			{
				fprintf (f, "%s\n", li->text);
				li = li->next;
			}
			fprintf (f, "\n");
		}
		{
			int header = 0;
			struct boxgate *g;

			g = b->gateways;
			while (g)
			{
				if (g->allowed && strstr (g->allowed, "*"))
				{
					if (!header)
					{
						fprintf (f, "Gateways:\n");
						header = 1;
					}
					fprintf (f, "%s\n", g->name);
				}
				g = g->next;
			}
			if (header)
				fprintf (f, "\n");
		}
		b = b->next;
	}
	if (strcmp (fname, "-"))
		fclose (f);
}

static void
write_isb (const char *fname, const char *mytelprefix)
{
	FILE *f;
	struct box *b;

	if (!strcmp (fname, "-"))
		f = stdout;
	else
	{
		f = fopen (fname, "w");
		if (!f)
		{
			perror (fname);
			exit (1);
		}
	}
	b = box_anker;
	while (b)
	{
		char buf[255];

		if (!b->telefon) 
			strcpy(buf,"[nicht angegeben]");
		else if (!strncmp (b->telefon->nummer, mytelprefix, strlen (mytelprefix)))
		{
			*buf = '0';
			strcpy (buf + 1, b->telefon->nummer + strlen (mytelprefix) + 1);
		}
		else
		{
			*buf = '0';
			buf[1] = '0';
			strcpy (buf + 2, b->telefon->nummer + 1);
		}

		fprintf (f, "** %3ld %-8.8s  %-30.30s  %s\n",
				 b->nummer, b->kuerzel, b->name, buf);
		fprintf (f, "\n");
		{
			struct list *li;

			li = b->sysopinfo;
			while (li)
			{
				fprintf (f, "%s\n", li->text);
				li = li->next;
			}
			fprintf (f, "\n");
		}
		b = b->next;
	}
	if (strcmp (fname, "-"))
		fclose (f);
}

struct mussnoch
{
	struct mussnoch *next;
	struct box *box;
}
 *mussnoch_anker;

static void
write_inp (const char *fname)
{
	FILE *f;
	struct mussnoch *akt;

	if (!strcmp (fname, "-"))
		f = stdout;
	else
	{
		f = fopen (fname, "w");
		if (!f)
		{
			perror (fname);
			exit (1);
		}
	}
	write_inp_sub (f, 0, top);

	for (akt = mussnoch_anker; akt; akt = akt->next)
	{
		struct boxgate *g;
		size_t len = 0;

		g = akt->box->gateways;
		while (g)
		{
			if (g->allowed && strchr (g->allowed, '*'))
			{
				if (len + strlen (g->name) > 72 || len == 0)
				{
					if (len)
						fprintf (f, "\n");
					len = fprintf (f, "%s: ", akt->box->kuerzel);
				}
				len += fprintf (f, "%s  ", g->name);
			}
			g = g->next;
		}
		if (len)
			fprintf (f, "\n");
	}

	if (strcmp (fname, "-"))
		fclose (f);
}


static void
write_inp_sub (FILE * f, int level, struct box *this)
{
	int i;
	size_t len = 0;
	struct box *akt;

	for (i = 1; i < level; i++)
	{
		fputs ("    ", f);
		len += 4;
	}
	if (level > 0)
	{
		fputs ("--- ", f);
		len += 4;
	}
	fprintf (f, "%-3.3s", this->kuerzel);
	len += 4;
	if (this->gateways)
	{
		struct boxgate *g;
		size_t gatelen = 0;

		g = this->gateways;
		while (g)
		{
			if (g->allowed && strchr (g->allowed, '*'))
				gatelen += strlen (g->name) + 2;
			g = g->next;
		}
		len += gatelen;
		if (len < 72 && gatelen <= 20 && gatelen)
		{
			int first = TRUE;

			fprintf (f, "  [");
			g = this->gateways;
			while (g)
			{
				if (g->allowed && strchr (g->allowed, '*'))
				{
					fprintf (f, "%s%s",
							 first ? "" : "  ", g->name);
					first = FALSE;
				}
				g = g->next;
			}
			fprintf (f, "]");
		}
		else if (len >= 42 || gatelen > 20)
		{
			struct mussnoch *new;
			new = malloc (sizeof (struct mussnoch));

			new->next = mussnoch_anker;
			new->box = this;
			mussnoch_anker = new;
			fprintf (f, "[-> unten]\n");
		}
	}

	fprintf (f, "\n");
	for (akt = this->down; akt; akt = akt->right)
		write_inp_sub (f, level + 1, akt);
}
