/*
    Copyright (C) 1994 1995 1996 Dirk Hagedorn, Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/

/*	----------------------------------------------------------------------
	Includes
	----------------------------------------------------------------------	*/
#include	"tomsysdep.h"
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	<portab.h>
#include	<errno.h>
#include	<getopt.h>
#include	<sys/stat.h> /* oder wie auch immer das auf dem Atari heisst */
#ifdef HAVE_UNISTD_H
#include	<unistd.h>
#endif

#include	"libquark.h"
#include	"lineedit.h"
#include	"messages.h"
#include	"long-options.h"

/*
 * Caveats: 
 * - Die Texte, die wir bearbeiten, haben den Zeichensatz des Benutzers der
 *   Box. Der Einsatz irgendwelcher Makros aus ctype.h ist daher 
 *   verboten. Abhilfe bietet der Einbau der Umlautwandlungsroutinen der
 *   Quark. 
 *   Nachteil: Vor allen Ausgaben mu die Routine aufgerufen werden.
 *   Vorteil:  Die Such/Ersetzen-Funktion arbeitet caseinsensitiv auch
 *             fr Umlaute.
 *   Lohnt sich das?
 */


/*	----------------------------------------------------------------------
	Makros
	----------------------------------------------------------------------	*/
#define min(a, b)		((a) < (b) ? (a) : (b))
#define max(a, b)		((a) > (b) ? (a) : (b))

#define	linefeed()		fputs("\r\n", stdout),fflush(stdout)




/*	----------------------------------------------------------------------
	Typen
	----------------------------------------------------------------------	*/
/*
typedef enum
{
	FALSE,
	TRUE
} boolean;
*/
#define boolean int

typedef enum
{
	DEUTSCH,
	ENGLISH
} t_lang;


typedef enum
{
	NO_ERROR,		/* Kein Fehler aufgetreten  */
	NO_FILES,		/* Keine Datei(en) gefunden */
	NO_MEMORY,		/* Kein Speicher vorhanden  */
	READ_ERROR,		/* Lesefehler               */
	WRITE_ERROR		/* Schreibfehler            */
} t_error;





/*	----------------------------------------------------------------------
	Variablen
	----------------------------------------------------------------------	*/
char	text[MAXLINES][LINESIZE];		/* Hier steht der Text drin         */

										
int		textlines;						/* Soviele Zeilen hat der Text      */
										/* die erste Zeile steht in text[1]!*/

const char	*program_name;				/* der Name dieses Programms        */

boolean	option_q;						/* Keine Fehlermeldungen ausgeben?  */

int 	changes;						/* Aenderung aufgetreten?           */
			
t_lang	language;						/* Sprache */





#ifdef USE_TERMIOS
void io_reset_stdin(void);
void io_set_stdin(void);
#endif
void show_version ( void );
void lineedit ( void );
void get_borders (const char *zeile, int *a, int *b, const char **rest);
void insert_line( int zeilennr );
void delete_lines(int von, int bis);
void check_borders( int *von, int *bis );
void show_error (const char *string);
void usage1 ( int ecode );
void usage ( int ecode, const char *what );
boolean load_text ( char *dateiname, int *fehler);
void output_help ( void );
boolean save_text ( char *dateiname, int *fehler);
void list_lines(int von, int bis, boolean with_numbers, const char *such, 
	const char *ersatz);
void output_help ( void );
void my_strlower(char *to, const unsigned char *from);


	/*
	 * Careful: Der Code ist hlich und geht von 8bit-Zeichenstzen aus,
	 * die in den unteren 128 Zeichen ASCII entsprechen.
	 * Die Funktion ist massiv unportabel, aber Benutzung der Makros
	 * aus ctype.h verbietet sich, weil das, was wir hier bekommen,
	 * nicht die Systemumlaute sind, sondern unter der Quark die 
	 * Umlaute des Benutzers. Auf die Dauer mu dieses Problem durch
	 * Einbindung der Quarkumlautwandlungsroutinen beseitigt werden.
	 */
void my_strlower(char *to, const unsigned char *from)
{
	while (*from)
	{
		if (*from>='A' && *from <= 'Z')
			*to=(*from) + 'a' - 'A';
		else 
			*to=*from;
		from++;
	}
}

/*	######################################################################
	#                                                                    #
	#    Programm-Informationen und Fehlermeldungen                      #
	#                                                                    #
	######################################################################	*/
/*	----------------------------------------------------------------------
	show_version() zeigt die Versionsnummer dieses Programms
	----------------------------------------------------------------------	*/
void show_version ( void )
{
	printf("%s - %s\r\n",program_name, PACKAGE_VERSION);
	printf("  Copyright (C) Dirk Hagedorn, Frank Baumgart, Uwe Ohse\r\n");
	
} /*show_version*/


void usage1 ( int ecode )
{
	usage(ecode,NULL);
}

void usage (int ecode, const char *errtext) 
{
	if (ecode)
	{
		if (errtext)
			fprintf(stderr, "%s: %s\n",program_name,errtext);
		fprintf (stderr, Use_Help[language],
			program_name);
		exit(ecode);
	}
	printf(Usage[language]);
	fputs(
"simple line editor. Options:\n"
"     --help         print this text and exit\n"
"  -q --quiet        don't print warnings\n"
"     --version      print version number and exit\n"
		,stdout);
	exit(0);
} /*usage*/


/*	----------------------------------------------------------------------
	show_error() gibt eine Fehlermeldung inklusive Programmnamen aus.
	Die	Ausgabe erfolgt auf STDERR oder STDOUT.
	->	string:	die Fehlermeldung
	----------------------------------------------------------------------	*/
void show_error (const char *string)
{
	FILE *out=stderr;
#ifdef __TOS__
	static int use_stderr=-1;
#endif
	if (!option_q)
#ifdef __TOS__
	/* 
	 * STDERR ist ein Fehler gewesen - DONT_USE_STDERR wre portabler 
	 * gewesen.
	 */
	if (use_stderr==-1)
	{
		if (getenv("STDERR"))
			use_stderr=1;
		else
			use_stderr=0;
	}
	if (!use_stderr)
		out=stdout;
#endif
	
	fprintf(out, "%s: %s\r\n", program_name, string);
}	/*show_error*/





/*	######################################################################
	#                                                                    #
	#    Text laden und sichern                                          #
	#                                                                    #
	######################################################################	*/

/*	----------------------------------------------------------------------
	load_text() liest den Inhalt der Datei mit dem Zugriffspfad dateiname
	in das globale Textarray <text>. Zeilen werden wortweise auf COLUMNS
	umgebrochen.
	<-	TRUE:	Datei korrekt geladen
		FALSE:	Der Fehler <fehler> ist aufgetreten.
	----------------------------------------------------------------------	*/
boolean load_text ( char *dateiname, int *fehler)
{
	char	*zeile;				/* Lesebuffer */
	char	*ptr;
	FILE	*file;		
	int zeilennummer=0;
	boolean broken=FALSE;

	/* Textspeicher anfordern. Hier anfordern, am Ende von */
	/* save_text() wieder freigeben!                       */

#if 0	
	text= (char *) malloc( (MAXLINES+1)*LINESIZE );
	
	if (!text)
	{
		show_error( Err_No_Memory[language] );
		*fehler= NO_MEMORY;
		return (FALSE);
	}
#endif

	/* Lesebuffer anfordern (Neu: 09.05.1994) */
	
	zeile = XMALLOC(BUFFSIZE);
	
	textlines= 0;	/* Anzahl der gelesenen Zeilen zuruecksetzen */

	/* Datei oeffnen */
	
	file= fopen(dateiname, "r");
	
	if (!file)
	{
#ifdef HAVE_UNISTD_H
		if (access(dateiname,F_OK)==0)
#else
		struct stat st;
		if (stat(dateiname,&st)!=0 && errno!=ENOENT)
#endif
		{
			int e=errno;
			error(0,errno,"%s: %s: %s",dateiname,Err_Load_Open[language],
				strerror(e));
			return ( FALSE ); /* nichts zu machen */
		}
		fprintf( stdout, "\r\n%s\r\n\r\n", New_File_Info[language]);
		free(zeile);
		*fehler= NO_ERROR;
		return TRUE;
		
	} /*if*/
	
	
	while ((textlines<MAXLINES) &&  fgets(zeile, BUFFSIZE, file) )
	{
			/* LF, CR etc. entfernen */
		size_t sl;
		size_t i;
		zeilennummer++;
		sl=strlen(zeile);

		if (sl && zeile[sl-1]!='\n') {
			/* no way - das koennen wir nicht */
			error(1,errno,"%s: line %d too long",dateiname,zeilennummer);
		}
		while ( sl > 0 && ((unsigned char) zeile[sl-1]) < 32 )
			zeile[--sl]= EOS;
			
			/* Tabulatoren zu Leerzeichen */
		for (i=0; i<sl; i++)
		{
			if ( zeile[i] == TAB )
				zeile[i]= ' ';			/* Momentan nur ein Leerzeichen */
		}
		
		
		/* Ist es eine Leerzeile oder eine Zeile mit Inhalt? (04.05.1994) */
		
		if (sl >0)
		{
			/* <zeile> darf nicht veraendert werden, daher wird mit einem */
			/* Hilfszeiger gearbeitet. */
	
			ptr= zeile;
					
			while ( (textlines<MAXLINES) && (strlen(ptr)>LINESIZE) )
			{
				/* Hier werden ueberlange Zeilen wortweise umgebrochen */
				char *pbreak=NULL;
				char *pbind=NULL;
				char *p;

				broken=TRUE; 

				p=ptr+LINESIZE-1;
				while (p>ptr)
				{
					if (*p==' ' || *p=='\t')
					{
						*p=EOS;
						pbreak=p;
						break;
					}
					if (*p=='-')
					{
						pbind=p;
						break;
					}
					p--;
				}
				if (pbreak)
				{
					/* In den Text uebernehmen */
					strcpy (text[++textlines], ptr);
					ptr=p+1;
					/* Spaces bergehen */
					while (*ptr == ' ' || *ptr == '\t')
						ptr++;
				}
				else if (pbind)
				{
					size_t l=pbind-ptr+1;
					p=text[++textlines];
					strncpy(p,ptr,l);
					p[l]=EOS;
					ptr+=l;
				}
				else
				{
					p=text[++textlines];
					strncpy(p,ptr,LINESIZE-1);
					p[LINESIZE-1]=EOS;
					ptr+=LINESIZE-1;
				}
			} /*while*/
	
			if ( (textlines<MAXLINES) && (strlen(ptr)>0) )
			{	/* in <zeile> steht nach dem Umbruch noch etwas, also mit aufnehmen, */
				/* bzw. kurze, nicht umgebrochene Zeile	uebernehmen                  */
				strcpy(text[++textlines], ptr);
			}
		}
		else
		{
			textlines++;		/* text[] wird beim Programmstart initialisiert */
								/* eine Leerzeile zu kopieren ist ueberfluessig */
		} /*if (strlen...)*/
		
	} /*while*/
	
	
	fclose( file );		/* Datei wieder schliessen */

	free ( zeile );		/* Speicher freigeben     */

	/* Das ist wichtig zu wissen, wenn irgendeine Datei mit einem */
	/* extrem widerlichen Format, wie z.B. die .CNFs im MausNet,  */
	/* bearbeitet wird. */

	if (broken)
		fprintf( stdout, "\r\n%s\r\n\r\n", Lines_Broken_Info[language] );

	*fehler= NO_ERROR;
	
	return ( TRUE );
	
}	/*load_text*/





/*	----------------------------------------------------------------------
	save_text() sichert den Text in Datei <dateiname>
	<-	TRUE:	Datei korrekt geladen
		FALSE:	Der Fehler <fehler> ist aufgetreten.
	----------------------------------------------------------------------	*/
boolean save_text ( char *dateiname, int *fehler)
{
	int		i;
	FILE	*file;		

	/* Datei oeffnen */
	
	file= fopen(dateiname, "w");
	
	if (!file)
	{
		show_error( Err_Save_Open[language] );
		*fehler= NO_FILES;
		/*free(text);	*/		/* Neu: 09.05.1994 */
		return ( FALSE );
	}


	/* Kurz und buendig: */
	/* Allen Zeilen ein Linefeed anhaengen (das CR haengt */
	/* fputs() im Textmodus selbstaendig unter GEMDOS an) */
	/* und die Zeilen in die Datei schreiben, bis ein     */
	/* Fehler auftritt.                                   */
	/* Das ganze for()-Konstrukt in einer Zeile, aber     */
	/* dafuer 5 Zeilen Kommentar, tss, tss ;-)            */

	for (i=1;i<=textlines;i++)
	{
		if (fputs(text[i],file)==EOF || fputs("\n",file)==EOF)
			break;
	}

	/* auch beim Schliessen kann ein Fehler auftreten, weil die Puffer */
	/* nicht mehr zurueckgeschrieben werden koennen */

	if (i<=textlines || fclose(file))
	{
		show_error( Err_Save_Write[language] );
		*fehler = WRITE_ERROR;
		return (FALSE);
	}

	*fehler= NO_ERROR;
	
	return ( TRUE );
	
}	/*save_text*/





/*	######################################################################
	#                                                                    #
	#    Editor-Routinen                                                 #
	#                                                                    #
	######################################################################	*/

/*	----------------------------------------------------------------------
	output_help() gibt eine kurze Uebersicht ueber die Steuerkommandos fuer
	den Zeileneditor aus.
	Kommando: .?
	----------------------------------------------------------------------	*/
void output_help ( void )
{
	printf(Edit_Help[language]);
	
#if 0	
	Nicht unterstuetzte Quark-1-Feature:
	.u  ASCII-Upload einer Textdatei beginnen
	    (wozu, ASCII-Upload ist nichts weiter als schnelles Tippen)
	.a  Empfaengeradresse aendern
	.b  Betreffstext aendern
	    Dafuer braeuchte man eine eigene Steuerdatei mit den Headerdaten.
	    Das sollte aber moeglichst mehr als nur die beiden Funktionen
	    bieten, z.B. noch:
	    - Gruppenwechsel (Problem: in welche Gruppen darf der User denn
	      schreiben).
	    - Followup-To setzen (es waere schoen, wenn man die Existenz 
	      wenigstens einer Gruppe pruefen koennte, und noch besser, wenn
	      man sagen koennte "in *die* darfst Du nicht schreiben, warum
	      also sollen Leute dahin kommentieren")
	    - Reply-To setzen (Problem: Darf der User die Adresse anschreiben?)
	    Alles machbar, aber aufwendig.
	.q  Quoten/Quotefenster
	    reichlich aufwendig.
#endif
	
} /*output_help*/





/*	----------------------------------------------------------------------
	check_borders() ueberprueft die Gueltigkeit der Zeilengrenzen fuer
	list_lines() und show_lines()
	----------------------------------------------------------------------	*/
void check_borders( int *von, int *bis )
{
	int tausch;
	
	/* Die Bereiche schliesslich nochmal ueberpruefen */

	*von= max(*von, 0);
	*von= min(*von, textlines);

	*bis= max(*bis, 0);
	*bis= min(*bis, textlines);

	if (*von > *bis)
	{
		tausch= *von;
		*von= *bis;
		*bis= tausch;
	}
} /*check_borders*/





/*	----------------------------------------------------------------------
	delete_lines() loescht eine oder mehrere Zeilen
	Kommando: .d
	----------------------------------------------------------------------	*/
void delete_lines(int von, int bis)
{
	int i, diff;

	check_borders(&von, &bis);
	
	if ( (von!=0) || (bis!=0) )
	{	
		if ( (von==0) && (bis!=0) )
		{	/* bis bis loeschen */
			von= 1;
		}
		
		if ( (von!=0) && (bis==0) )
		{	/* nur a loeschen!!! */
			bis= von;
		}
		

		diff= bis-von+1;
		
		for (i=von; i<=textlines; i++)
		{
			if ( (i+diff)<=textlines )
				strcpy(text[i], text[i+diff]);
			else
				strcpy(text[i], "");
		}
		
		textlines= textlines-diff;
	
		changes = TRUE;		/* neu: 03.04.1994 */
	}
	
}	/*delete_lines*/




/*	----------------------------------------------------------------------
	insert_line() fuegt eine Leerzeile in den Text ein und schiebt alle
	anderen Zeilen eine Position hoeher, notfalls gehen diese verloren
	->	zeilennr:	Zeilennummer
	----------------------------------------------------------------------	*/
void insert_line( int zeilennr )
{
	int i;

	for (i=textlines; i>=zeilennr; i--)
	{
		if (i<MAXLINES)					/* Abfrage: 04.05.1994 */
			strcpy(text[i+1], text[i]);
	}
	
	strcpy(text[zeilennr], "");


	/* Hier stand einmal "textlines++;", aber nach einem Return wird die   */
	/* Anzahl der Zeilen sowieso erhoeht, egal ob im Insertmode oder nicht */	
	
	changes = TRUE;		/* neu: 03.04.1994 */
	
}	/*insert_line*/





/*	----------------------------------------------------------------------
	list_lines() gibt Zeilen des Textes mit Zeilennummer aus
	und uebernimmt auch noch suchen/ersetzen.
	Kommando: .l
	----------------------------------------------------------------------	*/
void list_lines(int von, int bis, boolean with_numbers, const char *such, 
	const char *ersatz)
{
	int i;
	char buf[LINESIZE];
	char tmp[LINESIZE];
	char *p;

	if (such)
	{
		if (strlen(such) >= LINESIZE) /* Paranoia? */
		{
			linefeed();
			return;
		}

		/* Caseinsensitiv arbeiten */
		/* Umlaute funktionieren nicht caseinsensitiv. Tja */

		strcpy(buf,such);
		my_strlower(buf,such);
	}
	
	check_borders(&von, &bis);
	
	if (von==0)
	{
		von= 1;
		if (bis==0)	
			bis = textlines;
	}
	else
	{
		if (bis==0)	
			bis = von;
	}

	for (i=von; i<=bis; i++)
	{
		if (such)
		{
			char *s;
			my_strlower(tmp,text[i]);
			if ((s=strstr(tmp,buf))==0)
				continue;
			if (ersatz 
				&& strlen(text[i]) + strlen(ersatz) - strlen(buf) < LINESIZE)
			{
				/* Jetzt wird's heavy */
				p=tmp;
				if (s>tmp)
				{
					memcpy(p,text[i],s-tmp);
					p+=s-tmp;
				}
				memcpy(p,ersatz,strlen(ersatz));
				p+=strlen(ersatz);
				strcpy(p,s+strlen(buf));
				strcpy(text[i],tmp);
				changes = TRUE;
			}

		}
		if (with_numbers)
			fprintf(stdout, "%3d:%s\r\n", i, text[i]);
		else
			fprintf(stdout, "%s\r\n", text[i]);
	}

	linefeed();

} /*list_lines*/




/*	----------------------------------------------------------------------
	get_borders() liefert die Grenzen des gewuenschten Textbereichs und
	wird zur Ermittlung der Parameter mehrerer Kommandos eingesetzt.
	Beispiel: zeile== ".l 1-2" --> a=1, b=2
	Achtung: get_borders() liefert fuer eine Intervallgrenze 0 zurueck,
	         wenn keine Angaben gemacht wurden. Dies ist wichtig, da
	         .l alle Zeilen anzeigt, .d aber nicht alle Zeilen loescht.
	         get_borders() liefert aber jeweils (0,0) zurueck!
	->	zeile:	Kommandozeile
	<-	a:		Bereich-Start
		b:		Bereich-Ende
		rest:   Der Rest der Zeile, was nicht verarbeitet werden konnte
	----------------------------------------------------------------------	*/
void get_borders (const char *zeile, int *a, int *b, const char **rest)
{
	char *strich;
	char *p;
	char kopie[LINESIZE];
	char *start;
	char *hilfe;

	if ( strlen(zeile)>2 )
		strcpy(kopie, zeile+3);		/* Nur mit der Kopie (ohne Kommando) arbeiten */
	else
		strcpy(kopie, "");

	start=kopie;	
	/* Leerzeichen links entfernen */
	while (*start=='\t' || *start==' ')
		start++;
	/* Leerzeichen rechts entfernen */
	p=strlen(start)+start;
	while (p > start  && p[-1]==' ')
		p--;
	*p=EOS;

	hilfe=NULL;
		
	if ( (strich=strchr(start, '-'))!=NULL )
	{
		/* Bindestrich vorhanden, jetzt wird's kompliziert */
		/* a- : alle Zeilen ab a  */
		/* -b : alle Zeilen bis b */
		/* a-b: alle Zeilen zwischen a und b inklusive */
		
		
		if ( start[0]=='-' )
		{	/* Steht der Strich ganz links? */
			*a= 1;
			*b= (int) strtol(start+1,&hilfe,10);
		}
		else if ( start + strlen(start)-1 == strich )
		{	/* Oder ganz rechts? */
			start[strlen(start)-1]= EOS;
			*a= atoi(start);
			*b= textlines;
		}
		else
		{	/* Nee, anscheinend mittendrin */
			*b= (int) strtol(strich+1,&hilfe,10);
			strich[0]= EOS;			/* Weg mit dem Strich */
			*a= (int) strtol(start,NULL,10);
		}
		
	}
	else
	{
		if (strlen(start)==0)
		{
			/* Kein Bindestrich und Rest leer, daher keine Infos */
			*a= 0;
			*b= 0;
		}
		else
		{
			/* Nur eine Zeile */
			*a= (int) strtol(start,&hilfe,10);
			*b= *a;
		}
		
	}
	if (rest)
	{
		if (hilfe)
		{
			while (*hilfe==' ' || *hilfe=='\t')
				hilfe++;
			*rest=zeile+(hilfe-start);
			if ( strlen(zeile)>2 )
				(*rest)+=3;
		}
		else
			*rest=NULL;
	}

} /*get_borders*/




/*	----------------------------------------------------------------------
	lineedit() beeinhaltet den kompletten Editor
	----------------------------------------------------------------------	*/
void lineedit ( void )
{
	size_t	s, ex;
	int		a, b;
	int		altz, neuz;
	int		ey, old_ey;				/* Cursor-Position (textbezogen)  */
	int		newchar;				/* Zeichen-Eingabe                */
	boolean	exit_ed, exit_line;		/* ein paar Flags                 */
	boolean	correct_mode;			/* Korrektur-Modus aktiv?         */
	boolean	insert_mode;			/* Einfuege-Modus aktiv?          */
	int		inserted;				/* Anzahl der eingefuegten Zeilen */
	char	newline[LINESIZE];		/* die zu editierende Zeile       */
	char	*ptr;					/* ein Hilfspointer               */
	const char *rest;
	int     last_crlf;              /* zuletzt ein \r oder ein \n?    */
	
	
	ey= textlines+1;				/* Am Ende des Textes geht's los */
	old_ey= 0;
	exit_ed= FALSE;
	correct_mode= FALSE;
	insert_mode= FALSE;
	inserted= 0;
	last_crlf= 0;

	fprintf(stdout, "%s\r\n\r\n", First_Line[language]);	/* Hinweis auf ".?" */

	
	while (!exit_ed)
	{
		ex= 0;
		
		if (insert_mode)
			strcpy(newline, "");
		else
			strcpy(newline, text[ey]);
		
		fprintf(stdout, "%3d:%s", ey, newline);		/* gendert: 04.05.1994 */
		fflush(stdout);
	
		exit_line= FALSE;

		
		while ( (!exit_line) && (!exit_ed) )
		{
#if defined(__TURBOC__) || defined(__GO32__)
			newchar= getch();
#else
			newchar= fgetc(stdin);		/* Hier MiNT/Unix-Routinen benutzen! */
#endif
			if (newchar == EOF || newchar == 0)
			{
				exit_ed=TRUE;
				break;
			}

			/* Tabulator zu Leerzeichen */
			
			if ( newchar == TAB )
				newchar=' ';				/* nur ein Leerzeichen! */
				
			switch (newchar)							/* Jetzt geht's los ;-) */
			{
				case LF:	/* Linefeed        */
				case CR:	/* Carriage-Return */
					if (newchar==LF && last_crlf==CR)
					{
						last_crlf=LF;
						break;
					}
					last_crlf=newchar;

					if ( (!correct_mode) && ((strcmp(newline, ".")==0) || (strcmp(newline, "...")==0)) )
					{
						linefeed();
						exit_ed= TRUE;
					}
					else
					{
							/* Punktkommandos werden nicht im Korrektur- oder */
							/* Einfuegemodus ausgewertet!                      */
						if ( (newline[0]=='.') && (!correct_mode))
						{
							switch (newline[1])
							{
#ifdef TEST
								case '!':	/* Text ohne sichern beenden */
									linefeed();
									changes= 0;
									exit_ed= TRUE;
									break;
#endif
								case '?':	/* Hilfsanzeige */
									linefeed();
									output_help();
									exit_line= TRUE;
									break;
								case 'd':	/* Bereich loeschen */
									if (insert_mode)
									{
										linefeed();
										fprintf( stdout, "\r\n%s\r\n\r\n", Impossible_Insert[language] );
									}
									else
									{
										get_borders(newline, &a, &b, NULL);
										if ( (a>textlines) || (b>textlines) )
										{
											linefeed();
											fprintf( stdout, "\r\n%s\r\n\r\n", False_Params[language] );
											exit_line= TRUE;
										}
										else
										{
											delete_lines( a, b );
											ey= textlines+1;
											linefeed();
										}
									}
									exit_line= TRUE;
									break;
								case 'e':	/* Ersetzen */
									linefeed();
									linefeed();
									get_borders(newline, &a, &b, &rest);
									if (rest && strchr(rest,','))
									{
										char *q=XSTRDUP(rest);
										char *p;

										p=strchr(q,',');
										*p=EOS;
										list_lines( a, b, TRUE, q, p+1 );
									}
									else
									{
										linefeed();
										fprintf( stdout, "\r\n%s\r\n\r\n", 
											Substitution_Syntax[language] );
									}
									exit_line= TRUE;
									break;
								case 'f':	/* Suchen */
									linefeed();
									linefeed();
									get_borders(newline, &a, &b, &rest);
									if (rest)
										list_lines( a, b, TRUE, rest, NULL );
									exit_line= TRUE;
									break;
								case 'g':	/* Sprache wechseln */
									linefeed();
									if (language==DEUTSCH)
									{
										fprintf( stdout, "\r\n%s\r\n\r\n", " Do you speak English?" );
										language=ENGLISH;
									}
									else
									{	/* wer kennt Gerhard Polt nicht??? ;-) */
										fprintf( stdout, "\r\n%s\r\n\r\n", " Man spricht Deutsh!" );
										language=DEUTSCH;
									}
									exit_line= TRUE;
									break;
								case 'i':	/* Einfuegemodus aktivieren */
									if (insert_mode)
									{
										linefeed();
										fprintf( stdout, "\r\n%s\r\n\r\n", Already_Insert[language] );
									}
									else
									{
										get_borders(newline, &a, &b, NULL);
										if ( (a>=0) && (a<textlines) )
										{
											old_ey= ey;
											ey= a+1;			/* siehe Quark-I-Editor! */
											fprintf( stdout, "\r\n\r\n%s\r\n\r\n", Start_Insert[language] );
											insert_mode= TRUE;
											inserted= 0;
										}
										else
										{
											linefeed();
											fprintf( stdout, "\r\n%s\r\n\r\n", False_Params[language] );
										}
									}
									exit_line= TRUE;
									break;
								case 'k':	/* Zeile korrigieren */
									if (insert_mode)
									{
										fprintf( stdout, "\r\n\r\n%s\r\n\r\n", Impossible_Insert[language] );
										exit_line= TRUE;
									}
									else
									{
										get_borders(newline, &a, &b, NULL);
										if ( (a==0) || (a>textlines) )
										{
											linefeed();
											fprintf( stdout, "\r\n%s\r\n\r\n", False_Params[language] );
											exit_line= TRUE;
										}
										else
										{
											old_ey= ey;
											ey= a;
											correct_mode= TRUE;
											strcpy(newline, text[ey]);
											fprintf(stdout, "\r\n%3d:%s", ey, newline);
											ex= strlen(newline);
										}
									}
									break;
								case 'l':	/* Bereich mit Nummern anzeigen */
									linefeed();
									linefeed();
									get_borders(newline, &a, &b, NULL);
									list_lines( a, b, TRUE, NULL ,NULL );
									exit_line= TRUE;
									break;
								case 's':	/* Bereich ohne Nummern anzeigen */
									linefeed();
									linefeed();
									get_borders(newline, &a, &b, NULL);
									list_lines( a, b, FALSE, NULL, NULL );
									exit_line= TRUE;
									break;
								case 'v':	/* Versionsinformationen ausgeben */
									linefeed();
									linefeed();
									show_version();
									exit_line= TRUE;
									break;
								case 'z':	/* Einfuegemodus ausschalten */
									if (!insert_mode)
									{
										linefeed();
										fprintf( stdout, "\r\n%s\r\n\r\n", Not_Insert[language] );
										exit_line= TRUE;
									}
									else
									{
										fprintf( stdout, "\r\n\r\n%s\r\n\r\n", Stop_Insert[language] );
										insert_mode= FALSE;
										exit_line= TRUE;
										ey= old_ey+inserted;
									}
									break;
								default:
									linefeed();
									fprintf( stdout, "\r\n%s%s\r\n\r\n", No_Command[language], First_Line[language] );
									exit_line= TRUE;
							} /*switch(newline[1])*/
						}
						else
						{
							if (insert_mode)
							{
								insert_line(ey);
								inserted++;
							}
								
							strcpy(text[ey], newline);
							changes = TRUE;		/* neu: 03.04.1994 */

							
							if (correct_mode)
							{
								ey= old_ey;
								correct_mode= FALSE;
								linefeed();
							}
							else
							{
								textlines++;
								ey++;
							}

							linefeed();
							exit_line= TRUE;
						} /*if-else*/
						
					} /*if-else*/
					break;
				
				case DEL:	/* ASCII-127 */
				case BS:	/* Backspace */
					if ((s= strlen(newline))>0)
					{
						fputs( "\b \b", stdout);
						ex--;
						newline[s-1]= newline[s];
						changes = TRUE;		/* neu: 03.04.1994 */
					}
					break;

				case BEL:	/* Glocke                 */
				case FF:	/* Formfeed               */
				case TAB:	/* horizonatler Tabulator */
				case '\v':	/* vertikaler Tabulator   */
				case ESC:
				case EOF:	/* ... ignorieren         */
					break;

				default:	/* "normale" Taste der Zeile hinzufuegen */
							/* dabei das Nullbyte beachten!          */
					if ( (s= strlen(newline)) < (LINESIZE-1) )
					{
						fputc(newchar, stdout);
						newline[s+1]= newline[s];
						newline[s++]= newchar;
						ex++;
						changes = TRUE;		/* neu: 03.04.1994 */
					}
					
					if ( s >= (LINESIZE-1) )
					{
						if (ey<MAXLINES)
						{
							char *space,*bind;
							if (insert_mode)
							{
								insert_line(ey);
								inserted++;
							}
							
							strcpy(text[ey], newline);
							changes = TRUE;		/* neu: 03.04.1994 */
							
							altz= ey;			/* fuer wortweisen Umbruch */
							
							if (correct_mode)
							{
								ey= old_ey;
								correct_mode= FALSE;
							}
							else
							{
								textlines++;
								ey++;
							}
							
							neuz= ey;			/* fuer wortweisen Umbruch */
							
							/* wortweiser Umbruch (neu: 04.05.1994) */
							
							/* ist ueberhaupt ein Leerzeichen oder gar ein */
							/* Bindestrich in der Zeile vorhanden?         */
							
							bind= strrchr(text[altz], '-');
							space= strrchr(text[altz], ' ');
							if (bind && space)
								ptr=bind < space ? space : bind;
							else if (bind)
								ptr=bind;
							else
								ptr=space;
							
							if ( ptr )
							{
								ptr++;	/* Trennzeichen nicht beruecksichtigen */
								
								while (ptr[0]!=EOS)
								{
									s= strlen(text[neuz]);
									text[neuz][s+1]= text[neuz][s];	/* letztes Zeichen der alten Zeile */
									text[neuz][s]= ptr[0];			/* der neuen hinzufuegen           */
									ptr[0]= EOS;					/* Zeilenendung(en) setzen         */
									ptr++;
									fputs("\b \b", stdout);

								} /*while*/
								
							} /*if*/
							
							if (correct_mode)	/* 04.05.1994: stand vorher mit oben, kam aber */
								linefeed();		/* dem Loeschen der Zeichen ins Gehege         */
								
							linefeed();
							exit_line= TRUE;
						}
					}
			} /*switch(newchar)*/

			fflush(stdout);

		} /*while*/	

	} /*while*/
	

}	/*lineedit*/

#ifdef USE_TERMIOS
/*
 * Terminalhandling :-( Quark-Console needs it.
 */
/*
 * stdin auf RAW und NOECHO setzen
 */
struct termios cons;

void io_set_stdin(void)
{
	struct termios console;

	tcgetattr(0,&cons);
	console = cons;
#if 0
 - Function: int cfmakeraw (struct termios *TERMIOS_P)
      This function provides an easy way to set up *TERMIOS_P for what
           has traditionally been called "raw mode" in BSD.  It does exactly
                this:
#endif
console.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
console.c_oflag &= ~OPOST;
console.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
console.c_cflag &= ~(CSIZE|PARENB);
console.c_cflag |= CS8;

#if 0
	console.c_lflag &= ~ICANON;
	console.c_lflag &= ~ECHO;
	console.c_cc[VMIN]  = 1;
	console.c_cc[VTIME] = 0;
#endif
	atexit(io_reset_stdin);
	tcsetattr(0,TCSADRAIN,&console);
}

/*
 *  stdin auf Ursprungseinstellung zurcksetzen
 */
void io_reset_stdin(void)
{
	tcsetattr(0,TCSADRAIN,&cons);
} /* io_reset_stdin */
#endif





/*	######################################################################
	#                                                                    #
	#    Programmstart und -Initialisierungen                            #
	#                                                                    #
	######################################################################	*/

/*	----------------------------------------------------------------------
	main() wertet die Kommandozeile aus und ruft entsprechend die
	Suchroutine (eventuell mehrmals) auf.
	<-	0:	alles OK
	<-	1:	Datei nicht gefunden
	<-	2:	Nicht genuegend Speicher vorhanden
	----------------------------------------------------------------------	*/
int main ( int argc, char *argv[])
{
	int		i;
	int		result;
	boolean	argv_exit;			/* Flag                        */

	struct option opts[] =
	{
		{"quiet", no_argument, NULL, 'q'},
		{NULL, 0, NULL, 0}
	};

	/* globale Variablen initialisieren */
	
	language= ENGLISH;				/* default ist Englisch */
	argv_exit= FALSE;
	
	changes = 0;					/* Aenderungen aufgetreten?    */
	
	option_q= FALSE;				/* Quiet-Mode?       */
	
	for (i=0; i<=MAXLINES; i++)		/* Stringfeld leeren */
		strcpy(text[i], "");

	if (argv[0] && *argv[0]) {		/* Programmnamen ermitteln */ 
		program_name=strrchr(argv[0],'/');
		if (program_name)
			program_name++;
		else
			program_name=argv[0];
	}
	else
		program_name="linedit";
	
	/* Environment auswerten (neu: 03.05.1994)             */
	/* Die Sprachauswertung erfolgt so, wie Julian Reschke */
	/* es bei seinen Mupfel-Tools macht. (siehe *.1)       */
	
	if ( getenv("LANG") )
	{
		if ( strstr(getenv("LANG"), "german") )
			language=DEUTSCH;
	}

	if ( getenv("LC_ALL") )
	{
		if ( strstr(getenv("LC_ALL"), "german") )
			language=DEUTSCH;
	}
	
	if ( getenv("LC_MESSAGES") )
	{
		if ( strstr(getenv("LC_MESSAGES"), "german") )
			language=DEUTSCH;
	}

	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);

    parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage1);
	
	/* setvbuf(stdin,NULL,_IONBF,0); */
	save_console_attribs();
	set_console_attribs();

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "q", opts, NULL);
		if (c == -1)
			break;
		switch (c)
		{
		case -1:
		case 0:
			break;
		case '?':
			usage(1,No_Option[language]);
			break; /* NOTREACHED */
		case 'q':
			option_q=TRUE;
			break;
		}
	}

	if (optind==argc) {
		usage(1,NULL);
	}

	for (; optind < argc; optind++)
	{
		if ( load_text(argv[optind], &result) )
		{
			lineedit();
			if ( changes )
				save_text(argv[optind], &result);
		}
	}

	return (NO_ERROR);

}	/* main */


/*	##### lineedit.c ###################################################	*/
