/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef TOM_IO_MODEM_H
#define TOM_IO_MODEM_H

#include <stdarg.h>

#ifndef TOM_ATTRIB_PRINTF
/* offline includes this file, and we cannot include system.h since offline
 * uses another system.h
 */
#define TOM_ATTRIB_PRINTF(x,y)
#endif

#ifndef C_UP
/*
    Offline-intern normierte Sondertastencodes
*/
#define C_UP        512
#define C_DOWN      513
#define C_LEFT      514
#define C_RIGHT     515
#define C_PGDN      516
#define C_PGUP      517
#define C_HOME      518
#define C_END       519
#define C_CENTER    520
#define C_HELP      521
#define C_UNDO      522
#define C_PRINT     523

#define C_F1        530
#define C_F2        531
#define C_F3        532
#define C_F4        533
#define C_F5        534
#define C_F6        535
#define C_F7        536
#define C_F8        537
#define C_F9        538
#define C_F10       539
#define C_F11       540
#define C_F12       541
#endif


/*
 * Modemroutinen.
 */
void init_modem(int fd);
void hangup_modem(int fd);
void takeoff_modem(int fd);
void answer_modem(int fd);
void check_modem(int fd);
void dial_modem(int fd, const char *nummer);

#define CARRIER_ERROR   0
#define	CARRIER_LOGIN   1
#define CARRIER_SPEC    2
	/* liefert einen der obigen Werte zurck und setzt einiges akt_connection */
int warte_auf_carrier(int fd);

/* 
 * Highlevel-IO
 */
/* String der vor allen Ausgaben ausgegeben wird. Used by smtp code. */
extern const char *mb_output_prefix;

int  mboutput(const char *daten);

int  mbprintf(const char *format,...) TOM_ATTRIB_PRINTF(1,2);

int mbvprintf(const char *format, va_list param) TOM_ATTRIB_PRINTF(1,0); 

void mbfehler(const char *s);
void mbinput(char *textS,char *restS);
char * mbgetline(ULONG timeout_sec, BOOLEAN *timed_out, BOOLEAN with_echo);
char * mbrlquestion(const char *prompt, const char *vorgabe, char hint);

BOOLEAN mbquestion(size_t laengeL,int mode,BOOLEAN linie,const char *frage,
					const char *vorgabe,char *text);
/* linie ziehen */
#define MBF_LINE 1 
/* alles in Grossbuchstaben wandeln */
#define MBF_TOUPPER 2 
/* alles in Kleinbuchstaben wandeln */
#define MBF_TOLOWER 4 
/* Zeichen als * echoen */
#define MBF_STAR 8 
/* kein Linefeed hinterherschieben */
#define MBF_NOLF 16
/* nicht in Protokolldatei schreiben */
#define MBF_NOPROT 32
/* strtrim auf das Ergebnis anwenden */
#define MBF_TRIM 64

BOOLEAN mbfrage(size_t laengeL,int flags,const char *frage,char *text);

BOOLEAN userbreak(void);
void userglocke(void);
BOOLEAN wait_mehr(void);
int mbkey(void);

void online_protokoll_close(void);
BOOLEAN befehls_umlenkung(const char *datei);

	/* nicht ganz so highlevel, aber ... */
void tprint_fd(int fd, const char *cap);

void get_connection_quality(const char *, const char *);

/*
 * Definition der Scrollzeilen in Abhngigkeit vom Terminaltyp.
 */
#define SCROLLZEILEN (Max(vtmodeW==T_ASCII ? anz_lineW : anz_lineW-5,10))


/*
 * Systemabhngige Routinen
 */
/* Zeichenkette ans Modem senden und Echo verwerfen */
void send_modem(int fd, const char *befehl);
/* Maximal 3 Sekunden auf OK warten */
BOOLEAN modem_ok(int fd);
/* komplette Zeile vom Modem lesen */
char * modem_readline(int fd, ULONG timeout_sec, BOOLEAN *timed_out);
int mbchar(void);
#endif
