/* msg.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>

#ifdef OFFLINE2
#include "offline/inc/version.h"
#ifdef OFFLINE_VERSION
#include "offline/inc/mdb_global.h"
#else
#include "offline/struktur.h"
#include "offline/mdb_glob.h"
#endif
#else
#include "offline/struktur.h"
#include "offline/mdb_glob.h"
#endif

#include "prototyp.h"
#include "modem.h"
#include "user.h"
#include "brett.h"
#include "msg.h"
#include "befehle.h"			/* wg. bef_kopieren */
#include "bin_msg.h"
#include "routing.h"
#include "box_conf.h"
#include "tompaths.h"
#include "filelock.h" /* libquark */

extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;
extern long max_brettnr;


/* =========================  Message-Handling  ========================= */

/*
	MSG-ID erzeugen

	*muss* leider dem Format der langen Maus-IDs entsprechen, sonst 
	funktioniert die bergabe der Nachrichten zur Maus nicht.
*/
void make_msg_id(char *id,BOOLEAN pm, const char *mtzeit, time_t *tt_zeit)
{
	time_t zeit;
	char buf[20];
	const char *zp;
	long mnk_sys=get_new_id_nr(FALSE);

	if (mtzeit)
		zp=mtzeit;
	else
	{
		zp=buf;
		if (tt_zeit && *tt_zeit)
			time_t_to_mtzeit(tt_zeit,buf);
		else
		{
			time(&zeit);
			time_t_to_mtzeit(&zeit,buf);
		}
	}

	sprintf(id,"%s.%c%lu@%s", zp, pm ? 'p' : 'a', mnk_sys, box_conf.msgid_domain);

} /* make_msg_id */


/*
	darf der aktuelle User eine Binrdatei lesen
	(LESEN, INHALT, SUCHEN, DOWNLOAD, VERBOSE)

	15.02.93 Frank
*/
BOOLEAN bin_read_access(const BIN_MSG *bin_info)
{
	BRETTNR bnr=bin_info->brettnr;
	if (bin_info->status != BIN_FREE)
		return FALSE;
	if (bnr)
	{
		if (bnr > max_brettnr) /* paranoia */
			return FALSE;
		if (br_ind[bnr].brett && br_ind[bnr].brett->flags & B_READ)
			return TRUE;
	}
	else
	{
		/* paranoia check */
		if (brettart(brettnrW)!=BINAER_BRETT)
			return FALSE;
		if (user.nummer == bin_info->empfaenger)
			return TRUE;
	}
	return FALSE;

} /* bin_read_access */



/*
	darf der aktuelle User eine Binrdatei ndern
	(EDIT, ...)

	18.02.93 Frank
*/
BOOLEAN bin_change_access(const BIN_MSG *bin_info)
{
	BRETTNR bnr=bin_info->brettnr;

	/* Absender darf immer */
	if (user.nummer == bin_info->usernr)
		return TRUE;
	if (bnr==0)
		return (user.nummer == bin_info->empfaenger);
	if (!br_ind[bnr].brett)
		return FALSE;
	/* im Programmfach nur der Empfnger */
	if (brettart(bnr)!=BINAER_BRETT)
		return FALSE;
	if (level()==9)
		return TRUE;
	if (!(br_ind[bnr].brett->flags & B_READ))
		return FALSE;
	if (br_ind[bnr].brett->grp_chef==user.nummer)
		return TRUE;

	return FALSE;

} /* bin_change_access */



BOOLEAN 
msg_change_access(void)
{
	/* Tja, das mte eigentlich auf die aktuelle Nachricht bezogen
	 * werden ...
	 */
	if (brettnrW && (user.nummer == akt_brett.grp_chef || level()==9))
		return TRUE;
	else
		return FALSE;
}


static void
msg_clean_message_intern(BOOLEAN force,message_t *head)
{
	*head->msg_id='\0';
	*head->ref_id='\0';
	*head->absender='\0';
	*head->sender='\0';
	*head->reply_to='\0';
	*head->betreff='\0';
	*head->eingabezeit='\0';
	head->eingabezeit_tt=0;
	*head->bearbeitungsstatus='\0';
	*head->organisation='\0';
	*head->realname='\0';
	head->distribution_byte='\0';
	head->kopienzahl=0;
	head->junk_group=(BRETTNR) 0;
	head->msgtyp=false_msg;
	head->importzeit=(time_t) 0;
	if (force)
	{
		head->gruppen=NULL;
		head->followup=NULL;
		head->comments=NULL;
		head->uo_comments=NULL;
		head->to=NULL;
		head->body.buf=NULL;
		head->body.len=0;
		head->body.physlen=0;
		head->body.bufsize=0;
		head->body.next=head->body.buf;

		head->mimeinfo=NULL;
		head->fname=NULL;
		head->gateway_zeile=NULL;
		head->path_zeile=NULL;
		head->flags_zeile=NULL;
	}
	else
	{
		simple_free_chain(head->gruppen);
		head->gruppen=NULL;
		simple_free_chain(head->comments);
		head->comments=NULL;
		simple_free_chain(head->followup);
		head->followup=NULL;
		simple_free_chain(head->to);
		head->to=NULL;
		simple_free_chain(head->uo_comments);
		head->uo_comments=NULL;
		/* Wenn schon eine Nachricht gelesen wurde und ein Puffer
		 * existiert, dann auf Leerstring setzen. 
		 */
		if (head->body.buf)
		{
			*head->body.buf='\0';
			head->body.len=0;
			head->body.physlen=0;
			head->body.next=head->body.buf;
		}
#define DYN_CLEAN(ele) if (head->ele) {free(head->ele); head->ele=NULL;}
		DYN_CLEAN(fname);
		DYN_CLEAN(mimeinfo);
		DYN_CLEAN(gateway_zeile);
		DYN_CLEAN(path_zeile);
		DYN_CLEAN(flags_zeile);
	}
}

void
msg_clean_message(message_t *head)
{
	msg_clean_message_intern(FALSE,head);
}

message_t *
msg_alloc(void) 
{
	message_t *head;
	head=XMALLOC(sizeof(message_t));
	msg_clean_message_intern(TRUE,head);
	return head;
}

void
msg_free(message_t *msg)
{
	if (msg)
	{
		/* freigeben, was freigegeben werden kann */
		msg_clean_message_intern(FALSE,msg);
		if (msg->body.buf)
#ifdef MSG_MMAP
			large_free(msg->body.buf);
#else
			free(msg->body.buf);
#endif
		free(msg);
	}
}
