/* filter.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Filter-Routinen
	(c) 1992-94 bei Frank Baumgart

	begonnen am 28.04.1992
	Version vom 21.11.1994
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>

#include "system.h"
#include "struktur.h"
#include "tool.h"
#include "message.h"
#include "screen/io.h"
#include "filter.h"
#include "mdb_conf.h"

extern CONFIG config;

static size_t autoren = 0,		/* Anzahl zu filternder Autoren */
				  themen = 0;		/* und Themen */

static char *aut_text,			/* Autorenfiltertext = Datei */
				*thm_text;			/* dito Themenfiltertext */

static struct FILTER
{
	CRC crc;
	char *name;

} *filter;



/*
	Autor- und Themenfilter laden

	<-			TRUE = ok, FALSE = Fehler

	!!! spter einmal mit Parameter zur Angabe des Filterpfades bzw.
	der Usernummer
*/
BOOLEAN lade_filter(void)
{
	char datei[PATH_MAX];
	FILE *flt_f;
	char zeile[MAX_CPG];
	BOOLEAN ok = TRUE;

	autoren = 0;
	themen = 0;

	filter = NULL;
	aut_text = NULL;
	thm_text = NULL;

	/*
		Autorenfilterdatei verarbeiten
		(auf size_t Gre beschrnkt)

		- einlesen, Anzahl der Eintrge zhlen, Gre feststellen
		- auswerten, Zeiger und CRCs aufbauen, Stringumwandlung
	*/
	strcpy(datei,config.messages);
	strcat(datei,"filter.aut");
	flt_f = fopen(datei,"r");
	if (flt_f)
	{
		size_t bytes;
		size_t entries = 0;

		bytes = (size_t) filesize(flt_f);
		while (fgets(zeile,MAX_CPG,flt_f))
		{
			/* Leerzeilen zhlen nicht */
			if (*zeile != '\r' && *zeile != '\n')
				entries++;
			else
				bytes -= strlen(zeile);
		}

		/* enthlt die Autorenfilterdatei berhaupt Eintrge? */
		if (entries)
		{
			size_t len;
			char *pos;

			filter = (struct FILTER *) malloc(entries * sizeof(struct FILTER));
			if (filter == NULL)
			{
				ok = FALSE;
				goto no_mem;
			}

			if ((aut_text = (char *) malloc(bytes)) == NULL)
			{
				ok = FALSE;
				goto no_mem;
			}

			pos = aut_text;

			rewind(flt_f);
			while (fgets(pos,MAX_CPG,flt_f))
			{
				char *crlf;

				if ((crlf = strpbrk(pos,"\r\n")) != NULL)
					*crlf = 0;
				len = strlen(pos);
				/* Leerzeilen zhlen nicht */
				if (!len)
					continue;
				pos[len] = 0;
				filter[autoren].name = pos;
				filter[autoren].crc  = crc32ansi(pos,len);
				pos += len + 1;
				autoren++;
			}
		}
no_mem:
		fclose(flt_f);
	}


	/*
		die Verwaltung des Themenfilters ist wegen der zustzlichen
		Zeitangabe etwas komplizierter, darum die getrennte Abarbeitung
		Zu alte Angaben werden nmlich entfernt (FILTER)
	*/
	/*
		Themenfilterdatei verarbeiten
		(auf size_t Gre beschrnkt)

		- einlesen, Anzahl der Eintrge zhlen, Gre feststellen
		- zu alte Eintrge entfernen
		- auswerten, Zeiger und CRCs aufbauen, Stringumwandlung
	*/
	strcpy(datei,config.messages);
	strcat(datei,"filter.thm");
	flt_f = fopen(datei,"r");
	if (flt_f)
	{
		size_t bytes;
		size_t entries = 0;

		bytes = (size_t) filesize(flt_f);
		while (fgets(zeile,MAX_CPG,flt_f))
		{
			/* Leerzeilen zhlen nicht */
			if (*zeile != '\r' && *zeile != '\n')
				entries++;
			else
				bytes -= strlen(zeile);
		}

		/* enthlt die Themenfilterdatei berhaupt Eintrge ? */
		if (entries)
		{
			size_t len;
			char *pos;

			filter = (struct FILTER *) malloc(entries * sizeof(struct FILTER));
			if (filter == NULL)
			{
				ok = FALSE;
				goto no_mem2;
			}

			if ((thm_text = (char *) malloc(bytes)) == NULL)
			{
				ok = FALSE;
				goto no_mem2;
			}

			pos = thm_text;

			rewind(flt_f);
			while (fgets(pos,MAX_CPG,flt_f))
			{
				char *crlf;

				if ((crlf = strpbrk(pos,"\r\n")) != NULL)
					*crlf = 0;
				len = strlen(pos);
				/* Leerzeilen zhlen nicht */
				if (!len)
					continue;
				filter[themen].name = pos;
				filter[themen].crc  = crc32ansi(pos,len);
				pos += len + 1;
				themen++;
			}
		}
no_mem2:
		fclose(flt_f);
	}

	if (autoren)
	{
		if (autoren > 1)
			mprintf("%ld Autoren werden gefiltert\r\n",(long)autoren);
		else
			mputs("1 Autor wird gefiltert");
	}
	if (themen)
	{
		if (themen > 1)
			mprintf("%ld Themen werden gefiltert\r\n",(long)themen);
		else
			mputs("1 Thema wird gefiltert");
	}

	if (!ok)
	{
		mputs("Fehler beim Laden der Filter");
		mputs("Outfile wird ohne Filterung importiert");
	}

	return ok;

} /* lade_filter */



/*
	Filterspeicher wieder freigeben
*/
void verwerfe_filter(void)
{
	if (!filter)  { free(filter); filter = NULL; }
	if (!aut_text)  { free(aut_text); aut_text = NULL; }
	if (!thm_text)  { free(thm_text); thm_text = NULL; }

} /* verwerfe_filter */



/*
	feststellen, ob eines der Filterkriterien (Autor oder Thema)
	auf die aktuelle Mail zutrifft

	->			art	:	Art der Filterung (AUTOR oder TITEL)
				crc	:	CRC ber Filtertext
				name	:	Name des Filtertextes

	<-			TRUE = trifft zu (= Msg. filtern) , FALSE = nicht filtern
*/
BOOLEAN msg_filter(int art,CRC crc,const char *name)
{
	size_t nr = 0;
	size_t len;

	/* Abkrzung */
	if ((art == AUTOR && !autoren)  ||  (art == TITEL && !themen))
		return FALSE;

	/* Unterschiede von Zeile zu String ausbgeln */
	if (strpbrk(name,"\r\n"))
		len = strcspn(name,"\r\n");
	else
		len = strlen(name);

	if (art == AUTOR)
	{
		while (nr < autoren)
		{
			/*
				berprfung (nacheinander, da short-circuit-evaluation)
				auf CRC, Stringlnge und Stringinhalt
			*/
			if (filter[nr].crc == crc
			&&  strlen(filter[nr].name) == len
			&&  !strncmp(filter[nr].name,name,len))
				return TRUE;
			nr++;
		}
	}

	if (art == TITEL)
	{
		while (nr < themen)
		{
			/*
				berprfung (nacheinander, da short-circuit-evaluation)
				auf CRC, Stringlnge und Stringinhalt
			*/
			if (filter[nr].crc == crc
			&&  strlen(filter[nr].name) == len
			&&  !strncmp(filter[nr].name,name,len))
			{
				time_t t = time(NULL);

				/* neue Datumsinformation eintragen */
				strftime(filter[nr].name - 9,8,"%d.%m.%y",localtime(&t));
				return TRUE;
			}
			nr++;
		}
	}

	return FALSE;

} /* msg_filter */



/*
	feststellen, ob die aktuelle CRC mit einer der Filterkriterien
	(Autor oder Thema) bereinstimmt

	->			art	:	Art der Filterung (AUTOR oder TITEL)
				crc	:	CRC ber Filtertext

	<-			TRUE	:	trifft zu (gleiche CRC)
				FALSE	:	nicht filtern

	Im Gegensatz zur Funktion msg_filter() kann diese Routine keine 
	100%ige bereinstimmung garantieren sondern nur ausschlieen.
	Diese Funktion ist in erster Linie fr Indexdatenstze gedacht, 
	wobei i.d.R. keine Informationen aus der Datendatei vorhanden 
	sind.
*/
BOOLEAN msg_crc(int art,CRC crc)
{
	size_t nr = 0;

	if (art == AUTOR)
	{
		/* berprfung auf CRC */
		while (nr < autoren)
			if (filter[nr++].crc == crc)
				return TRUE;
	}
	else
	if (art == TITEL)
	{
		/* berprfung auf CRC */
		while (nr < themen)
			if (filter[nr++].crc == crc)
				return TRUE;
	}

	return FALSE;

} /* msg_crc */



/*
	Autor oder Thema in die Filterdatei aufnehmen

	->			art	:	AUTOR oder TITEL
				name	:	Zeiger auf Namen des Autors bzw. Titels
				zeit	:	Zeitpunkt der Msg. (fr Themen)
*/
void add_filter(int typ,const char *name,time_t zeit)
{
	char datei[PATH_MAX];
	FILE *flt_f;

	strcpy(datei,config.messages);
	if (typ == AUTOR)
		strcat(datei,"filter.aut");
	else
		strcat(datei,"filter.thm");

	/* Filterdatei zum Lesen und Anhngen (er)ffnen */
	flt_f = fopen(datei,"a+");
	if (flt_f)
	{
		time_t t = time(NULL);
		char datum[10];
		const char *eol;

		/* bei Thema auch Zeit/Datum speichern */
		if (typ == TITEL)
		{
/* ### offen ist noch, welche Zeit/Datum: Mail oder aktuelle */
/* MAILdatum ! evtl. wird die Msg. ja erst eine Woche spter gelesen */
			{
/*				time_t time;
				struct tm timeptr;
				timeptr.tm_sec  = 0;
				timeptr.tm_min  = xx;
				timeptr.tm_hour = xy;
				timeptr.tm_mon  = 0;
				timeptr.tm_year = xy - 1900;
				time = mktime(&timeptr);*/
			}
			/* Datumsinformation (noch aktuelles Datum) eintragen */
			strftime(datum,9,"%d.%m.%y",localtime(&t));
			datum[8] = ' ';
			datum[9] = ' ';

			fwrite(datum,9,1,flt_f);

			strftime(datum,9,"%d.%m.%y",localtime(&zeit));
		}

		eol = strpbrk(name,"\r\n");
		if (eol && (size_t)(eol-name) < strlen(name))
			fwrite(name,(size_t) (eol - name),1,flt_f);
		else
			fputs(name,flt_f);
		fputs("\r\n",flt_f);
		fclose(flt_f);
	}
} /* add_filter */
