/* kontroll.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Datenbank-Kontrolle
	(c) 1992-95 bei Frank Baumgart

	begonnen am 26.05.1992
	Version vom 27.01.1995
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>

#include "system.h"
#include "mdb_conf.h"
#include "struktur.h"
#include "mdb_intr.h"
#include "mdb_glob.h"
#include "message.h"
#include "screen/io.h"
#include "tool.h"
#include "fehler.h"
#include "puffer.h"

extern CONFIG config;				/* globale Einstellungen */



static BOOLEAN mdb_check_area(long *defekt,BOOLEAN pm,BOOLEAN correct);
static BOOLEAN check_db(const char *dat_name,const char *ind_name,BOOLEAN correct);



/*
	alle Gruppen oder User berprfen

	->		<-	defekt	:	Anzahl defekter Gruppen
				pm			:	User (TRUE) oder Gruppe (FALSE)
				correct	:	TRUE = Fehler auch korrigieren

	<-			TRUE		:	alle ok
				FALSE		:	mind. 1 Gruppe defekt
*/
static BOOLEAN mdb_check_area(long *defekt,BOOLEAN pm,BOOLEAN correct)
{
	BOOLEAN ok = TRUE;
	long anzahl = 0;
	char gruppenname[MAX_CPG];
	char dat_pfad[PATH_MAX],
	     ind_pfad[PATH_MAX];

	if (!open_grplst(pm))		/* alle PM-Gruppen berprfen */
		return FALSE;
	while (read_grplst(gruppenname))
	{
		if (group_path(gruppenname,dat_pfad,pm) < 0)
			continue;
		strcpy(ind_pfad,dat_pfad);
		strcat(ind_pfad,"ind");
		strcat(dat_pfad,"dat");

		mprintf("%s ",gruppenname);
		if (check_db(dat_pfad,ind_pfad,correct))
			mputs(" ok");
		else
		{
			mputs("defekt");
			ok = FALSE;
			anzahl++;
		}
	}
	close_grplst();

	if (defekt)
		*defekt = anzahl;

	return ok;

} /* mdb_check_area */



/*
	alle Gruppen- und User-Datenbanken auf Fehler berprfen bzw. korrigieren

	->			correct	:	TRUE = Fehler auch korrigieren

	<-			TRUE		:	alle ok
				FALSE		:	mind. 1 Gruppe/User defekt
*/
BOOLEAN mdb_check_all(BOOLEAN correct)
{
	BOOLEAN grp_ok = TRUE;
	BOOLEAN usr_ok = TRUE;
	long anzahl = 0;

	usr_ok = mdb_check_area(&anzahl,TRUE,correct);
	if (usr_ok)
		mputs("\nUserdatenbanken fehlerlos\n");
	else
		if (anzahl == 1)
			mputs("\n1 Userdatenbank hat Fehler!\n");
		else
			mprintf("\n%ld Userdatenbanken haben Fehler!\n",anzahl);

	grp_ok = mdb_check_area(&anzahl,FALSE,correct);
	if (grp_ok)
		mputs("\nGruppen fehlerlos\n");
	else
		if (anzahl == 1)
			mputs("\n1 Gruppe hat Fehler!\n");
		else
			mprintf("\n%ld Gruppen haben Fehler!\n",anzahl);

	return (grp_ok && usr_ok);

} /* mdb_check_all */



/*
	berprfung auf Konsistenz der Datenbankstrukturen einer Gruppe

	geprft werden:
	- Indexdateilnge
	- Verweise, also gesamte Kommentarverkettung
	- Positionsangaben
	- Lngenangaben
	- Text-CRC
	- Anzahl neuer und zu lschender Msgs.
	- ID-Text-Position
	- Msg.Text-Position

	->			name		:	Zeiger auf Namen der Gruppe bzw. NULL
				pm			:	TRUE = User, FALSE = Gruppe
				correct	:	Fehler korrigieren

	<-			TRUE		:	DB ok bzw. korrigierte/korrigierbare Fehler
				FALSE		:	DB fehlerhaft
*/
BOOLEAN mdb_check_group(const char *name,BOOLEAN pm,BOOLEAN correct)
{
	char dat_pfad[PATH_MAX];
	char ind_pfad[PATH_MAX];

	if (group_path(name,dat_pfad,pm) < 0)
		return FALSE;
	strcpy(ind_pfad,dat_pfad);
	strcat(ind_pfad,"ind");
	strcat(dat_pfad,"dat");

	return check_db(dat_pfad,ind_pfad,correct);

} /* mdb_check_group */



/*
	Fehlermeldung ausgeben

	->			nr		:	Messagenummer
				text	:	Fehlermeldung
*/
static void mfehler(long nr,const char *text)
{
	mprintf("Msg.%ld: ",nr+1);
	mputs(text);

} /* mfehler */



/*
	->				dat_name	:	Zeiger auf Pfad+Dateinamen der dat-Datei
					ind_name	:	dito ind-Datei
					correct	:	Fehler korrigieren, falls mglich

	<-				TRUE		:	DB ok
					FALSE		:	DB defekt

	Fr die berprfung werden sinnvollerweise die Pufferroutinen verwendet
*/
static BOOLEAN
check_db(const char *dat_name,const char *ind_name,BOOLEAN correct)
{
	BOOLEAN ok = TRUE;
	KENNUNG kennung;
	INDEX ix;						/* untersuchte Nachricht */
	INDEX verkettet;				/* Referenznachricht */
	MESSAGE gesamt;				/* Anzahl aller Messages */
	MESSAGE nr = 0;				/* aktuelle Messagenummer */
	MESSAGE neue = 0;				/* # ungelesener Nachrichten */
	MESSAGE deleted = 0;			/* # zu lschender Nachrichten */
	FILE *ind_f;
	FILE *dat_f;
	long next = 0;					/* nchste erwartete Datendatei-Position */
	char buf[512];					/* ID-Text-Puffer */

	if ((ind_f = fopen(ind_name,BINREAD)) == NULL)
	{
		/*
			eine nicht vorhandene "ind"-Datei kann auch eine leere Gruppe sein, 
			dann allerdings darf auch keine "dat" existieren.
		*/
		if ((dat_f = fopen(dat_name,"r")) != NULL)
		{
			mputs("dat ohne ind");
			fclose(dat_f);
			return FALSE;
		}
		return TRUE;
	}
	/* Gre der Indexdatei bzgl. Datenstrukturgre berprfen */
	if ((filesize(ind_f) - sizeof(KENNUNG)) % sizeof(INDEX))
	{
		fclose(ind_f);
		return FALSE;
	}
	fclose(ind_f);

	/* Indexdatei zum Lesen und Schreiben ffnen */
	if (!i_open(ind_name))
		return FALSE;

	/* enthlt die Gruppe berhaupt Nachrichten ? */
	gesamt = i_anzahl();
	if (!gesamt)
	{
		i_close(FALSE);

		/* leere Gruppen sind auch ok */
		return TRUE;
	}

	/* Datenbasis zum Lesen ffnen */
	if ((dat_f = fopen(dat_name,BINREAD)) == NULL)
	{
		i_close(FALSE);
		fehler(DAT_OPEN);
		return FALSE;
	}

	/* Kennung einlesen */
	i_read(&kennung,sizeof(KENNUNG));
	if (kennung.anzahl != gesamt)
	{
		mputs("");
		mputs("Anzahl Messages differiert");
		mprintf("Kennung: %u\n",kennung.anzahl);
		mprintf("real   : %u\n",gesamt);
		if (correct)
		{
			mputs("[korrigiert]");
			i_rewind();
			kennung.anzahl = gesamt;
			(void) i_write(&kennung,sizeof(KENNUNG));
			i_goto(0);
		}
		ok = FALSE;
	}

	while (nr < gesamt)
	{
		BOOLEAN fatal;						/* aktuelle Msg. irreparabel ? */
		BOOLEAN changed = FALSE;

		fatal = FALSE;

		/* Indexdatensatz lesen */
		if (!i_read(&ix,sizeof(INDEX)))
		{
			mputs("Einlesen aus Index-Puffer fehlerhaft");
			/* fatal */
			break;
		}

		/* Flags berprfen */
		if (ix.flags & TOTAL)
		{
			if (ix.flags & GESCHUETZT)
			{
				mfehler(nr,"geschtzte Msg. mit Lschflag");
				if (correct)
				{
					/* sicherheitshalber das Lschflag entfernen */
					ix.flags &= ~TOTAL;
					changed = TRUE;
				}
			}
			else
				deleted++;
		}
		if (!(ix.flags & GELESEN))
			neue++;

#if 0
		id (ix.pos > datendateilaenge)
		{
			mfehler(nr,"Indexstruktur zeigt auerhalb Datendatei");
			fatal = TRUE;
		}
#endif
		/*
			Zusammenhnge zwi. Index und Datendatei prfen
		*/
		if (next != -1 && ix.pos != next)
		{
			mfehler(nr,"Loch/berschneidung in der Datenbank");
			mprintf("Erwartete Position %ld, ",next);
			mprintf("laut Indexdatei %ld\r\n",ix.pos);
			fatal = TRUE;
			next = -1;
		}
		else if (fseek(dat_f,ix.pos,SEEK_SET))
		{
			mfehler(nr,"kann Nachricht nicht finden");
			fatal = TRUE;
			next = -1;
		}
		else
		{
			if (fread(buf,1,sizeof(buf),dat_f) < 2)
			{
				mfehler(nr,"kann ID(-Text) nicht lesen");
				fatal = TRUE;
			}
			else if (*buf != '#')
			{
				mfehler(nr,"Nachricht beginnt nicht mit ID-Text");
				fatal = TRUE;
			}
			else
			{
				size_t id_len;
				size_t crc;

				id_len = strcspn(buf+1,"\r\n");
				crc = crc32ansi(buf+1,id_len);
				if (crc != ix.id_crc)
				{
					mfehler(nr,"ID-CRC falsch");
					fatal = TRUE;
				}
			}

			if (ix.flags & XP_REF)
#ifdef REFS_WITH_HEADER
				next = ix.pos + ix.t_text;
#else
				next = ix.pos;
#endif
			else
				next = ix.pos + ix.len;

			/* ':' suchen, sofern nicht kodiert oder XP-Referenz */
			if (!(ix.flags & (CODIERT | XP_REF)))
			{
				if (fseek(dat_f,ix.pos + ix.t_text,SEEK_SET))
				{
					mfehler(nr,"kann Nachrichtentext nicht finden");
					fatal = TRUE;
				}
				else
				if (fgetc(dat_f) != ':')
				{
					mfehler(nr,"Nachrichtentext beginnt nicht mit Doppelpunkt");
					fatal = TRUE;
				}
			}
		}


		/*
			Kommentarverkettung berprfen
		*/
		if (ix.oben != LEER)
		{
			if (ix.oben > gesamt)
			{
				mfehler(nr,"oben falscher Verweis");
				if (correct)
				{
					ix.oben = LEER;
					changed = TRUE;
				}
				ok = FALSE;
			}
			else
			{
				/* verkettete Nachricht untersuchen */
				i_goto(ix.oben);
				i_read(&verkettet,sizeof(INDEX));
				if ((verkettet.kommentare == 1) && (verkettet.unten != nr))
				{
					mfehler(nr,"oben falsche Verkettung");
					mprintf("Kommentierte Nachricht verweist auf %ld statt %ld\r\n",
					        (long) verkettet.unten, (long) nr);
					if (correct)
					{
						ix.oben = LEER;
						changed = TRUE;
					}
				}
			}
		}
		if (ix.unten != LEER)
		{
			if (ix.unten > gesamt)
			{
				mfehler(nr,"unten falscher Verweis");
				if (correct)
				{
					ix.unten = LEER;
					changed = TRUE;
				}
				ok = FALSE;
			}
			else
			{
				/* verkettete Nachricht untersuchen */
				i_goto(ix.unten);
				i_read(&verkettet,sizeof(INDEX));
				if (verkettet.oben != nr)
				{
					mfehler(nr,"unten falsche Verkettung");
					if (correct)
					{
						ix.unten = LEER;
						changed = TRUE;
					}
					ok = FALSE;
				}
			}
		}
		if (ix.links != LEER)
		{
			if (ix.links > gesamt)
			{
				mfehler(nr,"links falscher Verweis");
				if (correct)
				{
					ix.links = LEER;
					changed = TRUE;
				}
				ok = FALSE;
			}
			else
			{
				/* verkettete Nachricht untersuchen */
				i_goto(ix.links);
				i_read(&verkettet,sizeof(INDEX));
				if (verkettet.rechts != nr)
				{
					mfehler(nr,"links falsche Verkettung");
					if (correct)
					{
						ix.links = LEER;
						changed = TRUE;
					}
					else
						ok = FALSE;
				}
			}
		}
		if (ix.rechts != LEER)
		{
			if (ix.rechts > gesamt)
			{
				mprintf("Msg.%ld: ",(long) nr+1);
				mputs("rechts falscher Verweis");
				if (correct)
				{
					ix.rechts = LEER;
					changed = TRUE;
				}
				ok = FALSE;
			}
			else
			{
				/* verkettete Nachricht untersuchen */
				i_goto(ix.rechts);
				i_read(&verkettet,sizeof(INDEX));
				if (verkettet.links != nr)
				{
					mfehler(nr,"rechts falsche Verkettung");
					if (correct)
					{
						ix.rechts = LEER;
						changed = TRUE;
					}
					ok = FALSE;
				}
			}
		}

		if (fatal)
		{
			ok = FALSE;

			if (correct)
			{
				/* aktuelle Nachricht als defekt kennzeichnen */
				ix.flags |= DEFEKT | TOTAL;
				ix.flags &= ~GESCHUETZT;
				changed = TRUE;
				deleted++;
			}
		}

		if (correct && changed)
		{
			i_goto(nr);
			(void) i_write(&ix,sizeof(INDEX));
		}

		nr++;
		i_goto(nr);
	}

	if (neue != kennung.neue)
	{
		mputs("");
		mputs("unterschiedliche Anzahl neuer Msgs.");
		mprintf("kennung: %lu\n",(ULONG) kennung.neue);
		mprintf("real   : %lu\n",(ULONG) neue);

		if (correct)
		{
			mputs("[korrigiert]");
			kennung.neue = neue;
		}
	}

	if (deleted != kennung.putzen)
	{
		mputs("");
		mputs("unterschiedliche Anzahl zu lschender Msgs.");
		mprintf("kennung: %lu\n",(ULONG) kennung.putzen);
		mprintf("real   : %lu\n",(ULONG) deleted);

		if (correct)
		{
			mputs("[korrigiert]");
			kennung.putzen = deleted;
		}
	}

	/* Zeitpunkt der letzten berprfung festhalten */
	kennung.tested = time(NULL);
	i_rewind();
	(void) i_write(&kennung,sizeof(KENNUNG));

	fclose(dat_f);
	i_close(TRUE);

	return ok;

} /* check_db */
