/* mt_cmd.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Auswertung der MausTausch-Prozessor Kommandos

	(c) 1992-95 bei Frank Baumgart

	begonnen am 20.12.1992
	Version vom 28.01.1995
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>

#include "system.h"
#include "mt_cmd.h"
#include "mdb_defs.h"
#include "mdb_conf.h"
#include "mdb_glob.h"
#include "screen/io.h"

extern CONFIG config;


static void save_info_crc(const char *zeile);



/*
	Infofile-CRC merken
	(kommt im LOG-Block als Reaktion auf ein I.. Kommando im Infile)

	->			zeile	:	z.B. "IGK=58278 (CRC unverndert)"
*/
static void save_info_crc(const char *zeile)
{
	FILE *crc_f;
	char name[PATH_MAX];
	const char *p;

	if ((p = strchr(zeile,'=')) == NULL)
		return;
	strcpy(name,config.infofiles);
	strcat(name,zeile);
	name[strlen(config.infofiles) + p - zeile] = 0;
	strcat(name,".crc");

	if ((crc_f = fopen(name,"w")) == NULL)
		return;
	fputs(p+1,crc_f);
	fclose(crc_f);

} /* save_info_crc */



/*
	Auswertung des LOG-Blockes
*/
void log_auswertung(FILE *imp_f)
{
	char log[PATH_MAX];
	FILE *log_f;
	char referenz[256];
	char zeile[256];
	int c;

	strcpy(log,config.messages);
	strcat(log,"log");
	log_f = fopen(log,"w");

	while ((c = fgetc(imp_f)) == ':')
	{
		(void) fgets(zeile,255,imp_f);

		switch (*zeile)
		{
			/* allgemeine Mitteilung */
			case '!' :
				continue;
/*				break;*/

			/* folgende Mitteilungen beziehen sich auf diese ID */
			case '#' :
				strcpy(referenz,zeile);
				break;

			/* folgende Angaben beziehen sich auf dieses Kommando */
			case '\"' :
				strcpy(referenz,zeile);
				break;

			/* Mitteilung unter nachfolgendem Namen eingetragen */
			case '=' :
				fputs(referenz,log_f);
				fputs(zeile+1,log_f);
				break;

			/* Fehler aufgetreten */
			case '?' :
				break;

			/* CRC fr ein Infofile */
			case '$' :
				save_info_crc(zeile+1);
				break;

			/* ... */
			case '%' :
				break;

#ifdef DEVELOPER
			default :
				mputs("ungltiges LOG-Kommando");
#endif
		}
	}

	ungetc(c,imp_f);

	if (log_f)
		fclose(log_f);

} /* log_auswertung */



/*
	Auswertung des REN-Blocks

	Dieser liefert die seit dem letzten Tausch erfolgten Gruppenumbenennungen
*/
void ren_auswertung(FILE *imp_f)
{
	char oldname[MAX_CPG];
	char newname[MAX_CPG];
	char zeile[MAX_CPG];
	char *p;
	int c;

	*oldname = *newname = 0;

	while ((c = fgetc(imp_f)) == ':')
	{
		c = fgetc(imp_f);
		(void) fgets(zeile,sizeof(zeile),imp_f);

		/* alles auer 'O' und 'N' (z.Z. 'D'atumsangabe) ignorieren */
		if (c != 'O' && c != 'N')
			continue;

		if ((p = strpbrk(zeile,"\r\n")) != NULL)
			*p = 0;

		if (c == 'O')
			strcpy(oldname,zeile+1);
		else
		if (c == 'N')
			strcpy(newname,zeile+1);

		if (*oldname && *newname)
		{
			mdb_rename_group(oldname,newname,FALSE);
			*oldname = *newname = 0;
		}
	}

	ungetc(c,imp_f);

} /* ren_auswertung */



/*
	Auswertung des HEAD-Blocks

	Dieser liefert Angaben ber die Box sowie das nachfolgende Outfile
*/
void head_auswertung(FILE *imp_f)
{
	char head[PATH_MAX];
	FILE *head_f;
	char zeile[256];
	int c;

	strcpy(head,config.messages);
	strcat(head,"head");
	head_f = fopen(head,"w");

	while ((c = fgetc(imp_f)) == ':')
	{
		(void) fgets(zeile,255,imp_f);

		if (head_f)
			fputs(zeile,head_f);

		switch (*zeile)
		{
			/* Tauschversion */
			case 'T' :
			/* Boxversion */
			case 'V' :
			/* Boxtyp (zZ. Maus, Quark oder Madness) */
			case 'B' :
			/* Netzkennung (ohne Domain dahinter) */
			case 'I' :
			/* Tauschdatum (JJJJMMDDHHMM) */
			case 'D' :
				break;

			/* Username */
			case 'U' :
#if 0
				if ((p = strpbrk(zeile,"@\r\n")) != NULL)
					*p = 0;
				while (strlen(zeile) && zeile[strlen(zeile)-1] == ' ')
					zeile[strlen(zeile)-1] = 0;
				strcpy(head_username,zeile);
#endif
				break;

			/*
				Sortierinfo (optional)

				U -> Alle PMs fr einen Benutzer sind am Stck hintereinander
				G -> Alle Nachrichten fr eine Gruppe sind am Stck hintereinander
				R -> PMs sind vor ffentlichen Nachrichten.
			*/
			case 'S' :
				continue;

#ifdef DEVELOPER
			default :
				mputs("unbekannte HEAD-Zeile");
#endif
		}
	}

	ungetc(c,imp_f);

	if (head_f)
		fclose(head_f);

} /* head_auswertung */



/*
	Zeile aus HEAD-Block auslesen

	->			typ			:	Zeilenkenn-Zeichen
			<-	eintrag		:	Puffer fr Zeile (ohne fhrendes Typzeichen),
									optional NULL, falls nur Funktionswert interessiert
				keep_open	:	head-Datei fr weitere Zugriffe geffnet halten
									fr nachfolgende Zugriffe TRUE/FALSE

	<-			TRUE			:	Eintrag vorhanden
				FALSE			:	kein Eintrag mit "typ" vorhanden

	Das Zeilenende (z.B. CR/LF) wird vor der Rckgabe entfernt
*/
BOOLEAN head_getinfo(char typ,char *eintrag,BOOLEAN keep_open)
{
	static FILE *head_f = NULL;
	char head[PATH_MAX];
	char zeile[256];
	BOOLEAN found = FALSE;
	char *p;

	if (head_f)
		rewind(head_f);
	else
	{
		strcpy(head,config.messages);
		strcat(head,"head");
		if ((head_f = fopen(head,"r")) == FALSE)
			return FALSE;
	}

	while (fgets(zeile,255,head_f))
	{
		if (*zeile == typ)
		{
			if (eintrag)
			{
				strcpy(eintrag,zeile+1);
				if ((p = strpbrk(eintrag,"\r\n")) != NULL)
					*p = 0;
			}
			found = TRUE;
			break;
		}
	}

	if (head_f && !keep_open)
	{
		fclose(head_f);
		head_f = NULL;
	}

	return found;

} /* head_getinfo */



/*
	unbekannten Block auslesen

	->			imp_f	:	Outfile-stream
				buf	:	Puffer von mind. MSG_SIZE Bytes
*/
void skip_block(FILE *imp_f,char *buf)
{
	int c;

	while ((c = fgetc(imp_f)) != EOF && (c != '#'))
		fgets(buf,MSG_SIZE,imp_f);
	ungetc(c,imp_f);

} /* skip_block */
