/* puffer.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Puffer-Routinen
	(c) 1992-94 bei Frank Baumgart

	begonnen am  x.04.1992
	Version vom 07.09.1994
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>				/* file locking */
#include <errno.h>

#include "system.h"
#include "mdb_conf.h"
#include "struktur.h"
#include "mdb_intr.h"
#include "fehler.h"
#include "screen/io.h"
#include "tool.h"
#include "puffer.h"

extern CONFIG config;				/* globale Einstellungen (Verkettung...) */



/*
	Routinen zur transparenten Bearbeitung der Indexdatei
	Ablauf entweder komplett auf der Datei oder komplett im RAM.
	(Die Kennung wird natrlich ebenfalls gepuffert)

	Diese Routinen sind fr die Bearbeitung existierender 
	Indexdateien gedacht und eignen sich nicht zum Anhngen 
	von Indexdatenstzen
*/


static FILE *ind_f = NULL;			/* Indexdatei */

static BOOLEAN index_gepuffert = FALSE;
static char *puffer_start;
static char *index_ptr;
static long ibuf_size;				/* Gre des Puffers */
static BOOLEAN write_back;			/* Puffer nur zurckschreiben */
											/* wenn Schreibzugriffe erfolgten */
static long dirty_start,			/* Anfang und Ende des vernderten */
				dirty_end;				/* Pufferbereiches */



/*
	Indexdatei ffnen und falls mglich im RAM puffern

	->			datei		:	Name der Indexdatei

	<-			Indexdatei konnte geffnet werden (TRUE) oder nicht (FALSE)

	Der Rckgabewert sagt nicht, OB die Indexdatei gepuffert wurde.
	Die Pufferung findet beim Putz, Markieren und Check statt, so da 
	hier auch das Filelocking gemacht wird, das notwendig ist, damit die 
	Gruppe nicht mit den Messagefunktionen geffnet wird.
*/
BOOLEAN i_open(const char *datei)
{
	long bytes;

	write_back = FALSE;
	index_gepuffert = FALSE;

	/* Indexdatei zum Lesen und Schreiben ffnen */
	if ((ind_f = fopen(datei,BIN_RW_OPEN)) == NULL)
	{
		fehler(IND_OPEN);
		return FALSE;
	}

#if defined(__UNIX__) || defined(__MINT__)
	/* file locking */
	{
		struct flock lock;

		lock.l_type = F_WRLCK;
		lock.l_whence = SEEK_SET;
		lock.l_start = lock.l_len = 0;

		if (fcntl(fileno(ind_f),F_SETLKW,&lock) == -1)
		{
			fprintf(stderr,"Locking error: %s\r\n",strerror(errno));
			fclose(ind_f);
			return FALSE;
		}
	}
#endif

	/* Gre der Indexdatei feststellen */
	bytes = filesize(ind_f);

	/* fr Dosen mit 64 KB Speicherblocklimit ist hier Schlu */
	if (bytes != (size_t) bytes)
		return TRUE;

	/* Puffergre setzen */
	ibuf_size = bytes;

	/* Speicher in der bentigten Gre anfordern */
	puffer_start = (char *) malloc((size_t)ibuf_size);
	if (puffer_start)
	{
		/* Indexdatei komplett einlesen */
		fread(puffer_start,(size_t)ibuf_size,1,ind_f);
		index_ptr = puffer_start;
		index_gepuffert = TRUE;

		dirty_start = ibuf_size;
		dirty_end = 0;
	}

	return TRUE;

} /* i_open */



/*
	Indexdatei schlieen

	->			save	:	Daten sichern ja/nein

	Der Parameter "save" sagt ebenso wie der Rckgabewert von i_open 
	nichts ber die Pufferung aus.
	Selbst wenn fr "save" FALSE angegeben wird, so sind ohne Pufferung  
	der Indexdatei ja bereits smtliche Daten geschrieben.
	Der Zweck von "save" ist es, auf das Sichern der Indexdatei verzichten 
	zu knnen, da sie nicht mehr bentigt wird. Dies ist derzeit fr den 
	Putz sinnvoll, da die alte Indexdatei gar nicht mehr bentigt wird.
*/
void i_close(BOOLEAN save)
{
	if (!save)
		write_back = FALSE;

	/* Pufferung und Datei gendert ? */
	if (index_gepuffert)
	{
		if (write_back)
		{
			if (config.opt_cache)
			{
				size_t dirty_len;

				/* nur vernderten Bereich zurckschreiben */
				fseek(ind_f,dirty_start,SEEK_SET);
				dirty_len = (size_t)(dirty_end - dirty_start);
				if (dirty_len)
					if (!fwrite(puffer_start+dirty_start,dirty_len,1,ind_f))
						fehler(WRT_BACK);
			}
			else
			{
				/* kompletten Puffer als Indexdatei zurckschreiben */
				rewind(ind_f);
				if (!fwrite(puffer_start,(size_t)ibuf_size,1,ind_f))
					fehler(WRT_BACK);
			}
		}
		free(puffer_start);
	}

#if defined(__UNIX__) || defined(__MINT__)
	/* release lock */
	{
		struct flock lock;

		lock.l_type = F_UNLCK;
		lock.l_whence = lock.l_start = lock.l_len = 0;

		fcntl(fileno(ind_f),F_SETLK,&lock);
	}
#endif

	/* geschlossen wird die reale Datei in jedem Falle */
	fclose(ind_f);

} /* i_close */



/*
	Auf Startposition zurckfahren
*/
void i_rewind(void)
{
	if (index_gepuffert)
		index_ptr = puffer_start;
	else
		rewind(ind_f);

} /* i_rewind */



/*
	aktuelle Datei/Pufferposition ermitteln
*/
long i_tell(void)
{
	if (index_gepuffert)
		return index_ptr - puffer_start;
	else
		return ftell(ind_f);

} /* i_tell */



/*
	bestimmte Dateiposition anfahren

	->			pos	:	Dateiposition
*/
void i_seek(long pos)
{
	if (index_gepuffert)
		index_ptr = puffer_start + (size_t)pos;
	else
		fseek(ind_f,pos,SEEK_SET);

} /* i_seek */



/*
	bestimmten Indexdatensatz anfahren

	->			nr	:	Nummer des Datensatzes
*/
void i_goto(MESSAGE nr)
{
	long pos = sizeof(KENNUNG) + (long)(nr) * sizeof(INDEX);

	if (index_gepuffert)
		index_ptr = puffer_start + pos;
	else
		fseek(ind_f,pos,SEEK_SET);

} /* i_goto */



/*
	einen Indexdatensatz einlesen

	->			puffer	:	Adresse des Datenpuffers vom Aufrufer
				len		:	Lnge dieses Puffers in Byte

	<-			Anzahl gelesener Bytes, bei Fehlern = 0
*/
size_t i_read(void *puffer,size_t len)
{
	if (index_gepuffert)
	{
		long pos = index_ptr - puffer_start;

		if (pos + len > ibuf_size)
		{
			fehler(MEM_ERR);
#ifdef TEXT_FRONTEND
			fprintf(stderr,"lnge = %ld\n",(long)len);
			fprintf(stderr,"%ld Bytes zuviel\n",(long)(pos + len - ibuf_size));
			fprintf(stderr,"ibuf_size = %ld\r\n",(long) ibuf_size);
			fprintf(stderr,"akt. pos  = %ld\r\n",pos);
			fputs("Datenbank mit Sicherheit defekt\r\n",stderr);
			taste();
#endif
			return 0;
		}
		memcpy(puffer,index_ptr,len);
		index_ptr += len;
		return len;
	}
	else
		return fread(puffer,1,len,ind_f);

} /* i_read */



/*
	einen Indexdatensatz schreiben

	->			puffer	:	Adresse des Datenpuffers vom Aufrufer
				len		:	Lnge dieses Puffers in Byte

	<-			Anzahl geschriebener Bytes, bei Fehlern = 0
*/
size_t i_write(void *puffer,size_t len)
{
	size_t written;

	if (!index_gepuffert)
	{
		written = fwrite(puffer,1,len,ind_f);
		/* r/w Wechsel */
		fseek(ind_f,0,SEEK_CUR);
	}
	else
	{
		/* aktuelle Pufferposition */
		long pos = index_ptr - puffer_start;

		/* berschreitung der Puffergrenze ? */
		if (pos + len > ibuf_size)
		{
			written = 0;
			/*
				fall through mit Update der index-Position, damit 
				die fehlerhafte Position nicht durch die Pufferung 
				verflscht wird und dadurch mglicherweise wieder 
				zurck in den gltigen Bereich gert
			*/
			fputs("Schreiben auf fehlerhafte Indexposition\n",stderr);
		}
		else
		{
			written = len;

			/*
				vergleichen, ob berhaupt Unterschiede zwischem neuen und
				alten Daten bestehen. Wenn bereits neue Daten geschrieben 
				wurden kann der relativ zeitaufwendige Vergleich entfallen
				(short circuit evaluation)
			*/
			if (write_back || memcmp(index_ptr,puffer,len))
			{
				/* dann Speicherbereich kopieren */
				memcpy(index_ptr,puffer,len);

				/* Information ber vernderten Bereich aktualisieren */
				if (dirty_start > pos)
					dirty_start = pos;
				if (dirty_end < pos+len-1)
					dirty_end = pos+len-1;

				/* merken, da ein Schreibzugriff erfolgte */
				write_back = TRUE;
			}
		}
		index_ptr += len;
	}

	if (!written)
		fputs("Fehler beim Index-Schreiben (auerhalb Puffer)\n",stderr);

	return written;

} /* i_write */



/*
	einen Datensatz zurckfahren

	ntzlich um einen gerade gelesenen und vernderten Datensatz an
	derselben Stelle wieder abspeichern zu knnen
*/
void i_back(void)
{
	if (index_gepuffert)
		index_ptr -= sizeof(INDEX);
	else
		fseek(ind_f, - (long)(sizeof(INDEX)),SEEK_CUR);

} /* i_back */



/*
	feststellen, ob das Datei/Pufferende erreicht ist
*/
BOOLEAN i_eof(void)
{
	if (index_gepuffert)
	{
		if (index_ptr >= puffer_start + ibuf_size)
			return TRUE;
		else
			return FALSE;
	}
	else
	{
		if (feof(ind_f))
			return TRUE;
		else
			return FALSE;
	}
} /* i_eof */



/*
	Anzahl der vorhandenen Indexdatenstze zurckgeben
*/
MESSAGE i_anzahl(void)
{
	if (index_gepuffert)
		return (MESSAGE) ((ibuf_size - sizeof(KENNUNG)) / sizeof(INDEX));
	else
	{
		long save_pos;
		MESSAGE anzahl;

		save_pos = ftell(ind_f);
		fseek(ind_f,0,SEEK_END);
		anzahl = (MESSAGE) ((ftell(ind_f) - sizeof(KENNUNG)) / sizeof(INDEX));
		fseek(ind_f,save_pos,SEEK_SET);

		return anzahl;
	}
} /* i_anzahl */
