/* mdb_glob.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	globale Datenbank-Funktionen
*/



BOOLEAN mdb_init(const char *);
int mdb_version(void);



/*
	outfile.txt importieren

	->			outfile	:	Dateiname inkl. Pfad des Outfiles
				box		:	TRUE = Boxmodus, FALSE = FrontEnd-Modus

	<-			TRUE		:	import verlief erfolgreich
				FALSE		:	import schlug fehl
*/
BOOLEAN mdb_import(const char *outfile,BOOLEAN box);



/*
	Infile-Reimport fr FrontEnd-Betrieb
*/
BOOLEAN infile_einfuegen(const char *infile);



/*
	Putzen
	zu lschende und teilzulschende Datenstze in einer Gruppe 
	entfernen und Index- und Datendatei neu aufbauen

	- ein evtl. in der kommentierten Mail gespeicherter Index des 
	  LETZTEN Kommentares wird bereits bearbeitet (?)

	->			name	:	Name der zu putzenden Gruppe
				pm		:	TRUE = PM , FALSE = Gruppe
				immer	:	unabh. von Anzahl zu lschender Mails immer putzen (TRUE)

	<-			FALSE	:	Fehler aufgetreten
				TRUE	:	alles ok

	Fehlermglichkeiten:
	- Datei enthlt berhaupt keine Datenstze
	- zuwenig Speicher fr Verweispuffer
	- zuwenig Platz auf externem Speicher
*/
BOOLEAN mdb_putze_gruppe(const char *name,BOOLEAN pm,BOOLEAN immer, BOOLEAN *dummy);
BOOLEAN mdb_putze_alles(BOOLEAN pm,BOOLEAN immer);
BOOLEAN mdb_markiere(const char *name,BOOLEAN pm,BOOLEAN modus,BOOLEAN zeit,...);
BOOLEAN mdb_markiere_size (const char *name,BOOLEAN pm,size_t max_size);



/*
	globale MsgID-Verwaltung
*/
BOOLEAN msgid_create(void);
BOOLEAN msgid_export(const char *name,BOOLEAN pm,const char *datei);
BOOLEAN msgid_export_all(const char *datei);



/*
	Parameter und Optionen fr Export
	(statt lnglicher Parameterliste)
*/
typedef struct
{
	BOOLEAN box;					/* TRUE = Boxmodus, FALSE = FE-Modus */
	ULONG dist;						/* Distribution (s. message.h) */
	time_t zeit;					/* Start-Zeitpunkt */
	long maximum;					/* max. Outfilegre in Bytes bzw. 0 */

	/* fr Box-Modus: */
	BOOLEAN one_group;			/* nur einen Gruppennamen bei XPs liefern */
	const char *site;				/* Msgs. mit "site" im Path weglassen */
	const char *noidfname;		/* Datei mit IDs nicht zu exportierender Msgs. */
	BOOLEAN stern;              /* '*'-Zeile mitschicken? */
	short status_mode;          /* 0: kein, 1: PSN, 2: PSA */

} EXPORT;

/*
	Gruppen im Outfile-Format exportieren

	->			liste		:	Name+Pfad der Exportlistendatei
				export	:	Name+Pfad der Exportdatei ("Outfile")
				opt		:	Parameter + Optionen

	<-			TRUE		:	Export erfolgreich
				FALSE		:	Fehler

	Die Distribution gibt die gewnschten Nachrichten an, ist dist also 
	V_MAUSNET, werden nur MAUSNET und NET Mails, aber keine lokalen exportiert.

	Im Boxmodus wird an ein evtl. vorhandenes Outfile angehngt.

	Zum einmaligen crossposting-Export werden via dynhash die IDs exportierter 
	Nachrichten in einer Hashtabelle gemerkt.
*/
BOOLEAN mdb_export(const char *liste,const char *export,EXPORT *opt);



/*
	berprfung auf Konsistenz der Datenbankstrukturen einer Gruppe

	->			name		:	Gruppen- bzw. Username
				pm			:	TRUE = PM , FALSE = Gruppe
				correct	:	TRUE = Fehler auch korrigieren

	<-			TRUE = DB ok , FALSE = DB fehlerhaft
*/
BOOLEAN mdb_check_group(const char *name,BOOLEAN pm,BOOLEAN correct);



/*
	alle Gruppen und User berprfen

	->			correct	:	TRUE = Fehler auch korrigieren

	<-			TRUE		:	alle ok
				FALSE		:	mind. 1 Gruppe/User defekt
*/
BOOLEAN mdb_check_all(BOOLEAN correct);



/*
	Gruppe lschen bzw. komplett entfernen

	->			name	:	Name der zu entfernenden Gruppe
				pm		:	Typ, PM (TRUE) oder ffentlich (FALSE)
				keep	:	TRUE = nur lschen, FALSE = komplett entfernen

	<-			TRUE = ok , FALSE = Fehler

	Ablauf:
	- Merken aller gruppenbergreifenden Referenzen
	- Auflsen derselben
	- Lschen der Datendatei
	- Lschen der Indexdatei bzw. Reduktion auf Kennung
	- Entfernen der Gruppe aus der Gruppenlistendatei bei keep = FALSE
*/
BOOLEAN mdb_remove_group(const char *name,BOOLEAN pm);



/*
	neue Gruppe/User anlegen

	->				name	:	Gruppen/Username
					pm		:	TRUE = User, FALSE = Gruppe
					pfad	:	Pfad bzw. NULL fr default

	<-				TRUE	:	ok, Gruppe/User angelegt
					FALSE	:	Fehler
*/
BOOLEAN mdb_create_group(const char *name,BOOLEAN pm,const char *pfad);



/*
	Gruppe zwischen Laufwerken/Pfaden verschieben

	->				name	:	Gruppenname
					pm		:	TRUE = User, FALSE = Gruppe
					dest	:	Zielpfad bzw. NULL

	<-				TRUE	:	ok
					FALSE	:	Fehler

	Wenn als Zielpfad NULL angegeben ist, wird die Gruppe in das Standard-
	gruppenverzeichnis (CNF: GroupPath/UserPath) verschoben
*/
BOOLEAN mdb_move_group(const char *name,BOOLEAN pm,const char *dest);



/*
	Gruppe umbenennen

	->			oldname	:	alter Name der Gruppe
				newname	:	neuer Name der Gruppe
				pm			:	Typ, PM (TRUE) oder ffentlich (FALSE)

	<-			TRUE		:	ok
				FALSE		:	Fehler
*/
BOOLEAN mdb_rename_group(const char *oldname,const char *newname,BOOLEAN pm);



/*
	Datei mit allen Gruppen (opt. +User) in sortierter Form liefern

	->			datei	:	Zieldatei, ein Name je Zeile
				pm		:	TRUE = auch Usernamen (z.Z. nicht ausgewertet)
*/
BOOLEAN mdb_grouplist(const char *datei,BOOLEAN pm,long *anzahl);



/*
	globale ID-Suche

	->			id			:	zu suchender ID-String (darf auf \r\n enden)
			<-	grname	:	Name der Gruppe, die die Nachricht enthlt bzw. NULL
			<-	usname	:	Name des Absenders der Nachricht bzw. NULL
			<-	autor		:	Autor der Nachricht an usname bzw. NULL

	<-			TRUE		:	ID gefunden, ggf. grname/usname/autor initialisiert
				FALSE		:	ID nicht gefunden
*/
BOOLEAN msgid_find_infos(const char *id,char *grname,char *usname,char *autor);




/*
	->			req_list	:	Datei mit ID-Zeilen
				outfile	:	Name der Ausgabedatei

	<-			TRUE		:	Kommando ok
				FALSE		:	Kommando fehlerhaft

	Daten werden an <outfile> angehngt, falls es bereits existiert.
	Das Funktionsergebnis ist unabh. davon, ob berhaupt eine ID gefunden 
	wurde. Wenn man daran interessiert ist, mu man sich das Outfile ansehen.
*/
BOOLEAN mdb_get_article(const char *req_list,const char *outfile, 
	BOOLEAN dummy_for_next_version);



/*
	Msg(s). mit der angegeben ID zum lschen markieren
	(also auch bei crosspostings)

	->			id		:	ID, optional mit \r\n

	->			TRUE	:	Nachricht (bereits) lschmarkiert
				FALSE	:	Nachricht nicht gefunden etc.

	bugs:
	z.Z. keine Verarbeitung bei crosspostings
*/
BOOLEAN mdb_kill_article(const char *id);



/*
	die folgenden 3 globalen Funktionen ttigen Bildschirmausgaben 
	und gehren nicht unbedingt in diese Headerdatei (FB 2.11.94)
*/



/*
	monatlichen Traffic einer Gruppe/User ausgeben

	->			name	:	Name von Gruppe bzw. User
				pm		:	TRUE = User, FALSE = Gruppe
*/
BOOLEAN gruppen_traffic(const char *name,BOOLEAN pm);



/*
	Gruppen/Userinfo ausgeben
	ausfhrliche Informationen ber eine einzelne Gruppe/User

	->			name	:	Name von Gruppe/User bzw. NULL (interaktiv)
				pm		:	TRUE = User, FALSE = Gruppe

	<-			TRUE = alles ok , FALSE = Fehler ist aufgetreten
*/
BOOLEAN gruppeninfo(const char *name,BOOLEAN pm);



/*
	Dump der Indexdaten .pos und .len 

	<-			TRUE	:	ok
				FALSE	:	Gruppe ex. nicht/konnte nicht geffnet werden
*/
BOOLEAN gruppen_dump(const char *name,BOOLEAN pm);



/*
	Anzeigen und Bearbeiten von Datenstzen

	->			gruppenname	:	klar
				pm				:	PM (TRUE) oder OM (FALSE)
				inhalt		:	TRUE = Inhalts-, FALSE = Maildarstellung
				start			:	Nummer der Einstiegsmail
									 0 = zur ersten Nachricht
									>0 = zur angegebenen Nachrichtennummer
									-1 = zur zuletzt gelesenen Nachricht
									-2 = zur nchsten neuen Nachricht
									-3 = zur nchsten ungelesenen Nachricht
				ref			:	Referenzzeitpunkt, falls start = -2

	<-			TRUE			:	ok
				FALSE			:	keine (neue) Nachricht vorhanden
*/
BOOLEAN
msg_browser(const char *gruppenname,BOOLEAN pm,BOOLEAN inhalt,long start,time_t ref);



/*
	Datenbankgre ermitteln

	->			pm			:	User (TRUE) oder Gruppen (FALSE)
			<-	gruppen	:	Anzahl Gruppen bzw. User-Datenbanken
			<-	mails		:	Anzahl Mails insgesamt
			<-	bytes		:	Anzahl Bytes insgesamt

	<-			TRUE		:	ok
				FALSE		:	(kann derzeit nicht vorkommen)
*/
BOOLEAN
mdb_info(ULONG *user,ULONG *gruppen,ULONG *mails,ULONG *bytes,ULONG *overall,
         ULONG *neue,ULONG *putzen);
