/* infoview.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Infofile-Anzeige
	(c) 1992-94 bei Frank Baumgart

	begonnen am 12.06.1992
	Version vom 15.11.1994
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "system.h"
#include "frontend/frontend.h"
#include "mdb_defs.h"
#include "kennung.h"
#include "io.h"
#include "tool.h"
#include "mdb_conf.h"
#include "fehler.h"

extern CONFIG config;


static SEL_EINTRAG *infonamen;
static size_t anz_infos;

static BOOLEAN read_infofiles(void);
static void forget_infofiles(void);



static int srt_name(const void *a,const void *b);

static int srt_name(const void *a,const void *b)
{
	return strcmp((*(const SEL_EINTRAG *)a).text,(*(const SEL_EINTRAG *)b).text);

} /* srt_name */



/*
	Achtung:
	schmutziger Code, geht davon aus, da eine ':'-Zeile direkt nach der 
	'#'-Zeile kommt

	In Abhngigkeit von "MarkNewInfofiles" werden entweder die beim letzten 
	Import gekommenen Infofiles (TRUE) oder die im Infofileverzeichnis vor-
	handenen Infofiles (FALSE) in fetter Darstellung angezeigt.
*/
static BOOLEAN read_infofiles(void)
{
	struct stat buf;
	time_t last_import = time(NULL) + 1;	/* Hauptsache, gro genug */
	char datei[PATH_MAX];
	char zeile[256];
	FILE *inf_f;
	char *p;

	if (config.mark_infofiles)
	{
		/* Datum des letzten Imports feststellen und merken */
		strcpy(datei,config.infofiles);
		strcat(datei,"import");
		if (!stat(datei,&buf))
			last_import = buf.st_mtime;		/* letzte Vernderung */
	}

	strcpy(datei,config.infofiles);
	strcat(datei,"ITI.txt");
	if ((inf_f = fopen(datei,"r")) == NULL)
		return FALSE;
	anz_infos = 0;
	while (fgets(zeile,sizeof(zeile),inf_f))
	{
		if (*zeile == '#')
			anz_infos++;
	}
	infonamen = (SEL_EINTRAG *) calloc(anz_infos,sizeof(SEL_EINTRAG));
	if (!infonamen)
	{
		fclose(inf_f);
		fehler(NO_MEMORY);
		return FALSE;
	}
	rewind(inf_f);
	anz_infos = 0;
	while (fgets(zeile,sizeof(zeile),inf_f))
	{
		if (*zeile != '#')
			continue;
		if ((p = strpbrk(zeile,",\r\n")) != NULL)
			*p = 0;
		infonamen[anz_infos].text = strdup(zeile+1);

		fgets(zeile,sizeof(zeile),inf_f);
		if ((p = strpbrk(zeile,",\r\n")) != NULL)
			*p = 0;
		if (*zeile == ':')
			infonamen[anz_infos].info = strdup(zeile+1);
		else
			infonamen[anz_infos].info = NULL;

		/* vorhandene/neue Infofiles werden fett dargestellt */
		strcpy(datei,config.infofiles);
		strcat(datei,infonamen[anz_infos].text);
		strcat(datei,".txt");
		if (config.mark_infofiles)
		{
			if (!stat(datei,&buf) && (buf.st_mtime >= last_import))
				infonamen[anz_infos].fett = TRUE;
			else
				infonamen[anz_infos].fett = FALSE;
		}
		else
		{
			infonamen[anz_infos].fett = fileexist(datei);
		}
		anz_infos++;
	}
	fclose(inf_f);

	qsort(infonamen,anz_infos,sizeof(SEL_EINTRAG),srt_name);

	return TRUE;

} /* read_infofiles */



static void forget_infofiles(void)
{
	while (anz_infos)
	{
		anz_infos--;
		if (infonamen[anz_infos].info)
			free(infonamen[anz_infos].info);
		free(infonamen[anz_infos].text);
	}
	free(infonamen);

} /* forget_infofiles */



#ifdef TEXT_FRONTEND
/*
	Anzeige von Infofiles
*/
void show_infofile(void)
{
	static int last = 0;
	char datei[PATH_MAX];
	char name[80];
	int tmp;

	if (!read_infofiles())
		return;

	if (!*config.pager && getenv("PAGER"))
		config.pager = getenv("PAGER");

	while (1)
	{
		/* Auswahl des Infofiles */
		*name = 0;
		tmp = uni_eingabe("Infofile-Verwaltung:",INPUT_INFOFILE,infonamen,anz_infos,last,FALSE,name);
		if (tmp < 0)
			break;
		if (tmp >= 0)
			last = tmp;

		if (!*name)
			break;

		strcpy(datei,config.infofiles);
		strcat(datei,name);
		strcat(datei,".txt");

		if (!fileexist(datei))
		{
			do_beep();
			continue;
		}

		if (*config.pager)
		{
			char cmd[PATH_MAX];

			sprintf(cmd,"%s %s",config.pager,datei);
			if (shell(cmd))
				taste();
		}
		else
		{
			FILE *inf_f;

			if ((inf_f = fopen(datei,BINREAD)) == NULL)
			{
				mputs("Infofile nicht vorhanden");
				taste();
			}
			else
			{
				char zeile[2000];

				while (fgets(zeile,1999,inf_f))
					mprintf("%s",zeile);
				fclose(inf_f);
				taste();
			}
		}
	}

	forget_infofiles();

} /* show_infofile */



/*
	Anzeige von Diffs auf ein Infofile (und das alte Infofile)
*/
BOOLEAN show_infofilediff(const char *name)
{
	char cmd[PATH_MAX];
	char neu[PATH_MAX];
	char alt[PATH_MAX];

	if (!*name)
		return FALSE;

	strcpy(neu,config.infofiles);
	strcat(neu,name);
	strcpy(alt,neu);
	strcat(neu,".txt");
	strcat(alt,".bak");

	if (!fileexist(alt) || !fileexist(neu))
		return FALSE;

	if (*config.pager)
		sprintf(cmd,"%s %s %s | %s",config.diff,neu,alt,config.pager);
	else
		sprintf(cmd,"%s %s %s",config.diff,neu,alt);

	if (shell(cmd))
		taste();

	return TRUE;

} /* show_infofilediff */

#endif /* TEXT_FRONTEND */
