/* tausch.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Infile-Erstellung
	(c) 1992-95 bei Frank Baumgart

	begonnen am 01.05.1992
	Version vom 15.01.1995
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>

#include "system.h"

#ifdef HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# ifdef HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# ifdef HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# ifdef HAVE_NDIR_H
#  include <ndir.h>
# endif
# ifdef HAVE_DIRECT_H
#  include <direct.h>
# endif
#endif

#include "message.h"
#include "frontend/frontend.h"
#include "frontend/post_fe.h"
#include "mdb_conf.h"
#include "io.h"
#include "tool.h"
#include "fehler.h"
#include "missing/missing.h"

extern CONFIG config;

extern int bs_zeilen;
extern int bs_spalten;


static void infile_init(void);
static void infile_exit(void);
static void infile_sort(void);
static void update_eingabe(void);
static void update_zeile(BOOLEAN show);
static BOOLEAN msg_change_header(const char *name);
static BOOLEAN msg_create_copy(long nr);
static BOOLEAN new_msg(BOOLEAN pm);



typedef struct
{
	long phys;				/* phys. Dateinummer */
	ULONG flags;			/* z.B. zurckgestellt */
	char an[40];			/* Empfnger/Gruppe */
	char wegen[40];		/* Betreff */
	time_t zeit;			/* Eingabezeit */

} MSGINFO;

static MSGINFO *msginfo = NULL;

/* Umrechnungstabelle: fortlaufende Nummer zu Dateinamennummer */
static int *nr2file = NULL;

/* max. Anzahl der Eintrge */
static unsigned max_anzahl = 200;

/* Anzahl der Eintrge */
static unsigned anzahl;

/*
	interne Struktur zur bersicht der zu versendenden Nachrichten aufbauen

	evtl. spter Parameter: Username
*/
static void infile_init(void)
{
	char hdr_name[PATH_MAX];
#if defined(__WATCOMC__)
	char ts[PATH_MAX];
#endif
	DIR *dir;
	struct dirent *entry;
	MSGINFO aktinfo;

	anzahl = 0;
	if ((nr2file = (int *) malloc(max_anzahl * sizeof(int))) == NULL)
		return;
	if ((msginfo = (MSGINFO *) calloc(max_anzahl,sizeof(MSGINFO))) == NULL)
	{
		free(nr2file);
		return;
	}

#if defined(__WATCOMC__)
	strcpy(ts,config.messages);
	strcat(ts,"*.*");
	if ((dir = opendir(ts)) == NULL)
#else
	if ((dir = opendir(config.messages)) == NULL)
#endif
		return;
	while ((entry = readdir(dir)) != NULL)
	{
		/* Header-Dateien auslesen */
#if defined(__WATCOMC__)
		strlwr(entry->d_name);
#endif
		if ((entry->d_name[0] == 'm') && (atol(entry->d_name+1))
		&& (strlen(entry->d_name) == strlen("m0000001.hdr"))
		&& (!strcmp(entry->d_name+9,"hdr") || !strcmp(entry->d_name+9,"hdz")))
		{
			char zeile[MAX_HDR];
			FILE *hdr_f;
			BOOLEAN zurueck;
			char *p;

			zurueck = !strcmp(entry->d_name+9,"hdz");
			strcpy(hdr_name,config.messages);
			strcat(hdr_name,entry->d_name);
			if ((hdr_f = fopen(hdr_name,"r")) == NULL)
				continue;
			/* ID ist uninteressant */
			fgets(zeile,sizeof(zeile),hdr_f);

			*aktinfo.an = *aktinfo.wegen = 0;
			aktinfo.zeit = 0;
			aktinfo.phys = atol(entry->d_name+1);
			aktinfo.flags = (zurueck) ? 1 : 0;

			/* Header auslesen */
			while (!*aktinfo.an || !*aktinfo.wegen || !aktinfo.zeit)
			{
				if (!fgets(zeile,sizeof(zeile),hdr_f))
					break;
				if ((p = strpbrk(zeile,"\r\n")) != NULL)
					*p = 0;

				switch (*zeile)
				{
					case 'A' :
					case 'G' :
						strncpy(aktinfo.an,zeile+1,39);
						aktinfo.an[39] = 0;
						break;

					case 'W' :
						strncpy(aktinfo.wegen,zeile+1,39);
						aktinfo.wegen[39] = 0;
						break;

					case 'E' :
						aktinfo.zeit = calctime(zeile+1);
						break;
				}
			}
			fclose(hdr_f);

			if (anzahl == max_anzahl)
			{
				int *tmp1;
				MSGINFO *tmp2;

				tmp1 = (int *) realloc(nr2file,max_anzahl + 200);
				tmp2 = (MSGINFO *) realloc(msginfo,max_anzahl + 200);
				if (tmp1 && tmp2)
				{
					nr2file = tmp1;
					msginfo = tmp2;
					max_anzahl += 200;
				}
				else
				{
					free(msginfo);
					fehler(NO_MEMORY);
				}
			}

			if (anzahl < max_anzahl)
			{
				nr2file[anzahl] = anzahl;
				msginfo[anzahl] = aktinfo;
				anzahl++;
			}
		}
	}
	closedir(dir);

} /* infile_init */



/*
	Speicher fr Infile-Nachrichten-Struktur wieder freigeben
*/
static void infile_exit(void)
{
	if (msginfo)
	{
#if 0
		int i;

		for (i = 0; i < anzahl; i++)
			if (msginfo[i])
				free(msginfo[i]);
			else
				break;
#endif
		free(msginfo);
		msginfo = NULL;
	}

	if (nr2file)
	{
		free(nr2file);
		nr2file = NULL;
	}

} /* infile_exit */



static int inf_srt_fn(const void *a,const void *b)
{
	MSGINFO *ma = &msginfo[*(const int *)a];
	MSGINFO *mb = &msginfo[*(const int *)b];

	/* erst zurckgestellte */
	if (ma->flags && !mb->flags)
		return -1;
	if (mb->flags && !ma->flags)
		return 1;

	/* alte vor neuen */
	if (ma->zeit < mb->zeit)
		return -1;
	if (ma->zeit > mb->zeit)
		return 1;

	return 0;	

} /* inf_srt_fn */



/*
	Infile-Nachrichten fr bersicht sortieren
	- zurckgestellte vor neuen
	- alte vor neuen
	wird spter konfigurierbar
*/
static void infile_sort(void)
{
	if (!anzahl)
		return;

	qsort(nr2file,anzahl,sizeof(int),inf_srt_fn);

} /* infile_sort */



/*
	neue Nachricht erstellen

	->			pm		:	TRUE = PM, FALSE = OM

	<-			TRUE	:	neue Nachricht wurde erstellt
				FALSE	:	keine neue Nachricht
*/
static BOOLEAN new_msg(BOOLEAN pm)
{
	char empfaenger[MAX_CPG];
	char betreff[MAX_CPG];
	OFFSET offset;

	*empfaenger = *betreff = 0;

	if (pm)
	{
		if (!input_adress(empfaenger))
			return FALSE;
		tprint("cl");
		mprintf("Empfnger: %s",empfaenger);
		if (!*empfaenger)
			return FALSE;
		offset.author = empfaenger;
		offset.name = empfaenger;
	}
	else
	{
		char *p;
		if ((p = gruppeneingabe("Gruppe",SEL_OM)) == NULL)
			return FALSE;

		tprint("cl");
		strcpy(empfaenger,p);
		mprintf("Gruppe: %s",empfaenger);
		offset.group = empfaenger;
	}
	mputs("");

	mprintf("Betreff  : ");
	mgets(betreff,MAX_CPG);
	if (!strlen(betreff))
		return FALSE;
	offset.topic = betreff;
	mputs("");

	message(0,(pm) ? NEUE_PM : NEUE_MSG,&offset);

	return TRUE;

} /* new_msg */



/*
	Header (Empfnger, Gruppe, Betreff) ndern

	->			name	:	Dateiname des Headers

	todo:
	- crosspostings
	- follow-ups
*/
static BOOLEAN msg_change_header(const char *name)
{
	char zeile[MAX_CPG];
	char i[MAX_HDR],w[MAX_CPG],a[MAX_CPG],g[MAX_CPG],e[MAX_CPG],v[MAX_CPG];
	FILE *hdr_f;
	BOOLEAN changed = FALSE;
	BOOLEAN pm = FALSE;
	char *p;

	if ((hdr_f = fopen(name,"r")) == NULL)
		return FALSE;

	*i = *w = *a = *g = *e = *v = 0;
	while (fgets(zeile,MAX_CPG,hdr_f))
	{
		if ((p = strpbrk(zeile,"\r\n")) != NULL)
			*p = 0;

		switch (*zeile)
		{
			case	'#'	:
				strcpy(i,zeile+1);
				break;

			case	'W'	:
				strcpy(w,zeile+1);
				break;

			case	'A'	:
				strcpy(a,zeile+1);
				if (!*g)
					pm = TRUE;
				break;

			case	'G'	:
				if (!*g)
					strcpy(g,zeile+1);
				pm = FALSE;
				break;

			case	'E'	:
				strcpy(e,zeile+1);
				break;

			case	'V'	:
				strcpy(v,zeile+1);
				break;
		}
	}
	fclose(hdr_f);

	/* Nachrichtentyp (PM/OM) ndern ? */
	{
		int typ;

		mprintf("Typ (O,P,<return>): %s",(*g) ? "OM" : "PM");
		typ = taste();
		if (*g && typ == 'p')
		{
			/* OM -> PM */
			mputs(" -> PM");
			*i = 'P';
			*g = 0;
			pm = TRUE;
		}
		else
		if (*a && typ == 'o')
		{
			/* PM -> OM */
			mputs(" -> OM");
			*i = 'O';
			*a = 0;
			pm = FALSE;
		}
		else
			mputs("");
	}

	if (pm)
	{
		mprintf("Empfnger: ");
		strcpy(zeile,a);
	}
	else
	{
		mprintf("Gruppe: ");
		strcpy(zeile,g);
	}
	mgets(zeile,78);
	if ((p = strpbrk(zeile,"\r\n")) != NULL)
		*p = 0;
	if (strlen(zeile))
	{
		if (pm)
			strcpy(a,zeile);
		else
			strcpy(g,zeile);
		changed = TRUE;
	}
	mputs("");

	mprintf("Betreff: ");
	strcpy(zeile,w);
	mgets(zeile,78);
	if ((p = strpbrk(zeile,"\r\n")) != NULL)
		*p = 0;
	if (strlen(zeile))
	{
		strcpy(w,zeile);
		changed = TRUE;
	}
	mputs("");

	if (changed)
	{
		if ((hdr_f = fopen(name,"w")) == NULL)
			return FALSE;
		fprintf(hdr_f,"#%s\r\n",i);
		if (*g)
			fprintf(hdr_f,"G%s\r\n",g);
		else
		if (*a)
			fprintf(hdr_f,"A%s\r\n",a);
		fprintf(hdr_f,"W%s\r\n",w);
		if (*v)
			fprintf(hdr_f,"V%s\r\n",v);
		if (*e)
			fprintf(hdr_f,"E%s\r\n",e);
		fclose(hdr_f);
	}

	return TRUE;

} /* msg_change_header */



/*
	Kopie einer Nachricht erzeugen

	Es wird der Editor aufgerufen, in dem zeilenweise alle Empfnger 
	einzugeben sind. Somit kann daraus spter wahlweise eine cc-Zeile 
	erzeugt werden als auch jetzt das MT-bliche Format.
	Zudem erlaubt die Eingabe per Editor das nachladen von alias-Dateien.
	ggf. spter "AliasPath" setzen.

	Das entsprechende Gegenstck (Auswertung bei Infile-Erstellung) und 
	Beachtung beim Infile-Import fehlt noch.
*/
static BOOLEAN msg_create_copy(long nr)
{
	char cc[PATH_MAX];
	BOOLEAN pm = TRUE;	/* ### */

	if (pm)
		mputs("Kopie(n) der Nachricht an:");
	else
		mputs("Crosspostings in folgende Gruppen:");
	sleep(1);

	sprintf(cc,"%sm%07ld.cc",config.messages,nr);
	if (call_editor(cc))
		return FALSE;
	else
		return TRUE;

} /* msg_create_copy */



static int top = 0;
static int cursor = 0;
static int i_anz = 0;

static void update_eingabe(void)
{
	int old_cursor = cursor;
	int nr;
	int max;

	max = top + i_anz;
	if (max > anzahl)
		max = anzahl;

	for (nr = top; nr < max; nr++)
	{
		cursor = nr - top;
		update_zeile(cursor == old_cursor);
	}

	cursor = old_cursor;
	gotoxy(0,i_anz + 2);

} /* update_eingabe */



/*
	Cursorzeile neu aufbauen
*/
static void update_zeile(BOOLEAN show)
{
	int nr = top + cursor;
	MSGINFO *mi;
	int rest = bs_spalten-9;

	if (!anzahl)
		return;

	mi = &msginfo[nr2file[nr]];

	gotoxy(0,cursor + 2);

	if (show)
		tprint("so");

	mprintf("%c %4d %-*.*s %-*.*s",	/* war 32/39 bei 80 */
	       (mi->flags) ? 'z' : ' ',
	       nr+1,
	       (rest*2)/5,(rest*2)/5,
	       mi->an,
	       (rest*3)/5,(rest*3)/5,
	       mi->wegen);

	if (show)
		tprint("se");

	gotoxy(0,i_anz + 2);

} /* update_zeile */



/*
	Nachrichten nachtrglich bearbeiten
*/
void infile_edit(void)
{
	char hdr[PATH_MAX];
	char hdz[PATH_MAX];
	char name[PATH_MAX];
	char tmp[200];
	char trenner[200];
	FILE *tst_f;
	MSGINFO *mi = NULL;
	BOOLEAN keep = FALSE;
	int wahl;
	int i;
	size_t max;

	get_screen();
	i_anz = bs_zeilen -2 -5;
	top = 0;
	cursor = 0;

	max = (bs_spalten > 200) ? 200-1 : bs_spalten-1;
	memset(trenner,'=',max);
	trenner[max] = 0;

	infile_init();
	infile_sort();

	do
	{
		long phys = 0;
		int old_top;
		int old_cursor;

		if (!keep)
		{
			tprint("cl");
			mputs("Nachrichten-bersicht:");
			mputs(trenner);
			update_eingabe();
			mputs(trenner);
			mputs("neue (p)m/(o)m, (k)opie, (d)elete, (z)urckstellen, (h)eader ndern");
			mputs("(a)nzeigen, tauschdat(u)m/(v)ertreter/pa(w)ort/tite(l) setzen");
			mputs("(i)nfile erstellen, (t)auschen, (q)uit");
		}

		keep = TRUE;
		old_top = top;
		old_cursor = cursor;

		/* phys. Nr. der aktuellen Mail */
		if (anzahl)
		{
			mi = &msginfo[nr2file[top + cursor]];
			phys = mi->phys;
		}
		else
			phys = 0;

		wahl = taste();

		/* Nachricht direkt via Nummer anspringen */
		if (wahl < 256 && isdigit(wahl))
		{
			char ziel[5];
			int nr;

			ziel[0] = wahl;
			ziel[1] = 0;
			mgets(ziel,5);
			nr = atoi(ziel);
			if (*ziel && nr && nr <= anzahl)
			{
				update_zeile(FALSE);
				nr--;
				if (nr < top || nr > top + i_anz)
				{
					top = nr;
					cursor = 0;
				}
				else
					cursor = nr - top;
			}
		}

		switch (wahl)
		{
			case C_UP :
			case C_LEFT :
				update_zeile(FALSE);
				if (cursor)
					--cursor;
				else
				if (top)
					top--;
				else
					do_beep();
				break;

			case C_DOWN :
			case C_RIGHT :
				update_zeile(FALSE);
				if (top + cursor +1 >= anzahl)
					do_beep();
				else
				if (cursor + 1 == i_anz)
					top++;
				else
					cursor++;
				break;

			case C_PGDN :
				update_zeile(FALSE);
				if (top + i_anz < anzahl)
					top += i_anz;
				else
				if (top + cursor +1 >= anzahl)
					do_beep();
				else
					cursor = anzahl - top -1;
				break;

			case C_PGUP :
				update_zeile(FALSE);
				if (!top && !cursor)
					do_beep();
				else
				if (top > i_anz)
					top -= i_anz;
				else
				if (top)
					top = 0;
				else
					cursor = 0;
				break;

			case '?' :
			case C_HELP :
				/* ... */
				break;

			case '\r' :
			case '\n' :
				if (!anzahl)
					break;
				sprintf(name,"%sm%07ld.txt",config.messages,phys);
				call_editor(name);
				keep = FALSE;
				break;

			case 'a'	:
				if (!anzahl)
					break;
				if (!*config.pager && getenv("PAGER"))
					config.pager = getenv("PAGER");
				if (*config.pager)
					sprintf(name,"%s %sm%07ld.txt",config.pager,config.messages,phys);
				else
					sprintf(name,"more %sm%07ld.txt",config.messages,phys);
				shell(name);
				keep = FALSE;
				break;

			case 'd' :
				if (!anzahl)
					break;
				mprintf("Nachricht Nr.%d wirklich lschen ?",top + cursor +1);
				if (taste() != 'j')
					break;
				sprintf(name,"%sm%07ld.hdr",config.messages,phys);
				unlink(name);
				sprintf(name,"%sm%07ld.hdz",config.messages,phys);
				unlink(name);
				sprintf(name,"%sm%07ld.txt",config.messages,phys);
				unlink(name);
#if 0
				for (i = top+cursor+1; i < anzahl; i++)
				{
					nr2file[i-1] = nr2file[i];
					msginfo[i-1] = msginfo[i];
				}
/*				free(msginfo[top + cursor]);
				msginfo[top + cursor] = NULL;*/
#endif
				keep = FALSE;
				infile_exit();
				anzahl--;
				if ((top + cursor + 1 >= anzahl) && (cursor))
					cursor--;
				infile_init();
				infile_sort();
				break;

			case 'z' :
				if (!anzahl)
					break;
				sprintf(hdr,"%sm%07ld.hdr",config.messages,phys);
				sprintf(hdz,"%sm%07ld.hdz",config.messages,phys);
				if ((tst_f = fopen(hdr,"r")) != NULL)
				{
					fclose(tst_f);
					if (!rename(hdr,hdz))
						mputs("Nachricht zurckgestellt");
				}
				else
				{
					if (!rename(hdz,hdr))
						mputs("zurckgestellte Nachricht fr Tausch aktiviert");
				}
				mi->flags = !mi->flags;
				sleep(1);
				break;

			case 'p' :
				new_msg(TRUE);
				infile_exit();
				infile_init();
				infile_sort();
				keep = FALSE;
				break;

			case 'o' :
				new_msg(FALSE);
				infile_exit();
				infile_init();
				infile_sort();
				keep = FALSE;
				break;

			case 'h' :
				if (!anzahl)
					break;
				sprintf(name,"%sm%07ld.hd%c",config.messages,phys,(mi->flags) ? 'z' : 'r');
				msg_change_header(name);
				break;

			case 'k' :
				if (!anzahl)
					break;
				mputs("Kopie erzeugen");
				msg_create_copy(phys);
				break;

			case 'u' :
				mputs("Tauschdatum setzen (nur fr Quarks)");
				mputs("Bitte exakt in der Form YYYYMMDDHHMM, z.B. 1994040231930 setzen");
				strcpy(name,"1994");
				mgets(name,12);
				if (!*name)
					break;
				if (strlen(name) != 12 || (strncmp(name,"19",2) && strncmp(name,"20",2)))
				{
					printf("\rfalsches Format");
					fflush(stdout);
					do_beep();
					sleep(1);
					break;
				}
				for (i = 0; i < 12; i++)
					if (!isdigit(name[i]))
					{
						printf("\rfalsches Format (nur Ziffern erlaubt)");
						fflush(stdout);
						do_beep();
						sleep(1);
						break;
					}
				infile_cmd("TD",name);
				break;

			case 'l' :
				mprintf("Fachtitel setzen: ");
				*name = 0;
				mgets(name,80);
				{
					mprintf("Fachtitel lschen (j/n) ?");
					if (taste() != 'j')
						break;
				}
				sprintf(tmp,"%s=%s",akt_username(),name);
				infile_cmd("UT",tmp);
				break;

			case 'v' :
				mprintf("Vertreter setzen: ");
				*name = 0;
				mgets(name,80);
				if (!*name)
				{
					mprintf("Vertreter lschen (j/n) ?");
					if (taste() != 'j')
						break;
				}
				sprintf(tmp,"%s=%s",akt_username(),name);
				infile_cmd("UF",tmp);
				break;

			case 'w' :
				mprintf("neues Pawort setzen: ");
				*name = 0;
				mgets(name,12);
				if (!*name)
					break;
				sprintf(tmp,"%s=%s",akt_username(),name);
				infile_cmd("UP",tmp);
				break;

			case 'i' :
				tprint("cl");
				if (!infile_erstellen())
					do_beep();
				keep = FALSE;
				break;

			case ' ' :
				/* bersicht neu einlesen und aufbauen */
				infile_exit();
				infile_init();
				infile_sort();
				keep = FALSE;
				break;
		}

		if (top != old_top)
			update_eingabe();
		else
			update_zeile(TRUE);

		/* nur nach Aktionen Statusbereich lschen, nicht nach Sondertasten */
		if (wahl < 256)
		{
			if (i_anz > anzahl)
				gotoxy(0,anzahl + 6);
			else
				gotoxy(0,i_anz + 6);
			tprint("cd");
		}
	}
	while (wahl != 'q');

	/* Speicher wieder freigeben */
	infile_exit();

} /* infile_edit */
