/* packer.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"
#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <syslog.h>
#include <ctype.h>
#include <errno.h>
#include "conflib.h" /* damit sie in . liegen darf */

#include "prototyp.h"
#include "packer.h"
#include "tompaths.h"
#include "box_conf.h"
#include "modem.h"

extern int logout_modus;

typedef struct packer_t 
{
	char key;
	const char *kennung;
	const char *endung;
	const char *packcmd;
	const char *unpackcmd;
	const char *listcmd;
	const char *testcmd;
	BOOLEAN tausch;
	const char *name;
	const char *magic;
	struct packer_t *next;
} packer_t;

static packer_t *anker;
static BOOLEAN read_packer_cnf(void);

	/* fr do_interpret */
const char *packer_archive; /* PA */
const char *packer_files;   /* PF */

static BOOLEAN interpret(char **buf,const char *command, 
	const char *archive, const char *files);
static BOOLEAN packer_run(const char *cmd);

static BOOLEAN
interpret(char **buf,const char *command, const char *archive, 
	const char *files)
{
	int i=0;
	size_t len=1024;
	*buf=NULL;

	do
	{
		len*=4;
		if (*buf)
			free(*buf);
		*buf=malloc(len);
		if (!*buf)
		{
			mbfehler ("Kommandointerpretation gescheitert");
			return FALSE;
		}
		packer_archive=archive;
		packer_files=files;
		if (do_interpret (command, *buf, len))
		{
			return TRUE;
		}
		i++;
	} while (i<4); /* 1024*4*4*4 = 64 K */
	mbfehler ("Kommandointerpretation gescheitert");
	free(*buf);
	return FALSE;
}

static BOOLEAN 
read_packer_cnf(void)
{
	int erg;
	static int init_done;
	char fname[PATH_MAX];
	cl_list_t *all_packer = NULL;
	cl_file_t *cfp;
	cl_var_t listtab[] =
	{
		{"packer", NULL, CL_LIST, NULL, 0, NULL},
		{NULL, NULL, 0, NULL, 0, NULL}
	};
	listtab[0].adr=&all_packer;

	if (init_done)
		return TRUE;
	sprintf(fname,"%spacker.cnf",box_conf.config_dir);

	init_done = TRUE;
	cfp = cl_openconf (fname, "r");
	if (!cfp)
	{
		syslog (LOG_ERR, "kann packer.cnf nicht lesen");
		return FALSE;
	}
	erg = cl_getstanza (cfp, listtab, "liste", 0, NULL);
	if (erg)
	{
		syslog (LOG_ERR, "kann Packerliste nicht lesen");
		cl_closeconf(cfp);
		return FALSE;
	}
	while (all_packer)
	{
		/* all_events->inhalt ist die ID des Events */
		cl_list_t *next;
		char key = 0;
		char *packcmd = NULL;
		char *unpackcmd = NULL;
		char *listcmd = NULL;
		char *testcmd = NULL;
		char *kennung = NULL;
		char *name = NULL;
		char *endung = NULL;
		char *magic = NULL;
		long maxfiles=-1;
		int tausch = 0;
		char *id;
		unsigned pcounter=0;
		cl_var_t packerdat[15];

		/* work around "initializers must be constant" problem */
#define ADD_VAR(var,x1,x2,x3,x4,x5,x6) do { \
	if (sizeof(packerdat) / sizeof(cl_var_t) <=var+1) { \
		error_at_line(1,0,__FILE__, __LINE__, "internal program error"); \
	} \
	packerdat[var].varname=x1; packerdat[var].secdata=x2; \
	packerdat[var].typ=x3; packerdat[var].adr=x4; \
	packerdat[var].flags=x5; packerdat[var].rangeexp=x6; \
	var++; } \
	while (0)

		ADD_VAR(pcounter,"key", NULL, CL_CHAR, &key, 0, NULL);
		ADD_VAR(pcounter,"name", NULL, CL_STRING, &name, 0, NULL);
		ADD_VAR(pcounter,"pack", NULL, CL_STRING, &packcmd, 0, NULL);
		ADD_VAR(pcounter,"kennung", NULL, CL_STRING, &kennung, 0, NULL);
		ADD_VAR(pcounter,"unpack", NULL, CL_STRING, &unpackcmd, 0, NULL);
		ADD_VAR(pcounter,"test", NULL, CL_STRING, &testcmd, 0, NULL);
		ADD_VAR(pcounter,"list", NULL, CL_STRING, &listcmd, 0, NULL);
		ADD_VAR(pcounter,"endung", NULL, CL_STRING, &endung, 0, NULL);
		ADD_VAR(pcounter,"magic", NULL, CL_STRING, &magic, 0, NULL );
		ADD_VAR(pcounter,"maxfiles", NULL, CL_NUM, &maxfiles, 0, NULL );
		ADD_VAR(pcounter,"tausch", NULL, CL_BOOLEAN, &tausch, 0, NULL);
		ADD_VAR(pcounter,NULL, NULL, 0, NULL, 0, NULL);

		id = all_packer->inhalt;
		erg = cl_getstanza (cfp, packerdat, id, 0, NULL);
		if (erg)
		{
			syslog (LOG_NOTICE, "kann Packer %s nicht lesen", id);
		}
		else
		{
			BOOLEAN ok=TRUE;

			if (!key)
				ok = FALSE;
			if (!name)
				ok = FALSE;
			if (!endung)
				ok = FALSE;
			if (!ok)
			{
				mbprintf ("unvollstndige Angaben zu Packer %s\n", id);
				syslog (LOG_NOTICE, "unvollstaendige Angaben zu Packer %s\n", id);
			}
			else
			{
				packer_t *neu;
#define W(data) if (data) neu->data=XSTRDUP(data); else neu->data=NULL;
				neu = XMALLOC (sizeof (packer_t));
				neu->key = key;
				W(packcmd);
				W(unpackcmd);
				W(listcmd);
				W(testcmd);
				W(endung);
				W(kennung);
				W(name);
				W(magic);
#undef W
				neu->kennung=XSTRDUP(id);
				neu->tausch=tausch ? TRUE : FALSE;
				neu->next = anker;
				anker = neu;
			}
		}
		cl_clearconf (packerdat);
		next = all_packer->next;
		free (all_packer->inhalt);
		free (all_packer);
		all_packer = next;
	}
	cl_closeconf (cfp);
	return TRUE;
}

#if 0
char *
packerauswahl (void)
{
	packer_t *packer;
	char buf[128];
	char dec[128] = "";
	char c;
	size_t len = 0;

	for (packer = anker; packer; packer = packer->next)
	{
		size_t sl;

		sl = sprintf (buf, "(%c) %s", packer->key, packer->name);
		if (sl + len > 65)
		{
			mbprintf ("\r\n  (%s)", buf);
			len = 0;
		}
		else
			mbprintf ("  %s", buf);
		len += 2 + sl;
		sprintf (dec + strlen (dec), "%c", tolower (packer->key));
	}
	mboutput ("  (ESC) Abbruch  ");
	mboutput ("?\r\n");
	c = '\0';
	while (!c)
	{
		c = mbchar ();
		if (logout_modus)
			break;
		switch (c)
		{
		case 27:				/* ESC */
		case 3:				/* ^C */
		case 24:				/* ^X */
			return '\0';
			break;
		}
		if (!strchr (dec, tolower (c)))
			c = '\0';
	}
	for (packer = anker; packer; packer = packer->next)
		if (tolower(packer->key) == tolower(c))
			return packer->kennung;
	return NULL; /* keep compiler quiet */
}
#endif

static packer_t *
find_packer_for_archive(const char *archive)
{
	packer_t *packer;
	size_t len=strlen(archive);
	const char *p;
	char *cmd;
	FILE *f;
	size_t clen;

	/* first try: Nach Endung */
	for (packer=anker;packer;packer=packer->next)
	{
		size_t l=strlen(packer->endung);
		if (l>=len)
			continue;
		p=archive+len-l;
		if (!stricmp(p,packer->endung))
			return packer;
	}
	/* Und wenn das nicht will: */
	clen=asprintf(&cmd,"file %s",archive);
	XNULLCHECK(cmd);
	f=popen(cmd,"r");
	if (!f)
	{
		mbprintf("kann keinen Packer fr %s finden: file fehlgeschlagen\r\n",
			archive);
		return NULL;
	}
	if (-1==getline(&cmd,&clen,f)) {
		fclose(f);
		mbprintf("kann keinen Packer fr %s finden: file fehlgeschlagen\r\n",
			archive);
		return NULL;
	}
	for (packer=anker;packer;packer=packer->next)
	{
		if (packer->magic && strstr(cmd,packer->magic))
			return packer;
	}
	mbprintf("kann keinen Packer fr %s finden\r\n", archive);
	return NULL;
}

BOOLEAN 
packer_list(const char *archive)
{
	char *p;
	BOOLEAN ret;
	packer_t *packer;

	if (!read_packer_cnf())
		return FALSE;

	packer=find_packer_for_archive(archive);
	if (!packer)	
		return FALSE;

	p=NULL;
	if (!interpret(&p,packer->listcmd,archive,NULL))
		return FALSE;
	ret=packer_run(p);
	free(p);
	return ret;
}

BOOLEAN
packer_unpack(const char *archive, const char *workdir, BOOLEAN tausch,
    char *packerkennung, char *packerendung)
{
	packer_t *packer;
	int ret;
	char *p;

	if (!read_packer_cnf())
		return FALSE;

	packer=find_packer_for_archive(archive);
	if (!packer)	
		return FALSE;
	if (tausch && !packer->tausch)
	{
		mbfehler("Packer fr den Tausch nicht erlaubt");
		return FALSE;
	}

	p=NULL;
	if (!interpret(&p,packer->unpackcmd,archive,NULL))
		return FALSE;

	if (workdir)
	{
		if (chdir(workdir))
		{
			mbfehler("kann Arbeitsverzeichnis nicht wechseln");
			free(p);
			return FALSE;
		}
	}
	ret=packer_run(p);
	if (p)
		free(p);
	if (workdir)
		chdir(box_conf.quark_dir);
	if (ret)
	{
		if (packerkennung)
			strcpy(packerkennung,packer->kennung);
		if (packerendung)
			strcpy(packerendung,packer->endung);
	}
	return ret;
}

BOOLEAN 
packer_pack(const char *archive, const char *workdir, char *packerkennung, 
	const char *file)
{
	packer_t *packer;
	int ret;
	char *p;

	if (!read_packer_cnf())
		return FALSE;

	if (packerkennung)
	{
		for(packer=anker;packer;packer=packer->next)
		{
			if (!stricmp(packerkennung,packer->kennung))
				break;
		}
		if (!packer)
		{
			mbprintf("Packer %s nicht gefunden\r\n",packerkennung);
			return FALSE;
		}
	}
	else
		packer=find_packer_for_archive(archive);
	if (!packer)	
		return FALSE;

	p=NULL;
	if (!interpret(&p,packer->packcmd,archive,file))
		return FALSE;

	if (workdir)
	{
		if (chdir(workdir))
		{
			mbfehler("kann Arbeitsverzeichnis nicht wechseln");
			free(p);
			return FALSE;
		}
	}
	ret=packer_run(p);
	if (p)
		free(p);
	if (workdir)
		chdir(box_conf.quark_dir);
	return ret;
}

static BOOLEAN
packer_run(const char *cmd)
{
	FILE *f;
	char *p;
	char buf[1024];
	f=popen(cmd,"r");
	if (!f)
	{
		mboutput(" Offenbar sind die Packer nicht richtig installiert\r\n");
		syslog(LOG_NOTICE,"packer.sh nicht gefunden: %s",strerror(errno));
		return FALSE;
	}
	while (fgets(buf,sizeof(buf),f))
	{
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		mbprintf("%s\r\n",buf);
	}
	return pclose(f)==0;
}
