/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Programmlisten-Verwaltung

	begonnen: 22.03.1993, Frank

	Das (alte) Konzept beruht darauf, in jedem Binrbrett eine ASCII-Datei mit 
	dem Inhalt aller im Brett befindlichen Dateien entweder zu aktualisieren  
	oder aber ggf. (nach Uploads/Lschungen) neu zu erstellen.
	Diese Programmlisten werden dann nur noch aneinandergehngt.
	Das etwas neuere Konzept sieht so aus, dass die Programmlisten direkt aus 
	den Indexdaten (index.dat) und den entsprechenden Infotexten on-the-fly 
	generiert werden. Das reicht vorerst aus.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>

#include "prototyp.h"
#include "modem.h"
#include "bin_msg.h"
#include "proglist.h"

#define PL_QUARK			1			/* Programmlistenexport im Quarkformat */
#define PL_MAUS			1			/* Programmlistenexport im Mausformat */
#define MAX_INFOLINES	10			/* max. 10 Infozeilen */

extern BRETTINFO *br_ind;
extern long max_brettnr;



/*
	Infodatei an Programmliste anhngen

	->			bnr	:	klar
				nr		:	globale Uploadnummer
*/
BOOLEAN pl_append_brett(BRETTNR bnr,long nr)
{
	char name[PATH_MAX];
	char info[PATH_MAX];
	char zeile[120];
	FILE *prl_f,*inf_f;
	int lines = 0;

	sprintf(info,"%s%8ld.txt",brettpfad(bnr),nr);
	if (!fileexist(info))
		return TRUE;

	if ((inf_f = fopen(name,BINREAD)) == NULL)
		return FALSE;

	sprintf(name,"%s%s",brettpfad(bnr),"inhalt.lst");
	if ((prl_f = fopen(name,BINAPPEND)) == NULL)
	{
		fclose(inf_f);
		return FALSE;
	}

	if (tom_fgetline(zeile,inf_f))
		fprintf(prl_f,"#%ld\r\n",nr);

	while (tom_fgetline(zeile,inf_f) && lines++ < MAX_INFOLINES)
		fprintf(prl_f," %s\r\n",zeile);

	fclose(prl_f);
	fclose(inf_f);

	return TRUE;

} /* pl_append_brett */



/*
	Programmliste eines Brettes exportieren

	->			bnr		:	klar
				ref_zeit	:	alles ab time_t Zeitangabe
				exp_f		:	geffneter Exportdatei-Stream
				lang		:	kurzes oder langes Format (FALSE/TRUE)
*/
BOOLEAN pl_export_brett(BRETTNR bnr,time_t ref_zeit,FILE *exp_f, BOOLEAN lang)
{
	char zeile[120];
	BIN_MSG bin_info;
	FILE *bin_f;
	FILE *lst_f;
	BOOLEAN header = FALSE;

	if ((bin_f = binaer_dat_oeffnen(bnr,FALSE)) == NULL)
		return FALSE;

	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (bin_info.datime < ref_zeit)
			continue;

		if (lang && !header)
		{
			fprintf(exp_f,":+---------------------------------------"
			              "-----------------------------------+\r\n");
#ifdef PL_MAUS
			fprintf(exp_f,":! Brett: %-65s !\r\n",br_ind[bnr].brett->name);
#else
			fprintf(exp_f,":=== %s ===========\r\n",br_ind[bnr].brett->name);
#endif
			fprintf(exp_f,":+---------------------------------------"
			              "-----------------------------------+\r\n");
			fputs(":\r\n",exp_f);
#ifdef PL_MAUS
			fprintf(exp_f,":  Nr. Filename                  Bytes  Abruf  Datum\r\n");
#elif PL_QUARK
			fprintf(exp_f,":  Nr. Brett                Dateiname                 Gre    Datum  Zug.\r\n");
#endif
			fputs(":\r\n",exp_f);
			header = TRUE;
		}

#ifdef PL_MAUS
		fprintf(exp_f,":%5ld %-22.22s %8ld  %4ld  %s\r\n",
		        bin_info.nr,bin_info.name,bin_info.bytes,
		        bin_info.zugriffe,bin_info.datum);
#elif PL_QUARK
		fprintf(exp_f,":%5ld %-20.20s %-22.22s %8ld  %s  %ld\r\n",
							bin_info.lokal,br_ind[bnr].name,bin_info.name,
							bin_info.bytes,bin_info.datum,bin_info.zugriffe);
#endif

		if (!lang)
			continue;

		fprintf(exp_f,": %s\r\n",bin_info.betreff);
		lst_f = fopen(tprintf("%s%ld.txt",brettpfad(bnr),(long)bin_info.nr),"r");
		if (lst_f)
		{
			while (tom_fgetline(zeile,lst_f) && *zeile)
			{
				fputc(':',exp_f);
#ifdef PL_MAUS
				fputs("  ",exp_f);
#endif
				fputs(zeile,exp_f);
				fputs("\r\n",exp_f);
			}
			fclose(lst_f);
		}
#ifdef PL_MAUS
		fprintf(exp_f,":  Von: %s\r\n",bin_info.absender);
#endif
		fputs(":\r\n",exp_f);
	}

	binaer_dat_schliessen(bin_f);

	return TRUE;

} /* pl_export_brett */



/*
	Infotexte entfernen

	Infotexte aller in nr[] angegebenen Dateinummern entfernen
*/
BOOLEAN pl_filter_brett(BRETTNR bnr,long nr[])
{
	char name[PATH_MAX];
	char tmp [PATH_MAX];
	char zeile[120];
	FILE *old_f,*new_f;

	*zeile = 0;

	sprintf(name,"%s%s",brettpfad(bnr),"inhalt.lst");
	sprintf(tmp ,"%s%s",brettpfad(bnr),"inhalt.tmp");

	if ((old_f = fopen(name,BINREAD)) == NULL)
		return FALSE;

	if ((new_f = fopen(tmp ,BINWRITE)) == NULL)
	{
		fclose(old_f);
		return FALSE;
	}

	while (!feof(old_f))
	{
		if (!*zeile && !tom_fgetline(zeile,old_f))
			break;

		if (isdigit(*zeile))
		{
			long i = 0;
			long eintrag = atol(zeile);

			/* Nummer in der Lschliste suchen */
			while (nr[i] && nr[i] != eintrag)
				i++;

			if (nr[i] && nr[i] == eintrag)
			{
				while (tom_fgetline(zeile,old_f) && !isdigit(*zeile));
			}
			else
			{
				while (tom_fgetline(zeile,old_f) && !isdigit(*zeile))
					fputs(zeile,new_f);
			}
		}
	}

	unlink(name);
	rename(tmp,name);

	return TRUE;

} /* pl_filter_brett */


#if 0
/*
	kurze Programmliste eines Brettes exportieren/ausgeben

	->			bnr		:	klar
				ref_zeit	:	alles ab time_t Zeitangabe
				exp_f		:	geffneter Exportdatei-Stream
*/
BOOLEAN pl_kurz_export_brett(BRETTNR bnr,time_t ref_zeit,FILE *exp_f)
{
	char zeile[120];
	BIN_MSG bin_info;
	FILE *bin_f;
	BOOLEAN header = FALSE;

	if ((bin_f = binaer_dat_oeffnen(bnr,FALSE)) == NULL)
		return FALSE;

	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (bin_info.datime < ref_zeit)
			continue;

		if (!header)
		{
			fputc(':',exp_f);
			sprintf(zeile,"  Nr. Brett                Dateiname                 Gre    Datum  Zug.\r\n");
			fprintf(exp_f,"%s",zeile);
			header = TRUE;
		}

		sprintf(zeile,"%5ld %-20s %-22.22s %8ld  %s  %ld\r\n",
							bin_info.lokal,br_ind[bnr].name,bin_info.name,
							bin_info.bytes,bin_info.datum,bin_info.zugriffe);
		fprintf(exp_f,":%s",zeile);
	}
	if (header)
		fputs(":\r\n",exp_f);

	binaer_dat_schliessen(bin_f);

	return TRUE;

} /* pl_kurz_export_brett */
#endif


/*
	Programmlisten aller Bretter im MT-Format exportieren

	->			ref_zeit	:	alles ab time_t Zeitangabe
				exp_f		:	geffneter Exportdatei-Stream

	Die globale Indexdatei wird nach Binrbrettern mit neuen Eintrgen
	durchsucht und dann ggf. brettweise exportiert.
*/
BOOLEAN pl_export_all(time_t ref_zeit,FILE *exp_f)
{
	long i;

	for (i = 1; i < max_brettnr; i++)
	{
		if (br_ind[i].brett
		&& (br_ind[i].brett->flags & B_READ)
		&& (br_ind[i].brett->art == BINAER_BRETT)
		&& (br_ind[i].brett->datum >= ref_zeit))
		{
			pl_export_brett(i,ref_zeit,exp_f,TRUE);
		}
	}

	return TRUE;

} /* pl_export_all */
