/* r_conf.c: Laden der Konfiguration
 */
/*
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/

#include "tomsysdep.h"

#include "portab.h"
#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <ctype.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "tompaths.h"
#include "box_conf.h"
#include "routing.h"
#include "route_int.h"

	/* Tabelle der Domainpartauflser */
static r_aufloeser_t r_dom_aufloeser_field[]=
{
	{ "RFC Sourceroute Auflser",  
	                        'S', "Source", r_821sourceroute_aufloeser,  0 },
	{ "Rewriter Auflser",  'R', "Rewriter", r_rewriter_aufloeser,  0 },
	{ "Domainbounce Auflser",  'B', "DomBounce", r_domainbounce_aufloeser,  0 },
	{ "Hostalias Auflser", 'A', "Hostalias", r_hostalias_aufloeser,  0 },
		/* Localhost auf jeden Fall vor FQDN, denn letzterer knnte
		 * eine Wildcarddomain haben.
		 */
	{ "Localhost Auflser", 'L', "Localhost", r_localhost_aufloeser,  0 },
	{ "FIDO Hack Auflser", 'F', "Fido", r_fido_aufloeser, 0 },
	{ "MAUS Bindestrich Hack Auflser", 
	                        '-', "MAUS", r_bindestrich_aufloeser,0},
	{ "FQDN Auflser",      '@', "FQDN", r_fqdn_aufloeser, 0 },
	{ NULL, 0, NULL, NULL, 0}
};

	/* Tabelle der Localpartauflser */
static r_aufloeser_t r_loc_aufloeser_field[]=
{
	{ "Prozent-Hack Router", '%', "percent", r_percent_aufloeser,  0 },
	{ "Bang Router",         '!', "bang", r_bang_aufloeser,  0 },
	{ "Useralias",           'a', "Useralias", r_useralias_aufloeser, 0 },
	{ "Gruppenchefrouter",   'c', "Chef", r_chef_aufloeser, 0 },
	{ "Usernamenrouter",     'u', "User", r_user_aufloeser, 0 },
	{ NULL, 0, NULL, NULL, 0}
};

	/* Konfigdateien gelesen? */
BOOLEAN r_init_done;


static void r_add_gate(char *s);
static void r_add_domain(char *s);
static void r_add_alias_chain(char *s, r_alias_t **anker);
static void load_routing_file(const char *fname);
static void r_add_useralias_chain(char *s);
static void r_add_order(r_aufloeser_t **anker, char *inhalt);
static void r_conf_zeile_auswerten(char *lbuf);

static void r_aufloeser_verkettung(r_aufloeser_t *field,
	r_aufloeser_t **anker);

const char *r_default_config=
"deftransport error error\n"
"deftransport intern discard\n"
"deftransport intern bounce\n"
"deftransport intern notice\n"
"deftransport intern echo\n"
;

static void 
r_aufloeser_verkettung(r_aufloeser_t *field, r_aufloeser_t **anker)
{
	*anker=field;
	for (;field && field->call;field++)
	{
		if (field[1].call)
			field->next=&field[1];
		else
			field->next=NULL;
	}
}

void
r_do_init(void)
{
	char *dies,*next;
	char * kopie;
	char fname[PATH_MAX];
	
	if (r_init_done)
		return; /* can happen */	
	r_init_done=TRUE;
	kopie=XSTRDUP(r_default_config);

	/* Defaultkonfig laden */
	for (dies=kopie;dies && *dies; dies=next)
	{
		next=strchr(dies,'\n');
		if (next)
			*next++=0;
		while (isspace(*dies))
			dies++;
		r_conf_zeile_auswerten(dies);
	}
	free(kopie);

	sprintf(fname,"%srouting.cnf",box_conf.config_dir);
	if (!fileexist(fname))
		sprintf(fname,"%srouting.dat",box_conf.config_dir);
	load_routing_file(fname);
	if (!r_dom_aufloeser_anker)
		r_aufloeser_verkettung(r_dom_aufloeser_field,&r_dom_aufloeser_anker);
	if (!r_loc_aufloeser_anker)
		r_aufloeser_verkettung(r_loc_aufloeser_field,&r_loc_aufloeser_anker);
}

static void
r_conf_zeile_auswerten(char *lbuf)
{
	char *inhalt;
	/* # Kommentarzeilen ignorieren.*/
	if (*lbuf==';' || *lbuf=='#' || !*lbuf)
		return;

	inhalt=strpbrk(lbuf," \t");
	if (inhalt)
	{
		*inhalt++='\0';
		while(isspace(*inhalt))
			inhalt++;
	}
	strlwr(lbuf);
	switch(*lbuf)
	{
	case 'd':
		if (!strcmp(lbuf,"domain"))	
			r_add_domain(inhalt);
		else if (!strcmp(lbuf,"domainrewrite"))	
			r_add_alias_chain(inhalt,&r_rewrite_anker);	
		else if (!strcmp(lbuf,"domainbounce"))	
			r_add_alias_chain(inhalt,&r_domainbounce_anker);	
		else if (!strcmp(lbuf,"deftransport"))	
		{
			char *paras=strpbrk(inhalt,"\t ");
			if (!paras)
			{
				syslog(LOG_WARNING,"deftransport %s: no parameter",inhalt);
				return;
			}
			*paras++=0;
			while(isspace(*paras))
				paras++;
			if (0==stricmp(inhalt,"maustausch"))
				r_t_maustausch_define(paras);
			else if (0==stricmp(inhalt,"error"))
				r_t_error_define(paras);
			else if (0==stricmp(inhalt,"discard"))
				r_t_error_define(paras);
			else if (0==stricmp(inhalt,"intern"))
				r_t_intern_define(paras);
			else if (0==stricmp(inhalt,"extern"))
				r_t_extern_define(paras);
			else
				syslog(LOG_WARNING,"deftransport %s: unknown type",inhalt);
		}
		break;
	case 'f':
		if (!strcmp(lbuf,"foreignorder"))
			r_add_order(&r_dom_aufloeser_anker,inhalt);
		break;
	case 'g':
		if (!strcmp(lbuf,"gate") || !stricmp(lbuf,"gateway"))	
			r_add_gate(inhalt);
		break;
	case 'h':
		if (!strcmp(lbuf,"hostalias"))	
			r_add_alias_chain(inhalt,&r_hostalias_anker);	
		break;
	case 'i':
		if (!strcmp(lbuf,"include"))
		{
			char fname2[PATH_MAX];
			if (*inhalt=='/')
				sprintf(fname2,inhalt);
			else
			{
				sprintf(fname2,"%s%s",box_conf.config_dir,inhalt);
				if (!fileexist(fname2))
					sprintf(fname2,"%s%s",box_conf.var_dir,inhalt);
			}
			load_routing_file(fname2);
		}
		break;
	case 'l':
		if (!strcmp(lbuf,"localorder"))	
			r_add_order(&r_loc_aufloeser_anker,inhalt);
		break;
	case 'r':
		if (!strcmp(lbuf,"remoteorder"))	
			r_add_order(&r_dom_aufloeser_anker,inhalt);
		break;
	case 't':
		if (!strcmp(lbuf,"transport"))	
			r_t_add_to_transport(inhalt);
		break;
	case 'u':
		if (!strcmp(lbuf,"useralias"))	
			r_add_useralias_chain(inhalt);	
		break;
	}
}

static void
load_routing_file(const char *fname)
{
	static unsigned int recursion_counter=0;
	char buf[2048];
	char *lbuf=NULL;
	size_t lbufsize=0;
	FILE *f;
	char *p;
	recursion_counter++;
	if (recursion_counter>4)
	{
		syslog(LOG_ERR,"load_routing_file: zu tief verschachtelt: %s",fname);
		recursion_counter--;
		return;
	}
	f=fopen(fname,"r");
	if (!f)
	{
		recursion_counter--;
		return;
	}
	while (fgets(buf,sizeof(buf),f))
	{
		size_t l;
		char *start;
		p=strchr(buf,'\n');
		if (p)
		{
			if (p>buf && p[-1]=='\r')
				p--;
			*p=0;
		}
		else
			p=buf+strlen(buf)-1;
		while (isspace(*p) && p>buf)
			p--;
		p[1]=0;
		if (isspace(*p))
			*p=0;
		start=buf;
		while (isspace(*start))
			start++;

		/* Der folgende Aufwand soll Fortsetzungszeilen ermglichen.
		 * Aus
		 *   blah \
		 *   blubb\
		 *     fasel \
		 *     last
		 * wird
		 *   blah blubbfasel last
		 * Mit anderen Worten, Leerzeichen am Anfang verfallen.
		 */
		l=strlen(start);
		if (!lbuf || l+strlen(lbuf)+1>lbufsize)
		{
			/* Puffer vergrern */
			if (lbufsize==0)
			{
				lbufsize=sizeof(buf);
				lbuf=XMALLOC(lbufsize);
				*lbuf=0;
			}
			else
			{
				lbufsize*=2;
				lbuf=XREALLOC(lbuf,lbufsize);
			}
		}
		strcat(lbuf,start);
		l=strlen(lbuf);
		p=lbuf+l-1;
		if (l && *p=='\\')
		{
			*p=0;
			continue;
		}
		r_conf_zeile_auswerten(lbuf);
		*lbuf=0;
	}
	fclose(f);
	if (lbuf)
		free(lbuf);
	recursion_counter--;
}

static void 
r_add_order(r_aufloeser_t **anker, char *inhalt)
{
	char *p;
	r_aufloeser_t *last;
	if (!*anker)
		last=NULL;
	else
	{
		for (last=*anker;last->next;last=last->next)
			; /* nothing */
	}
	while (inhalt && *inhalt)
	{
		r_aufloeser_t *akt;
		p=strpbrk(inhalt,"\t ");
		if (p)
		{
			*p++='\0';
			while (isspace(*p))
				p++;
		}
		for (akt=r_dom_aufloeser_anker;akt;akt=akt->next)
		{
			if (!stricmp(akt->magic,inhalt))
				break;
		}
		if (!akt)
		{
			for (akt=r_loc_aufloeser_anker;akt;akt=akt->next)
			{
				if (!stricmp(akt->magic,inhalt))
					break;
			}
		}
		if (!akt)
		{
			syslog(LOG_WARNING,"unbekannter Aufloeser %s",inhalt);
		}
		else
		{
			if (last)
				last->next=akt;
			else
				(*anker)->next=akt;
			last=akt;
		}
		inhalt=p;
	}
}

static void 
r_add_useralias_chain(char *s)
{
	char buf[UDAT_NAME_LEN];
	r_alias_t *n,*last;
	char *p;

	p=strchr(s,'=');
	if (!p)
		p=strpbrk(s,"\t ");
	if (p)
	{
		*p++='\0';
		strtrim(s);
		strtrim(p);
		if (!*p)
			p=NULL;
	}
	if (strlen(s)>UDAT_NAME_LEN)
	{
		syslog(LOG_NOTICE,"Useraliasname zu lang: %s",s);
		return;
	}
	ub_canonical(s,buf);
	n=r_useralias_anker;
	last=NULL;
	while (n)
	{
		if (!stricmp(n->from,buf))
		{
			free(n->to);
			if (p)
				n->to=XSTRDUP(p);
			else
				n->to=NULL;
			return;
		}
		last=n;
		n=n->next;
	}
	n=XMALLOC(sizeof(r_alias_t));
	n->from=XSTRDUP(buf);
	n->fromlen=strlen(n->from);
	if (p)
		n->to=XSTRDUP(p);
	else
		n->to=NULL;
	n->next=NULL;
	if (last)
		last->next=n;
	else
		r_useralias_anker=n;
}

static void 
r_add_alias_chain(char *s, r_alias_t **anker)
{
	r_alias_t *n,*last;
	char buf[512];
	char *p=strpbrk(s,"\t ");
	if (!p)
		return;
	*p++='\0';
	strtrim(p);
	if (!*p)
		return;
	strtrim(s);

	r_hcanonical(s,buf);
	n=*anker;
	last=NULL;
	while (n)
	{
		if (n->from[0]==buf[0] && 0==strcmp(n->from,buf))
		{
			free(n->to);
			if (p)
				n->to=XSTRDUP(p);
			else
				n->to=NULL;
			return;
		}
		last=n;
		n=n->next;
	}
	n=XMALLOC(sizeof(r_alias_t));
	n->from=XSTRDUP(buf);
	n->fromlen=strlen(n->from);
	if (p)
		n->to=XSTRDUP(p);
	else
		n->to=NULL;
	n->next=NULL;
	if (last)
		last->next=n;
	else
		*anker=n;
}

static void 
r_add_gate(char *s)
{
	/* inhalt= "tirka.gun.de 2 1 @ GATE TIRKA" */
	USERNR unr=0;
	r_gateway_t *n;

	char *path;
	char *erlaubtp;
	char *kostenp;
	char *userp;

	userp=strtok(s,"@");
	if (!userp)
	{
		syslog(LOG_ERR,"Gateeintrag --unknown-- unparsbar");
		return;
	}
	path=strtok(NULL,"\t ");
	if (!path)
	{
		syslog(LOG_ERR,"Gateeintrag f. %s unparsbar",userp);
		return;
	}
	erlaubtp=strtok(NULL,"\t ");
	if (!erlaubtp)
	{
		syslog(LOG_ERR,"Gateeintrag %s unparsbar",path);
		return;
	}
	kostenp=strtok(NULL,"\t ");
	if (!kostenp)
	{
		syslog(LOG_ERR,"Gateeintrag %s unparsbar",path);
		return;
	}

	strtrim(userp);
	if (!ub_find_username(&unr,userp))
	{
		syslog(LOG_ERR,"User %s zu Gate %s unbekannt",userp,path);
		return;
	}

	if (!strchr("0987654321$",*erlaubtp))
	{
		syslog(LOG_ERR,"Gateeintrag %s, Erlaubnis %c unbekannt",s,*erlaubtp);
		return;
	}

	n=XMALLOC(sizeof(r_gateway_t));
	n->path_entry=XSTRDUP(path);
	n->nur_zahler=FALSE;
	n->ab_level=0;

	while (*erlaubtp)
	{
		if (*erlaubtp=='$')
			n->nur_zahler=TRUE;
		if (isdigit(*erlaubtp))
			n->ab_level=*erlaubtp-'0';
		erlaubtp++;
	}

	n->kosten=strtol(kostenp,NULL,10);
	n->via=unr;
	n->next=r_gateway_anker;
	r_gateway_anker=n;
}

static void 
r_add_domain(char *s)
{
	char buf[512];
	/* inhalt= "*.maus.de du2.maus.de" */
	r_gateway_t *gate;
	r_domain_t *n,*last;
	char *gatep;
	char *costp;
	char c;

	gatep=strpbrk(s,"\t ");
	if (!gatep)
	{
		syslog(LOG_ERR,"Domaineintrag %s unparsbar",s);
		return;
	}
	*gatep++='\0';

	while (isspace(*gatep))
		gatep++;
	costp=strpbrk(gatep,"\t ");
	if (costp)
	{
		*costp++='\0';
		while (isspace(*costp))
			costp++;
		if (!*costp)
			costp=NULL;
		else
		{
			char *p;
			p=costp+strlen(costp)-1;
			while (isspace(*p))
				*p--=0;
		}
	}

	c=tolower(*gatep);
	for (gate=r_gateway_anker;gate;gate=gate->next)
	{
		if (c == tolower(*gate->path_entry)
			&& !r_hostnamecmp(gate->path_entry,gatep))
			break;
	}	
	if (!gate)
	{
		syslog(LOG_ERR,"Domaineintrag %s: Gate %s unbekannt ",s,gatep);
		return;
	}

	last=NULL;
	r_hcanonical(s,buf);

	for (n=r_domain_anker;n;n=n->next)
	{
		/* strcmp statt stricmp mglich, weil beides canonical ist */
		if (*buf==*n->domain && !strcmp(n->domain,buf))
			break;
		last=n;
	}

	if (!n)
	{
		char *p;
		n=XMALLOC(sizeof(r_domain_t));
		p=buf;
		if (*p=='@')
		{
			n->fidostyle=TRUE;
			p++;
		}
		else
			n->fidostyle=FALSE;
		n->domain=XSTRDUP(p);
		if (costp && *costp)
			n->kosten=strtol(costp,NULL,10);
		else
			n->kosten=0;
		if (last)
			last->next=n;
		else
			r_domain_anker=n;
		n->next=NULL;
	}
	n->via=gate;
}

