/* routing.c: Hauptroutinen, Routing an sich.  */
/*
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/

#include "tomsysdep.h"

#include "portab.h"
#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <ctype.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "box_conf.h"
#include "tompaths.h"
#include "routing.h"
#include "route_int.h"

/* 
 * Anker fr verschiedenen Listen
 */
	/* Liste der Domainauflser. Zeigt auf Elemente der obigen
	 * Tabelle, allerdings ist die Reihenfolge ganz anders.
	 */
r_aufloeser_t *r_dom_aufloeser_anker;
	/* Liste der Lokalauflser. Zeigt auf Elemente der obigen
	 * Tabelle, allerdings ist die Reihenfolge ganz anders.
	 */
r_aufloeser_t *r_loc_aufloeser_anker;

	/* Liste der Gateways */
r_gateway_t *r_gateway_anker;
	/* Liste der bekannten Domains (incl. Wildcarddomain) 
	 * Die "from"-Domainnamen sind canonical, rewrite.c depend on it!
	 */
r_domain_t *r_domain_anker;
	/* Liste der bekannten Domainnamen, die wir in andere umschreiben
	 * knnen (XYZ -> ABC).
	 * Die "from"-Domainnamen sind canonical, rewrite.c depend on it!
	 */
r_alias_t *r_rewrite_anker;
	/* Liste der Aliasnamen fr Benutzer (sysop -> uwe_ohse) */
r_alias_t *r_useralias_anker;
	/* Liste der Aliasnamen fr andere Rechner (du3.maus.de -> mausdu3.gun.de).
	 * Diese Namen werden als gleichwertig betrachtet, aber nicht 
	 * umgeschrieben.
	 */
r_alias_t *r_hostalias_anker;
	/* Liste der Domains, die wir bouncen ... 
	 * nicht wirklich ein Alias, aber ganz gleich zu behandeln 
	 */
r_alias_t *r_domainbounce_anker;

r_t_zuord_t *r_transport_zuord_anker;
char *r_dsn_out_of_memory=NULL;

static BOOLEAN r_route_one(r_adr_t *adr);

static void
r_show_flags(r_adr_t *adr)
{
	R_PRINTF(3,("Flags:"));
#define DOIT(what,str) if (adr->flags & (what)) \
	R_PRINTF(3,(" %s",str));

	DOIT(R_F_NEUE_ID,"+id");
	DOIT(R_F_TAUSCHLOG,"taulog");
	DOIT(R_F_SPAETER_AUFL,"spter");
	DOIT(R_F_LOKAL,"lokal");
	DOIT(R_F_NO_WILDCARD,"-wild");
	DOIT(R_F_NEWENVELOPE,"+env");

	DOIT(R_F_AUFGELOEST,"aufgeloest");
	DOIT(R_F_GEROUTET,"geroutet");
	DOIT(R_F_ZUGESTELLT,"zugestellt");
	DOIT(R_F_E_TRIED,"tried");

	DOIT(R_F_TO_POSTMASTER,"postm");
	DOIT(R_F_TO_OWNER,"owner");
	DOIT(R_F_TO_SENDER,"sender");
#undef DOIT
	R_PRINTF(3,("\r\n"));
}

/*
 * Deep magic starts here!
 *
 * diese Funktion ist schwer zu dokumentieren, weite Teile ihrer
 * Ttigkeit hngen von den einzelnen Adressauflsern ab.
 * Eine Adresse wird zerlegt in Local- und Domainpart (anhand des @).
 * Danach werden die Domainaufloeser auf den Domainpart angesetzt.
 * Pat einer von ihnen, wird die resultierende Adresse erneut zerlegt.
 * Falls kein Domainpart existiert, wird der Localpart durch die 
 * verschiedenen Localpartauflser geschickt.
 * Die Localpartauflser sind aber teilweise verkappte Domainaufloeser,
 * so werden UUCP-Bangpath-Adressen und der %-Hack hier erkannt.
 * Auch wenn der Zieluser einen Vertreter eingesetzt hat oder fr
 * ihn ein Alias existiert, entsteht eine neue Adresse, mit oder ohne 
 * Domain, fr die die Bearbeitung dann erneut beginnt.
 *
 * Schluendlich entsteht eine Adresse, die 
 * a) durch ein bekanntes Gateway geschickt werden kann (eine Adresse
 *    mit Domain)
 * b) an einen lokalen User verschickt werden kann
 * c) an ein Programm verschickt wird (not yet)
 * d) gebounced wird (unauflsbar)
 *
 * Die Aufloeser lsen nicht alle wirklich eine Adresse auf - einige
 * schreiben sie auch einfach nur etwas um, oder deklarieren sie als
 * Alias fr einen andere.
 * Hacker, sei gewarnt: Als ich das schrieb, wute ich sehr gut, was
 * ich tat. Diese Routine und die Auflser sind hochgradig 
 * fehlerempfindlich, wer nicht wei, was sie tun sollte davon die
 * Finger lassen.
 *
 * Beschrnkungen:
 * - absolut ungeeignet, Adressen auf fremden Rechnern umzuschreiben
 *   (Designentscheidung - ich wei nicht, ob der Zielrechner 
 *    user!name nicht als lokalen Usernamen zult!).
 *
 * unerwartete Seiteneffekte:
 * - erzeugt neue Adressen in der Liste.
 */
static BOOLEAN 
r_route_one(r_adr_t *adr)
{
	char *p;
	r_a_work_t work;
	BOOLEAN returncode=FALSE;
	int conversions=0;

	adr->user_gate_nr=0;
	adr->transport=NULL;

	if (!r_init_done)
		r_do_init();

	if (!r_dsn_out_of_memory) {
		r_dsn_out_of_memory=XSTRDUP("4.3.0 (out of memory)");
	}

	R_PRINTF(4,("r_route_one to %s, erroradr %s\r\n",
			adr->inputadr, adr->erroradr ? adr->erroradr : "<>"));

	adr->resultadr=strdup(adr->inputadr);
	work.local=strdup(adr->inputadr);
	if (!adr->resultadr || !work.local)
	{
		R_PRINTF(0,("r_route_one: out of memory\r\n"));
		r_writecomment(R_LOG_FATAL, adr,"out of memory");
		adr->errorflags |= R_E_TEMP;
		adr->errorflags |= R_E_INTERN;
		adr->dsn=r_dsn_out_of_memory;

		returncode=FALSE;
		goto cleanup;
	}

	p=strchr(work.local,'<');
	if (p)
	{
		char *q=strchr(p,'>');
		if (q)
		{
			*q=0;
			q=work.local;
			work.local=strdup(p+1);
			free(q);
			if (!work.local)
			{
				R_PRINTF(0,("r_route_one: out of memory\r\n"));
				r_writecomment(R_LOG_FATAL, adr,"out of memory");
				adr->errorflags |= R_E_TEMP;
				adr->errorflags |= R_E_INTERN;
				adr->dsn=r_dsn_out_of_memory;

				returncode=FALSE;
				goto cleanup;
			}
			strtrim(work.local);
		}
		conversions++;
	}

	p=strchr(work.local,'@');
	if (p)
	{
		*p++=0;
		work.domain=strdup(p);
		if (!work.domain)
		{
			R_PRINTF(0,("r_route_one: out of memory\r\n"));
			r_writecomment(R_LOG_FATAL, adr,"out of memory");
			adr->errorflags |= R_E_TEMP;
			adr->errorflags |= R_E_INTERN;
			adr->dsn=r_dsn_out_of_memory;
			returncode = FALSE;
			goto cleanup;
		}
		strtrim(work.domain);
		strtrim(work.local);
	}
	else
	{
		work.domain=NULL;
		strtrim(work.local);
	}

	p=strchr(work.local,'\"');
	if (p)
	{
		char *q=strchr(p+1,'\"');
		if (q)
		{
			*q=0;
			q=work.local;
			work.local=strdup(p+1);
			free(q);
			if (!work.local)
			{
				R_PRINTF(0,("r_route_one: out of memory\r\n"));
				r_writecomment(R_LOG_FATAL, adr,"out of memory");
				adr->errorflags |= R_E_TEMP;
				adr->errorflags |= R_E_INTERN;
				adr->dsn=r_dsn_out_of_memory;

				returncode=FALSE;
				goto cleanup;
			}
			conversions++;
		}
		else
			R_PRINTF(4,
				("r_route_one: stray \" in localpart, hope that's ok: %s\r\n",
				work.local));
	}
	if (conversions)
	{
		R_PRINTF(4,("r_route_one: converted to local: %s, domain: %s\r\n",
			work.local,work.domain));
	}

	/* adr->flags &= ~R_F_NEWENVELOPE; Warum? */
	adr->error = FALSE;
	adr->errorflags = 0;

	while (TRUE)
	{
		adr->loopcount++;
		R_PRINTF(4,("r_route_one: loop %ld\r\n",(long) adr->loopcount));
		if (adr->loopcount>20)
		{
			r_writecomment(R_LOG_ERROR,adr,
				"Abbruch des Aufloesens nach %d Schleifen", adr->loopcount);
			R_PRINTF(2,("r_route_one: breaking after %d loops\r\n",
				adr->loopcount));
			adr->dsn=r_get_dsn("5.4.6 (loop suspected)");
			adr->errorflags|=R_E_ADR;
			adr->error=TRUE;
			returncode = FALSE;
			goto cleanup;
		}
		R_PRINTF(4,(" work.local: %s\r\n",
			work.local ? work.local : "<>"));
		R_PRINTF(4,(" work.domain: %s\r\n",
			work.domain ? work.domain : "<>"));

		if (work.domain && *work.domain)
		{
			r_aufloeser_t *aufl;

			for (aufl=r_dom_aufloeser_anker;aufl;aufl=aufl->next)
			{
				int thisret;
				R_PRINTF(5,("r_route_one: calling dom %s\r\n",aufl->name));
				thisret=aufl->call(adr,&work);
				if (thisret==R_A_DONE)
				{
					R_PRINTF(4,("r_route_one: %s returned DONE\r\n",aufl->name));
					R_PRINTF(4,(" work.local: %s\r\n",
						work.local ? work.local : "<>"));
					R_PRINTF(4,(" work.domain: %s\r\n",
						work.domain ? work.domain : "<>"));
					break;
				}
				R_PRINTF(5,("r_route_one: %s returned %s\r\n",aufl->name,
					thisret==R_A_ERROR ? "ERROR" : "NOTDONE" ));
				if (thisret==R_A_ERROR)
				{	
					adr->error=TRUE;
					goto cleanup;
				}
			}
			if (!aufl)
			{
				if (!adr->logstr)
					r_writecomment(R_LOG_ERROR, adr,"keine Route gefunden");
				if (!adr->errorflags)
					adr->errorflags |= R_E_ADR;
				adr->dsn=r_get_dsn("5.4.4 (no route found)");
				returncode = FALSE;
				adr->error=TRUE;
				goto cleanup;
			}

			r_show_flags(adr);

			if (adr->user_gate_nr)
				/* Das passende Gate ist gefunden */
				goto finalize;

			if (work.domain && *work.domain)
				continue;
		}

		/* Lokal oder pseudolokal (%, !) */
		{
			r_aufloeser_t *aufl;

			for (aufl=r_loc_aufloeser_anker;aufl;aufl=aufl->next)
			{
				int thisret;
				R_PRINTF(5,("r_route_one: calling loc %s\r\n",aufl->name));
				thisret=aufl->call(adr,&work);
				if (thisret==R_A_DONE)
				{
					R_PRINTF(4,("r_route_one: %s returned DONE\r\n",aufl->name));
					R_PRINTF(4,(" work.local: %s\r\n",
						work.local ? work.local : "<>"));
					R_PRINTF(4,(" work.domain: %s\r\n",
						work.domain ? work.domain : "<>"));
					break;
				}
				R_PRINTF(5,("r_route_one: %s returned %s\r\n",aufl->name,
					thisret==R_A_ERROR ? "ERROR" : "NOTDONE" ));
				if (thisret==R_A_ERROR)
				{
					adr->error=TRUE;
					goto cleanup;
				}
			}
			if (!aufl)
			{
				if (!adr->logstr)
				{
					r_writecomment(R_LOG_ERROR, adr,
						"Benutzername nicht gefunden");
				}
				if (!adr->dsn)
					adr->dsn=r_get_dsn("5.1.1 (user name not found)");
				if (!adr->errorflags)
					adr->errorflags |= R_E_ADR;
				returncode = FALSE;
				adr->error=TRUE;
				goto cleanup;
			}

			r_show_flags(adr);

			if (adr->user_gate_nr)
				goto finalize;

			continue;
		}
	}

finalize:
	R_PRINTF(5,("r_route_one: finalize\r\n"));

	/* 
	 * lets do some magic: Wenn mglich den Hostnamen voll qualifizieren 
	 */
	if (work.domain && !strpbrk(work.domain,"@%! "))
	{
		char *fqdn=r_domainize(work.domain);
		if (fqdn)
		{
			if (stricmp(work.domain,fqdn)!=0)
				adr->flags |= R_F_NEWENVELOPE;
			free(work.domain);
			work.domain=fqdn;
		}
	}

	/* Alternative transports angemeldet? */
	if (r_transport_zuord_anker)
	{
		/* Dann in deren Liste nachsehen und transport in Adresse eintragen */
		r_t_zuord_t *z;
		for (z=r_transport_zuord_anker; z!=NULL; z=z->next)
		{
			if (z->user_nr==adr->user_gate_nr)
				adr->transport=z->transport;
		}
	}

	returncode=TRUE;
	if (adr->flags & R_F_NEWENVELOPE)
	{
		char *neu;
		if (work.domain)
			neu=concat(work.local,"@",work.domain,NULL);
		else if (strpbrk(work.local,"!%"))
			neu=strdup(work.local);
		else
#if 0
			neu=concat(work.local,"@",MYDOMAINPART,NULL);
#else
			neu=concat(work.local,"@",box_conf.domainname,NULL);
#endif
		if (!neu)
		{
			R_PRINTF(0,("r_route_one: out of memory\r\n"));
			r_writecomment(R_LOG_FATAL,adr,"out of memory");
			adr->errorflags |= R_E_TEMP;
			adr->errorflags |= R_E_INTERN;
			adr->dsn=r_dsn_out_of_memory;
			returncode = FALSE;
		}
		else
		{
			free(adr->resultadr);
			adr->resultadr=neu;
		}
	}
	if (returncode)
		adr->flags|=R_F_AUFGELOEST | R_F_GEROUTET;

cleanup:
	R_PRINTF(5,("r_route_one: cleanup\r\n"));
	if (work.local)
		free(work.local);
	if (work.domain)
		free(work.domain);
	return returncode;

}

BOOLEAN 
r_router(message_t *msg TOM_ATTRIB_UNUSED, r_adr_t *adr)
{
	BOOLEAN one_ok=FALSE;
	BOOLEAN fatal=FALSE;
	r_adr_t *first;
	r_adr_t *last;

	first=adr;
	while (adr)
	{
		if (fatal)
		{
			adr->error=TRUE;
			adr->errorflags|=R_E_TEMP;
			adr->errorflags|=R_E_INTERN;
		}
		else
		{
			int ret;
			BOOLEAN schon_getan=FALSE;
			if (first!=adr)
			{
				/* hatten wir die schon? Regelmig auftretender Fall
				 * bei aliasexpansionen.
				 */
				r_adr_t *akt;
				for(akt=first; akt && akt!=adr; akt=akt->next)
				{
					if (akt->inputadr && adr->inputadr)
					{
						if (!stricmp(akt->inputadr,adr->inputadr))
						{
							schon_getan=TRUE;
							break;
						}
					}
				}
			}
			if (!schon_getan)
			{
				ret=r_route_one(adr);
				if (!ret && adr->errorflags & R_E_INTERN)
					fatal=TRUE;
				else if (ret)
				{
					r_adr_t *akt;
					one_ok=TRUE;
					/* So, jetzt sehen, ob diese Zieladresse doppelt ist */
					for(akt=first; akt && akt!=adr; akt=akt->next)
					{
						if (akt->resultadr && adr->resultadr)
						{
							if (!stricmp(akt->resultadr,adr->resultadr))
							{
								adr->flags |= R_F_ZUGESTELLT;
								break;
							}
						}
					}
				}
			}
			else
			{
				adr->flags |= R_F_AUFGELOEST;
				adr->flags |= R_F_GEROUTET;
				adr->flags |= R_F_ZUGESTELLT;
			}
		}
		adr=adr->next;
	}

	/*
	 * berflssige Adressen beseitigen.
	 */
	adr=first;
	last=NULL; /* keep gcc happy */
	while(adr)
	{
		if (adr->error || !(adr->flags & R_F_ZUGESTELLT))
		{
			last=adr;
			adr=adr->next;
		}
		else
		{ 
			r_adr_t *next;
			next=adr->next;
			adr->next=NULL;
			r_free(adr);
			last->next=next;
			adr=next;
		}
	}
	if (fatal)
		return FALSE;
	return one_ok;
}


/*
 * Ist "host_name" ein Name fr die eigene Kiste?
 * Das ist im wesentlichen eine gekrzte Version von r_route_one,
 * die allerdings keine Lokalrouter aufruft.
 */
BOOLEAN 
r_is_me(const char *host_name)
{
	char *p;
	r_a_work_t work={NULL,NULL};
	BOOLEAN returncode=FALSE;
	r_adr_t *adr;

	if (!r_init_done)
		r_do_init();

	adr=r_alloc();
	if (!adr)
		goto cleanup;

	adr->inputadr=concat("sysop","@",host_name,NULL);
	if (!adr->inputadr)
		goto cleanup;

	adr->resultadr=strdup(adr->inputadr);
	work.local=strdup(adr->inputadr);
	if (!adr->resultadr || !work.local)
		goto cleanup;

	p=strchr(work.local,'@');
	if (p)
	{
		*p++=0;
		work.domain=strdup(p);
		if (!work.domain)
			goto cleanup;
		strtrim(work.domain);
		strtrim(work.local);
	}
	else
	{
		work.domain=NULL;
		strtrim(work.local);
	}

	while (TRUE)
	{
		r_aufloeser_t *aufl;
		adr->loopcount++;
		if (adr->loopcount>20)
			goto cleanup;

		if (!work.domain || !*work.domain) {
			returncode=TRUE;
			break;
		}

		for (aufl=r_dom_aufloeser_anker;aufl;aufl=aufl->next)
		{
			int thisret=aufl->call(adr,&work);
			if (thisret==R_A_DONE)
				break;
			else if (thisret==R_A_ERROR)
				goto cleanup;
		}
		if (!aufl)
			goto cleanup;
		
		if (adr->user_gate_nr)
		{
			if (adr->flags & R_F_LOKAL)
				returncode=TRUE;
			goto cleanup;
		}
	}


cleanup:
	if (work.local)
		free(work.local);
	if (work.domain)
		free(work.domain);
	if (adr)
		r_free(adr);
	return returncode;

}


const char *
r_get_router_name(char kenn)
{
	r_aufloeser_t *aufl;
	for (aufl=r_dom_aufloeser_anker; aufl; aufl=aufl->next)
	{
		if (aufl->kenn==kenn)
			return aufl->name;
	}
	for (aufl=r_loc_aufloeser_anker; aufl; aufl=aufl->next)
	{
		if (aufl->kenn==kenn)
			return aufl->name;
	}
	return NULL;
}

BOOLEAN
r_vrfy(const char *to, char **com, UWORD lv, BOOLEAN bezahlt)
{
	BOOLEAN ok;
	r_adr_t *adr;
	message_t *msg;

	if (com)
		*com=NULL;
	adr=r_alloc();
	if (adr)
		adr->inputadr=strdup(to);
	msg=msg_alloc();
	if (!msg || !adr || !adr->inputadr)
	{
		if (com)
			*com=strdup("out of memory");
		if (adr)
			r_free(adr);
		if (msg)
			msg_free(msg);
		return FALSE;
	}

	adr->level=lv;
	adr->sec_level=lv;
	adr->bezahlt=bezahlt;
	adr->sec_bezahlt=bezahlt;
	adr->owner=0;

	ok=r_router(msg,adr);
	if (adr->logstr)
	{
		if (com)
			*com=strdup(adr->logstr);
	}
	r_free_chain(adr);
	msg_free(msg);
	return ok;
}

USERNR
r_is_local_user(const char *to)
{
	BOOLEAN ok;
	USERNR ret=0;
	r_adr_t *adr;
	message_t *msg;

	adr=r_alloc();
	if (adr)
		adr->inputadr=strdup(to);
	msg=msg_alloc();
	if (!msg || !adr || !adr->inputadr)
	{
		if (adr)
			r_free(adr);
		if (msg)
			msg_free(msg);
		return FALSE;
	}

	adr->level=ULV_MAX;
	adr->sec_level=ULV_MAX;
	adr->bezahlt=TRUE;
	adr->sec_bezahlt=TRUE;
	adr->owner=0;

	ok=r_router(msg,adr);
	if (ok)
	{
		if (!adr->next && adr->flags & R_F_LOKAL)
		{
			ret=adr->user_gate_nr;
		}
	}
	r_free_chain(adr);
	msg_free(msg);
	return ret;
}

