/* resize.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "config.h"

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
#include <fnmatch.h>
#include <conflib.h>
#include <locale.h>
#include <fcntl.h>

#ifdef MAJOR_IN_MKDEV
#include <sys/mkdev.h>
#else
# ifdef MAJOR_IN_SYSMACROS
# include <sys/sysmacros.h>
# endif
#endif


#include "tomsysdep.h"

#include "tom_termcap.h"

/* manche curses definieren TRUE/FALSE unbedingt */
#include <portab.h>

#include "prototyp.h"
#include "io.h"
#include "modem.h"
#include "terminal.h"
#include "user.h"
#include "brett.h"
#include "log.h"
#include "protokoll.h"
#include "aktdata.h"


static void alarm_handler(int signo);
static BOOLEAN read_answer(int fdin, const char *tmpl, char *buf, size_t max);

static int got_alarm=FALSE;

#ifdef HAVE_TERMCAP
static int output_fd=0;
static int my_putchar(int c) 
{
	return write(output_fd, &c,1);
}
#endif

/*
 * Wie stellt man fest, wie gro der lokale Screen ist?
 * Ganz einfach: 
 * 1. man setzt den Cursor utopisch weit (999,999).
 * 2. man sendet eine Escapesequenz an das Terminal
 * 3. man liest und parst die Antwort des Terminals.
 * Nein, kein Witz. Die ioctls fr die Windowsize erfragen
 * nur, was eingestellt ist, das Terminal wei es im Zweifelsfall
 * besser.
 * Und will man nicht auf die Nase fallen, wenn die Ein/Ausgabe
 * umgelenkt wird, wird das alles noch *viel* komplizierter.
 * Modi:
 * RESIZE_MODE_NORMAL   - normal
 * RESIZE_MODE_INQUIRE  - feststellen ob resize mglich ist
 * RESIZE_MODE_FORCE_VT - force mode: vt100 sequenzen annehmen.
 * Return, normaler und force modus:
 * RESIZE_ANS_OK       - ok.
 * RESIZE_ANS_NOT_TERM - geht nicht, kein Terminal
 * RESIZE_ANS_NOT_CAP  - Termcapeintrge fehlen (nicht force mode)
 * RESIZE_ANS_FAILED   - Ausfhrung schiefgegangen.
 * Return, inquire mode:
 * RESIZE_ANS_OK       - geht
 * RESIZE_ANS_NOT_TERM - geht nicht, kein Terminal
 * RESIZE_ANS_NOT_CAP  - force ntig, Termcapeintrag unvollstndig
 */
int
io_resize_terminal(int fdin, int fdout, int mode)
{
	const char *cap_request=NULL;
	const char *cap_report=NULL;
	const char *cap_cursor=NULL;
	const char *cap_store=NULL;
	const char *cap_restore=NULL;
	const char *cap_scrollregion=NULL;
#ifdef HAVE_TERMCAP
	char buf_request[1024];
	char buf_report[1024];
	char buf_cursor[1024];
	char buf_store[1024];
	char buf_restore[1024];
	char buf_scrollregion[1024];
#endif
	char *p;
	struct stat st0;
	struct stat st1;
	char rbuf[128];
	int y=-1,x=-1;
	BOOLEAN ok=FALSE;
	const char *termname;
#ifdef HAVE_TERMCAP
	output_fd=fdout;
#endif

#ifdef __MSDOS__
	if (fdin==0)
	{
		 /* das geht nunmal nicht ... */
		 return RESIZE_ANS_NOT_TERM;
	}
#endif

	if (!akt_connection.is_tty)
	{
		syslog(LOG_DEBUG,"resize: 0 ist not a tty");
		return RESIZE_ANS_NOT_TERM;
	}
	if (!isatty(fdout))
	{
		syslog(LOG_DEBUG,"resize: 1 ist not a tty");
		return RESIZE_ANS_NOT_TERM;
	}

	/* nun prfen, ob STDIN/OUT auf dasselbe Terminal zeigen 
	 * ohne HAVE_ST_RDEV ist das allerdings nichts wert, aber
	 * fr den Zweck ist das Timeout weiter unten gedacht
	 */
	if (fstat(fdin,&st0) || fstat(fdout,&st1) 
		|| !S_ISCHR(st0.st_mode) || !S_ISCHR(st1.st_mode))
	{
		syslog(LOG_DEBUG,"resize: stat failed, or no char device");
		return RESIZE_ANS_NOT_TERM;
	}
#ifdef HAVE_ST_RDEV
#if defined(major) && defined(minor)
	if (major(st0.st_rdev) != major(st1.st_rdev)
		|| minor(st0.st_rdev) != minor(st1.st_rdev))
#else
	if (st0.st_rdev!=st1.st_rdev)
#endif
	{
		syslog(LOG_DEBUG,"resize: stdin/out are different devices");
		return RESIZE_ANS_NOT_TERM;
	}
#endif

	termname=getenv("TERM");
	if (!termname)
		termname="unknown";

#ifdef HAVE_TERMCAP
	/* ncurses header files don't use "const". If this annoys you
	 * fix them. Don't ruin my sources.
	 */
	p=buf_request;      cap_request=tgetstr("u7",&p);
	p=buf_report;       cap_report=tgetstr("u6",&p);
	p=buf_cursor;       cap_cursor=tgetstr("cm",&p);
	p=buf_store;        cap_store=tgetstr("sc",&p);
	p=buf_restore;      cap_restore=tgetstr("rc",&p);
	p=buf_scrollregion; cap_scrollregion=tgetstr("cs",&p);
#endif

	/* die ncurses sind braindead: Damit der Termcap-Eintrag nicht 
	 * grer als 1024 Bytes wird, sind diese Codes hier eingespart ...
	 */
	if (!strcasecmp(termname,"linux") 
#ifdef linux
	|| !strcasecmp(termname,"console")
	|| !strcasecmp(termname,"cons80x25")
	|| !strcasecmp(termname,"cons80x30")
	|| !strcasecmp(termname,"cons80x40")
	|| !strcasecmp(termname,"cons80x50")
	|| !strcasecmp(termname,"con80x25")
	|| !strcasecmp(termname,"con80x30")
	|| !strcasecmp(termname,"con80x40")
	|| !strcasecmp(termname,"con80x50")
#endif
	)
	{
		if (!cap_report)
			cap_report="\033[%d;%dR";
		if (!cap_request)
			cap_request="\033[6n";
	}
	/* fr VT100 und xterm knnen wir sicher sein, da die entsprechenden
	 * Fhigkeiten in jedem Fall vorhanden sind
	 */
	if (!cap_report && (!stricmp(termname,"vt100") || strstr(termname,"xterm")))
		cap_report="\033[%d;%dR";
	if (!cap_request && (!stricmp(termname,"vt100") || strstr(termname,"xterm")))
		cap_request="\033[6n";
	if (!cap_report && mode==2)
		cap_report="\033[%d;%dR";
	if (!cap_request && mode==2)
		cap_request="\033[6n";
	if (!cap_cursor && mode==2)
		cap_cursor="\033[%d;%dH";


	if (!cap_request || !cap_report || !cap_cursor)
	{
		if (mode==1)
		{
			/* inquire mode, force ntig */
			return RESIZE_ANS_NOT_CAP;
		}
		syslog(LOG_DEBUG,"resize: terminal %s lacks capability",
			termname);
		return RESIZE_ANS_NOT_CAP; /* das hat keinen Zweck */
	}
	if (mode==1)
		return RESIZE_ANS_OK; /* kann unternommen werden */

	io_mode_work(fdin, FALSE); /* kein ECHO ... */
	if (cap_store && cap_restore)
	{
#ifdef HAVE_TERMCAP
		output_fd=fdout;
		tputs(cap_store,1,my_putchar);
#else
		fputs(cap_store,stdout);
#endif
		fflush(stdout);
	}

	/* Ausgabefenster ausschalten */
	if (cap_scrollregion)
	{
		char tbuf[256];
		tparam(cap_scrollregion,tbuf,sizeof(tbuf)-1,999,999);
		fputs(tbuf,stdout);
		fflush(stdout);
	}

	/* Cursor weit, weit vorstellen */
#ifdef HAVE_TERMCAP
	p=tgoto(cap_cursor,999,999);
	tputs(p,1,my_putchar);
#else
	{
		 char buf2[128];
		 sprintf(buf2,cap_cursor,999,999);
		 fputs(buf2,stdout);
	}
#endif
	fflush(stdout);

	/* Das Terminal fragen */
#ifdef HAVE_TERMCAP
	tputs(cap_request,1,my_putchar);
#else
	fputs(cap_request,stdout);
#endif
	fflush(stdout);

	/* die Antwort lesen. Mit 1 Sekunde timeout */
	x=80;
	y=25;
	if (read_answer(fdin, cap_report, rbuf, sizeof(rbuf)))
	{
		char *rbufp=rbuf;
		/* skip garbage ... */
		while(*rbufp && *rbufp!=*cap_report)
			rbufp++;
		if (*rbufp)
		{
			BOOLEAN inc;
			int got;
			/* be careful. We read evil input from the user */
			if (strstr(cap_report,"%i"))
				inc=TRUE;
			else
				inc=FALSE;
			cap_report="\033[%d;%dR";
			got=sscanf(rbufp,cap_report,&y,&x);
			if (got == 2 && x>0 && y>0 && x<999 && y<999)
			{
				io_change_term_size(0,x,y);
				ok=TRUE;
			}
			else
				syslog(LOG_DEBUG,"resize: scanned x,y=%d,%d, got %d, out of range",x,y,got);
		}
		else
			syslog(LOG_DEBUG,"resize: got no answer");
	}
	else
		syslog(LOG_DEBUG,"resize: timed out");

	/* Ausgabefenster nochmals ausschalten - diesmal richtig */
	if (cap_scrollregion)
	{
		char tbuf[256];
		tparam(cap_scrollregion,tbuf,sizeof(tbuf)-1,0,ok ? (y-1) : 24);
		fputs(tbuf,stdout);
		fflush(stdout);
	}

	if (cap_store && cap_restore)
	{
#ifdef HAVE_TERMCAP
		tputs(cap_restore,1,my_putchar);
#else
		fputs(cap_restore,stdout);
#endif
		fflush(stdout);
	}
	if (ok)
		return RESIZE_ANS_OK;
	return RESIZE_ANS_FAILED;
}

static void
alarm_handler(int signo TOM_ATTRIB_UNUSED)
{
	got_alarm=TRUE;
}

static BOOLEAN
read_answer(int fdin, const char *tmpl, char *buf, size_t max)
{
	RETSIGTYPE (*old_alarm)(int);
	char last;
	char c=0;
	char *p;
	size_t cnt;

	old_alarm=signal(SIGALRM,alarm_handler);
	got_alarm=FALSE;
	alarm(1);

    last = tmpl[strlen(tmpl) - 1];
	cnt=0;
	p=buf;
    while (read(fdin,&c,1)!=-1) 
	{
		if (got_alarm)
			break;
		if (!c) /* telnet-nullbytes stren hier nur */
			continue;
		*p++=c;
		if (++cnt==max-1)
			break;
		if (c==last)
			break;	
	}
	alarm(0);
	signal(SIGALRM,old_alarm);
	*p=0;
	if (last==c)
		return TRUE;
	return FALSE;
}

