#include "tomsysdep.h"
#include "portab.h"

#ifdef WANT_READLINE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/errno.h>

#include "libquark.h"
#include "tomsysdep.h"
#include "user.h"
#include "brett.h"
#include "cmd.h"
#include "modem.h"
#include "prototyp.h"
#include "io.h"
#include "box_conf.h"

extern int rl_ignore_completion_duplicates;

static char *command_generator (const char *,int);
static char *user_generator (const char *,int);
static void calc_command_common(char **matches);
static char *brett_generator (const char *,int);
static void calc_brett_common(char **matches);
static char * twoword_command_generator(const char *text, int state);
static char **quark_completion (const char *text, int start, int end);

static BOOLEAN init_done=FALSE;
static USERNR last_user_nummer;
static char hint;
extern USER user;
extern BRETT akt_brett;


/* Tell the GNU Readline library how to complete.  We want to try to complete
   on command names if this is the first word in the line, or on filenames
   if not. */
static void
initialize_readline (void)
{
	/* Allow conditional parsing of the ~/.inputrc file. */
	rl_readline_name = XSTRDUP("tom");

	/* Tell the completer that we want a crack first. */
	rl_attempted_completion_function = (CPPFunction *)quark_completion;

	init_done=TRUE;
}

extern int logout_modus;
extern BOOLEAN mehr_disabled;

static int 
my_rl_getc_func(FILE *f TOM_ATTRIB_UNUSED)
{
	int x=mbchar();
	if (x==0)
		return EOF;
	return x;
}

char *
do_readline(const char *prompt, const char hint_para)
{
	char *line;
	if (!init_done)
		initialize_readline();
	rl_ignore_completion_duplicates=0; /* Wegen Bug in alten Versionen */
	if (last_user_nummer != user.nummer)
	{
		HIST_ENTRY *got;
		while ((got= remove_history (0))!=NULL)
		{
			free(got->data);
			free(got->line);
			free(got);
		}
		last_user_nummer = user.nummer;
	}
	hint=hint_para;
	io_set_ECHO(0,TRUE); /* readline gibt nur aus, wenn Echo an */
	io_set_ONLCR(0,TRUE); /* readline gibt nur LF aus */
	rl_getc_function=my_rl_getc_func;
	line=readline(prompt ? prompt : "");
	io_set_ECHO(0,FALSE);
	io_set_ONLCR(0,FALSE);

	/* Hack Alarm */
	if (!akt_connection.is_tty && akt_connection.sends_cr
		&& akt_connection.sends_lf)
		mbchar(); /* eat \n */

	if (!line)
		return line;
	strtrim(line);
	if (*line)
		add_history(line);
	return line;
}

static void
quark_ignore_nothing (char **names TOM_ATTRIB_UNUSED)
{
	return;
}
static void
quark_ignore_anything (char **names)
{
	if (!*names)
		return;
#if 0
	names++; /* erstes Element wird gebraucht, es ist auch kein Resultat,
	          * sondern der gemeinsame Teil aller Resultate */
#endif
	while (*names)
	{
		free(*names);
		*names=NULL;
		names++;
	}
	return;
}

/* Hack fr Benutzereingaben - die Namen bestehen ja aus mehreren Worten */
static const char *completion_start_hint;

/* der scheinbare Unfug mit den ignore-Funktionen hat einen guten 
 * Grund: readline versucht filenamenexpandierung, wenn quark_completion
 * NULL zurckliefert.
 */
char **
quark_completion (const char *text, int start, int end TOM_ATTRIB_UNUSED)
{
	const char *first_char;
	BOOLEAN at_start=FALSE;
	char **matches;
	completion_start_hint=NULL;

	rl_ignore_some_completions_function = (Function *)quark_ignore_nothing;

	if (hint=='U') {
		/* Userkontext: Hier mu ein Username eingegeben werden */
		completion_start_hint=rl_line_buffer;
		matches= completion_matches (text, user_generator);
		if (!matches)
			rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
		return matches;
	} else if (hint=='B') {
		/* Brettkontext: Hier mu ein Brettname eingegeben werden */
		matches= completion_matches (text, brett_generator);
		if (!matches)
			rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
		calc_brett_common(matches);
		return matches;
	} else if (hint!='C') {
		/* irgendetwas anderes - da mu sicherlich noch etwas getan werden */
		char **matches1;
		char **matches2;
		char **x,**y;
		int z1=0,z2=0;
		completion_start_hint=rl_line_buffer;
		matches1= completion_matches (text, user_generator);
		if (!strchr(rl_line_buffer,' ')) /* Gruppennamen haben keine Spaces */
			matches2= completion_matches (text, brett_generator);
		else
			matches2=NULL;
		if (matches1)
		{
			if (!matches1[1])
				z1=1;
			else
			{
				for (x=&matches1[1];*x;x++)
					z1++;
			}
		}
		if (matches2)
		{
			if (!matches2[1])
				z2=1;
			else
			{
				for (x=&matches2[1];*x;x++)
					z2++;
			}
		}
		if (z1+z2==0)
		{
			rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
			return NULL;
		}
		matches=XMALLOC(sizeof(char *) * (1+z1+z2+1));
		matches[0]=XSTRDUP(""); /* da mu etwas stehen */
		y=&matches[1];
		if (matches1)
		{
			if (z1==1)
				*y++=matches1[0];
			else
			{
				free(matches1[0]);
				for (x=&matches1[1];*x;x++)
					*y++=*x;
				free(matches1);
			}
		}
		if (matches2)
		{
			if (z2==1)
				*y++=matches2[0];
			else
			{
				free(matches2[0]);
				for (x=&matches2[1];*x;x++)
					*y++=*x;
				free(matches2);
			}
		}
		*y=NULL;
		if (!matches[2])
		{
			free(matches[0]);
			matches[0]=matches[1];
			matches[1]=NULL;
		}
		else
			calc_brett_common(matches);

		rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
		return matches;
	}

	/* sind wir am Anfang der Zeile? */
	for (first_char=rl_line_buffer;isspace(*first_char);first_char++)
		/* nothing */;
	if (start==first_char-rl_line_buffer)
		at_start=TRUE;

	if (at_start) {
		/* Wort am Anfang einer Kommandozeile: Kommando zu vervollst. */
		matches= completion_matches (text, command_generator);
		if (!matches)
			rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
		calc_command_common(matches);
		return matches;
	} else {
		/* das *kann* sein:
		 * edit user kar:     COMMAND COMMAND2 incomplete-PARA
		 * edit use           COMMAND incomplete-COMMAND2
		 * edit blahbla       COMMAND incomplete-PARA
		 * edit blah bla      COMMAND PARA incomplete-para
		 * tzende Sache ... Wer hat diese blde Kommandosyntax erfunden?
		 */
		char *s;
		char *p;
		box_cmd_t *cmd=NULL;

		/* Kopie der fertigen Sachen anlegen */
		s=XMALLOC(start+1);
		memcpy(s,first_char,start);
		s[start]=0;
		strtrim(s);

		/* s enthlt nun den vollstndigen Teil */

		/* Enthlt der fertige Teil Leerzeichen? */
		p=strpbrk(s,"\t ");
		if (!p)
		{
			/* Nope, es ist: 
			 * "edit "use
			 * "edit "bla
			 * Ergo: Es ist ein einfaches oder ein zusammengesetztes Kommando.
			 * Einmal raten bitte ...
			 */
			matches=completion_matches (first_char, twoword_command_generator);
			if (matches)
			{
				free(s);
				calc_command_common(matches);
				return matches;
			}

			cmd=befehl_find(s,"");
			if (!cmd)
			{
				free(s);
				rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
				return NULL;
			}
			completion_start_hint=text;
		}
		else
		{
			/* "edit user uwe "o
			 * "edit user "uw
			 * "edit uwe "o
			 * dann brauchen wir das Kommando nicht mehr komplettieren,
			 * mssen aber herausbekommen, welches Kommando es ist
			 */
			char *q;
			*p++=0;
			while (isspace(*p))
				p++;
			strtrim(p);
			q=strpbrk(s,"\t ");
			if (q)
			{
				/* "edit user uwe "o */ 
				*q++=0;
				while (isspace(*q))
					q++;
				cmd=befehl_find(s,p);
				/* wenn cmd!=NULL -> edit user */
				if (!cmd)
				{
					/* "edit uwe karl "o */
					cmd=befehl_find(s,"");
					if (!cmd)
					{
						free(s);
						rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
						return NULL;
					}
					completion_start_hint=strstr(rl_line_buffer,p);
				}
				else
					completion_start_hint=strstr(rl_line_buffer,q);
			}
			else
			{
			 	/* "edit user "uw
			 	 * "edit uwe "o
			 	 */
				cmd=befehl_find(s,"");
				if (!cmd)
				{
					free(s);
					rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
					return NULL;
				}
				completion_start_hint=strstr(rl_line_buffer,p);
			}
		}
		free(s);

		/* an dieser schnen Stelle haben wir das Kommando (cmd)  */

		if (cmd->parameter_hint & CMDPARA_USER)
		{
			matches=completion_matches (text, user_generator);
			if (!matches)
				rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
			return matches;
		}
		if (cmd->parameter_hint & CMDPARA_BRETT)
		{
			matches=completion_matches (text, brett_generator);
			if (!matches)
				rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
			calc_brett_common(matches);
			return matches;
		}
		rl_ignore_some_completions_function = (Function *)quark_ignore_anything;
		return NULL;
	}
	return (NULL);
}

char *
command_generator(const char *text, int state)
{
	static box_cmd_t *ptr;
	static int len;
	int lvl=level();

	/* If this is a new word to complete, initialize now.  This includes
	saving the length of TEXT for efficiency, and initializing the index
	variable to 0. */
	if (!state) {
		ptr=befehl_get_anker();
		len = strlen (text);
	}
	else
		ptr=ptr->next;

	/* Return the next name which partially matches from the command list. */
	for (;ptr;ptr=ptr->next) {
		if (0==strnicmp(ptr->name_ext,text,len)
			&& (COMMAND_ALLOWED(ptr,lvl)))
		{
			char *s=XSTRDUP(ptr->name_ext);
			box_cmd_t *last;
			if (!ptr->next)
				return s;
			/* gleichlautende Befehle berspringen */
			for (last=ptr,ptr=ptr->next;ptr;last=ptr,ptr=ptr->next)
			{
				if (0!=stricmp(ptr->name_ext,last->name_ext))
					break;
			}
			ptr=last;
			return s;
		}
	}
	/* If no names matched, then return NULL. */
	return (NULL);
}

/*
 * we get called with text:
 * "edit    use"TAB
 * or:
 * "ed use"TAB
 * The latter case is somewhat ugly ...
 */
static char *
twoword_command_generator(const char *text, int state)
{
	static char *cmd=NULL;
	static char *para=NULL;
	static box_cmd_t *ptr;
	static int para_len;
	static int cmd_len;
	int lvl=level();

	if (!state) {
		const char *p;
		char *q;
		if (cmd)
			free(cmd);
		if (para)
			free(para);

		p=text;
		while (isspace(*p))
			p++;
		cmd=XSTRDUP(p);
		q=strpbrk(cmd,"\t ");
		*q++=0;
		while (isspace(*q))
			q++;
		para=XSTRDUP(q);

		para_len=strlen(para);
		cmd_len=strlen(cmd);

		ptr=befehl_get_anker();
	}
	else
		ptr=ptr->next;

	/* Return the next name which partially matches from the command list. */
	for (;ptr;ptr=ptr->next) {
		if (0==strnicmp(ptr->name_ext,cmd,cmd_len))
		{
			/* der Befehl mu einen Parameter haben */
			if (!ptr->para_ext || !*ptr->para_ext)
				continue;
			if (0!=strnicmp(ptr->para_ext,para,para_len))
				continue;
			if (COMMAND_ALLOWED(ptr,lvl))
			{
				char *s;
				char *q;
				/* return XSTRDUP(ptr->para_ext); reicht nicht!
				 * in der befehle.cnf stehen noch hufig genug
				 * wildcards ...
				 */
				s=XSTRDUP(ptr->para_ext);
				q=strstr(s,"{*}");
				if (q)
				{
					*q=0;
					strtrim(s);
				}
				return s;
			}
		}
	}
	/* If no names matched, then return NULL. */
	return (NULL);
}

/*
 * diese Funktion hat eine besondere Schwierigkeit: Benutzernamen 
 * sind lnger als ein Wort ... und man kann sich nicht darauf
 * verlassen, da der User nicht nur den Nachnamen eingibt, oder
 * keine Abkrzungen ...
 * Wir verwenden daher ub_find_username_multi.
 */
char *
user_generator(const char *text TOM_ATTRIB_UNUSED, int state)
{
	static LISTUSER *ptr =NULL;
	static char *vorbereich =NULL;
	static char *kopie =NULL;
	if (!state)
	{
		const char *pc;
		char *p;
		char *q;
		char last;
		ptr=ub_get_first();
		for (pc=completion_start_hint;isspace(*pc);pc++)
			/* nix */;

		free(kopie);
		free(vorbereich);
		kopie=entumlauten1(pc);
		XNULLCHECK(kopie);
		strlwr(kopie);
		for (p=kopie, q=kopie,last=0;*p;p++)
		{
			if (*p!=' ' || last!=' ')
				*q++=*p;
			last=*p;
		}
		*q=0;
		p=strrchr(kopie,' ');
		if (p)
		{
			*p++=0;
			vorbereich=kopie;
			while (isspace(*p))
				p++;
			kopie=XSTRDUP(p);
		}
		else
			vorbereich=NULL;
	}
	else
		ptr=ptr->next;
	if (!ptr)
		return NULL;

	while (TRUE)
	{
		char buf[512];
		char *p,*q;
		char *start;
		BOOLEAN ok=FALSE;
		entumlauten(ptr->name,buf);
		strlwr(buf);

		if (vorbereich)
		{
			/* Vorname schon eingegeben. */
			for (p=buf,q=vorbereich;*p==*q && *p; p++,q++)
				/* nichts */;
			if (*q==0)
			{
				ok=TRUE;
				while (isspace(*p))
					p++;
			}
		}
		else
		{
			ok=TRUE;
			p=buf;
		}
		/* herausfinden, wie weit beides identisch ist:
		 * buf:   Uwe Ohse
		 * kopie: Uwe O
		 */
		if (ok)
		{
			start=p;
			for (q=kopie;*p==*q && *p; p++,q++)
				/* nichts */;
			if (*q==0)
			{
				/* das ist ein Match */
				return XSTRDUP(start);
			}
		}
		ptr=ptr->next;
		if (!ptr)
			return NULL;
	}
	return NULL;
}

char *
brett_generator(const char *text, int state)
{
	static BRETTNR bnr;
	static char name[256];
	static size_t len;
	if (!state)
	{
		bb_canonical(text,name);
		len=strlen(name);

		bnr=0;
	}
	bnr++;
	for (;bnr<max_brettnr;bnr++)
	{
		if (!br_ind[bnr].brett)
			continue;
		if (0==strnicmp(br_ind[bnr].canon,name,len))
			return XSTRDUP(br_ind[bnr].brett->name);
	}
	return NULL;
}

static void
calc_brett_common(char **matches)
{
	int low=999999;
	char buf[256];
	char lowbuf[256];
	int i;
	/* bei 0..1 matches kommt die Readline damit klar */
	if (!matches || !matches[0]||!matches[1]||!matches[2])
		return;
	bb_canonical(matches[1],lowbuf);
	low=strlen(lowbuf);
	for (i=2;matches[i];i++)
	{
		char *p,*q;
		int l;
		bb_canonical(matches[i],buf);
		for (p=buf,q=lowbuf,l=0;*p==*q && *p;p++,q++,l++)
			/* nothing */;
		if (l<low)
		{
			low=l;
			strcpy(lowbuf,buf);
		}
	}
	free(matches[0]);
	matches[0]=XMALLOC(low+1);
	if (low)
		memcpy(matches[0],lowbuf,low);
	matches[0][low]=0;
}

static void
calc_command_common(char **matches)
{
	int low=999999;
	char buf[256];
	char lowbuf[256];
	int i;
	/* bei 0..1 matches kommt die Readline damit klar */
	if (!matches || !matches[0]||!matches[1]||!matches[2])
		return;
	strcpy(lowbuf,matches[1]);
	strlwr(lowbuf);
	low=strlen(lowbuf);
	for (i=2;matches[i];i++)
	{
		char *p,*q;
		int l;
		strcpy(buf,matches[i]);
		strlwr(buf);
		for (p=buf,q=lowbuf,l=0;*p==*q && *p;p++,q++,l++)
			/* nothing */;
		if (l<low)
		{
			low=l;
			strcpy(lowbuf,buf);
		}
	}
	free(matches[0]);
	matches[0]=XMALLOC(low+1);
	if (low)
		memcpy(matches[0],lowbuf,low);
	matches[0][low]=0;
}
#endif
