/* rm_dir_content(): delete everything in a directory */
/*
    Copyright (C) 1994-1998 1999 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <syslog.h>
#include <errno.h>

#include "tompaths.h"
#include "prototyp.h"

#ifndef S_IFMT
#error S_IFMT nicht definiert
#endif
#if !defined(S_ISDIR)
#ifndef S_IFDIR
#error S_IFDIR nicht definiert
#endif
#define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
#endif


/* Ordnerinhalt lschen. */
void 
rm_dir_content(const char *para)
{
	char pfad[PATH_MAX];
	char *p;
	DIR *d;
	struct dirent *e;
	struct stat st;

	strcpy(pfad,para);
	p=pfad+strlen(pfad)-1;
	while (p>=pfad && *p=='/')
		*p--=0;
	d=opendir(pfad);
	if (!d)
	{
		syslog(LOG_WARNING,"cannot opendir %s: %m", pfad);
		return;
	}
	p++; /* Auf Ende */
	*p++='/';

	while ((e=readdir(d))!=NULL)
	{
		if (e->d_name[0]=='.' 
			&& (!e->d_name[1] || (e->d_name[1]=='.' && !e->d_name[2])))
			continue;
		strcpy(p,e->d_name);
		if (0!=stat(pfad,&st))
		{
			/* Hm? Kein Leserecht? Weg damit ... */
			if (0!=unlink(pfad))
			{
				syslog(LOG_WARNING,"cannot unlink %s: %m", pfad);
				continue;
			}
		}
		if (S_ISDIR(st.st_mode))
		{
			rm_dir_content(pfad);
			if (0!=rmdir(pfad))
				syslog(LOG_WARNING,"cannot rmdir %s: %m", pfad);
			continue;
		}
		if (0!=unlink(pfad))
		{
			syslog(LOG_WARNING,"cannot unlink %s: %m", pfad);
			continue;
		}
	}
	closedir(d);
} /* loesche_ordner */
