/* routing.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef ROUTING_H
#define ROUTING_H

#include "msg.h"

/*
 *
 * Fr die Box wichtige Routinginformationen/Prototypen.
 *
 *
 */

typedef enum 
{
	R_LOG_INFO,   /* !Sie haben aber heute schon 100 mal getauscht! */
	R_LOG_NOTICE, /* !weitergeleitet an /dev/null@somewhere */
	R_LOG_ERROR,  /* ?Der Benutzer NixIs ist unbekannt */
	R_LOG_FATAL   /* ?Out of memory, aborting */
} r_loggrade_t;

/*
 * Diese Struktur beschreibt eine Adresse aus einer Liste.
 */
typedef struct r_adr_t 
{

	char *erroradr;    /* Adresse fr Fehlermeldungen. Aufrufer initial. */
	char *inputadr;    /* Adresse, wie sie einging */
	char *resultadr;   /* Envelopeadresse, das Resultat */
	char *username;    /* assozierter Username, nicht unbedingt lokal! */

	char *logstr;      /* "?Out of memory\r\n" */
	r_loggrade_t loggrade; 
	BOOLEAN error;     /* irgendein Fehler */
	ULONG errorflags;  /* ... hier genauer beschrieben */
	char *dsn;         /* DSN laut RFC 1893 */

#define R_E_TEMP            0x01       /* Try it again, Sam - but later */
#define R_E_ADR             0x02       /* Adresse falsch */
#define R_E_ACCESS          0x04       /* nicht erlaubt */
#define R_E_INTERN          0x08       /* out of memory etc */

	UWORD loopcount;   /* Zwecks Schleifenbegrenzung */

	BOOLEAN bezahlt;   /* Sender hat Zahlerstatus? */
	BOOLEAN sec_bezahlt;/* "secure bezahlt", fr Sicherheitskritisches */
	UWORD level;       /* Level des Senders */
	UWORD sec_level;   /* fr sicherheitskritische Abfragen */
	USERNR  owner;     /* "Besitzer" der Nachricht */

	ULONG flags;       /* diverse Flags */
#define R_F_NEUE_ID         0x01      /* neue ID erzeugen */
#define R_F_TAUSCHLOG       0x02      /* ins #LOG schreiben */
#define R_F_SPAETER_AUFL    0x04      /* Mailingliste, spter expandieren */
#define R_F_LOKAL           0x08      /* geht in Userpostfach */
#define R_F_NO_WILDCARD     0x10      /* keine Wildcardvergleiche */
#define R_F_NEWENVELOPE     0x20      /* Envelopeadresse verndert */
#define R_F_PSEUDO          0x40      /* unbenutzt, war Pseudo-Hack */

#define R_F_AUFGELOEST      0x0100    /* aufgeloest */
#define R_F_GEROUTET        0x0200    /* geroutet */
#define R_F_ZUGESTELLT      0x0400    /* zugestellt */

#define R_F_E_TRIED         0x1000    /* Errortransport hat sie angefat */

#define R_F_TO_POSTMASTER 0x010000    /* Kopie an Postmaster */
#define R_F_TO_OWNER      0x020000    /* Kopie an Owner */
#define R_F_TO_SENDER     0x040000    /* Kopie an Sender */
#define R_F_TO_MASK       0x070000    /* Maske fr die "Kopie an ..." */

	struct r_transport_t *transport; /* transportiert die Nachricht */
	USERNR user_gate_nr;      /* An oder durch ihn geht die Nachricht */
	void   *thint;         /* Hints fr den Transport */

	struct r_adr_t *next;     /* nchste Adresse */
	struct r_adr_t *bezug;    /* Interner Gebrauch only, errortransport */

	char *infotext; /* etwas das errortransport mit in den Text schreiben soll, mehrere Zeilen mglich */

} r_adr_t;

/**************************************************************************
 *                           Transports
 **************************************************************************
 */

typedef BOOLEAN (*r_t_deliver_fn) (message_t *head, r_adr_t *);
typedef BOOLEAN (*r_t_flush_fn) (struct r_transport_t *);

typedef struct r_t_option_t 
{
	const char *name;
	const char *value;
	struct r_t_option_t *next;
} r_t_option_t;

typedef struct r_transport_t
{

	const char *name;       /* Name des Transports */

	r_t_deliver_fn deliver; /* bernimmt die Zustellung */
	r_t_flush_fn flush;     /* flusht */

	void *priv_data;

	struct r_transport_t *next; /* ja, das ist eine Liste */
} r_transport_t;

#define TRANSPORT_ALLOWED(adr) (!adr->error && !(adr->flags & R_F_ZUGESTELLT))
#define ROUTING_ALLOWED(adr) (!adr->error && !(adr->flags & R_F_GEROUTET))

#if 0
	NOTYET
	/*
	 * Adressen auflsen, Teil 1.
	 */
BOOLEAN r_aufloeser(r_adr_t *adr);

	/*
	 * Adressen auflsen, Teil 2.
	 */
BOOLEAN r_aufloeser2(r_adr_t *adr);
#endif

	/*
	 * Liste von aufgelsten Adressen routen.
	 */
BOOLEAN r_router(message_t *msg, r_adr_t *adr);

	/*
	 * Analog SMTP VRFY: Prfen auf Zustellbarkeit.
	 * Teurer Aufruf - im Prinzip wird der meiste Teil des Routers
	 * verwendet.
	 */
BOOLEAN r_vrfy(const char *adr, char **comment, UWORD level, BOOLEAN bezahlt);

	/*
	 * Nachricht an alle Adressen zustellen.
	 */
BOOLEAN r_transport(message_t *msg, r_adr_t *adr, BOOLEAN do_errors);
BOOLEAN r_flush(void);

const char * routing_suche_alias(const char *uname);

	/* Hostnamen in vergleichbare Form bringen */
void r_hcanonical(const char *from, char *to);

	/* Uwe Ohse @ DU3 -> Uwe_Ohse@DU3.maus.de */
char *r_convert_to_inetadr(const char *iname);

	/* Strcmp-hnlicher Vergleich von Hostnamen */
int r_hostnamecmp(const char *dom1, const char *dom2);

	/* liefert den angegebenen Hostnamen domainisiert zurck, oder
	 * das Original, falls keine domainisierte Version erzeugbar ist
	 * Zurckgeliefert wird in jedem Fall ein malloc()eter Bereich!
	 */
char *r_domainize(const char *idomain);

r_adr_t *r_alloc(void);
void r_free(r_adr_t *adr);
void r_free_chain(r_adr_t *adr);
	/* Force nicht setzen, internal use only! */
void r_clean(r_adr_t *adr, BOOLEAN force);

/*
 * Prft ob der Name ein Alias fr die eigene Kiste ist.
 * "Adresse" kann "mausdu3.gun.de" oder "du3" sein, Usernamen
 * haben hier allerdings nichts verloren.
 */
BOOLEAN r_is_me(const char *host_name);

/*
 * Prft, ob "adr" a) auf genau eine Zieladresse expandiert wird und
 * b) diese Zieladresse ein lokaler User ist.
 * Return: 0 -> nein, sonst Nummer des Users.
 */
USERNR r_is_local_user(const char *adr);

/*
 * herausfinden, was fr den Benutzer in den Path eingetragen werden mu.
 * Funktioniert auch fr User (return NULL).
 */
const char * get_path_entry(USERNR usernr) TOM_ATTRIB_CONST;

/*
 * Name eines Routers zu einem Kennbuchstaben holen.
 */
const char * r_get_router_name(char kenn);

	/*
	 * Benutzung:
	 * einfachnachricht(FALSE,9,TRUE,"/tmp/nachricht",
	 * 'W', "betreff", 'G', "gruppe", ..., NULL);
	 * Kennzeichen und Inhalt also paarweise!
	 */
BOOLEAN einfachnachricht(BOOLEAN mail, UWORD lv, BOOLEAN bezahlt,
	const char *fname,...);

struct veinfachpara 
{
	char kenn;
	char *str;
};

BOOLEAN veinfachnachricht(BOOLEAN mail, UWORD lv, BOOLEAN bezahlt,
	const char *fname, struct veinfachpara *para);

void r_paniclog (const char *format,...) TOM_ATTRIB_PRINTF(1,2);

	/* if (!adr) then gruppennachricht! */
void r_tauschuserlog(r_adr_t *adr, const char *format, ...);

int r_set_verbose_grade(int ng);

BOOLEAN r_queuerun(void);

BOOLEAN r_st_newmail(const char *id, const char *from, simplelist_t *to,
    time_t eingabezeit);
BOOLEAN r_st_set_status(const char *id, const char *from, const char *to,
    const char *status, time_t zeit);

BOOLEAN r_st_export(FILE *to_file, time_t seit);
BOOLEAN r_st_expire(void);

/* don't ever free() this string! */
extern char *r_dsn_out_of_memory;
/* like vasprintf, with special treatment of "out of memory" */
char *r_get_dsn(const char *format,...);

#endif
