/*
    Copyright (C) 1994 1995 1996 1997 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
 * Wartebildschirm
 * Smtliche Funktionen drfen nur aufgerufen werden, wenn kein User
 * eingeloggt ist.
 */
#include "tomsysdep.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <signal.h>

#include "tom_termcap.h"

#include "portab.h"
#include "prototyp.h"
#include "befehle.h"
#include "user.h"
#include "brett.h"			/* wg. brettanzahl */
#include "modem.h"
#include "io.h"
#include "aktdata.h"
#include "box_conf.h"
#include "event.h"
#include "tompaths.h"
#include "argument.h"
#include "tomtcl.h"

extern BOOLEAN modem_vorhanden;

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif
#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif

static void print_at(int x,int y);

static int spaltenzahl=-1;
static int zeilenzahl=-1;
static char *linie;

#ifdef SIGWINCH
static RETSIGTYPE
winch_handler(int signo TOM_ATTRIB_UNUSED)
{
	spaltenzahl=-1;
}
#endif

static void
init_screen_data(void)
{
	int old_s=spaltenzahl;
	int old_z=zeilenzahl;
	if (!io_get_term_size(STDIN_FILENO,&spaltenzahl,&zeilenzahl))
		spaltenzahl=80;
	else 
	{
		if (spaltenzahl<80)
			spaltenzahl=80;
#ifdef SIGWINCH
		signal(SIGWINCH,winch_handler);
#endif
	}
	if (spaltenzahl!=old_s) {
		char tmp[128];
		sprintf(tmp,"COLUMNS=%d",spaltenzahl);
		putenv(tmp);
		free(linie);
		linie=NULL;
	}
	if (zeilenzahl!=old_z) {
		char tmp[128];
		sprintf(tmp,"LINES=%d",zeilenzahl);
		putenv(tmp);
	}
	if (!linie)
		linie=XMALLOC(spaltenzahl+1);
}

void init_menue(void)
{
#ifdef HAVE_LIBTCL
	char fname[PATH_MAX];
#endif
	if (spaltenzahl==-1)
		init_screen_data();
	tprint_fd(STDOUT_FILENO, "cl");

	printf("                   TOM (c)1989-97 FB/UO, Version: %s\n", VERSION);

	print_at(5,2); printf("Port ...........>");
	print_at(5,3); printf("Auslastung .....>");
	print_at(5,4); printf("Status .........>");

	print_at(spaltenzahl/2+5,2); printf("Nchster in ....>");
	print_at(spaltenzahl/2+5,3); printf("Event um .......>");
	print_at(spaltenzahl/2+5,4); printf("      ist ......>");

	update_menue();

	fflush(stdout);
#ifdef HAVE_LIBTCL
	sprintf(fname,"%sscreen.tcl",box_conf.tcl_dir);
	if (fileexist(fname)) {
		print_at(2,8); printf(" ");
		fflush(stdout);
		ttcl_script(fname,NULL);
	}
#endif

} /* init_menue */



void update_menue(void)
{
	time_t dif;
	event_t *ev;
	int ml;
	const char *port;
	if (argument_port)
		port=argument_port;
	else if (box_conf.modem_portname)
		port=box_conf.modem_portname;
	else
		port="";

	ml=(spaltenzahl/2)-25;

	print_at(23,2); printf("%s", port);
	print_at(23,3); printf("%d %%",auslastung());
	print_at(23,4); printf("------   ");
	ev=event_next_soft(&dif);
	if (ev)
	{
		dif /= 60; /* rechnen wir in Minuten */
		print_at(spaltenzahl/2+23,2);
		if (dif < 6*60)
			printf("%02ld:%02ld hh:min  ",dif / 60,dif % 60);
		else
			printf("%ld h  ",dif/60);
		print_at(spaltenzahl/2+23,3);
		printf("%02d:%02d  ",(int) (ev->soft_begin / 100), 
			(int) (ev->soft_begin % 100));
		print_at(spaltenzahl/2+23,4);
		printf("%-*.*s ",ml,ml,ev->id);
	}
	fflush(stdout);

} /* update_menue */



void display_status(const char *text)
{
	long len;
	if (spaltenzahl==-1)
		init_screen_data();

	len = strlen(text);

	memset(linie,' ',spaltenzahl);
	linie[spaltenzahl] = 0;
	memcpy(linie+(spaltenzahl-5-len)/2+1,text,len);
	print_at(1,6); printf("%s\r",linie);
	fflush(stdout);

} /* display_status */

#ifdef putchar
static int my_putchar(int c)
{
	return fputc(c,stdout);
}
#else
#define my_putchar putchar
#endif


static void print_at(int x,int y)
{
	static const char *tm = NULL;

#ifdef TGETENT_NULL
	if (!tm)
	{
		tm = tgetstr("cm",NULL);
#else
	if (!tm)
	{
		char *buf=malloc(256);
		if (buf)
			tm = tgetstr("cm",&buf);
			
#endif
		/* mit "skip parameter" (ncurses-Erweiterung) kommt unser tgoto() nicht zurecht */
		if (tm && strstr(tm,"%p")) {
			/* "increment the next two arguments" gesetzt? */
			if (strstr(tm,"%i"))
				tm = "\033[%i%d;%dH";
			else
				tm = "\033[%d;%dH";
		}
	}

	if (tm) {
		char *tmp=tgoto(tm,x,y);
		tputs(tmp,1,my_putchar);
	}

} /* print_at */
