/* t_maustausch.c: Deliver via maustausch */
/*
    Copyright (C) 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>
#include <time.h>
#include <sys/stat.h>

#include "box_conf.h"
#include "offline/mdb_glob.h"
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "modem.h"
#include "tompaths.h"
#include "routing.h"
#include "route_int.h"
#include "msg.h" /* write_msg fr #LOG */

#include "lockfile.h" /* libquark */

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

extern BRETTINFO *br_ind;

struct transports {
	FILE *f;
	char *fname;
	BOOLEAN do_news;
	BOOLEAN do_mail;
	char *cmd;
	struct transports *next;
};
static struct transports *t_anker;

static BOOLEAN r_t_maustausch_deliver(message_t *head, r_adr_t *adr);
static BOOLEAN r_t_maustausch_flush(struct r_transport_t *);

static BOOLEAN r_t_maustausch_deliver_one(FILE *f,message_t *head,r_adr_t *adr, const char *id);
static BOOLEAN r_t_maustausch_open(void);

r_transport_t *r_t_maustausch=NULL;

void
r_t_maustausch_define(char *str)
{
	static int basic_done=0;
	/* "deftransport maustausch [news|mail|*] [offline|CMD|{nix}]
	 * wir sehen:               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
	 */
	char *cmd;
	struct transports *nt;
	BOOLEAN do_news=FALSE;
	BOOLEAN do_mail=FALSE;
	cmd=strpbrk(str,"\t ");
	if (cmd) {
		*cmd++=0;
		while (isspace(*cmd))
			cmd++;
		if (!*cmd)
			cmd=NULL;
	}

	/* nicht unntig - gelesen aus routing.cnf! */
	if (!stricmp(str,"news"))
		do_news=TRUE;
	else if (!stricmp(str,"mail"))
		do_mail=TRUE;
	else if (!stricmp(str,"*")) {
		do_news=TRUE;
		do_mail=TRUE;
	} else {
		syslog(LOG_WARNING,"maustausch_define called with illegal filter %s",str);
		return;
	}
	nt=XMALLOC(sizeof(struct transports));
	nt->do_mail=do_mail;
	nt->do_news=do_news;
	nt->fname=NULL;
	nt->f=NULL;
	if (cmd) {
		nt->cmd=XSTRDUP(cmd);
	} else
		nt->cmd=XSTRDUP("offline");
	nt->next=t_anker;
	t_anker=nt;

	if (!basic_done) {
		r_transport_t *n;
		basic_done=1;
		n=XMALLOC(sizeof(r_transport_t));
		strlwr(str);
		n->name=XSTRDUP(str);
		n->deliver=r_t_maustausch_deliver;
		n->flush=r_t_maustausch_flush;
		n->next=NULL;
		r_t_maustausch=n;
		r_t_chain_transport(n);
	}
}

static FILE * 
r_t_maustausch_open_one(char **fname) 
{
	char *p;
	int i=0;

	char fn[PATH_MAX];

	sprintf (fn, "%s_i.%lu."  , box_conf.spool_dir, (ULONG) getpid());
	p=strlen(fn)+fn;
	while (TRUE)
	{	
		struct stat st;
		sprintf(p,"%d",++i);
		if (stat(fn,&st))
			break;
	}
	if (*fname)
		free(*fname);
	*fname=strdup(fn);
	if (!*fname)
	{
		r_paniclog("out of memory\n");
		r_tauschuserlog(NULL,":?out of memory\r\n");
		msglog("   ?out of memory\r\n");
		return FALSE;
	}
	
	/*
	 * Der Filename ist also uniq. Nun mssen wir noch dafr sorgen, da
	 * niemand auf die Idee kommt, die Datei einzufgen, whrend wir
	 * sie noch schreiben.
	 * Groartige Warteschleifen und Fehlerprfungen sind nicht 
	 * ntig. Nur die Box legt diese Datei an, und der Filename
	 * *IST* uniq.
	 */
	{
		char lockfname[PATH_MAX];
		strcpy(lockfname,*fname);
		strcat(lockfname,".LCK");
		lock_create(lockfname, NULL);
	}
	return fopen (*fname, BINWRITE);
}

static BOOLEAN
r_t_maustausch_open(void)
{
	return TRUE;
}

static BOOLEAN
r_t_maustausch_deliver_one(FILE *f,message_t *head,r_adr_t *adr, const char *id)
{
	LISTUSER *lu=NULL; /* keep gcc quiet */
	static int counter=0;
	FILE *in=NULL; /* keep gcc quiet */

	if (head->msgtyp==pers_msg)
	{
		/* Paranoia-Check */
		lu=ub_find_usernummer(adr->user_gate_nr);
		if (!lu)
		{
			r_paniclog("cannot find user nr. %lu\n",adr->user_gate_nr);
			r_tauschuserlog(adr,":?TSNH: kann Benutzer nicht finden: %lu\r\n",
				adr->user_gate_nr);
			msglog("   ?TSNH: kann Benutzer nicht finden: %lu\r\n",
				adr->user_gate_nr);
			adr->errorflags|=R_E_INTERN;
			adr->flags|=R_F_TO_POSTMASTER;
			adr->flags|=R_F_TO_SENDER;
			adr->error=TRUE;
			return FALSE;
		}
	}
	if (head->fname)
	{
		/* Text aus der Datei lesen */
		in=fopen(head->fname,"r");
		if (!in)
		{
			const char *s=strerror(errno);
			r_paniclog("cannot open inputfile %s: %s\n",head->fname,s);
			r_tauschuserlog(adr,":?TSNH: kann Nachricht nicht ffnen: %s\r\n",s);
			msglog("   ?TSNH: kann Nachricht nicht ffnen: %s\r\n",s);
			if (adr)
			{
				adr->errorflags|=R_E_INTERN;
				adr->flags|=R_F_TO_POSTMASTER;
				adr->flags|=R_F_TO_SENDER;
				adr->error=TRUE;
			}
			return FALSE;
		}
	}

	/* printf und Abarten sind in allen Abkmmlingen der GNU-Library
	 * extrem langsam - und vielleicht auch noch in anderen Systemen.
	 * Deshalb werden sie hier vermieden. Das Zeilenende einer Zeile
	 * wird immer vor der Ausgabe der nchsten Zeile geschrieben!
	 */
#define ONELINE(chr,x) do { fputs(LINEEND chr,f); fputs(x,f); } while(0)
#define IFEX(chr,x) if (head->x && *head->x) ONELINE(chr,head->x)

	putc('#',f); fputs(id,f); 
	fputs(LINEEND "W",f); fputs(head->betreff,f);
	fputs(LINEEND "E",f); fputs(head->eingabezeit,f);
	IFEX("-",ref_id);
	IFEX("V",absender); /* anonym? */
	IFEX("S",sender); 
	IFEX("T",reply_to);
	IFEX("O",organisation);
	IFEX("N",realname);
	IFEX("Y",gateway_zeile);
	if (head->msgtyp==pers_msg)
	{
		const char *p_A_name;
		ONELINE("B",head->bearbeitungsstatus);

		/*
		 * Header/Envelope-Trennung lt sich mit MausTausch nicht machen,
		 * es klappt nur in der lokalen Datenbank, nicht jedoch bei 
		 * Nachrichten an andere Boxen.
		 */

		/* In welche DB einsortieren? */
		ONELINE("a",lu->name);

		if (adr->flags & R_F_LOKAL)
			p_A_name=head->to->data;
		else
			p_A_name=adr->resultadr;

		/* das sollte zwar weiter oben erledigt sein, aber sicher ist 
		 * sicher.
		 */
		if (strpbrk(p_A_name,"@!%"))
			ONELINE("A",p_A_name);
		else
			fprintf(f,LINEEND "A%s @ %s",p_A_name,MYDOMAINPART);

		msglog("  Tmaustausch\n");
		msglog("  >U%lu %s\n",adr->user_gate_nr, p_A_name);
	}
	else
	{
		simplelist_t *akt;

		/*
		 * Gruppenzeilen schreiben.
		 */
		for(akt=head->gruppen;akt;akt=akt->next)
		{
			ONELINE("G",akt->data);
			if (head->junk_group)
				msglog("  >G%s (junked)\n",akt->data);
			else
				msglog("  >G%s\n",akt->data);
		}

		/*
		 * Followup-To:
		 */
		for(akt=head->followup;akt;akt=akt->next)
			ONELINE("F",akt->data);
		
		if (head->distribution_byte)
		{
			fputs(LINEEND "D",f);
			fputc(head->distribution_byte,f);
		}

		if (head->junk_group)
			ONELINE("g",box_conf.junk_group);
	}
	/* MIME-Headerzeile */
	IFEX("M",mimeinfo);
	IFEX("X",flags_zeile);
	/* 
	 * Erst nach allen anderen Headerzeilen, zur Optimierung von Offline. 
	 */
	if (head->msgtyp==gruppen_msg)
		IFEX("p",path_zeile);

	if (adr && adr->logstr && *adr->logstr)
		ONELINE(">",adr->logstr);

	if (head->uo_comments)
	{
		simplelist_t *com;
		for (com=head->uo_comments;com!=NULL;com=com->next)
		{
			ONELINE("!",com->data);
		}
	}
	if (head->comments)
	{
		simplelist_t *com;
		for (com=head->comments;com!=NULL;com=com->next)
		{
			ONELINE(">",com->data);
		}
	}

	/* die letzte Headerzeile mit CR/LF abschliessen */
	fputs(LINEEND,f); 

	if (head->fname)
	{
		char buf[1024];
		while (fgets(buf,sizeof(buf),in))
		{
			char *p=strpbrk(buf,"\r\n");
			if (p)
				*p=0;
			fprintf(f,":%s"LINEEND,buf);
		}
		fclose(in);
	}
	else
		fputs(head->body.buf,f);

	if (head->msgtyp==gruppen_msg)
	{
		/* Bei Netpoll und Gateways extrem ineffizient */
		simplelist_t *akt;
		for(akt=head->gruppen;akt;akt=akt->next)
			br_ind[akt->xulong].brett->flags|=B_UPDATE;
		if (head->junk_group)
			br_ind[head->junk_group].brett->flags|=B_UPDATE;
	}
	if (head->msgtyp==pers_msg || ++counter>20)
	{
		counter=0;
		if (fflush(f) != 0)
		{
			/* AUA! */
			const char *s=strerror(errno);
			r_paniclog("error flushing outputfile: %s",s);
			r_tauschuserlog(adr,":?Fehler bei fflush(): %s\r\n",s);
			msglog("   ?Fehler bei fflush(): %s\r\n",s);
			return FALSE;
		}
	}
	msglog("  =%s\n",id);
	r_tauschuserlog(adr,":=%s\r\n",id);
	return TRUE;
}

/* 
 * Hierhin kommen nur vollstndige Nachrichten, keine Spezialnachrichten
 * und keine Statusmeldungen.
 */
BOOLEAN
r_t_maustausch_deliver(message_t *head, r_adr_t *adr)
{
	BOOLEAN ok=TRUE;

	if (!r_t_maustausch_open())
	{
		adr->errorflags|=R_E_INTERN;
		adr->flags|=R_F_TO_POSTMASTER;
		adr->flags|=R_F_TO_OWNER;
		adr->error=TRUE;
		return FALSE;
	}

	if (head->msgtyp==pers_msg)
	{
		for ( ; adr; adr=adr->next)
		{
			if (adr->transport==r_t_maustausch
				&& TRANSPORT_ALLOWED(adr))
			{
				char idbuf[256];
				char *id;
				struct transports *t;

				if (adr->flags & R_F_NEUE_ID)
				{
					make_msg_id(idbuf,TRUE, NULL, NULL);
					id=idbuf;
				}
				else
					id=head->msg_id;
				for (t=t_anker;t;t=t->next) {
					if (!t->do_mail)
						continue;
					if (!t->f) {
						t->f=r_t_maustausch_open_one(&t->fname);
						if (!t->f) {
							const char *s=strerror(errno);
							r_paniclog("cannot open outputfile: %s\n",s);
							r_tauschuserlog(NULL,":?TSNH: kann Ausgabedatei nicht ffnen: %s\r\n",s);
							msglog("   ?TSNH: kann Ausgabedatei nicht ffnen: %s\r\n",s);
							continue;
						}
					}
					if (!r_t_maustausch_deliver_one(t->f,head,adr,id)) {
						r_writecomment(R_LOG_ERROR,adr,"Transport gescheitert");
						adr->error=TRUE;
						adr->errorflags|=R_E_INTERN;
						ok=FALSE;
					} else
						adr->flags |= R_F_ZUGESTELLT;
				}
			}
		}
	}
	else if (head->msgtyp==status_msg)
	{
		/* kommt hier wirklich irgendetwas an? */
#if 0
		for ( ; adr; adr=adr->next)
		{
			LISTUSER *lu;
			fprintf(f,"#%s\r\n",head->msg_id);
			fprintf(f,"B%s\r\n",head->bearbeitungsstatus);
			if (*head->eingabezeit)
				fprintf(f,"E%s\r\n",head->eingabezeit);
			else
			{
				time_t_to_mtzeit(&head->importzeit, head->eingabezeit);
				fprintf(f,"E%s\r\n",head->eingabezeit);
			}
			fprintf(f,"A%s\r\n",adr->resultadr);

			lu=ub_find_usernummer(adr->user_gate_nr);
			if (lu)
				fprintf (f, "a%s" LINEEND,lu->name);

			adr->flags |= R_F_ZUGESTELLT;
			r_tauschuserlog(adr,":=%s\r\n",head->msg_id);
			msglog("  =%s (Status)\r\n",head->msg_id);
		}
#else
		mbprintf("DEBUG: maustausch-deliver fr status_msg aufgerufen\r\n");
#endif

	}
	else
	{
		struct transports *t;
		for (t=t_anker;t;t=t->next) {
			if (!t->do_news)
				continue;
			if (!t->f) {
				t->f=r_t_maustausch_open_one(&t->fname);
				if (!t->f) {
					const char *s=strerror(errno);
					r_paniclog("cannot open outputfile: %s\n",s);
					r_tauschuserlog(NULL,":?TSNH: kann Ausgabedatei nicht ffnen: %s\r\n",s);
					msglog("   ?TSNH: kann Ausgabedatei nicht ffnen: %s\r\n",s);
					continue;
				}
			}
			if (!r_t_maustausch_deliver_one(t->f,head,NULL,head->msg_id))
				ok=FALSE;
		}
	}
	return ok;
}

BOOLEAN
r_t_maustausch_flush(struct r_transport_t *dummy TOM_ATTRIB_UNUSED)
{
	BOOLEAN ok=TRUE;
	char lockfname[PATH_MAX];
	struct transports *t;

	for (t=t_anker;t;t=t->next) {
		if (!t->f)
			continue;
		fputs("#\r\n",t->f);
		fclose(t->f);
		t->f=NULL;
		if (0==strcmp(t->cmd,"offline")) {
			if (!mdb_import (t->fname, TRUE)) {
				const char *s=strerror(errno);
				ok=FALSE;
				/* XXX hier mte das File gespoolt werden */
				r_paniclog("error importing into database: %s",s);
				r_tauschuserlog(NULL,":?Fehler bei Datenbankimport: %s\r\n",s);
				msglog("   ?Fehler bei offline-Import: %s\r\n",s);
			} else
				unlink(t->fname);
			strcpy(lockfname,t->fname);
			strcat(lockfname,".LCK");
			lock_delete(lockfname);
			free(t->fname);
			t->fname=NULL;
		} else {
			int ret;
			uostr_t cmd;cmd.data=0;

			uostr_xdup_cstr(&cmd,t->cmd);
			uostr_xadd_char(&cmd,' ');
			uostr_xadd_cstr(&cmd,t->fname);
			uostr_x0(&cmd);

			ret=system(cmd.data);
			uostr_freedata(&cmd);
			if (ret==0) {
				unlink(t->fname);
			} else {
				ok=FALSE;
				r_paniclog("error importing into external database: %d",ret);
				r_tauschuserlog(NULL,":?Fehler bei externem Datenbankimport: %d\r\n",ret);
				msglog("   ?Fehler bei externem Datenbank-Import: %d\r\n",ret);
			}
		}
	}

	if (ok) {
		BRETTNR bnr;
		for (bnr=1;bnr<max_brettnr;bnr++)
		{
			if (br_ind[bnr].brett && (br_ind[bnr].brett->flags & B_UPDATE))
			{
				bb_update_brett(bnr);
				br_ind[bnr].brett->flags &= ~(B_UPDATE);
			}
		}
	}
	return ok;
}
