/* ta_artcheck.c: Prfung eines bereits gelesenen Artikels.  */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "config.h"
/* have to include this early because of AIX */
#include "strdupa.h"

#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <syslog.h>
#include <time.h>

#ifdef OFFLINE2
#include "offline/inc/mdb_global.h"
#else
#include "offline/mdb_global.h"
#endif
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "modem.h"
#include "tompaths.h"
#include "routing.h"
#include "msg.h" /* write_msg fr #LOG */
#include "dynhash.h"
#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"
#include "crc32.h"


static int badmailfrom(const char *addrese) ;
#define BMF_NOTFOUND 0
#define BMF_USERFOUND 1
#define BMF_HOSTFOUND 2

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

extern BRETTINFO *br_ind;
enum id_typ { fqdn_id,spec_id,user_id, illegal_id };

/* **************************************************************************
 * Beschreibung der Funktionen mitsamt Prototypen
 ****************************************************************************
 */

#define ERRLOG(x) { \
	ta_msg_log("  ?%s\n",x); \
	ta_do_log_error(x); \
	}

static inline void canon_split_id(char *id) ;
static inline void canon_split_id_domainpart(char *id) ;
static BOOLEAN ta_is_in_path(char *path,const char *entry);
static void artcheck_id_setup(void);
static BOOLEAN artcheck_id(BOOLEAN is_mid, char *ziel, 
	const char *orig_id, const char **errortext);
static BOOLEAN ta_kombicheck(message_t *head, const char *was, const char *muss, 
	const char *darf);
static enum id_typ artcheck_get_idtyp(char *id, char **at);

/*****************************************************************************
 * Ende der Prototypen
 *****************************************************************************
 */

/****************************************************************************
 *
 * Message-ID-Prfung.
 *
 ****************************************************************************
 */

/*
** Weite Teile des folgenden Codes basieren auf Teilen der INN-Sourcen:
**  Rich Salz:
**    Check Message-ID format based on RFC 822 grammar, except that (as per
**    RFC 1036) whitespace, non-printing, and '>' characters are excluded.
**    Based on code by Paul Eggert posted to news.software.b on 22-Nov-90
**    in <#*tyo2'~n@twinsun.com>, with additional email discussion.
**    Thanks, Paul.
** Danke, Rich und Paul.
** Der Code ist auf die Quark angepat worden.
*/

/*
**  Flag array, indexed by character.  Character classes for Message-ID's.
*/
static char     ARTcclass[256];
#define CC_MSGID_ATOM   01
#define CC_MSGID_NORM   02
#define CC_HOSTNAME 04
#define ARTnormchar(c)  ((ARTcclass[(unsigned char)(c)] & CC_MSGID_NORM) != 0)
#define ARTatomchar(c)  ((ARTcclass[(unsigned char)(c)] & CC_MSGID_ATOM) != 0)
#define ARThostchar(c)  ((ARTcclass[(unsigned char)(c)] & CC_HOSTNAME) != 0)

static void
artcheck_id_setup (void)
{
	const char *p;
	int i;

	/* Set up the character class tables.  These are written a
	   * little strangely to work around a GCC2.0 bug. 
	 */
	memset (ARTcclass, 0, sizeof(ARTcclass));
	p = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	while ((i = *p++) != 0)
	{
		ARTcclass[i] = CC_HOSTNAME | CC_MSGID_ATOM | CC_MSGID_NORM;
	}
	p = "!#$%&'*+-/=?^_`{|}~";
	while ((i = *p++) != 0)
	{
		ARTcclass[i] = CC_MSGID_ATOM | CC_MSGID_NORM;
	}
	p = "\"(),.:;<@[\\]";
	while ((i = *p++) != 0)
	{
		ARTcclass[i] = CC_MSGID_NORM;
	}

	/* The RFC's don't require it, but we add underscore to the list of valid
	 * hostname characters. */
	ARTcclass['.'] |= CC_HOSTNAME;
	ARTcclass['-'] |= CC_HOSTNAME;
	ARTcclass['_'] |= CC_HOSTNAME;
}
 
 
	/*
	 * artcheck_id erfllt zwei Funktionen:
	 * 1. < und > entfernen, falls sie kommen. Bei References wird auch 
	 *    ein "," am Ende entfernt *vor* dem Suchen nach ">".
	 *    Die ID wird dann nach "ziel" geschrieben.
	 * 2. Die ID in "ziel" wird dann nach den von INN geklauten Regeln
	 *    auf Korrektheit geprft.
	 * "ziel" wird auf jeden Fall gesetzt. Es ist Aufgabe der aufrufenden
	 * Funktion, den Fehlercode zu prfen, da nur sie wei, ob es News
	 * oder Mail (dann sollte die Nachricht trotzdem angenommen werden)
	 * ist.
	 * is_mid: ID (TRUE) oder Referenz-ID (FALSE)?
	 * ziel:   Ausreichend groer Bereich fr die fertige ID.
	 * orig_id: Wie der Name sagt, die Eingangs-ID
	 * errortext:   Pointer auf eine Fehlerbeschreibung.
	 */
static BOOLEAN 
artcheck_id (BOOLEAN is_mid, char *ziel,
			 const char *orig_id, const char **errortext)
{
	char *p;
	char *id;
	static BOOLEAN init_done = FALSE;
	char *kopie;

	if (!init_done)
		artcheck_id_setup ();

	kopie=tom_strdupa(orig_id);
	id = kopie;

	/*
	 * Zuerst mal mit <x@y> verfahren (weg mit < und >).  */
	if (*id == '<')
	{
		BOOLEAN falsch = FALSE;

		id++;

		p = id + strlen (id) - 1;

		/* hufigen Bug ausgleichen, aber nur bei References */ 
		if (!is_mid && *p == ',' && p[-1] == '>')
			*p-- = 0;

		if (*p != '>')
		{
			if (errortext)
				*errortext = "Unbalanciertes <";
			falsch = TRUE;
		}
		else
			*p = '\0';
		strcpy (ziel, id);
		if (falsch)
			return FALSE;
	}
	else
	{
		/* beginnt nicht mit <, dann sollte da aber auch kein
		   > drin sein */
		if (strchr (id, '>'))
		{
			if (errortext)
				*errortext = "Unbalanciertes >";
			return FALSE;
		}
		strcpy (ziel, id);
	}

	p = strchr (ziel, '@');
	if (!p)
	{
		/* Ich knnte darauf schwren, da das hier nicht ankommt */ 
		 if (errortext)
			*errortext = "kein @";
		return FALSE;
	}
	if (strchr (p + 1, '@'))
	{
		if (errortext)
			*errortext = "mehrere @";
		return FALSE;
	}
	if (strpbrk (ziel, "\t "))
	{
		if (errortext)
			*errortext = "enthlt Whitespace";
		return FALSE;
	}

	/*
	 * Nun die kompliziertere Routine: */
	/* Scan local-part:  "< atom|quoted [ . atom|quoted]" */ 
	p = ziel;
	for (;; p++)
	{
		if (ARTatomchar (*p))
		{
			while (ARTatomchar (*++p))
				continue;
		}
		else
		{
			if (*p++ != '"')
			{
				if (errortext)
					*errortext="Illegales Zeichen im Localpart";
				return FALSE;
			}
			for (;;)
			{
				char c;
				switch (c = *p++)
				{
				case '\\':
					c = *p++;	/* FALLTHROUGH */
				default:
					if (ARTnormchar (c))
						continue;
					if (errortext)
						*errortext="Illegales gequotetes Zeichen";
					return FALSE;
				case '"':
					break;
				}
				break;
			}
		} /* else */
		if (*p != '.')
			break;
	}

	if (*p++ != '@')
	{
		if (errortext)
			*errortext="@ erwartet";
		return FALSE;
	}
	for (;; p++)
	{
		if (ARTatomchar (*p))
		{
			while (ARTatomchar (*++p))
				continue;
		}
		else
		{
			if (*p++ != '[')
			{
				if (errortext)
					*errortext="Illegales Zeichen im Domainpart";
				return FALSE;
			}
			for (;;)
			{
				char c;
				switch (c = *p++)
				{
				case '\\':
					c = *p++;	/* FALLTHROUGH */
				default:
					if (ARTnormchar (c))
						continue;
					/* FALLTHROUGH */
				case '[':
					if (errortext)
						*errortext="Illegale Verschachtelung von [";
					return FALSE;
				case ']':
					break;
				}
				break;
			}
		}
		if (*p != '.')
			break;
	}
	if (*p!='\0')
	{
		if (errortext)
			*errortext="Domainpart unparsbar";
		return FALSE;
	}
	/* ### nur, weil MAUS Probleme damit haben duerfte! */
	if (p-ziel>=HEADLEN-5)
	{
		if (errortext)
			*errortext="ID zu lang";
		return FALSE;
	}
	return TRUE;
}


#define KC(was,buch,bedingung) \
	if (bedingung) \
	{ \
		if (!strchr(muss,buch) && !strchr(darf,buch)) \
		{ \
			char zwi[256]; \
			sprintf(zwi,"%c-Zeile in %s nicht erlaubt", buch, was); \
			ERRLOG(zwi); \
			return FALSE; \
		} \
	} \
	else if (strchr (muss,buch)) \
	{ \
		char zwi[256]; \
		sprintf(zwi,"%c-Zeile in %s erforderlich", buch, was); \
		ERRLOG(zwi); \
		return FALSE; \
	} 

static BOOLEAN 
ta_kombicheck(message_t *head, const char *was, const char *muss, 
	const char *darf)
{
	KC(was,'#',*head->msg_id);
	KC(was,'-',*head->ref_id);
	KC(was,'V',*head->absender);
	KC(was,'S',*head->sender);
	KC(was,'T',*head->reply_to);
	KC(was,'A',head->to);
	KC(was,'W',*head->betreff);
	KC(was,'E',*head->eingabezeit);
	KC(was,'G',head->gruppen);
	KC(was,'F',head->followup);
	KC(was,'B',*head->bearbeitungsstatus);
	KC(was,'O',*head->organisation);
	KC(was,'N',*head->realname);
	KC(was,'D',head->distribution_byte);
	KC(was,'Y',head->gateway_zeile);
	KC(was,'p',head->path_zeile);
	KC(was,'X',head->flags_zeile);
	KC(was,'M',head->mimeinfo);
	return TRUE;
}

static BOOLEAN 
ta_is_in_path(char *path,const char *entry)
{
	char *p;
	char *orig=path;
	size_t l;
	if (!path || !*path)
		return FALSE;
	l=strlen(entry);
	while ((p=strstr(path,entry))!=NULL)
	{
		if ((p==orig || p[-1]=='!' || path[-1]==' ')
			&& (!p[l] || p[l]=='!' || p[l]==' '))
		{
			return TRUE;
		}
		path=p+1;
	}
	return FALSE;
}

#if 0
static void
ta_add_path_entry(char *path, const char *entry)
{
#define NOT_FOR_MAIL "!not-for-mail"
	size_t l=strlen(path);
	size_t el=strlen(entry);
	if (!*path)
	{
		memcpy(path,entry,el);
		memcpy(path+el,NOT_FOR_MAIL,sizeof(NOT_FOR_MAIL));
		return;
	}
	if (el+l+1<=HEADLEN)
	{
		memmove(path+el+1,path,l+1);
		memcpy(path,entry,el);
		path[el]='!';
		return;
	}
	/* Wre zu lang fr den Maustausch. Was jetzt kommt, entspricht
	 * keineswegs RFC1036 oder seinem Nachfolger. Naja.
	 */
	{
		char *s=malloc(el+l+2);
		char *p;
		if (!s)
		{
			sprintf(path,"%s!not-for-mail",entry);
			return;
		}
		sprintf(s,"%s!%s",entry,path);
		p=s+HEADLEN-2;
		while (*p!='!')
			p--;
		*p='\0';
		strcpy(path,s);
		free(s);
		return;
	}
}
#endif

/*
 * Unterscheidet drei Arten IDs:
 * 1. FQDN (enthlt @)
 * 2. spec (nur Buchstaben)
 * 3. User-IDs (sonstige)
 * 2 und 3 enthalten keine Spaces, sonst return illegal_id;
 */
static enum id_typ 
artcheck_get_idtyp(char *id, char **at_p)
{
	char *at;
	UWORD buch=0;
	UWORD ziff=0;
	UWORD space=0;
	UWORD sonst=0;
	char *p;

	at=strchr(id,'@');
	if (at) {
		if (at_p)
			*at_p=at;
		return fqdn_id;
	}
	if (!*id)
		return illegal_id;

	p=id;
	while (*p)
	{
		if (isalpha(*p))
			buch++;
		else if (isdigit(*p))
			ziff++;
		else if (isspace(*p))
			space++;
		else 
			sonst++;
		p++;
	}
	if (space)
		return illegal_id;
	if (ziff || sonst)
		return user_id;
	else
		return spec_id;
}


/* IDs gesplitteter Nachrichten wieder auf Ursprungsnachricht zurueckfuehren */
/* Verwendung: Statusnachrichten, References */
static inline void
canon_split_id_domainpart(char *dp) 
{
	char *p,*q;
	p=strchr(dp,':');
	if (!p)
		return;
	q=p+1;
	if (!*q)
		return;
	while(isdigit(*q))
		q++;
	if (*q)
		return;
	*p=0;
}
static inline void
canon_split_id(char *id) 
{
	char *at=strchr(id,'@');
	if (!at)
		return;
	canon_split_id_domainpart(at+1);
}


static BOOLEAN
artcheck_dupecheck(message_t *head)
{
	if (user.typ==UTYP_USER)
	{
		DUSTR *du;
		du=ta_userdupecheck (head->msg_id);
		if (du)
		{
			ta_do_log(":=%s" LINEEND,du->realid);
			ta_do_log(":!?Dupe zu %s\n",du->realid);
			ta_msg_log("  Dupe zu %s\n",du->realid);
			return FALSE;
		}
	}
#ifdef QUARK
	else
	{
		/* Policy: Hier wird nur auf Dupes in ffentlichen Nachrichten
		 * getestet. Der fr PMs, bei denen es darauf ankommt, in 
		 * welchem Postfach die Nachricht liegt, bleibt bei der
		 * Datenbank.
		 */
		char g[256];
		if (!msgid_find_infos(head->msg_id,g,NULL,NULL))
			return TRUE;
		if (head->msgtyp==gruppen_msg && *g)
		{
			ta_do_log(":=%s" LINEEND,head->msg_id);
			ta_do_log(":!?Dupe zu %s\n",head->msg_id);
			ta_msg_log("  Dupe zu %s\n",head->msg_id);
			return FALSE;
		}
	}
#endif
	return TRUE;
}

static BOOLEAN
artcheck_infileverkettung(message_t *head)
{
	DUSTR *du;
	/* Innerhalb des Infiles verkettete Nachrichten? */
	if (*head->ref_id && !strchr(head->ref_id,'@'))
	{
		/*
		 * Der User darf folgendes schicken:
		 *   #BLAH122
		 *   .....
		 *   #BLAH123
		 *   -BLAH122
		 * Anhand der Dupecheckinformationen wird das verkettet. 
		 * Kleiner Nachteil (?): Es kann auch ber Monate hinweg 
		 * verkettet werden, wenn die Dupecheckinformationen so
		 * lange gehalten werden.
		 */
		du=ta_userdupecheck (head->ref_id);
		if (!du)
		{
			ERRLOG("Kommentar-ID nicht zuzuordnen");
			return FALSE;
		}
		ta_msg_log("  !IV <- %s\n",head->ref_id);
		ta_msg_log("  !IV -> %s\n",du->realid);
		strcpy(head->ref_id,du->realid);
	}
	return TRUE;
}

static BOOLEAN
artcheck_get_ag(message_t *head)
{
	char g[HEADLEN]; /* Gruppe */
	char u[HEADLEN]; /* Postfach */
	char emp[HEADLEN];

	ta_msg_log("  !Zuord. von %s\n",head->ref_id);
	if (!msgid_find_infos(head->ref_id,g,u,emp))
	{
		ERRLOG("Kommentierte Nachricht nicht gespeichert");
		return FALSE;
	}

	ta_msg_log("  !Gruppe %s\n",g);
	ta_msg_log("  !Postfach %s\n",u);
	ta_msg_log("  !Absender %s\n",emp);
	
	if (*g || *emp)
	{
		simplelist_t *l;
		l=simple_alloc(*g ? g : emp);
		if (!l)
		{
			ERRLOG("out of virtual memory");
			return FALSE;
		}
		if (*g)
			head->gruppen=l;
		else
			head->to=l;
	}
	return TRUE;
}

static BOOLEAN
artcheck_zeilenkombinationspruefung(message_t *head)
{
	if (head->msgtyp==pers_msg)
	{
		switch(user.typ)
		{
		case UTYP_USER:
			/* wegen MAUS-Gehirnschaden - Benutzer drfen "Sender:" setzen,
			 * aber nicht "From:" 
			 */
			if (*head->absender && *head->sender) {
				ERRLOG("V- und S-Zeile nicht zusammen erlaubt");
				return FALSE;
			}
			if (!ta_kombicheck(head,"persnliche Nachricht",
				USER_KOMBI_MAIL_ZWANG,USER_KOMBI_MAIL_OPT))
				return FALSE;
			break;
		case UTYP_GATE:
			/* PMs prfen */
			if (!ta_kombicheck(head,"persnliche Nachricht",
				GATE_KOMBI_MAIL_ZWANG,GATE_KOMBI_MAIL_OPT))
				return FALSE;
			break;
		case UTYP_BOX:
			if (!ta_kombicheck(head,"persnliche Nachricht",
				BOX_KOMBI_MAIL_ZWANG,BOX_KOMBI_MAIL_OPT))
				return FALSE;
			break;
		}
	}
	else /* ffentliche Nachricht */
	{
		switch(user.typ)
		{
		case UTYP_USER:
			if (*head->absender && *head->sender) {
				ERRLOG("V- und S-Zeile nicht zusammen erlaubt");
				return FALSE;
			}
			if (!ta_kombicheck(head,"ffentliche Nachricht",
				USER_KOMBI_NEWS_ZWANG,USER_KOMBI_NEWS_OPT))
				return FALSE;
			break;
		case UTYP_GATE:
			if (!ta_kombicheck(head,"ffentliche Nachricht",
				GATE_KOMBI_NEWS_ZWANG,GATE_KOMBI_NEWS_OPT))
				return FALSE;
			head->msgtyp=gruppen_msg;
			break;
		case UTYP_BOX:
			/* V/A sind optional. A wegen Fido ( :-/ ), V wegen WAR */
			if (!ta_kombicheck(head,"ffentliche Nachricht",
				BOX_KOMBI_NEWS_ZWANG,BOX_KOMBI_NEWS_OPT))
				return FALSE;
			break;
		}
	}
	return TRUE;
}

static BOOLEAN
artcheck_mengenpruefung(message_t *head)
{
	ULONG newbytes = akt_anruf.total_msg_bytes + head->body.len;
	ULONG newzahl = akt_anruf.total_msg_zahl + 1;
	BOOLEAN ok = TRUE;
	BOOLEAN alarm_sysop = FALSE;

	if (box_conf.tausch_max_kb
		&& newbytes > box_conf.tausch_max_kb*1024UL)
	{
		ERRLOG ("Kilobytelimit berschritten");
		/* beim ersten berschreiten der Grenze den Sysop 
		 * anschreiben.
		 */
		if (akt_anruf.total_msg_bytes <= box_conf.tausch_max_kb)
			alarm_sysop=TRUE;
		ok = FALSE;
		logout_modus=TRUE;
		if (user.level>0 && user.level <8)
			user.level=1;
	}
	else if (box_conf.tausch_max_zahl
			 && newzahl > box_conf.tausch_max_zahl)
	{
		ERRLOG ("Mengenlimit berschritten");
		/* beim ersten berschreiten der Grenze den Sysop 
		 * anschreiben.
		 */
		if (akt_anruf.total_msg_zahl <= box_conf.tausch_max_zahl)
			alarm_sysop=TRUE;
		ok = FALSE;
		logout_modus=TRUE;
		if (user.level>0 && user.level <8)
			user.level=1;
	}
	if (alarm_sysop)
	{
		char *empfaenger=concat("Sysop @ ", MYDOMAINPART, NULL);
		char *absender=concat("Quark @ ", MYDOMAINPART, NULL);
		char *wegen=concat(
			"Nachrichtenzahl/Grenberschreitung durch ", 
			user.name, NULL);

		if (absender && empfaenger && wegen)
		{
			char mfname[PATH_MAX];
			time_t t=time(NULL);
			FILE *f;

			sprintf(mfname,"%swarn.tmp",tom_tmp_dir);
			f=xfopen(mfname,"w");
			fprintf(f,"Datum/Uhrzeit: %s\n\n",ctime(&t));
			fprintf(f, "Der Benutzer %s\n"									 
"berschritt mit %lu Nachrichten mit zusammen %lu KB das Limit\n",
				user.name, newzahl , newbytes / 1024);
			fprintf(f, 
"Die Nachrichten wurden protokolliert, aber nicht gesendet\n");
			fclose(f);
			einfachnachricht(TRUE,9,TRUE,mfname,
				'A', empfaenger,
				'V', absender,
				'W', wegen,
				0);
			unlink(mfname);
		}
		free(empfaenger);
		free(wegen);
		empfaenger=concat(user.name, " @ ", MYDOMAINPART, NULL);
		wegen=concat("Limitberschreitung", NULL);
		if (absender && empfaenger && wegen)
		{
			char mfname[PATH_MAX];
			FILE *f;

			sprintf(mfname,"%swarn.tmp",tom_tmp_dir);
			f=xfopen(mfname,"w");
			fprintf(f, 
"Sie berschritten das Mengenlimit. Bitte setzen Sie sich mit dem Sysop\n"
"in Verbindung.\n"
"\n"
"Ihre Box\n"
				);
			fclose(f);
			einfachnachricht(TRUE,9,TRUE,mfname,
				'A', empfaenger,
				'V', absender,
				'W', wegen,
				0);
			unlink(mfname);
		}

		free(empfaenger);
		free(absender);
		free(wegen);
		syslog(LOG_ERR,
			"Limitueberschreitung durch %s", user.name);
	}
	akt_anruf.total_msg_zahl = newzahl;
	akt_anruf.total_msg_bytes = newbytes;
	return ok;
}

static BOOLEAN
artcheck_check_user_flags(message_t *head)
{
	char *p;
	if (!head->flags_zeile || !*head->flags_zeile)
		return TRUE;
	p=head->flags_zeile;
	while (*p) {
		if (!p[1])
			return FALSE;
		if (*p=='E') {
			if (p[1]!='+' && p[1]!='-')
				return FALSE;
		} else
			return FALSE;
		p+=2;
	}
	return TRUE;
}

/* 
 * Message prfen, klassifizieren, Zugriffsrechte bei PMs und AMs prfen.
 * Vervollstndigen.
 * head: Die Nachricht.
 * first_zuordnung: FALSE=="Erste Zuordnung von G/A aufgrund von '-' gemacht"
 *                  TRUE =="diese kommt erst noch"
 *                  NULL: Irrelevant.
 * bounce_it: Rckgabeparameter, wird auf TRUE gesetzt, wenn eine Nachricht
 *            gebounced werden mu.
 * path_entry: Zu benutzender Eintrag fr die Pathzeile oder NULL.
 */
BOOLEAN
ta_artcheck(message_t *head, r_adr_t **adr, BOOLEAN *first_zuordnung, 
	const char *path_entry, BOOLEAN *bounce_it)
{
	enum id_typ id_typ=illegal_id;
	BOOLEAN ohne_ag_angabe=FALSE;
	BOOLEAN user_ist_moderator=FALSE;
	BOOLEAN absender_selbst_eingesetzt=FALSE;
	char *id_at;
	char *orig_id=tom_strdupa(head->msg_id);
	char *bmf_err=NULL;
	char *bmf_dsn=NULL;

	if (bounce_it)
		*bounce_it=FALSE;

	/* wenn keine *-Zeile kam, dann den Nachrichtentyp anhand der 
	 * Zeilenkombination feststellen.
	 *  
	 */
	if (head->msgtyp==false_msg)
	{
		/*
		 * Was fr eine Sorte IDs ist es denn?
		 */
		id_typ=artcheck_get_idtyp(head->msg_id,&id_at);
		if (id_typ==illegal_id)
		{
			/* 
			 * der ID-Typ wird fr viel zu viel Logik in den folgenden
			 * Zeilen bentigt. Daher *muss* die ID einer der drei 
			 * Gruppen angehren, sonst geht die Nachricht - egal was sie
			 * ist - in die Tonne. Der Fall sollte nicht auftreten.
			 */
			ERRLOG("Message-ID-Mll: Whitespace und kein @");
			if (head->gateway_zeile)
				ta_msg_log("  !via %s\n",head->gateway_zeile);
			return FALSE;
		}

		/* 
		 * Nachrichten mit Spezial-IDs sind Spezialnachrichten, und werden 
		 * gesondert behandelt. Um die Programmierung diverser Unterroutinen
		 * zu erleichtern wird angenommen, da keine Spezialnachricht leer
		 * ist, mit Ausnahme der CMD/LOG/REN-Blcke.
		 */
		if (id_typ==spec_id)
		{
			/* Nachricht ohne Text? */
			if ((!head->body.buf || !*head->body.buf)
				&& strcmp(head->msg_id,"CMD")!=0
				&& strcmp(head->msg_id,"LOG")!=0
				&& strcmp(head->msg_id,"REN")!=0)
			{
				ERRLOG("Nachrichten ohne Text illegal");
				return FALSE;
			}
			if (!strcmp(head->msg_id,"REN"))
				head->msgtyp=ren_msg;
			else if (!strcmp(head->msg_id,"NOCEM"))
				head->msgtyp=nocem_msg;
			else if (!strcmp(head->msg_id,"HEAD"))
				head->msgtyp=head_msg;
			else if (!strcmp(head->msg_id,"LOG"))
				head->msgtyp=log_msg;
			else if (!strcmp(head->msg_id,"CMD"))
				head->msgtyp=cmd_msg;
			else if (!strcmp(head->msg_id,"CNF"))
				head->msgtyp=cnf_msg;
			else if (!strncmp(head->msg_id,"OUT",3))
				head->msgtyp=out_msg;
			else  if (ta_find_xinfofile(head->msg_id,user.nummer))
				head->msgtyp=info_msg;
			else
			{
				char zwi[256]; 
				sprintf(zwi,"Illegale Spezialnachrichten-ID %s", head->msg_id);
				ERRLOG(zwi); 
				return FALSE; /* unbekannter Block */
			}
			goto got_typ;
		} /* spec_id */
		if (head->bearbeitungsstatus  && *head->bearbeitungsstatus
			&& (!head->body.buf || !*head->body.buf))
		{
			head->msgtyp=status_msg;
			goto got_typ;
		}
		/* Nachricht ohne Text? */
		if (!head->body.buf || !*head->body.buf)
		{
			if (head->msgtyp==pers_msg && user.typ!=UTYP_USER) {
				/* persnliche leere Nachrichten nicht wegwerfen. */
				if (head->body.bufsize<1024) {
					free(head->body.buf);
					head->body.bufsize=0;
					head->body.buf=NULL;
				}
				if (!head->body.buf) {
					head->body.bufsize=1024;
					head->body.buf=XMALLOC(1024);
				}
				sprintf(head->body.buf,
						":Quark %s: Nachricht kam ohne Text\r\n",
						box_conf.systemname);
				head->body.next=strlen(head->body.buf)+head->body.buf;
			} else {
				ERRLOG("Nachrichten ohne Text illegal");
				return FALSE;
			}
		}
		/*
		 * Nachrichten ohne Gruppen/Empfnger-Angabe bearbeiten. 
		 */
		if (!head->to && !head->gruppen && user.typ==UTYP_USER)
		{
			ohne_ag_angabe=TRUE;
			if (!artcheck_get_ag(head))
				return FALSE;
		}

		/*
		 * Empfnger oder Gruppen brauchen wir auf jeden Fall.
		 */
		if (!head->to && !head->gruppen)
		{
			ERRLOG("Weder Gruppe noch Empfnger angegeben");
			return FALSE;
		}

		/* Feststellen ob ffentlich oder persnlich */
		if (head->gruppen)
			head->msgtyp=gruppen_msg;
		else
			head->msgtyp=pers_msg;

		goto got_typ;
	} /* keine *-Zeile erhalten */

got_typ:

	/* kein Defaultblock, Fehlerprfung des Compilers ist hier wichtig */
	switch(head->msgtyp)
	{
		case false_msg: 
			return FALSE;

		case ren_msg:
		case nocem_msg:
		case quark_msg:
		case head_msg:
		case log_msg:
		case cmd_msg:
		case cnf_msg:
		case out_msg:
			/* Nachricht ohne Text? */
			if (!head->body.buf || !*head->body.buf)
				return FALSE;
			return TRUE;
		case info_msg:
			if (!head->body.buf || !*head->body.buf)
				return FALSE;
			if (!head->msg_id || !*head->msg_id)
			{
				ERRLOG("Kein Infofilename angegeben");
				return FALSE;
			}
			return TRUE;

		case pers_msg:
		case gruppen_msg:
		case status_msg:
			break;
	}

	/* ab hier kennen wir sicher den Typ der Nachricht */

	/*
	 * Kmmern wir uns nun um Statusnachrichten. Die haben keinen
	 * Text, aber einen Bearbeitungsstatus.
	 */
	if (head->msgtyp==status_msg)
	{
		/* Prfen wir erst mal, ob die ID ok ist */
		const char *errortext=NULL;
		if (!head->bearbeitungsstatus || !*head->bearbeitungsstatus)
		{
			ERRLOG("Statusnachricht braucht Status");
			return FALSE;
		}
		if (!head->msg_id || !*head->msg_id)
		{
			ERRLOG("Statusnachricht ohne ID");
			return FALSE;
		}
		if (head->body.buf && *head->body.buf)
		{
			ERRLOG("Statusnachricht mit Text illegal");
			return FALSE;
		}
		if (id_typ!=fqdn_id)
		{
			ERRLOG("Statusnachricht braucht FQDN-ID");
			return FALSE;
		}
		/* mit gesplitteten Nachrichten verfahren */
		canon_split_id_domainpart(id_at); /* x@y:25 -> x@y */
		/* dies ist eventuell bertrieben */
		if (!artcheck_id(TRUE,head->msg_id,head->msg_id,&errortext))
		{
			char *s;
			s=concat("Message-ID-Mll: ",errortext,NULL);
			XNULLCHECK(s);
			ERRLOG(s);
			if (head->gateway_zeile)
				ta_msg_log("  !via %s\n",head->gateway_zeile);
			free(s);
			return FALSE;
		}
		/* Soweit, so gut. Nun Zeilenkombinationen prfen.
		 * Protokollerweiterungen gg. MAUS: A-Zeile erlaubt,
		 * V-Zeile fr Gates und Boxen 
		 */
		/* So, es ist eine Statusnachricht */
		if (user.typ==UTYP_BOX
			&& !ta_kombicheck(head,"Statusmitteilung","#EB","AV"))
			return FALSE;
		if (user.typ==UTYP_GATE
			&& !ta_kombicheck(head,"Statusmitteilung","#B","EAV"))
			return FALSE;
		if (user.typ==UTYP_USER
			&& !ta_kombicheck(head,"Statusmitteilung","#B","EA"))
			return FALSE;
		return TRUE;
	}


	/* 
	 * Eine ganz normale Nachricht (News oder Mail, weder Spezial noch
	 * Status).
	 */

	/* moeglicherweise ist das weiter oben schon erledigt worden! */
	if (id_typ==illegal_id)
		id_typ=artcheck_get_idtyp(head->msg_id,&id_at);

	/* Schutzmanahme gegen den Idioten ... */
	if (user.typ == UTYP_USER && head->msgtyp==gruppen_msg)
	{
		if (!artcheck_mengenpruefung(head))
			return FALSE;
	}

	/* 
	 * ID-Typen prfen.
	 */
	if (user.typ==UTYP_USER)
	{
		if (id_typ==fqdn_id)
		{
			ERRLOG("Illegale Message-ID (FQDN nicht erlaubt)");
			return FALSE;
		}
		else if (strlen(head->msg_id)>250)
		{
			ERRLOG("ID zu lang (maximal 250 Zeichen)");
			return FALSE;
		}
	}
	else
	{
		if (id_typ!=fqdn_id)
		{
			ERRLOG("Illegale Message-ID (FQDN erforderlich)");
			return FALSE;
		}
	}

	/*
	 * Dupecheck durchfhren.
	 */
	if (!artcheck_dupecheck(head))
		return FALSE;

	/* 
	 * Kmmern wir uns nun um innerhalb des Infiles verkettete Nachrichten.
	 */
	if (user.typ==UTYP_USER)
	{
		if (!artcheck_infileverkettung(head))
			return FALSE;
	}

	if (user.typ!=UTYP_USER && head->msgtyp==pers_msg && head->absender) {
		switch(badmailfrom(head->absender)) {
		case BMF_USERFOUND:
			bmf_err=XSTRDUP("sender is in badmailfrom list / Absender unerwnscht");
			bmf_dsn=r_get_dsn("5.7.1 (sender is in badmailfrom list)");
			break;
		case BMF_HOSTFOUND:
			bmf_err=XSTRDUP("sending host is in badmailfrom list / Absender unerwnscht");
			bmf_dsn=r_get_dsn("5.7.1 (sending host is in badmailfrom list)");
			break;
		default: /* nicht gefunden -> not bad -> ok */
			break;
		}
	}

	/*
	 * Nachrichten ohne Gruppen/Empfnger-Angabe bearbeiten. Z.Z. nur
	 * fr User - aber man knnte das auch fr Quark 1.x erlauben wollen.
	 * Hm.
	 */
	if (!head->to && !head->gruppen && user.typ==UTYP_USER)
	{
		ohne_ag_angabe=TRUE;
		if (!artcheck_get_ag(head))
			return FALSE;
	}

	if (head->msgtyp==pers_msg)
	{
		if (!head->to || !*head->to->data)
		{
			ERRLOG("kein Empfnger angegeben");
			return FALSE;
		}
	}
	else if (head->msgtyp==gruppen_msg)
	{
		if (!head->gruppen)
		{
			ERRLOG("keine Gruppe angegeben");
			return FALSE;
		}
	}

	/*
	 * Empfnger oder Gruppen brauchen wir auf jeden Fall.
	 */
	if (!head->to && !head->gruppen)
	{
		/* Kann man von Gates irgendwo verlangen, oder? */
		ERRLOG("Weder Gruppe noch Empfnger angegeben");
		return FALSE;
	}
	/* 
	 * Gruppen *und* Empfnger ist nicht gut (gewesen?? Check offline!)
	 */
	if (head->gruppen && head->to)
	{
		/* Wegen Fido-Misfeature */
		if (!strchr(head->to->data,'@'))
		{
			free(head->to->data);
			if (head->to->xstring)
				free(head->to->xstring);
			free(head->to);
			head->to=NULL;
		}
		else
		{
			ERRLOG("Gruppe *und* Empfnger verboten");
			return FALSE;
		}
	}

	/* Zeilenkombinationsprfung */
	if (!artcheck_zeilenkombinationspruefung(head))
		return FALSE;


	if (id_typ==illegal_id)
	{
		if (head->msgtyp==gruppen_msg)
		{
			ERRLOG("Message-ID-Mll: Whitespace und kein @");
			if (head->gateway_zeile)
				ta_msg_log("  !via %s\n",head->gateway_zeile);
			return FALSE;
		}
		else
		{
			ta_do_log(":!Message-ID-Mll: %s" LINEEND , head->msg_id);
			ta_msg_log("  !Message-ID-Mll: %s\n", head->msg_id);
			if (head->gateway_zeile)
				ta_msg_log("  !via %s\n",head->gateway_zeile);
		}
	}

	/* 
	 * Syntaxcheck auf Referenz-ID
	 */
	if (*head->ref_id)
	{
		enum id_typ ref_typ;
		const char *errortext=NULL;
		BOOLEAN fehler=FALSE;
		char *at;

		ref_typ=artcheck_get_idtyp(head->ref_id,&at);
		if (ref_typ!=fqdn_id)
		{
			fehler=TRUE;
			errortext="Keine FQDN-ID";
		}
		else 
		{
			/* Kommentare auf gesplittete Nachrichten auf Teil 1 umbiegen.
			 * Sonst kann die Nachricht bei anderen Empfaenger, die sie
			 * ungesplittete bekamen, nicht verkettet werden.
			 */
			canon_split_id_domainpart(at+1); /* x@y:25 -> x@y */
			/* Dies bernimmt auch das Wegwerfen von < und > */
			if (!artcheck_id(FALSE,head->ref_id,head->ref_id,&errortext))
				fehler=TRUE;
		}

		if (fehler)
		{
			if (!errortext)
				errortext="";
			if (user.typ==UTYP_USER)
			{
				ta_do_log(":?Referenz-ID-Mll: %s: %s" LINEEND ,errortext,
					head->ref_id);
				ta_msg_log("  ?Referenz-ID-Mll: %s: %s" LINEEND ,errortext,
					head->ref_id);
				if (head->gateway_zeile)
					ta_msg_log("  !via %s\n",head->gateway_zeile);
				return FALSE;
			}
			else
			{
				ta_do_log(":!Referenz-ID-Mll: %s: %s" LINEEND ,errortext,
					head->ref_id);
				ta_msg_log("  !Referenz-ID-Mll: %s: %s" LINEEND ,errortext,
					head->ref_id);
				if (head->gateway_zeile)
					ta_msg_log("  !via %s\n",head->gateway_zeile);
				/*
				 * Die Referenz-ID wird nicht zerstrt. Grund: *Vielleicht*
				 * kann sie ja noch irgendwo verkettet werden. Mag sein,
				 * da man das nochmal berdenken mu.
				 */
			}
		}
	}
	/* 
	 * Syntaxcheck auf Message-ID
	 */
	if (id_typ==fqdn_id)
	{
		const char *errortext=NULL;
		if (!artcheck_id(TRUE,head->msg_id,head->msg_id,&errortext))
		{
			if (head->msgtyp==gruppen_msg)
			{
				/* Bei News ist die Policy: Weg mit allem, was Dupe sein
				 * knnte. Und das bedeutet: Weg mit dieser Nachricht.
				 * Zu viele Gateways schreddern IDs. Schne Gre ans
				 * Fido, AOL und Fidozerb :-(
				 */
				char *buf;
				asprintf(&buf,"Message-ID-Mll: %s",errortext);
				XNULLCHECK(buf);
				ERRLOG(buf);
				free(buf);
				if (head->gateway_zeile)
					ta_msg_log("  !via %s\n",head->gateway_zeile);
				return FALSE;
			}
			else
			{
				/*
				 * Bei Mail ist die Policy: Dafr sorgen, da es ankommt.
				 * Ja, ich halte News fr weniger wertvoll als Mail.
				 */
				ta_do_log(":!Message-ID-Mll: %s: %s" LINEEND ,errortext,head->msg_id);
				ta_msg_log("  !Message-ID-Mll %s: %s" LINEEND ,errortext,head->msg_id);
				if (head->gateway_zeile)
					ta_msg_log("  !via %s\n",head->gateway_zeile);
			}
		}
	}

	/* 
	 * Falls kein Datum angegeben ist, dann eines erzeugen. Hm, ob das
	 * bei Gates und Boxen sinnvoll ist, darf bezweifelt werden.
	 */
	if (!*head->eingabezeit
		&& (user.typ==UTYP_USER || head->msgtyp!=gruppen_msg))
	{
		time_t_to_mtzeit(&head->importzeit,head->eingabezeit);
		head->eingabezeit_tt=head->importzeit;
	}
	else if (!*head->eingabezeit)
	{
		ERRLOG("Eingabedatum fehlt"); /* should not happen *here* */
		return FALSE;
	}
	else
	{
		if (!mtzeit_to_time_t(head->eingabezeit,&head->eingabezeit_tt))
		{
			if (head->msgtyp==gruppen_msg || user.typ==UTYP_USER)
			{
				ERRLOG("Eingabezeit unparsbar");
				return FALSE;	
			}
			else
			{
				ta_do_log(":!datum unparsbar" LINEEND);
				ta_msg_log("  !datum %s unparsbar\n",head->eingabezeit);
			}
		}
	}
	/*
	 * PMs ohne Status bekommen nun BNnow.
	 */
	if (head->msgtyp==pers_msg && !*head->bearbeitungsstatus)
	{
		*head->bearbeitungsstatus='N';
		time_t_to_mtzeit(&head->importzeit,head->bearbeitungsstatus+1);
	}
	/*
	 * Kein Absender? Dann diesen User einsetzen. Das passiert wegen
	 * der obigen Zeilenkombinationsprfung nur fr User.
	 */
	if (!*head->absender)
	{
		size_t l=strlen(user.name);
		memcpy(head->absender,user.name,l);
		memcpy(head->absender+l," @ ",3);
		strcpy(head->absender+l+3,MYDOMAINPART);
		absender_selbst_eingesetzt=TRUE;
	}

	if (user.typ==UTYP_USER) {
		int got=0;
		if (head->flags_zeile) {
			if (!artcheck_check_user_flags(head)) {
				ERRLOG("Flags unparsbar");
			}
		}
		/* Adressverfremdung */
		if (head->flags_zeile) {
			char *fp;
			for (fp=head->flags_zeile;*fp;fp+=2) {
				if (*fp=='E')
					got=(fp[1]=='+') ? 1 : -1;
			}
		}
		if (!got) {
			if (user.flags & UB_FLAG_IMMER_AEXPIRE && user.address_hash_expire) {
				if (head->flags_zeile) {
					char *nf=xconcat(head->flags_zeile,"E+",NULL);
					free(head->flags_zeile);
					head->flags_zeile=nf;
				} else {
					head->flags_zeile=XSTRDUP("E+");
				}
			}
		}
	}


	/* Altersprfungen */
	{
		/* Altersprfungen erlaubt? 
		 * Nur fr News und PMs von Usern (weil MAUS PMs wegwirft, wenn
		 * sie zu alt sind).
		 * ### box_conf.local_mail_age_limit notwendig.
		 */
		if (box_conf.news_age_limit>0 
			&& (head->msgtyp==gruppen_msg ||user.typ==UTYP_USER))
		{
			if (head->eingabezeit_tt + box_conf.news_age_limit*86400L
				< head->importzeit)
			{
				ERRLOG("Nachricht zu alt");
				return FALSE;	
			}
		}
		/* Nachricht zu jung? n Stunden Zeitfehler erlaubt, aber nicht mehr.
		 * Bei INN ist das Limit 24 Stunden.
		 */
		if (box_conf.news_youth_limit >= 0 
			&& head->eingabezeit_tt > 
				head->importzeit + box_conf.news_youth_limit * 3600L
			&& (user.typ==UTYP_USER || head->msgtyp==gruppen_msg))
		{
			ERRLOG("Nachrichtendatum in der Zukunft");
			return FALSE;	
		}
	}

	/*
	 * Die Nachricht ist nun technisch OK. Nun Routing und Schreibrechts-
	 * prfung durchfhren.
	 */
	if (head->msgtyp==pers_msg)
	{
		simplelist_t *to;
		r_adr_t *last;

		last=*adr;
		if (last)
		{
			while (last->next)
				last=last->next;
		}
		if (!head->to)
		{
			ERRLOG("keine Empfnger angegeben");
			(*adr)->dsn=r_get_dsn("5.1.0 (no recipient given)");
			return FALSE;
		}
		to=head->to;
		while (to)
		{
			r_adr_t *this;
			this=r_alloc();
			if (!this)
			{
				ERRLOG("out of virtual memory");
				(*adr)->dsn=r_get_dsn(NULL);
				return FALSE;
			}
			this->inputadr=strdup(to->data);
			this->erroradr=strdup(head->absender);
			this->bezahlt=user.bezahlt;
			this->sec_bezahlt=user.bezahlt;
			this->level=level();
			this->sec_level=user.level; /* ??? Paranoia? */
			this->owner=user.nummer;
			if (!this->inputadr||!this->erroradr)
			{
				if (this->inputadr)
					free(this->inputadr);
				if (this->erroradr)
					free(this->erroradr);
				r_free(this);
				ERRLOG("out of virtual memory");
				(*adr)->dsn=r_get_dsn(NULL);
				return FALSE;
			}
			this->flags |= R_F_TAUSCHLOG;

			/*
			 * Nun an die Kette aneinhngen 
			 */
			this->next=NULL;
			if (last)
				last->next=this;
			else
				*adr=this;
			last=this;

			if (bmf_err){
				(*adr)->logstr=bmf_err;
				(*adr)->dsn=bmf_dsn;
				if (bounce_it)
					*bounce_it=TRUE;
				this->error=TRUE;
			}

			to=to->next;
		}
		if (ta_passinfo.send_pe)
		{
			/* Kopie an mich selbst - mal sehen, wie das ausfllt */
			r_adr_t *this;
			this=r_alloc();
			if (!this)
			{
				ERRLOG("out of virtual memory");
				(*adr)->dsn=r_get_dsn(NULL);
				return FALSE;
			}
			this->inputadr=strdup(user.name);
			if (!this->inputadr)
			{
				r_free(this);
				ERRLOG("out of virtual memory");
				(*adr)->dsn=r_get_dsn(NULL);
				return FALSE;
			}
			this->erroradr=NULL;
			this->bezahlt=user.bezahlt;
			this->sec_bezahlt=user.bezahlt;
			this->level=level();
			this->next=NULL;
			if (last)
				last->next=this;
			else
				*adr=this;
			last=this;

			to=to->next;
		}
		if (bmf_err || !r_router(head,*adr))
		{
			if (bounce_it)
				*bounce_it=TRUE;
			return FALSE;
		}

		/* mal eben schaun, ob eine Adresse gebouncet werden mute */
		{
			r_adr_t *this;
			this=*adr;
			while (this)
			{
				if (this->error)
				{
					if (bounce_it)
						*bounce_it=TRUE;
					break;
				}
				this=this->next;
			}
		}
	}
	else if (head->msgtyp!=pers_msg)
	{
		/*
		 * ffentliche Nachricht
		 */
		BOOLEAN found;
		BOOLEAN all_anonym;
		simplelist_t *akt;
		BOOLEAN first;
		char moderator[256];
		first=TRUE;

	recheck:

		*moderator=0;
		all_anonym=TRUE;
		found=FALSE;

		/* Gibt es die Gruppe berhaupt? */
		for (akt=head->gruppen;akt != NULL; akt=akt->next)
		{
			BRETTNR bnr;
			BRETT *b;
			bnr = brettnr (akt->data);
			if (bnr)
			{
				b=br_ind[bnr].brett;
				/* offiziellen Brettnamen verwenden. 
				 * wir erkennen "maus.info" und "maus,info" als gleich,
				 * das gilt jedoch nicht fr die Transportebene und
				 * Datenbanken.
				 */
				if (0!=stricmp(akt->data,b->name)) {
					free(akt->data);
					akt->data=XSTRDUP(b->name);
				}
				akt->xulong=(ULONG) bnr;
				found=TRUE;
				if (b->flags & B_MITGLIED)
				{
					/* schreiben nur fr Mitglieder */
					if (!b_tst_access(bnr,user.nummer))
					{
						ERRLOG(tprintf(
							"Schreiben in dieser Gruppe nur fr Mitglieder: %s",
							akt->data));
						return FALSE;
					}
				}
				else
				{
					/* Normale Zugriffsberechtigung */
					if (b->flags & B_PASSWORT && level() < 8)
					{
						ERRLOG(tprintf(
							"Passwortgeschtzte Gruppe %s: "
							"Schreiben nur online mglich",
							akt->data));
							return FALSE;
					}
					if (b->flags & B_ACL)
					{
						char fname[PATH_MAX];
						const char *err=NULL;
						sprintf(fname,"%s%s",brettpfad(bnr),"acl.lst");
						if (!acl_check(fname,user.name,level(), 'W', &err))
						{
							char buf[1024];
							if (!err)
								err="Sie sind nicht die in "
								    "Zugriffskontrolliste eingetragen.\r\n";
							mbprintf(" Schreibzugriff verweigert." "\r\n"
								" %s." "\r\n"
								" Bitte an 'Chef %s' schreiben." "\r\n" "\n",
								err, akt->data);
							ERRLOG(tprintf(buf,"%s: %s",akt->data,err));
							return FALSE;
						}
					}
					if (!(b->flags & B_WRITE))
					{
						ERRLOG(tprintf(
							"Schreiben in dieser Gruppe nicht erlaubt: %s",
							akt->data));
						return FALSE;
					}
				}
				if (b->art == BINAER_BRETT)
				{
					ERRLOG(tprintf("Tausch in diese Gruppe nicht mglich: %s",
						akt->data));
					return FALSE;
				}
				if (b->flags & B_MODERIERT && user.typ == UTYP_USER)
				{
					/* Aieee ... alles, was wir gemacht haben, war fr die
					 * Katz.
					 * Probleme:
					 * 1. Was passiert bei Nachrichten in mehrere mod. Gruppen?
					 *   -> geht nur an einen Mod. Hm, das Usenet hat 
					 *      dasselbe Problem.
					 * 2. Informationen ber Crosspostings knnen wir nicht
					 *    transportieren, da die G-Zeile in PMs nicht
					 *    weiter transportiert werden darf.
					 */
					char *mod;
					size_t modlen;
					if (*moderator)
					{
						/* hmm ... wir hatten schon einen? */
						ERRLOG( "Es kann nur in jeweils eine "
							"moderierte Gruppe geschrieben werden");
						return FALSE;
					}
					mod=GetModeratorAddress(akt->data);
					if (!mod || !*mod)
					{
						ERRLOG("Keine Moderatoradresse bekannt.");
						return FALSE;
					}
					modlen=strlen(mod);
					if (modlen>sizeof(moderator)-1)
					{
						ERRLOG("Moderatoradresse zu lang.");
						return FALSE;
					}
					memcpy(moderator,mod,modlen+1);
				}
				if (!(b->flags & B_ANONYM))
					all_anonym=FALSE;
			}
		} /* for durch alle angegebenen Bretter */
		if (*moderator)
		{
			/* So, jetzt mssen wir noch herausfinden, ob der
			 * User selbst der Mod. ist und posten darf.
			 */
			USERNR unr;
			unr=r_is_local_user(moderator);
			if (unr!=user.nummer)
			{
				/* Nein */
				head->msgtyp=pers_msg;
				simple_free_chain(head->gruppen);
				head->gruppen=NULL;
				*head->absender=0;
				simple_free_chain(head->to);
				head->to=NULL;
				head->to=simple_alloc(moderator);
				if (!head->to)
				{
					ERRLOG("out of virtual memory");
					return FALSE;
				}
				ta_do_log(":!an Moderator: %s" LINEEND , moderator);
				goto got_typ;
			}
			ta_do_log(":!Sie sind der Moderator: %s" LINEEND , moderator);
			user_ist_moderator=TRUE;
		}
		if (first && !found && user.auto_grp_create 
			&& head->gruppen->next==NULL)
		{
			/* Das ist ein Hack. Der Level fr Boxen und Gates mu temporr 
			 * gendert werden, weil die Kommandos zum Anlegen der Gruppe
			 * und zum ndern der Rechte erst ab hohem Level aufrufbar 
			 * sind.
			 */
			char cmd[256+20];
			int old_tmp_level=temp_levelW;

			if (user.typ!=UTYP_USER)
				temp_levelW=9;

			sprintf(cmd,"GA%s",head->gruppen->data);
			ta_cmd_work (cmd);
			/* Zugriffsrechte setzen */
			sprintf(cmd,"GZ%s=R%dW%d",head->gruppen->data,	
				(int) user.level, (int) user.level);
			ta_cmd_work (cmd);
			if (user.typ!=UTYP_USER)
			{
				/* Vernetzen */
				sprintf(cmd,"GS%s=+",head->gruppen->data);
				ta_cmd_work (cmd);
			}

			temp_levelW=old_tmp_level;

			ta_do_log(":!Gruppe angelegt, bitte editi%seren: %s" LINEEND , 
				box_conf.ti ? "ti" : "", head->gruppen->data);
			first=FALSE;
			/* "cmd" enthlt hinterher Fehlermeldung */
			ta_brett_bestellen(brettnr (head->gruppen->data),cmd);
			goto recheck;
		}	
		if (!found)
		{
			if (!box_conf.junk_group || !*box_conf.junk_group 
				|| user.typ==UTYP_USER)
			{
				ERRLOG(tprintf("Gruppe(n) unbekannt: %s",head->gruppen->data));
				return FALSE;
			}
			else
			{
				static BRETTNR bnr=(BRETTNR) -1;
				if (bnr == (BRETTNR) -1)
					bnr = brettnr (box_conf.junk_group);
				if (bnr<=0)
				{
					ERRLOG(tprintf("Gruppen und junk unbekannt: %s",
						head->gruppen->data));
					return FALSE;
				}	
				/* nicht ERRLOG, junk wird spter noch gelogt */
				ta_do_log(":!Gruppe unbekannt -> %s" LINEEND,
					box_conf.junk_group);
				ta_msg_log("  !Gruppe unbekannt -> %s" LINEEND,
					box_conf.junk_group);
				head->junk_group=bnr;
			}
			all_anonym=FALSE; /* don't know ... */
		}
		if (head->distribution_byte)
		{
			char c=head->distribution_byte;
			if (c!='M' && c!='N' && c!='L' && user.typ!=UTYP_BOX)
			{
				ERRLOG(tprintf("Illegale Distribution: %s",c));
				return FALSE;
			}
		}
		else
		{
			/* MAUS holt das aus der kommentierten Nachricht - es sein denn,
			 * die ist nicht da, dann wird Net eingesetzt. Ein *sehr*
			 * zuverlssiges und vorhersagbares Verfahren. Sarkasmus off.
			 */
			head->distribution_byte='N';
		}

		if (all_anonym)
			*head->absender='\0';
		else if (!*head->absender)
		{
			size_t l=strlen(user.name);
			strcpy(head->absender,user.name);
			strcpy(head->absender+l," @ ");
			strcpy(head->absender+l+3,MYDOMAINPART);
		}

		/* Path ergnzen */
		{
			static size_t ldom=0;
			if (ldom==0)
				ldom=strlen(box_conf.domainname);
		}
		if (!head->path_zeile)
		{
			/* wir mssen selbst eine Path-Zeile erzeugen */
			if (path_entry)
				head->path_zeile=concat(box_conf.domainname, "!", 
										path_entry, "!not-for-mail",NULL);
			else
				head->path_zeile=concat(box_conf.domainname, "!not-for-mail",
										NULL);
			XNULLCHECK(head->path_zeile);
		}
		else if (!ta_is_in_path(head->path_zeile,box_conf.domainname))
		{
			/* schon eine vorhanden. Ergnzen */
			char *old=head->path_zeile;
			head->path_zeile=concat(box_conf.domainname,"!",old,NULL);
			free(old);
			XNULLCHECK(head->path_zeile);
		}
	}

	/*
	 * Gatewaykennung anhngen.
	 */
	if (user.typ==UTYP_GATE)
	{
		if (head->gateway_zeile)
			free(head->gateway_zeile);
		head->gateway_zeile=concat(user.name," @ ", MYDOMAINPART,NULL);
		XNULLCHECK(head->gateway_zeile);
	}

#if 0
	/* sollte nun, dank S/V-Zeilen Behandlung in artcheck_zeilenkombinationsprfung,
	 * anders gemacht werden.
	 */
	/*
	 * Spezialbehandlung fr lokale Moderatoren.
	 */
	if (user.typ==UTYP_USER)
	{
		if (!user_ist_moderator && !absender_selbst_eingesetzt)
		{
			ERRLOG("Absenderangabe nicht gestattet");
			return FALSE;
		}
	}
#else
	if (user.typ==UTYP_USER)
	{
		/* wenn `V' von Benutzer gesetzt, dann `S'ender erzeugen */
		if (!absender_selbst_eingesetzt) {
			size_t l=strlen(user.name);
			memcpy(head->sender,user.name,l);
			memcpy(head->sender+l," @ ",3);
			strcpy(head->sender+l+3,MYDOMAINPART);
#if 0
		} else if (*head->sender) {
			/* im umgekehrten Fall - der nur wegen gehirntotem Verhaltens der MAUS
			 * notwendig ist - Absender erzeugen */
			size_t l=strlen(user.name);
			memcpy(head->absender,user.name,l);
			memcpy(head->absender+l," @ ",3);
			strcpy(head->absender+l+3,MYDOMAINPART);
#endif
		}
	}

#endif

	/* 
	 * Wenn die Nachricht noch keine gltige ID hat, dann jetzt eine 
	 * erzeugen. Dies ist *keine* Dupequelle, alle notwendigen 
	 * Prfungen sind oben schon geschehen. Hier bekommen nur 
	 * Nachrichten lokaler Benutzer neue IDs.
	 */
	if (id_typ!=fqdn_id)
	{
		make_msg_id(head->msg_id,head->msgtyp==pers_msg,head->eingabezeit,NULL);
	}

	/* 
	 * Nun keine Fehler mehr melden, eine ID ist erzeugt! 
	 */
	if (ohne_ag_angabe && *first_zuordnung)
	{
		if (head->msgtyp==pers_msg)
		 	ta_do_log(":!Empfngerangabe fehlt, wurde automatisch ergnzt!\r\n");
		 else
		 	ta_do_log(":!Gruppenangabe fehlt, wurde automatisch ergnzt!\r\n");
		 *first_zuordnung=FALSE;
	}

#if 0
	/* 
	 * Logeintrag vervollstndigen. Geschieht nun in der Transportebene.
	 */
	ta_do_log (":=%s\r\n", head->msg_id);
	ta_msg_log("  =%s\n",head->msg_id);
#endif

	/*
	 * Und Dupecheckinformationen eintragen.
	 */
	if (user.typ==UTYP_USER)
	{	
		ta_userdupecheck_add_info (orig_id, head->msg_id, head->importzeit);

		user.txt_bytes+=head->body.len;
		user.geschriebene_mails++;
	}
	return TRUE;
}


static int
badmailfrom(const char *addresse) 
{
	static BOOLEAN init_done=FALSE;
	static void *hash_handle = NULL;
	DYN_ENTRY *e;
	DYN_ENTRY such;
	char *p,*q;

	if (!box_conf.badmailfrom)
		return BMF_NOTFOUND;
	if (!init_done) {
		FILE *f;
		char buf[1024];
		init_done++;
		hash_handle=hhcreate(2000);
		XNULLCHECK(hash_handle);
		if (box_conf.badmailfrom[0]=='/')
			f=fopen(box_conf.badmailfrom,"r");
		else {
			char fname[PATH_MAX];
			sprintf(fname,"%s%s",box_conf.config_dir,box_conf.badmailfrom);
			f=fopen(fname,"r");
		}
		if (!f)
			return BMF_NOTFOUND;
		while (fgets(buf,sizeof(buf),f)) {
			p=strchr(buf,'\n');
			if (p)
				*p=0;
			strlwr(buf);
			such.u.ptr = 0;;
			such.crc = crc32ansi (buf, strlen (buf));
			e = hhsearch (hash_handle, &such, DYN_FIND);
			if (e)
				continue;
			such.u.ptr = XSTRDUP(buf);
			hhsearch (hash_handle, &such, DYN_ENTER);
		}
		fclose(f);
	}
	p=r_convert_to_inetadr(addresse);
	XNULLCHECK(p);
	strlwr(p);
	such.u.ptr = 0;;
	such.crc = crc32ansi (p, strlen (p));
	e=hhsearch(hash_handle,&such,DYN_FIND);
	if (e && !strcmp(e->u.ptr,p)) {
		free(p);
		return BMF_USERFOUND;
	}
	q=strchr(p,'@');
	if (q) {
		such.u.ptr = 0;;
		such.crc = crc32ansi (q, strlen (q));
		e=hhsearch(hash_handle,&such,DYN_FIND);
		if (e && !strcmp(e->u.ptr,q)) {
			free(p);
			return BMF_HOSTFOUND;
		}
	}
	free(p);
	return BMF_NOTFOUND;
}
