#line 1 "ta_cmd_tcl.c4"
/* hi emacs, this is -*- C -*-
 */

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <syslog.h>
#include <ctype.h>
#include <errno.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "aktdata.h"
#include "user.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_cmd.h"
#include "box_conf.h"
#include "tomtcl.h"

#ifdef HAVE_LIBTCL
/* TOMTCL_FUNCTION_DOKU([[
tom_add_saugfile DATEI
    hngt DATEI an die Liste der zu downzuloadenden Files an.
]])
TOMTCL_FUNCTION(tom_add_saugfile,ttcl_add_saugfile)
*/
int
ttcl_add_saugfile(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
{
	FILE *f;
	const char *p;

    Tcl_ResetResult (interp);
    TTCL_EXITCHECK ();

    if (argc != 2)
    {
        Tcl_SetResult (interp, "Benutzung: tom_add_saugfile DATEI", TCL_STATIC);
        return TCL_ERROR;
    }

    f = fopen (tprintf ("%s%ld/%s", box_conf.spool_dir, user.nummer, SAUGCNTRL), BINAPPEND);
    if (!f) {
        ta_do_log (":?nicht moeglich: fopen: %s" LINEEND, strerror (errno));
        ta_msg_log ("  ?nicht moeglich: fopen: %s\n", strerror (errno));
        Tcl_SetResult (interp, "cannot open file", TCL_STATIC);
        return TCL_ERROR;
    }
    p=strrchr(argv[1],'/');
    if (!p)
        p=argv[1];
    fprintf (f, "%s\t%s\r\n", argv[1], p);
    fclose (f);

    return TCL_OK;
}
#endif


#ifndef HAVE_LIBTCL
void
ta_cmd_read_tcl(void)
{
}
#else
void
ta_cmd_read_tcl(void)
{
#define MAXDESCLEN 8192
	char fname[PATH_MAX];
	char *dbuf;
	size_t len;
	DIR *d;
	struct dirent *e;
	char *ep;

	sprintf(fname,"%sta_cmd",box_conf.tcl_dir);

	d=opendir(fname);
	if (!d)
		return;
	dbuf=XMALLOC(MAXDESCLEN);
	len=0;
	ep=fname+strlen(fname);
	*ep++='/';

	while ((e = readdir (d)) != NULL) {
		FILE *f;
		BOOLEAN ok=FALSE;
		char buf[2048];
		ta_command_t *neu=NULL;
		if (*e->d_name == '.')
			continue;
		strcpy(ep,e->d_name);
		f=xfopen(fname,"r");
		while (fgets(buf,sizeof(buf),f)) {
			char *p;
			p=strpbrk(buf,"\r\n");
			if (p)
				*p=0;
			/* Ende nach einer Nicht-Kommentar-Zeile */
			if (*buf && *buf!='#')
				break;
			if (!*buf)
				continue;
			if (!neu) {
				neu=XMALLOC(sizeof(ta_command_t));
				neu->magic=NULL;
				neu->magic2=NULL;
				neu->minlevel=0;
				neu->textpara=NULL;
				neu->numpara=0;
				neu->id=(unsigned int) strtoul(buf+1,NULL,10);
				neu->kommandogruppe=NULL;
				neu->version=0;
				neu->typ=NULL;
				neu->name=NULL;
				neu->kurzbeschreibung=NULL;
				neu->definition=NULL;
				neu->next=NULL;
				len=0;
				ok=TRUE;
			}
			p=strpbrk(buf,"\t ");
			if (!p) {
				ok=FALSE;
				break;
			}
			*p++=0;
			while (isspace(*p))
				p++;
			if (!*p) {
				ok=FALSE;
				break;
			}
			if (!stricmp(buf,"#magic")) {
				if (neu->magic)
					neu->magic2=XSTRDUP(p);
				else
					neu->magic=XSTRDUP(p);
			} else if (!stricmp(buf,"#level"))
				neu->minlevel=(unsigned int) strtol(p,NULL,10);
			else if (!stricmp(buf,"#rem"))
				continue;
			else if (!stricmp(buf,"#id"))
				neu->id=(unsigned int) strtol(p,NULL,10);
			else if (!stricmp(buf,"#gruppe"))
				neu->kommandogruppe=XSTRDUP(p);
			else if (!stricmp(buf,"#version"))
				neu->version=(unsigned int) strtol(p,NULL,10);
			else if (!stricmp(buf,"#typ"))
				neu->typ=XSTRDUP(p);
			else if (!stricmp(buf,"#name"))
				neu->name=XSTRDUP(p);
			else if (!stricmp(buf,"#text"))
				neu->kurzbeschreibung=XSTRDUP(p);
			else if (!stricmp(buf,"#desc")) {
				size_t l=strlen(p);
				if (l+len>=MAXDESCLEN-4)
					ok=FALSE;
				else {
					dbuf[len++]=':';
					strcpy(dbuf+len,p);
					len+=l;
					dbuf[len++]='\r';
					dbuf[len++]='\n';
					dbuf[len]='\0';
				}
			}
		} /* fgets-Schleife */
		fclose(f);
		if (!neu || !neu->magic || !neu->kommandogruppe || !neu->name || !neu->kurzbeschreibung
			|| !len || !neu->id || !neu->typ)
			ok=FALSE;
		if (!ok) {
			if (neu) {
				syslog(LOG_ERR,"incomplete/wrong command description in %s",fname);
				
				/* i should free them, but they are const, and this code
                 * should really not be executed! 
				 * free(neu->magic);
				 * free(neu->magic2);
				 * free(neu->kommandogruppe);
				 * free(neu->typ);
				 * free(neu->name);
				 * free(neu->kurzbeschreibung);
				 */
				free(neu);
			}
			len=0;
			dbuf[0]=0;
			continue;
		}
		neu->definition=XSTRDUP(dbuf);
		neu->next=kommandos;
		neu->fn=ta_cmd_tcl;
		neu->textpara=XSTRDUP(fname);
		kommandos=neu;
	}
}

void
ta_cmd_tcl (char *cmd, const char *fname, LONG dummy2)
{
    int ret;
	char *res=NULL; 

    ret = ttcl_call_func (fname, "command", cmd,&res);

    if (ret == -1) {
        ta_do_log_error ("kann Funktion nicht ausfuehren");
        return;
    }
    /* Achtung, Hack Alarm ... */
    if (!res || !*res) {
		ta_do_log(":!ok\r\n");
    }
    if (!strncmp(res,"out-mem:",8)) {
    	/* Moeglicherweise langer Text in res */
		FILE *f;
		char *s,*p;
		s=res+8;
		f = ta_open_out_block(cmd); /* cannot fail */
		while (s && *s) {
			p=strpbrk(s,"\r\n");
			if (p) {
				if (*p=='\r') {
					*p++=0;
					if (*p=='\n')
						p++;
				} else
					*p++=0;
			}
			fprintf(f,":%s\r\n",s);
				s=p;
		}
		fclose(f);
	} else if (!strncmp(res,"out-file:",9)) {
    	/* res enthaelt Dateinamen mit Ergebis */
		char buf[1024];
		FILE *f,*g;
		char *p;
		g = fopen(res+9,"r");
		if (!g) {
			ta_do_log_error(":Kommando ausgefhrt, aber Resultatsdatei unlesbar\r\n");
			return;
		}
		f = ta_open_out_block(cmd); /* cannot fail */
		while (fgets(buf,sizeof buf,g)) {
			p=strpbrk(buf,"\r\n");
			if (p)
				*p=0;
			fprintf(f,":%s\r\n",buf);
		}
		fclose(f);
		fclose(g);
		remove(res+9);
    } else {
		ta_do_log(":!%s\r\n",res);
    }
}

#endif
