/* ta_dupe.c : Dupecheck fr Benutzer */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "brett.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"

static DUSTR *first;
static DUSTR *last;

static BOOLEAN list_changed;
static BOOLEAN init_done;
static anzahl;
static BOOLEAN dupecheck_inaktiv = FALSE;

void
ta_userdupecheck_end (void)
{
	DUSTR *akt;

	if (!init_done)
		return;
	if (list_changed)
	{
		/* Dann speichern */
		FILE *f;

		f = xfopen (tprintf ("%s%ld.dup", box_conf.tausch_dir, 
							 (long) user.nummer), BINWRITE);
		akt = first;
		while (akt)
		{
			fprintf (f, "%s,%s,%ld\n", akt->userid, akt->realid, (long) akt->zeit);
			akt = akt->next;
		}
		fclose (f);
	}
	akt = first;
	while (akt)
	{
		if (akt->userid)
			free (akt->userid);
		if (akt->realid)
			free (akt->realid);
		first = akt->next;
		free (akt);
		akt = first;
	}
	init_done = FALSE;
	mboutput (" Dupecheck deinitialisiert\r\n");
}

BOOLEAN
ta_userdupecheck_init (void)
{
	FILE *f;
	DUSTR *new;
	char buf[1024];
	char *p;

	mboutput (" initialisiere Dupecheck\r\n");
	if ((f = fopen (tprintf ("%s%ld.dup", box_conf.tausch_dir, 
							 user.nummer), BINREAD)) == NULL)
	{
		init_done = TRUE;
		return TRUE;			/* Keine Dupecheckdatei - kein Problem */
	}

	first = NULL;
	last = NULL;
	anzahl = 0;
	list_changed = FALSE;

	while (fgets (buf, sizeof (buf) - 1, f))
	{
		p = strpbrk (buf, "\r\n");
		if (p)
			*p = '\0';

		new = XMALLOC (sizeof (DUSTR));
		new->userid = NULL;
		new->realid = NULL;
		new->zeit = (time_t) 0;
		p = strrchr (buf, ',');
		if (p)
		{
			*p = '\0';
			new->zeit = strtol (p + 1, NULL, 10);
		}
		p = strchr (buf, ',');
		if (p)
		{
			*p = '\0';
			new->realid = XSTRDUP (p + 1);
		}
		new->userid = XSTRDUP (buf);
		new->next = first;
		new->next = NULL;
		anzahl++;
		if (!first)
		{
			first = new;
			last = new;
		}
		else
		{
			last->next = new;
			last = new;
		}
	}
	fclose (f);
	init_done = TRUE;
	dupecheck_inaktiv = FALSE;
	return TRUE;
}

void
ta_dupe_on (void)
{
	dupecheck_inaktiv = FALSE;
	mboutput (" Dupecheck ist nun eingeschaltet\r\n");
}

void
ta_dupe_off (void)
{
	dupecheck_inaktiv = TRUE;
	mboutput (" Dupecheck ist nun ausgeschaltet\r\n");
}

DUSTR *
ta_userdupecheck (char *id)
{
	DUSTR *akt;

	if (dupecheck_inaktiv)
		return FALSE;

	if (!init_done)
	{
		if (!ta_userdupecheck_init ()) {
			mboutput (" Dupecheck nicht initialisiert!\r\n");
			return NULL;
		}
	}
	akt = first;
	while (akt)
	{
		if (!strcmp (id, akt->userid))
			return akt;
		akt = akt->next;
	}
	return NULL;				/* kein Dupe */
}

void
ta_userdupecheck_add_info (char *userid, char *realid, time_t zeit)
{
	DUSTR *akt,
	*new;

	if (!init_done) {
		if (!ta_userdupecheck_init ()) {
			mboutput (" Dupecheck nicht initialisiert!\r\n");
			return;
		}
	}

	new = malloc (sizeof (DUSTR));
	if (new)
	{
		new->userid = strdup (userid);
		new->realid = strdup (realid);
		new->zeit = zeit;
		new->next = NULL;
	}
	if (new && new->userid && new->realid)
	{
		if (last)
		{
			last->next = new;
			last = new;
		}
		else
		{
			last = new;
			first = new;
		}
		anzahl++;
		list_changed = TRUE;
		/* nur maximal 500 Dupecheckinfos aufbewahren */
		while (anzahl > 500)	/* ## arbitrary limit! */
		{
			akt = first->next;
			free (first->userid);
			free (first->realid);
			free (first);
			first = akt;
			anzahl--;
		}
		return;					/* kein Dupe */
	}
	else
	{
		/* Kein Speicher mehr frei */
		list_changed = FALSE;	/* Um Himmels Willen nicht sichern! */
		/* Daten freigeben */
		if (new)
		{
			if (new->realid)
				free (new->realid);
			if (new->userid)
				free (new->userid);
			free (new);
		}
		ta_userdupecheck_end ();
		mboutput (" Dupecheck fehlgeschlagen (kein Speicher)\r\n");
		return;					/* annehmen, da es ein Dupe ist */
	}
}
