/* ta_info.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef TA_INFO_H
#define TA_INFO_H

/*
	Tausch

	Infofilebehandlung.

*/
typedef enum 
{ 
	ITYP_LOKAL,  /* lokales Infofile IL*, Festtext ohne Doppelpunkte,
	              * CRC lokal erzeugt. 
	              */
	ITYP_NET,    /* netzweites Infofile, Festtext mit Doppelpunkten,
	              * CRC aus Serverbox bernommen 
	              */
	ITYP_CMD,    /* Infofile wird durch externes Kommando erzeugt */
	ITYP_PROC,   /* Infofile wird durch Funktion erzeugt */
	ITYP_INTERN  /* Infofile wird durch Boxbefehl erzeugt */
} INFOTYP;

typedef enum { TA_INFO_NETIDENT, TA_INFO_USERIDENT, TA_INFO_LOKALIDENT }
	ta_info_ident;

typedef ULONG (*ta_info_fn_t) (const char *tmpfname, const char *strpara, 
					long numpara);

typedef struct infostruct
{
	const char *name; /* Name/Kuerzel des Infofiles. */
	const char *group; /* Gruppierung */
	char zeichen;
	const char *text; /* Einzeiliger Infotext zum Infofile, ohne : und \r\n */
	INFOTYP typ;      /* siehe oben */
	UWORD level;      /* ab Level ... erlaubt */
	USERNR sender;    /* Benutzernummer des Senders, 0 unbekannt */
	BOOLEAN mit_crc;  /* Infofile mit CRC? */
	LONG crc; 	      /* CRC des Infofiles. -1: unbekannt (nicht vorhanden) */
	time_t zeit;      /* Zeit des Uploads, 0 unbekannt */ 
	const char *textpara; /* spezieller Wert/ID fr fn, oder 
	                       * Kommando fr ITYP_CMD 
	                       */
	long numpara;     /* spezieller Wert/ID fr fn */
	ta_info_ident ident; /* Identisch nur fuer User? Oder Lokal? Oder Net? */
	ta_info_fn_t fn;  /* Funktion zum Erstellen des Infofiles, liefert 
	                   * CRC zurck. 
	                   */
	struct infostruct *next; /* Zeiger auf das nchste Element der Liste */
} infofile_t;

typedef struct infofilemenu_t
{
	const char *kuerzel;
	const char *beschreibung;
	char zeichen;
	UWORD minlevel;
	simplelist_t *submenues;
	struct infofilemenu_t *next;
} infofilemenu_t;

typedef struct serverinfofile
{
	USERNR server;      /* Eintrag fr welchen Server */
	const char *name;   /* Name, z.B. ITG */
	BOOLEAN mit_crc;    /* Infofile mit CRC */
	LONG crc;
	struct serverinfofile *next;
} xinfofile_t;

	/* Maximale Lnge eines Infofilenamens - gem TauschBau 
	 * Konstante wird z.Z. nirgends benutzt.
	 */
#define TA_INFO_MAXNAME 8

	/* 
	 * Liefert erstes Element der Liste der Infofiles.
	 */
infofile_t *ta_get_info_anker(void);

/*
 * schaut nach, ob irgendein Infofile neu ist oder sich gendert hat, und
 * erzeugt eine CRC.
 */
void ta_info_sync(void);

/* 
 * Sucht das Infofile, dessen Krzel angegeben wurde, und 
 * liefert einen Pointer darauf zurck 
 */
infofile_t *ta_find_infofile(const char *name);

/*
 * Erzeugt den Filenamen. Nuetzlich wegen Caseinsensitiv.
 * Ret: TRUE OK,
 *      FALSE nicht OK.
 */
BOOLEAN ta_info_filename(infofile_t *inf,char *fname);

/*
 * Eine nderung an der Liste signalisieren.
 */
void ta_info_mark_change(void);

ULONG ta_info_itb (const char *fname, const char *dummy1, long is_its);
ULONG ta_info_iti (const char *fname, const char *dummy1, long dummy2);
ULONG ta_info_igx (const char *fname, const char *dummy1, long is_long);
ULONG ta_info_itg (const char *fname, const char *dummy1, long is_long);


infofilemenu_t * ta_get_infofilemenu_anker(void);





/****************************************************************************
 * Handling der Serverinfofiles (eXterne Infofiles)
 ****************************************************************************
 */
xinfofile_t *ta_find_xinfofile(const char *name, USERNR nummer);
void ta_xinfo_mark_change(void);
xinfofile_t * ta_xinfo_get_anker(void);
BOOLEAN ta_xinfo_filename(xinfofile_t *inf,char *fname);

#endif
