/* ta_info_itb.c : generate ITB/ITS */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <ctype.h>
#include <errno.h>

#ifdef OFFLINE2
#include "offline/inc/mdb_global.h"
#else
#include "offline/mdb_global.h"
#endif
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "mmread.h"


#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"


ULONG
ta_info_itb (const char *fname, const char *dummy1, long is_its)
{
#if 1
	ULONG crc = 0;
	FILE *f;
	mm_file_t *in;
	char *buffer;
	size_t buffersize;
	size_t got;

	char inname[PATH_MAX];
	UWORD lv=level();
	BOOLEAN own=FALSE;
	char *my_kennung;

	/* get rid of warnings */
	(void) dummy1;

	/* z.Z. sind ITB und ITS fr Fremdboxen noch identisch - und den
	 * Kompatibilittshack brauchen wir noch.
	 */
	sprintf(inname,"%sITS",tom_info_net_dir);
	in=MM_READ_OPEN(inname);
	if (!in)
	{
		sprintf(inname,"%sITB",tom_info_net_dir);
		in=MM_READ_OPEN(inname);
	}
	if (!in)
		return (ULONG) -1;
	f = xfopen (fname, BINWRITE);
	if (!(MM_READ_IS_MMAPPED(in)))
	{
		buffersize=10240;
		buffer=XMALLOC(buffersize);
	}
	my_kennung=XSTRDUP(box_conf.mausnet_kennung);
	strlwr(my_kennung);

	while ((got=MM_READ_GETLINE(in,&buffer,&buffersize))!=-1)
	{
		BOOLEAN ok;
		const char *p;
		if (*buffer=='*')
		{
			const char *q;
			size_t l;
			for (p=buffer+1, q=my_kennung, l=got-1; 
				l && *p && tolower(*p)==*q;
				p++, q++, l--)
				/* do nothing */;
			if (!*p || *p=='\r' || *p=='\n')
				own=TRUE;
			else
				own=FALSE;
		}
		ok=FALSE;
		if (!islower(*buffer) && *buffer!='%')
			ok=TRUE;
		else if (is_its && lv>6)
			ok=TRUE;
		/* geheime Telefonnummer der eigenen Box? */
		if ((*buffer=='t' || *buffer=='p') && own)
			ok=TRUE;
		if (!ok)
			continue;

		/* Ausgeben, aber Vorsicht mit Zeilenende! */
		ok=FALSE;
		p=buffer+got-1;
		if (*p=='\n')
		{
			if (p > buffer && p[-1]=='\r') 
			{
				got--;
				ok=TRUE;
			}
			got--;
		}
		crc += line_checksum_n (buffer, got);
		fputc(':',f); /* das fehlt in jedem Fall */
		fwrite(buffer,1,ok ? got + 2 : got,f);
		if (!ok)
			fwrite("\r\n",1,2,f);
	}

	if (!(MM_READ_IS_MMAPPED(in)))
		free(buffer);
	MM_READ_CLOSE(in);

	fclose (f);
	return (ULONG) (crc & 0xffff);
#else
	ULONG crc = 0;
	FILE *f;
	FILE *in;
	char inname[PATH_MAX];
	char buf[10240];
	char *p;
	UWORD lv=level();
	BOOLEAN own=FALSE;

	/* get rid of warnings */
	(void) dummy1;

	/* z.Z. sind ITB und ITS fr Fremdboxen noch identisch - und den
	 * Kompatibilittshack brauchen wir noch.
	 */
	sprintf(inname,"%sITS",tom_info_net_dir);
	in=fopen(inname,"r");
	if (!in)
	{
		sprintf(inname,"%sITB",tom_info_net_dir);
		in=fopen(inname,"r");
	}
	if (!in)
		return (ULONG) -1;
	f = xfopen (fname, BINWRITE);

	buf[0]=':';
	while (fgets(buf+1,sizeof(buf)-1,in))
	{
		BOOLEAN ok;
		p=strchr(buf+1,'\n');
		if (p) 
		{
			if (p[-1]!='\r')
			{
				*p++='\r';
				*p++='\n';
				*p=0;
			}
		}
		else
		{
			p=strlen(buf)+buf;
			*p++='\r';
			*p++='\n';
			*p=0;
		}
		if (buf[1]=='*')
		{
			own=!stricmp(buf+2,box_conf.mausnet_kennung);
		}
		ok=FALSE;
		if (!islower(buf[1]) && buf[1]!='%')
			ok=TRUE;
		else if (is_its && lv>6)
			ok=TRUE;
		/* geheime Telefonnummer der eigenen Box? */
		if ((buf[1]=='t' || buf[0]=='p') && own)
			ok=TRUE;
		if (!ok)
			continue;
		crc += line_checksum (buf+1);
		fputs(buf,f);
	}
	fclose (f);
	fclose(in);
	return (ULONG) (crc & 0xffff);
#endif
}
