/* ta_info_itg.c : generate ITG and ITE */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/

#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <syslog.h>
#include <ctype.h>
#include <errno.h>

#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "mmread.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"

ULONG
ta_info_itg (const char *fname, const char *dummy1, long mit_zeiten)
{
	FILE *f;
	BRETTNR i;
	LISTUSER *u;
	char zeile[1024];
	ULONG crc = 0;
	BOOLEAN ist_mitglied;
	BRETTNR *sortiert;

	ta_lade_installation(FALSE);

	sortiert=b_sortierung();
	if (!sortiert || !*sortiert)
		return 0;

	f = xfopen (fname, BINWRITE);

	/* 
	 * Programmfach-Pseudogruppe.
	 */
	crc += 1; /* um nach Einbau d. Progfachs CRC-nderung zu erzwingen */
	fputs (":GProgrammfach" LINEEND,f );
	fputs (":UPersnlicher Programmteil" LINEEND ,f);
	fprintf (f,":C%s"LINEEND, user.name);
	if (mit_zeiten)
		fputs (":L199501092110" LINEEND,f);
	fputs (":Rposter" LINEEND,f);
	fputs (":FLPS+V-G*P+$-" LINEEND,f);

	for ( ; sortiert && *sortiert; sortiert++)
	{
		BRETT *b;
		const char *hilf;
		char *wp;
		i=*sortiert;

		b=br_ind[i].brett;
		if (!b) /* Kann eigentlich nicht sein */
			continue;
		if (!(b->flags & (B_READ | B_WRITE)))
			continue;

		ist_mitglied = FALSE;
		if (b->flags & B_MITGLIED)
			ist_mitglied = b_tst_access (i, user.nummer);

		/*
		 * G-Zeile leitet einen Block ein: Gruppenname
		 */
		zeile[0]=':';
		zeile[1]='G';
		strcpy(zeile+2,b->name);
		crc += line_checksum (zeile);
		fputs (zeile,f);

		/*
		 * U-Zeile: Userinfotext. Mu vorhanden sein.
		 */
		if (b->userinfo)
			hilf=b->userinfo;
		else
			hilf="Keine Beschreibung verfgbar";
		crc += line_checksum (hilf);

		fwrite( LINEEND ":U", 4, 1, f);
		fputs( hilf,f);

		/*
		 * A-Zeile: Aliasname.
		 */
		if (b->aliasname)
		{
			crc += line_checksum (b->aliasname);
			fwrite( LINEEND ":A", 4, 1, f);
			fputs( b->aliasname,f);
		}

		/*
		 * S-Zeile: Sysopinfotext, optional.
		 */
		if (level()>6 && b->sysopinfo)
		{
			crc += line_checksum (b->sysopinfo);
			fwrite( LINEEND ":S", 4, 1, f);
			fputs( b->sysopinfo,f);
		}

		/*
		 * C-Zeile: Gruppenchef.
		 */
		u = ub_find_usernummer (b->grp_chef);
		if (u && u->name)
			hilf=u->name;
		else
			hilf="unbekannt";
		fwrite( LINEEND ":C", 4, 1, f);
		fputs( hilf,f);
		crc += line_checksum (hilf);

		/*
		 * E-Zeile: Aenderungsdatum netzweit.
		 */
		if (b->netzaenderung && mit_zeiten)
		{
			memcpy(zeile,LINEEND ":E", 4);
			time_t_to_mtzeit(&b->netzaenderung,zeile+4);
			crc += line_checksum (zeile);
			fputs (zeile,f);
		}

		/* 
		 * L-Zeile: Aenderungsdatum lokal.
		 */
		if (mit_zeiten) {
			memcpy(zeile,LINEEND ":L", 4);
			if (b->configaenderung>b->created)
				time_t_to_mtzeit (&b->configaenderung, zeile+4);
			else
				time_t_to_mtzeit (&b->created, zeile+4);
			crc += line_checksum (zeile);
			fputs (zeile,f);
		}

		/* 
		 * R-Zeile: Followup-To-Default.
		 */
		if (b->followup_to_default)
		{
			BRETTNR fup=b->followup_to_default;
			if (fup >=0 && fup < max_brettnr
				&& br_ind[b->followup_to_default].brett)
			{
				const char *p=br_ind[b->followup_to_default].brett->name;

				crc += line_checksum (p);
				fwrite (LINEEND ":R", 4, 1, f);
				fputs (p,f);
			}
		}

		/* 
		 * H-Zeile: Ursprungsnetz.
		 */
		crc += b->ursprungsnetz;
		{
			size_t cnt;
			memcpy(zeile,LINEEND ":H",4);
			cnt=libquark_litoa((long) b->ursprungsnetz,zeile+4);
			zeile[cnt+4]=':';
			fwrite(zeile,cnt+5,1,f);
			fputs(bb_ursprungsnetz(b->ursprungsnetz),f);
		}

		/* 
		 * P-Zeile: Sprache.
		 */
		crc += b->sprache;
		{
			size_t cnt;
			memcpy(zeile,LINEEND ":P",4);
			cnt=libquark_litoa((long) b->sprache,zeile+4);
			zeile[cnt+4]=':';
			fwrite(zeile,cnt+5,1,f);
			fputs(bb_sprache(b->sprache),f);
		}

		/*
		 * M-Zeile: Boxinterne Flags
		 */
		{
			brettflags_to_string(b->flags, zeile,sizeof(zeile)-1);
			crc += line_checksum (zeile);
			fwrite (LINEEND ":M", 4, 1, f);
			fputs (zeile,f);
		}

		/*
		 * F-Zeile: Flags.
		 */
		/* 
		 * L-Flag: Leseerlaubnis. N und S knnen wegen fehlender
		 * Netzgruppenverwaltung nicht geliefert werden.
		 */
		memcpy(zeile, LINEEND ":FL",5);
		wp=zeile+5;
		if (br_ind[i].brett->flags & B_ZWANG)
			*wp++='P'; /* Pflicht */
		else if (ta_blist[i]!=GRP_UNBESTELLT)
			*wp++='+'; /* bestellt */
		/* Wenn Gruppe Geheim und Mitgliedschaft und nicht mitglied */
		else if ((b->flags & (B_GEHEIM | B_MITGLIED)) == (B_GEHEIM | B_MITGLIED)
				 && !ist_mitglied)
			*wp++='+'; /* vom Chef eintragen lassen */
		else if (!(br_ind[i].brett->flags & B_READ))
			*wp++='-'; /* Lesen verboten */
		else
			*wp++='B'; /* Einfach bestellen */

		/* 
		 * S-Flag:Schreibzugriff 
		 */
		*wp++='S';
		if ((br_ind[i].brett->flags & B_WRITE))
			*wp++='+';	/* Schreiben erlaubt */
		/* Wenn Mitgliedschaft, aber nicht Mitglied */
		else if (b->flags & B_MITGLIED && !ist_mitglied)
			*wp++='C';	/* Vom Chef eintragen lassen */
		else
			*wp++='-';	/* Kein Schreibzugriff */
		/* 
		 * V-Flag: Vernetzungsstatus
		 */
		*wp++='V';
		if (b->netzstatus)
			*wp++='+'; /* vernetzt */
		else
			*wp++='-'; /* nicht vernetzt */
		/* 
		 * G-Flag: Verhltnis des Users zur Gruppe 
		 */
		*wp++='G';
		if (b->grp_chef == user.nummer)
			*wp++='*'; /* Chef */
		else if (ist_mitglied)
			*wp++='+'; /* Mitglied */
		else
			*wp++='-'; /* nichts besonderes */
		/*
		 * P-Flag: Programmteil vorhanden 
		 */
		*wp++='P';
		if (b->art == BINAER_BRETT)
			*wp++='+'; 
		else
			*wp++='-'; 
		/*
		 * $-Flag: Zugriff fr nichtzahler eingeschrnkt?
		 */
		*wp++='$';
		if (b->flags & (B_ZAHLER|B_PAYWRITE))
			*wp++='+'; 
		else
			*wp++='-'; 
		*wp=0; /* abschlieen */
		crc += line_checksum (zeile);
		fwrite( zeile,wp-zeile,1,f);

		fwrite (LINEEND,2,1,f); /* letzte Zeile abschlieen */
	}
	fclose (f);
	return crc & 0x7fff;
}

