/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>

#include "prototyp.h"
#include "brett.h"
#include "user.h"
#include "modem.h"
#include "msg.h"
#include "terminal.h"

#include "box_conf.h"

#include "tausch.h"
#include "ta_ext.h"
#include "ta_int.h"

extern WORD vtmodeW;
extern BOOLEAN mehr_disabled;

static void ta_datum_subroutine (time_t *last_news,time_t *last_mail);
static void ta_inst_bretter (void);
static void ta_inst_bretter_extern (void);
static void zeige_brett (BRETTNR i);

static void
ta_inst_bretter_extern (void)
{
	/* UO, 29.9.92: VT100 nicht mehr immer aktiv */
	/* Aufruf des externen Tauschbretteditors */
	char cmd[PATH_MAX*2+128];

	if (!box_conf.tausch_editor) {
		mboutput (" Der externe Editor ist leider nicht installiert!\r\n\n");
		return;
	}
	if (*box_conf.tausch_editor=='/')
		strcpy(cmd,box_conf.tausch_editor);
	else
		sprintf (cmd, "%s%s", box_conf.bin_dir, box_conf.tausch_editor);
	if (!fileexist (cmd)) {
		mboutput (" Der externe Editor ist leider nicht installiert!\r\n\n");
		return;
	}
	sprintf (cmd+strlen(cmd), " %s%ld.tau", box_conf.tausch_dir, user.nummer);
	if (vtmodeW == T_VT100)
		strcat (cmd, " /V");

	/* Bretterliste Laden */
	ta_lade_installation (FALSE);
	/* und mit eventuellen neuen Brettern sichern. */
	ta_sichere_installation (FALSE);
	ta_mark_invalid ();
	(void) call_extern(cmd,
		NULL,                 /* spezielles Arbeitsverzeichnis */
		akt_anruf.soft_limit, /* killzeitpunkt,just for the case */
		FALSE,                /* ignore_exitcode */
		FALSE);               /* nicht unter uid des Users */
}								/* ta_inst_bretter_extern */

static void
zeige_brett (BRETTNR i)
{
	const char *s;

	if (br_ind[i].brett->flags & B_ZWANG)
	{
		if (user.pflicht_befreit)
		{
			if (ta_blist[i]!=GRP_UNBESTELLT)
				s = "Pflicht";
			else
				s = "nein/Pfl";
		}
		else
		{
			s = "Pflicht";
			ta_blist[i] = GRP_BESTELLT;
		}
	}
	else if (ta_blist[i]!=GRP_UNBESTELLT)
		s = "bestellt";
	else
		s = "nicht";

	mbprintf ("  %-42.42s [%s] (JNQ VRS?) ", br_ind[i].brett->name, s);
}


static void
tbi_help (UWORD scroll)
{
	mboutput ("    Gruppe bestellen:                    J\r\n");
	mboutput ("        abbestellen:                     N\r\n");
	mboutput ("    Ohne nderung zur nchsten Gruppe:   Leertaste (Space)\r\n");
	mboutput ("    Eine Gruppe zurck:                  Backspace\r\n");
	mbprintf ("    %d Gruppen vor:                      V\r\n", (int) scroll);
	mbprintf ("    %d Gruppen zurck:                   R\r\n", (int) scroll);
	mboutput ("  * Gruppe suchen:                       S\r\n");
	mboutput ("  * Zum Anfang der Liste                 0\r\n");
	mboutput ("  * Zum Ende der Liste                   E\r\n");
	mboutput ("  * Zur Position vor dem letzten Sprung: L (*)\r\n");
	mboutput ("    Beenden:                             Q\r\n");
	mboutput ("    Hilfe (dieser Text):                 ?\r\n");
}

static void
ta_inst_bretter (void)
{
	/* 30.11.93, Uwe, auf neue Gegebenheiten angepat.
	   - neue Tauschinstallation,
	   - Postfach nicht mehr hier bestellbar
	   - einiges am Userinterface getan, die Portierung des
	   externen Bretteditors oder die neuprogrammierung eines
	   Ersatzes wird wohl noch etwas dauern.
	 */

	BRETTNR i;
	BRETTNR *sort;
	UWORD akt;
	UWORD last;
	BOOLEAN ende;
	UWORD scrollzeilen;
	BOOLEAN found;
	BOOLEAN old_disabled;
	int c;

	sort=b_sortierung();

	if (!sort)
	{
		mbfehler ("Abbruch wegen Speichermangel");
		return;
	}

	if (!sort[0])
	{
		mboutput ("Keine Gruppen vorhanden!\r\n");
		return;
	}

	scrollzeilen = user.bs_zeilen;
	if (!scrollzeilen)
		scrollzeilen = 20;
	old_disabled=mehr_disabled;
	old_disabled=TRUE;


	set_status_line (" Gruppenbestellung fr den Quarktausch");

	mboutput ("\r\n");
	mboutput ("  Gruppeninstallation:\r\n");
	tbi_help (scrollzeilen);
	mboutput ("\r\n");

	ta_lade_installation (FALSE);

	akt = 0;
	last = 0;
	ende = FALSE;
	while (!ende)
	{
		i = sort[akt];

		zeige_brett (i);

		c = mbkey();
		if (c < 256)
			c = toupper(c);

		switch (c)
		{
		case C_LEFT: /* togglen */
		case C_RIGHT: /* togglen */
			if (ta_blist[i]==GRP_UNBESTELLT)
			{
				mboutput ("bestellt!");
				ta_blist[i] = GRP_BESTELLT;
			}
			else
			{
				mboutput ("abbestellt!");
				ta_blist[i] = GRP_UNBESTELLT;
			}
			ta_opts.changed = TRUE;
			mboutput ("\r\n");
			break;
		case 'J':				/* Ja, bestellen */
			if (ta_blist[i]==GRP_UNBESTELLT)
			{
				mboutput ("bestellt!");
				ta_opts.changed = TRUE;
				ta_blist[i] = GRP_BESTELLT;
			}
			mboutput ("\r\n");
			akt++;
			break;
		case 'N':				/* Abbestellen */
			if (!(br_ind[i].brett->flags & B_ZWANG) || user.pflicht_befreit)
			{
				if (ta_blist[i]!=GRP_UNBESTELLT)
				{
					mboutput ("abbestellt!");
					ta_blist[i] = GRP_UNBESTELLT;
					ta_opts.changed = TRUE;
				}
				akt++;
			}
			mboutput ("\r\n");
			break;
		case 'Q':				/* Quit */
			mboutput ("Fertig!\r\n");
			ende = TRUE;
			break;
		case ' ':				/* Space = nchste */
		case C_DOWN:
			mboutput ("\r\n");
			if (sort[akt+1])
				akt++;
			else
				mboutput ("Ende der Liste! Ende mit `Q'.\r\n");
			break;
		case '\b':				/* Backspace */
		case '\177':			/* Delete */
		case C_UP:
			mboutput ("\r\n");
			if (akt > 0)
				akt--;
			else
				mboutput ("Anfang der Liste!\r\n");
			break;
		case 'V':
			mboutput ("\r\n");
			{
				int cnt = scrollzeilen;

				if (!sort[akt+1])
					mboutput ("Ende der Liste!\r\n");

				while (cnt > 0 && sort[akt + 1])
				{
					cnt--;
					akt++;
					if (cnt)
					{
						zeige_brett (sort[akt]);
						mboutput ("\r\n");
					}
				}
			}
			break;
		case 'R':
			mboutput ("\r\n");
			{
				int cnt = scrollzeilen;

				if (akt<=0)
					mboutput ("Anfang der Liste!\r\n");

				while (akt > 0 && cnt > 0)
				{
					cnt--;
					akt--;
					if (cnt)
					{
						zeige_brett (sort[akt]);
						mboutput ("\r\n");
					}
				}
			}
			break;
		case 'S':
			mboutput ("\r\n");
			{
				char such[sizeof (akt_brett.name)];

				mbquestion (65, 0, TRUE, " Gruppenname: ", "", such);

				if (*such)
				{
					BRETTNR bnr;

					bnr = find_brett (such);
					if (bnr)
					{
						UWORD tmp;

						found = FALSE;

						for (tmp = 0; sort[tmp]; tmp++)
						{
							if (sort[tmp] == bnr)
							{
								found = TRUE;
								break;
							}
						}
						if (!found)
							bnr = 0;
						else
						{
							last = akt;
							akt = tmp;
						}
					}
					if (!bnr)
						mboutput ("  nicht gefunden\r\n");
				}
			}
			break;
		case '0':				/* Anfang der Liste */
		case C_HOME:
			mboutput ("\r\n");
			last = akt;
			akt = 0;
			break;
		case 'E':				/* Ende der Liste */
		case C_END:
			mboutput ("\r\n");
			last = akt;
			while (sort[akt + 1])
				akt++;
			break;
		case 'L':				/* Zur Position vor dem letzten Sprung */
			mboutput ("\r\n");
			{
				UWORD zwi;

				zwi = last;
				last = akt;
				akt = zwi;
			}
			break;
		case '?':				/* Hilfe */
			mboutput ("\r\n");
			tbi_help (scrollzeilen);
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			{
				char buf[2];
				char nummer[10];
				BRETTNR bnr;

				*buf = c;
				buf[1] = '\0';
				mbquestion (10, 0, TRUE, " Gruppennr. : ", buf, nummer);
				bnr = atoi (nummer);
				if (bnr)
				{
					UWORD tmp;

					found = FALSE;

					for (tmp = 0; sort[tmp]; tmp++)
					{
						if (sort[tmp] == bnr)
						{
							found = TRUE;
							break;
						}
					}
					if (!found)
						bnr = 0;
					else
					{
						last = akt;
						akt = tmp;
					}
				}
				if (!bnr && *nummer)
					mboutput ("  nicht gefunden\r\n");
			}
			break;
		default:
			mboutput ("\r\n");
			break;
		}						/* switch */
		while (!sort[akt])
			akt--;
		if (logout_modus)
			ende = TRUE;
	}							/* while ! ende */

	/* ta_sichere_installation (FALSE); ta_inst erledigt das */
	mehr_disabled=old_disabled;

}								/* ta_inst_bretter */


/*
   25.02.93 Frank, neu
 */
static void
ta_datum_subroutine (time_t *last_news,time_t *last_mail)
{
	char alt[20];
	char neu[20];
	struct tm *tm;
	struct tm neu_tm;
	time_t ref_zeit;

	if (last_news && last_mail)
	{
		if (*last_news<*last_mail)
			ref_zeit = *last_news;
		else
			ref_zeit = *last_mail;
	}
	else if (last_news)
		ref_zeit=*last_news;
	else
		ref_zeit=*last_mail;

	tm = localtime (&ref_zeit);

	/* insb. gleiche Sommerzeit, sonst "korrigiert" mktime die Stunde */
	neu_tm = *tm;

	mboutput("Bitte whlen Sie das neue Datum. Abbruch mit Leereingabe.\r\n");
	do
	{
		sprintf (alt, "%02d", tm->tm_mday);
		mbquestion (2, 2, TRUE, " >> Tag    (TT): ", alt, neu);
		if (!*neu)
			return;
		neu_tm.tm_mday = atoi (neu);
	}
	while (neu_tm.tm_mday < 0 || neu_tm.tm_mday > 31);

	do
	{
		sprintf (alt, "%02d", tm->tm_mon + 1);
		mbquestion (2, 2, TRUE, " >> Monat  (MM): ", alt, neu);
		if (!*neu)
			return;
		neu_tm.tm_mon = atoi (neu) - 1;
		if (logout_modus)
			return;
	}
	while (neu_tm.tm_mon < 0 || neu_tm.tm_mon > 12);

	sprintf (alt, "%02d", tm->tm_year);
	mbquestion (4, 2, TRUE, " >> Jahr   (JJJJ): ", alt, neu);
	if (!*neu)
		return;
	neu_tm.tm_year = atoi (neu);
	if (neu_tm.tm_year<100) {
		time_t t1=time(0);
		struct tm *tm1=localtime(&t1);
		if (tm1->tm_year >= 100) 
			neu_tm.tm_year+=100;
	} else if (neu_tm.tm_year>1900)
		neu_tm.tm_year-=1900;
	
	mboutput ("\r\n");

	do
	{
		sprintf (alt, "%02d", tm->tm_hour);
		mbquestion (2, 2, TRUE, " >> Stunde (SS): ", alt, neu);
		if (!*neu)
			return;
		neu_tm.tm_hour = atoi (neu);
		if (logout_modus)
			return;
	}
	while (neu_tm.tm_hour < 0 || neu_tm.tm_hour > 24);

	do
	{
		sprintf (alt, "%02d", tm->tm_min);
		mbquestion (2, 2, TRUE, " >> Minute (MM): ", alt, neu);
		if (!*neu)
			return;
		neu_tm.tm_min = atoi (neu);
		if (logout_modus)
			return;
	}
	while (neu_tm.tm_min < 0 || neu_tm.tm_min > 59);

	neu_tm.tm_sec = 0;
	if (last_mail)
		*last_mail = mktime (&neu_tm);
	if (last_news)
		*last_news = mktime (&neu_tm);

}								/* ta_datum_subroutine */

void
ta_do_inst (void)
{
	char hilf1S[12];
	char hilf2S[12];
	WORD decideW;
	ULONG zwi;

	/* 
	   Damit hinterher eventuell die nderungen restauriert werden knnen,
	   mu die Installation im jetzigen Zustand gesichert werden.
	 */
	if (ta_opts.usernr && ta_opts.changed)
		ta_sichere_installation (FALSE);
	else if (ta_opts.usernr != user.nummer)
		ta_lade_installation (FALSE);


   /*
    * User, die noch nie getauscht haben, darauf hinweisen, dass sie beim
    * ersten Tausch per Voreinstellung bis zu 3 Tage alte Msgs. erhalten.
    */
	if (!ta_opts.last_newstausch)
	{
		set_status_line ("Erster Tausch");

		mboutput (" Hinweis fr Ihren ersten Tausch hier:\r\n");
		mboutput (" Sie erhalten bereits alle Nachrichten der letzten 5 Tage\r\n");
		mboutput (" aus den bestellten Gruppen.\r\n");
		mboutput (" Um dies zu ndern geben Sie im Men bitte 'D' fr \"Datum\" ein.\r\n");
		ta_opts.last_newstausch = time (NULL) - 86400L * 5;
		mboutput ("\r\n (TASTE)\r\n");
		mbchar ();
	}

	do
	{
		struct tm *tm;
		struct mbselect_t sel[20];
		int selcount=0;


		set_status_line ("Tausch-Optionen");

		mboutput ("\r\n");
		mbprintf ("(D)atum des letzten Tausch setzen");
		if (ta_opts.last_newstausch==ta_opts.last_mailtausch)
		{
			tm = localtime (&ta_opts.last_mailtausch);
			mbprintf ("   (%02d.%02d.%02d,%02d:%02d)\r\n",
				tm->tm_mday, tm->tm_mon + 1, tm->tm_year, 
				tm->tm_hour, tm->tm_min);
		}
		else
		{
			tm = localtime (&ta_opts.last_mailtausch);
			mbprintf ("   (PM %02d.%02d.%02d,%02d:%02d",
				tm->tm_mday, tm->tm_mon + 1, tm->tm_year, 
				tm->tm_hour, tm->tm_min);
			tm = localtime (&ta_opts.last_newstausch);
			mbprintf (", AM %02d.%02d.%02d,%02d:%02d)\r\n",
				tm->tm_mday, tm->tm_mon + 1, tm->tm_year, 
				tm->tm_hour, tm->tm_min);
		}
		sel[selcount].key='D';
		sel[selcount++].output="Tauschdatum setzen\r\n";

		mbprintf ("Tauschdatum fr (P)ersnliche Nachrichten setzen");
		tm = localtime (&ta_opts.last_mailtausch);
		mbprintf ("   (%02d.%02d.%02d,%02d:%02d)\r\n",
		 tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour, tm->tm_min);
		sel[selcount].key='P';
		sel[selcount++].output=
			"Tauschdatum persnlicher Nachrichten setzen\r\n";

		mbprintf ("Tauschdatum fr (O)effentlicher Nachrichten setzen");
		tm = localtime (&ta_opts.last_newstausch);
		mbprintf ("   (%02d.%02d.%02d,%02d:%02d)\r\n",
		 tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour, tm->tm_min);
		sel[selcount].key='O';
		sel[selcount++].output=
			"Tauschdatum ffentlicher Nachrichten setzen\r\n";

		mboutput ("(U)mlautwandlung (");
		if (ta_opts.umlaut)
			mboutput ("an");
		else
			mboutput ("aus");
		mboutput (")\r\n");
		sel[selcount].key='U';
		sel[selcount++].output="Umlautwandlung\r\n";

		mboutput ("(C)rosspostings (");
		if (!ta_opts.only_one_group)
			mboutput ("an");
		else
			mboutput ("aus");
		mboutput (")\r\n");
		sel[selcount].key='C';
		sel[selcount++].output="Crosspostings\r\n";

		if (!ta_opts.umbruch)
			mboutput ("(Z)eilenlnge: kein Umbruch\r\n");
		else
			mbprintf ("(Z)eilenlnge: %ld Zeichen\r\n", (long) ta_opts.umbruch);
		sel[selcount].key='Z';
		sel[selcount++].output="Zeilenlnge\r\n";

		if (ta_opts.maxlen==0)
			mboutput ("(N)achrichtenlnge: ungegrenzt\r\n");
		else
			mbprintf ("(N)achrichtenlnge: %ld KB\r\n", ta_opts.maxlen/1000);
		sel[selcount].key='N';
		sel[selcount++].output="Nachrichtenlnge\r\n";

		mboutput ("(G)ruppenliste installieren\r\n");
		sel[selcount].key='B';
		sel[selcount++].output="Gruppeninstallation\r\n";
		sel[selcount].key='G';
		sel[selcount++].output="Gruppeninstallation\r\n";

		if (box_conf.tausch_editor)
		{
			mboutput ("(E) Externen Gruppeneditor verwenden\r\n");
			sel[selcount].key='E';
			sel[selcount++].output="externer Gruppeneditor\r\n";
		}

		if (user.typ!=UTYP_USER)
		{
			mboutput ("(Q)uarkoutfile (");
			if (ta_opts.quarkoutfile)
				mboutput ("an");
			else
				mboutput ("aus");
			mboutput (")\r\n");
			sel[selcount].key='Q';
			sel[selcount++].output="Quarkoutfile\r\n";
		}
		if (!ta_opts.async_export)
			mboutput ("(A)synchroner Export: Nein\r\n");
		else
			mboutput ("(A)synchroner Export: Ja\r\n");
		sel[selcount].key='A';
		sel[selcount++].output="Asynchroner Export\r\n";

		mboutput ("\r\n(F)ertig\r\n");
		sel[selcount].key='F';
		sel[selcount++].output="Fertig\r\n";
		sel[selcount].key='\0';
		sel[selcount++].output=NULL;
		mboutput ("\r\n >> Ihre Wahl: ");

		decideW = mbselect(sel);
		if (!decideW)
		{
			ta_mark_invalid ();
			return;
		}

		switch (decideW)
		{
		case 'F':				/* fertig */
			break;

		case 'D':				/* Tauschdatum beide */
			ta_datum_subroutine (&ta_opts.last_mailtausch,
				&ta_opts.last_newstausch);
			ta_opts.changed = TRUE;
			break;
		case 'P':				/* Tauschdatum persnliche */
			ta_datum_subroutine (&ta_opts.last_mailtausch,NULL);
			ta_opts.changed = TRUE;
			break;
		case 'O':				/* Tauschdatum ffentliche */
			ta_datum_subroutine (NULL,&ta_opts.last_newstausch);
			ta_opts.changed = TRUE;
			break;


		case 'E':				/* externer Bretteditor */
			ta_inst_bretter_extern ();
			break;
		case 'B':				/* interner Bretteditor */
		case 'G':				/* interner Gruppeneditor */
			ta_inst_bretter ();
			break;

		case 'U':
			ta_opts.umlaut = !ta_opts.umlaut;
			mboutput ("  ... umgeschaltet\r\n");
			ta_opts.changed = TRUE;
			break;

		case 'A':
			ta_opts.async_export = !ta_opts.async_export;
			mboutput ("  ... umgeschaltet\r\n");
			ta_opts.changed = TRUE;
			break;

		case 'C':
			ta_opts.only_one_group = !ta_opts.only_one_group;
			mboutput ("  ... umgeschaltet\r\n");
			ta_opts.changed = TRUE;
			break;

		case 'Q':
			ta_opts.quarkoutfile = !ta_opts.quarkoutfile;
			mboutput ("  ... umgeschaltet\r\n");
			ta_opts.changed = TRUE;
			break;

		case 'Z':
			sprintf (hilf1S, "%d", ta_opts.umbruch);
			mboutput (" (0 = kein Umbruch, 1 bis 9 werden nicht akzeptiert)\r\n");
			mbquestion (3, 2, TRUE, " Zeilenlnge:", hilf1S, hilf2S);
			strtrim (hilf2S);
			zwi = strtoul (hilf2S,NULL,10);
			if (isdigit (*hilf2S) && (!zwi || zwi >= 10))
			{
				ta_opts.umbruch = strtol (hilf2S,NULL,10);
				ta_opts.changed = TRUE;
			}
			else
				mboutput ("  ... unverndert\r\n");
			break;
			
		case 'N':
			sprintf (hilf1S, "%lu", (ta_opts.maxlen/1000));
			mboutput (" (0 = keine Begrenzung, 16=Mininum)\r\n");
			mbquestion (4, 2, TRUE, " Nachrichtenlnge:", hilf1S, hilf2S);
			strtrim (hilf2S);
			zwi = strtol (hilf2S,NULL,10);
			if (isdigit (*hilf2S))
			{
				/*
				 * 16  ->  16000
				 * 99  ->  99999
				 * 150 -> 150000
				 * 1050->   1050
				 */
				if (zwi<1000)
					zwi*=1000;
				if (zwi && zwi<8192)
				{
					mboutput ("  ineffizient klein, bleibt unverndert\r\n");
				}
				else
				{
					ta_opts.maxlen=zwi;
					ta_opts.changed = TRUE;
				}
			}
			else
				mboutput ("  ... unverndert\r\n");
			break;
		}

		if (logout_modus)
		{
			ta_mark_invalid ();
			return;
		}
	}
	while (decideW != 'F');

	if (ta_opts.changed)
	{
		mboutput ("\r\n >> Neue Installation speichern? (Ja/Nein): ");
		if (mbdecide ("JN", " Ja ...\r\n| Nein\r\n") == 1)
		{
			mboutput ("\r\n -> Fein, wird gemacht ...");
			ta_sichere_installation (FALSE);
			mboutput ("OK\r\n");
		}
		else
		{
			ta_mark_invalid ();	/* nderungen ungltig machen */
		}
	}

}								/* ta_do_inst */
