/* ta_log.c: Logfilehandling (#LOG,*.msg) */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <stdarg.h>

#include "prototyp.h"
#include "modem.h"
#include "tompaths.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"

/*
 * #LOG
 */
static FILE *log_f;

void 
ta_open_log(void)
{
	char fname[PATH_MAX];
	if (log_f)
		return;

	sprintf (fname, "%s" TAUSCH_LOG_F , tom_tmp_dir);
	if (fileexist(fname))
		log_f = xfopen (fname, BINAPPEND);
	else
	{
		time_t zeit;

		time (&zeit);

		log_f = xfopen (fname, BINAPPEND);
		fputs ("#LOG\r\n",log_f);
	}
}

void
ta_do_log1 (const char *str)
{
	if (log_f)
		(void) fputs (str,log_f);
	mboutput ("  ");
	mboutput(str);
}

void
ta_do_log (const char *format,...)
{
	va_list param;
	char buffer[1024];

	va_start (param, format);
	(void) vsprintf (buffer, format, param);
	va_end (param);
	ta_do_log1(buffer);
}

void
ta_do_log_error (const char *s)
{
	char buf[1024];
	size_t l=strlen(s);
	if (l>sizeof(buf)-5)
		ta_do_log (":?%s\r\n", s);
	else
	{
		buf[0]=':';
		buf[1]='?';
		strcpy(buf+2,s);
		strcpy(buf+2+l,"\r\n");
		ta_do_log1(buf);
	}
}

/*
 * Messagelog
 */

void
ta_msg_log (const char *format,...)
{
	va_list param;
	char buf[2048];
	
	va_start (param, format);
	(void) vsprintf (buf, format, param);
	va_end (param);

	msglog("%s",buf);
}

void
ta_do_log_end (void)
{
	if (log_f)
	{
		fclose (log_f);
		log_f = NULL;
	}
}

