/*
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>
#include <time.h>
#include <sys/stat.h>

#include "box_conf.h"
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "modem.h"
#include "tompaths.h"
#include "routing.h"
#include "route_int.h"
#include "msg.h" /* write_msg fr #LOG */

#include "lockfile.h" /* libquark */

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

static BOOLEAN r_t_pseudo_deliver(message_t *head, r_adr_t *adr);
static BOOLEAN r_t_pseudo_flush(void);

static BOOLEAN r_t_pseudo_deliver_one(message_t *head,r_adr_t *adr, const char *id);

r_transport_t r_t_pseudo=
{
	"pseudo",              /* Name */
	NULL,
	r_t_pseudo_deliver,    /* Zustellung */
	r_t_pseudo_flush,
	NULL                    /* Next (wird zur Verkettung genutzt) */
};

static BOOLEAN
pseudo_fn_misc(r_adr_t *adr, const char *buf, char **errtext)
{
	FILE *f;
	BOOLEAN ret;
	char fname[PATH_MAX];
	char *absender;
	const char *md;

	if (0==stricmp(adr->thint,"bounce"))
		return FALSE;

	if (!adr->erroradr)
	{
		*errtext=strdup("Keine Antwortadresse");
		return FALSE;
	}

    md= box_conf.mailer_daemon ? box_conf.mailer_daemon : "MAILER-DAEMON";
	absender =concat(md, " @ ",(MYDOMAINPART),NULL);
	if (!absender)
	{
		*errtext=strdup("Out of memory");
		return FALSE;
	}

	sprintf(fname,"%s%misc%lu.rt",tom_tmp_dir,(ULONG) getpid());
	f=fopen(fname,"w");
	if (!f)
	{
		free(absender);
		*errtext=concat("cannot open ",fname,": ",strerror(errno),NULL);
		return FALSE;
	}

	if (0==stricmp(adr->thint,"echo"))
		fwrite(buf,strlen(buf),1,f);
	else
		fprintf(f,"notice\r\n");
	fclose(f);
	ret=einfachnachricht(TRUE, adr->level, adr->bezahlt,
		fname,
		'W', adr->thint,
		'V', absender,
		'A', adr->erroradr
		,0);
	free(absender);
	unlink(fname);
	return ret;
}

static BOOLEAN
r_t_pseudo_deliver_one(message_t *head,r_adr_t *adr, const char *id)
{
	FILE *in=NULL; /* keep gcc quiet */
	pseudo_fn_t fn=NULL;
	char *buffer=NULL;
	char *errtext=NULL;

	if (!adr->thint)
	{
		/* wenn wir keinen "transport hint" bekommen, wissen wir nicht,
		 * an welche Funktion die Nachricht tatschlich geht */
		r_paniclog("no transport hint in r_t_pseudo_deliver_one for: %s\n",
			adr->resultadr);
		r_tauschuserlog(adr,":?TSNH: kein transport hint ...\r\n");
		msglog("   ?TSNH: kein transport hint fr Nachricht an: %s\r\n",
			adr->resultadr);
		adr->errorflags|=R_E_INTERN;
		adr->flags|=R_F_TO_POSTMASTER;
		adr->flags|=R_F_TO_SENDER;
		adr->error=TRUE;
		return FALSE;
	}
	if (!stricmp(adr->thint,"echo"))
		fn=pseudo_fn_misc;
	if (!stricmp(adr->thint,"bounce"))
		fn=pseudo_fn_misc;
	if (!stricmp(adr->thint,"notice"))
		fn=pseudo_fn_misc;
	if (!fn)
	{
		r_paniclog(
			"transport hint %s notexistant in r_t_pseudo_deliver_one for %s\n",
			(char *) adr->thint, adr->resultadr);
		r_tauschuserlog(adr,":?TSNH: illegaler transport hint ...\r\n");
		msglog("   ?TSNH: illegaler transport hint %s fr Nachricht an %s\r\n",
			(char *) adr->thint, adr->resultadr);
		adr->errorflags|=R_E_INTERN;
		adr->flags|=R_F_TO_POSTMASTER;
		adr->flags|=R_F_TO_SENDER;
		adr->error=TRUE;
	}

	if (head->fname)
	{
		/* Text aus der Datei lesen */
		long pos;
		in=fopen(head->fname,"r");

		if (!in)
		{
			const char *s=strerror(errno);
			r_paniclog("cannot open inputfile %s: %s\n",head->fname,s);
			r_tauschuserlog(adr,":?TSNH: kann Nachricht nicht ffnen: %s\r\n",s);
			msglog("   ?TSNH: kann Nachricht nicht ffnen: %s\r\n",s);
			adr->errorflags|=R_E_INTERN;
			adr->flags|=R_F_TO_POSTMASTER;
			adr->flags|=R_F_TO_SENDER;
			adr->error=TRUE;
			return FALSE;
		}
		pos=filesize(in);
		if (pos<0)
		{
			const char *s=strerror(errno);
			r_paniclog("cannot seek on inputfile %s: %s\n",head->fname,s);
			r_tauschuserlog(adr,":?TSNH: kann Nachrichtendatei nicht seeken: %s\r\n",s);
			msglog("   ?TSNH: kann Nachrichtendatei nicht seeken: %s\r\n",s);
			adr->errorflags|=R_E_INTERN;
			adr->flags|=R_F_TO_POSTMASTER;
			adr->flags|=R_F_TO_SENDER;
			adr->error=TRUE;
			fclose(in);
			return FALSE;
		}
		buffer=malloc(pos+1);
		if (!buffer)
		{
			r_paniclog("cannot malloc %ld bytes\n",pos);
			r_tauschuserlog(adr,":?cannot malloc %ld bytes\r\n",pos);
			msglog("   ?cannot malloc %ld bytes\r\n",pos);
			adr->errorflags|=R_E_INTERN;
			adr->flags|=R_F_TO_SENDER;
			adr->error=TRUE;
			fclose(in);
			return FALSE;
		}
		if (1!=fread(buffer,pos,1,in))
		{
			const char *s=strerror(errno);
			r_paniclog("cannot read %lu bytes from %s: %s\n",pos,
				head->fname,s);
			r_tauschuserlog(adr,":?cannot read %lu bytes from %s: %s\r\n",pos,
				head->fname,s);
			msglog("   ?cannot read %lu bytes from %s: %s\r\n",pos,
				head->fname,s);
			adr->errorflags|=R_E_INTERN;
			adr->flags|=R_F_TO_SENDER;
			adr->error=TRUE;
			fclose(in);
			return FALSE;
		}
		buffer[pos]=0;
		fclose(in);
	}
	else
		buffer=head->body.buf;

	if (!fn(adr,buffer,&errtext))
	{
		const char *s=errtext ? errtext : "unknown error";
		r_paniclog("function call %s failed: %s\n",
			(char *) adr->thint,s);
		r_tauschuserlog(adr,":?function call %s failed: %s\r\n",
			(char *) adr->thint,s);
		msglog("   ?function call %s failed: %s\r\n",
			(char *) adr->thint,s);
		adr->errorflags|=R_E_INTERN;
		adr->flags|=R_F_TO_SENDER;
		adr->error=TRUE;
		if (errtext)
			free(errtext);
		return FALSE;
	}

	/* cleanup */
	if (errtext)
		free(errtext);

	if (head->fname)
		free(buffer);

	msglog("  =%s\n",id);
	r_tauschuserlog(adr,":=%s\r\n",id);
	return TRUE;
}

/* 
 * Hierhin kommen nur vollstndige Nachrichten, keine Spezialnachrichten
 * und keine Statusmeldungen.
 */
BOOLEAN
r_t_pseudo_deliver(message_t *head, r_adr_t *adr)
{
	BOOLEAN ok=TRUE;

	if (head->msgtyp==pers_msg)
	{
		for ( ; adr; adr=adr->next)
		{
			if (adr->transport==&r_t_pseudo
				&& TRANSPORT_ALLOWED(adr))
			{
				char idbuf[256];
				char *id;

				if (adr->flags & R_F_NEUE_ID)
				{
					make_msg_id(idbuf,TRUE, NULL, NULL);
					id=idbuf;
				}
				else
					id=head->msg_id;

				if (!r_t_pseudo_deliver_one(head,adr,id))
				{
					r_writecomment(R_LOG_ERROR,adr,"Transport gescheitert");
					adr->error=TRUE;
					adr->errorflags|=R_E_INTERN;
					ok=FALSE;
				}
				else
					adr->flags |= R_F_ZUGESTELLT;
			}
		}
	}
	else if (head->msgtyp==status_msg)
	{
		/* Hh .. */
		r_tauschuserlog(adr,":!not delivered to internal function\r\n");
		r_tauschuserlog(adr,":=%s\r\n",head->msg_id);
		msglog("  =%s (Status, not delivered through pseudo router)\r\n",
			head->msg_id);
	}
	return ok;
}

BOOLEAN
r_t_pseudo_flush(void)
{
	return TRUE; /* really a dummy */
}
