/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <errno.h>
#include <utime.h>

	/* damit sie in . liegen darf */
#include "conflib.h"

#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "tompaths.h"
#include "modem.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "bin_msg.h"
#include "bin_out.h"

extern USER user;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;


static void ta_s_index(const char *para, BOOLEAN wc);
static ULONG ta_s_index_extern(const char *para, BOOLEAN wc);
static ULONG ta_s_index_extern(const char *para, BOOLEAN wc);
static void ta_s_send(const char *para);
static BOOLEAN ta_saug_init(void);
static void ta_s_upload(const char *para);

static BOOLEAN init_done;

#define ERRLOG(x) do { \
    ta_msg_log("  ?%s\n",x); \
    ta_do_log_error(x); \
} while(0)


typedef struct saug_dir_t
{
	const char *dir;
	const char *acl;
	const char *indexcmd;
	struct saug_dir_t *next;
} saug_dir_t;

saug_dir_t *saug_anker;

extern const char *external_parameter;

void
ta_saugblock(message_t *msg)
{
	char *start;
	if (!init_done)
	{
		ta_saug_init();
	}

	start=msg->body.buf;
	while (start && *start)
	{
		char *sep;
		char sepc=0;
		const char *para=NULL;
		char *nl;
		char nlc=0;
		nl=strpbrk(start,"\r\n");
		if (nl)
		{
			nlc=*nl;
			*nl=0;
		}
		if (*start)
		{
			ta_do_log(":\"%s\"\r\n",start+1);
			ta_msg_log("  :%s\n",start);
			start++; /* Doppelpunkt berspringen */
			while (isspace(*start))
				start++;
			sep=strpbrk(start,"\t ");
			if (sep)
			{
				sepc=*sep;
				*sep=0;
				para=sep+1;
				while (isspace(*para))
					para++;
			}
			else
				para="";
			if (!stricmp(start,"grep") 
					|| !stricmp(start,"regex"))
				ta_s_index(para, FALSE);
			if (!stricmp(start,"suche") 
					|| !stricmp(start,"wildcard")
					|| !stricmp(start,"glob"))
				ta_s_index(para,TRUE);
			if (!stricmp(start,"send") 
					|| !stricmp(start,"download")
					|| !stricmp(start,"put"))
				ta_s_send(para);
			if (!stricmp(start,"upload")) 
				ta_s_upload(para);
			if (!stricmp(start,"cancel"))
				unlink(tprintf("%s%ld/%s",box_conf.spool_dir,user.nummer,SAUGCNTRL));
			if (sep)
				*sep=sepc;
		}
		if (nl)
		{
			*nl=nlc;
			nl++;
			while (isspace(*nl))
				nl++;
			start=nl;
		}
		else
			start=NULL;
	}
}

static ULONG 
ta_s_index_intern(const char *para, BOOLEAN wc)
{
    BIN_MSG bin_info;
    long nr;
    ULONG found=0;

	nr = bin_global_suchen(wc ? 'N' : 'R',para,0,&bin_info);
	while (nr >= 0)
	{
		if (br_ind[bin_info.brettnr].brett->flags & B_READ)
		{
			char buf[40];
			struct tm *tm;
			if (bin_info.brettnr)
			ta_do_log(":!%s:%s" LINEEND,
				bin_info.brettnr ? 
					br_ind[bin_info.brettnr].brett->name 
					: "Programmfach",
				bin_info.name);
			ta_do_log(":!%s" LINEEND, bin_info.betreff);

			tm=localtime(&bin_info.datime);
			strftime(buf,sizeof(buf)-1,"%d.%m.%Y, %H:%M:%S",tm);
			ta_do_log(":!%ld Bytes, %s" LINEEND,
				(long) bin_info.bytes, buf);
			found++;
		}

		nr = bin_global_suchen('R',para,nr+1,&bin_info);
	}
	return found;
}

static void
ta_s_index(const char *para, BOOLEAN wc)
{
	ULONG found;
    if (!para || !*para)
    {
    	ta_do_log(":?Was suchen?" LINEEND);
		ta_msg_log("  ?Was suchen?\n");
    	return;
    }
    found=ta_s_index_intern(para,wc);
    found+=ta_s_index_extern(para,wc);
	ta_msg_log("  !fand %lu\n",found);
}

static ULONG 
ta_s_index_extern(const char *para,BOOLEAN wc)
{
	ULONG found=0;
	char regbuf[1024];
	saug_dir_t *akt;
	if (wc)
	{
		/* 
		 * Wildcard to very special regular expression. You are not supposed
		 * to understand this, unless you know about regex's.
		 * 
		 */
		/* Wildcard nach regex */
		const char *p;
		char *q;
		p=para;
		q=regbuf;
		if (*p=='*')
		{
			*q++='.';
			*q++='*';
			p++;
		}
		while (*p)
		{
			switch(*p)
			{
			case '?': 
				*q++='.';
				break;
			case '*':
				*q++='.';
				*q++='+';
				break;
			case '.':
			case '|':
			case '^':
			case '+':
			case '$':
				*q++='\\';
				*q++=*p;
				break;
			default:
				*q++=*p;
				break;
			}
			p++;
		}
		/* Wenn das letzte Zeichen kein * ist, dann $ ergnzen */
		if (p>para && p[-1]!='*')
			*q++='$';
		*q=0;
		external_parameter=regbuf;
	}
	else
	{
		if (*para=='^')
			external_parameter=para+1;
		else
		{
			sprintf(regbuf,".*%s",para);
			external_parameter=regbuf;
		}
	}
	for(akt=saug_anker;akt;akt=akt->next)
	{
		if (akt->indexcmd)
		{
			char cmd[PATH_MAX*2+2048];
			if (!do_interpret(akt->indexcmd, cmd, sizeof(cmd)))
				syslog(LOG_INFO,"can't interpret %s",akt->indexcmd);
			else
			{
				FILE *f;
				f=popen(cmd,"r");
				if (!f)
					syslog(LOG_INFO,"can't popen %s",akt->indexcmd);
				else
				{
					char buf[1024];
					while (fgets(buf,sizeof(buf),f))
					{
						char *p=strpbrk(buf,"\r\n");
						if (p)
							*p=0;
						ta_do_log(":!%s" LINEEND,buf);
						found++;
					}
					pclose(f);
				}
			}
		}
	}
	external_parameter=NULL;
	return found;
}

static void
ta_s_send(const char *para)
{
    BIN_MSG bin_info;
    struct stat st;
    char *p;
    char kopie[PATH_MAX];
    BRETTNR bnr;
    FILE *bin_f;

	if (*para=='/')
	{
		BOOLEAN ok=TRUE;
		FILE *f;
		if (strstr(para,"../"))
		{
			ERRLOG(".. nicht erlaubt");
			ok=FALSE;
		}
		if (ok)
		{
			saug_dir_t *akt=saug_anker;
			saug_dir_t *best=NULL;
			size_t bestlen=0;
			/*
			 * In der Liste der Verzeichnisse ist das zu suchen, da 
			 * a) auf "para" pat und b) am lngsten ist.
			 */
			while (akt)
			{
				size_t l=strlen(akt->dir);
				if (l>bestlen)
				{
					if (!strncmp(akt->dir,para,l))
					{
						bestlen=l;
						best=akt;
					}
				}
				akt=akt->next;
			}
			if (!best)
			{
				ok=FALSE;
				ERRLOG("Zugriff auf das Verzeichnis nicht erlaubt");
			}
			else
			{
				const char *errtext=NULL;
				if (!acl_check(best->acl,user.name,level(),'R',&errtext))
				{
					if (errtext)
						ERRLOG(errtext);
					else
						ERRLOG("Zugriff nicht erlaubt (ACL)");
					ok=FALSE;
				}
			}
		}
		if (!ok)
			return;
		if (stat(para,&st) || access(para,R_OK))
		{
			ta_do_log(":?nicht mglich: %s"LINEEND,strerror(errno));
			ta_msg_log("  ?nicht mglich: %s\n",strerror(errno));
			return;
		}
		p=strrchr(para,'/'); /* klappt immer, para fngt mit / an */
		p++;
		f=fopen(tprintf("%s%ld/%s",box_conf.spool_dir,user.nummer,SAUGCNTRL),BINAPPEND);
		if (!f)
		{
			ta_do_log(":?nicht mglich: fopen: %s"LINEEND,strerror(errno));
			ta_msg_log("  ?nicht mglich: fopen: %s\n",strerror(errno));
			return;
		}
		fprintf(f,"%s\t%s\r\n",para,p);
		ta_do_log(":!ok, in Liste bernommen"LINEEND);
		ta_msg_log("  !ok, in Liste bernommen\n");
		fclose(f);
		return;
	}
	p=strchr(para,':');
	if (!p)
	{
		ERRLOG("unparsbarer Pfad");
		return;
	}
	strcpy(kopie,para);
	p=strchr(kopie,':');
	*p++=0;
	bnr=brettnr(kopie);
	if (!bnr)
	{
		ta_do_log(":?unbekannte Gruppe: %s"LINEEND,kopie);
		ta_msg_log("  ?unbekannte Gruppe: %s\n",kopie);
		return;
	}
	while (isspace(*p))
		p++;
	bin_f = binaer_dat_oeffnen(bnr,FALSE);
	if (bin_f == NULL)
	{
		ta_do_log(":?kann Gruppe nicht oeffnen: %s"LINEEND,kopie);
		ta_msg_log("  ?kann Gruppe nicht oeffnen: %s\n",kopie);
		return;
	}
	while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
	{
		if (!stricmp(bin_info.name,p))
		{
			FILE *f;
			f=fopen(tprintf("%s%ld/%s",box_conf.spool_dir,user.nummer,SAUGCNTRL),BINAPPEND);
			if (!f)
			{
				ta_do_log(":?nicht mglich: fopen: %s"LINEEND,strerror(errno));
				ta_msg_log("  ?nicht mglich: fopen: %s\n",strerror(errno));
			}
			else
			{
				const char *bp;
				const char *prefix;
				bp=brettpfad(bnr);
				if (*bp=='/')
					prefix="";
				else
					prefix=box_conf.quark_dir;

				fprintf(f,"%s%s%ld.bin\t%s\r\n",
					prefix, brettpfad(bnr),bin_info.nr, p);
				fclose(f);
				ta_do_log(":!ok, in Liste bernommen"LINEEND);
				ta_msg_log("  !ok, in Liste bernommen\n");
			}
		}
	}
	binaer_dat_schliessen(bin_f);
}

static BOOLEAN 
ta_saug_init(void)
{
	int erg;
	cl_list_t *progteile = NULL;
	cl_list_t *akt_progteil = NULL;
	cl_file_t *f;
	saug_dir_t *last=NULL;
	char fname[PATH_MAX];

	cl_var_t listtab[] =
	{
		{"progteil",      NULL,       CL_LIST,  &progteile, 0, NULL},
		{"directory",     "progteil", CL_ALIAS, NULL,       0, NULL},
		{"programmteil",  "progteil", CL_ALIAS, NULL,       0, NULL},
		{NULL, NULL, 0, NULL, 0, NULL}
	};
	sprintf(fname,"%s/saugtausch.cnf",box_conf.config_dir);
	f = cl_openconf (fname, "r");
	if (!f)
	{
		/* No problem */
		return TRUE;
	}
	erg = cl_getstanza (f, listtab, "liste", 0, NULL);
	if (erg)
	{
		syslog (LOG_ERR, "kann Verzeichnisliste nicht lesen");
		cl_closeconf (f);
		return FALSE;
	}
	for(akt_progteil=progteile;akt_progteil!=NULL;akt_progteil=akt_progteil->next)
	{
		char *dir=NULL;
		char *acl=NULL;
		char *indexcmd=NULL;
		saug_dir_t *neu;

		cl_var_t parameter[] =
		{
			{"indexcmd", NULL, CL_STRING, &indexcmd, 0, NULL},
			{"dir",      NULL, CL_STRING, &dir,      0, NULL},
			{"acl",      NULL, CL_STRING, &acl,      0, NULL},
			{NULL,       NULL, 0,         NULL,      0, NULL}
		};
		if (cl_getstanza(f,parameter,akt_progteil->inhalt,0,NULL))
		{
			syslog (LOG_INFO, "kann Programmteil %s nicht lesen", 
				akt_progteil->inhalt);
		}
		else if (!dir)
		{
			syslog (LOG_INFO, "kann Programmteilpfad %s nicht lesen", 
				akt_progteil->inhalt);
		}
		else
		{
			neu=XMALLOC(sizeof(saug_dir_t));
			neu->dir=XSTRDUP(dir);
			if (acl)
				neu->acl=XSTRDUP(acl);
			else
				neu->acl=XSTRDUP(tprintf("%s/.acl",dir));
			if (indexcmd)
				neu->indexcmd=XSTRDUP(indexcmd);
			else
				neu->indexcmd=NULL;
			neu->next=NULL;
			if (last)
				last->next=neu;
			else
				saug_anker=neu;
			last=neu;
		}
		cl_clearconf(parameter);
	}
	cl_clearconf(listtab);
	cl_closeconf(f);
	return TRUE;
}

static void
ta_s_upload(const char *para)
{
    struct stat st;
    char *p;

	if (*para=='/')
	{
		saug_dir_t *best=NULL;
		BOOLEAN ok=TRUE;
		char tmpfname[PATH_MAX];
		char uplfname[PATH_MAX];
		struct utimbuf ut;

		if (strstr(para,"../"))
		{
			ERRLOG(".. nicht erlaubt");
			ok=FALSE;
		}
		if (ok)
		{
			saug_dir_t *akt=saug_anker;
			size_t bestlen=0;
			/*
			 * In der Liste der Verzeichnisse ist das zu suchen, da 
			 * a) auf "para" pat und b) am lngsten ist.
			 */
			while (akt)
			{
				size_t l=strlen(akt->dir);
				if (l>bestlen)
				{
					if (!strncmp(akt->dir,para,l))
					{
						bestlen=l;
						best=akt;
					}
				}
				akt=akt->next;
			}
			if (!best)
			{
				ok=FALSE;
				ERRLOG("Zugriff auf das Verzeichnis nicht erlaubt");
			}
			else
			{
				const char *errtext=NULL;
				if (!acl_check(best->acl,user.name,level(),'W',&errtext))
				{
					if (errtext)
						ERRLOG(errtext);
					else
						ERRLOG("Zugriff nicht erlaubt (ACL)");
					ok=FALSE;
				}
			}
		}
		if (!ok)
			return;
		if (stat(para,&st)==0)
		{
			const char *errtext;
			if (!acl_check(best->acl,user.name,level(),'D',&errtext))
			{
				ERRLOG(errtext ? errtext : "Zugriff nicht erlaubt (ACL)");
				return;
			}
		}
		else
			st.st_mtime=time(NULL);

		p=strrchr(para,'/'); /* klappt immer, para fngt mit / an */
		p++;
		sprintf(tmpfname,"%s%s",tom_utmp_dir,p);

		if (!fileexist(tmpfname))
		{
			ERRLOG("Datei nicht vorhanden");
			return;
		}

		if (!file_move(tmpfname,para))
		{
			ERRLOG("Kann Datei nicht verschieben");
			return;
		}

		/* Dateidatum anpassen. Es wre schn, wenn cp -a berall 
		 * funktionieren wrde.
		 */
		ut.actime=time(NULL);
		ut.modtime=st.st_mtime;
		/* Fehler werden ignoriert. Das ist es nicht wert ... */
		utime(para,&ut);

		ta_do_log(":!Ok"LINEEND);
		ta_msg_log("  !Ok\n");
		unlink(tmpfname);
		p=strrchr(tmpfname,'.');
		if (p && ((size_t)(p-tmpfname)) > strlen(tom_utmp_dir))
		{
			strcpy(p+1,"upl");
			if (!fileexist(tmpfname))
			{
				*p=0;
				p=strrchr(tmpfname,'.');
				if (p && ((size_t)(p-tmpfname))>strlen(tom_utmp_dir))
				{
					strcpy(p+1,"upl");
					if (!fileexist(tmpfname))
						*tmpfname=0;
					else
					{
						strcpy(uplfname,para);
						strcpy(uplfname+(p-tmpfname)+1,".upl");
					}
				}
				else
					*tmpfname=0;
			}
			else
			{
				strcpy(uplfname,para);
				strcpy(uplfname+(p-tmpfname)+1,".upl");
			}
		}
		else
		{
			strcat(tmpfname,".upl");
			if (!fileexist(tmpfname))
				*tmpfname=0;
			else
			{
				strcpy(uplfname,para);
				strcat(uplfname,".upl");
			}	
		}
		if (*tmpfname && stat(tmpfname,&st)==0)
		{
			if (!file_move(tmpfname,uplfname))
			{
				int errno1=errno;
				ta_do_log(":!Problem mit Uploadinfo: %s"LINEEND,
					strerror(errno1));
				ta_msg_log(
					"  ?Fehler bei Uploadinfo:\n"
					"  ?  Upl: %s\n"
					"  ?  Nach: %s\n"
					"  ?  Error: %s\n",
						tmpfname,uplfname,strerror(errno1)
					);
				unlink(tmpfname);
				return;
			}
			else
			{
				ta_do_log(":!Uploadinfo Ok"LINEEND);
				ta_msg_log("  !Uploadinfo OK!\n");
			}
			/* Und Datum des Uploadinfos anpassen */
			ut.actime=time(NULL);
			ut.modtime=st.st_mtime;
			/* Fehler werden ignoriert. Das ist es nicht wert ... */
			utime(para,&ut);
		}
		else
		{
			ta_do_log(":!Kein Uploadinfo gefunden"LINEEND);
			ta_msg_log("  !Kein Uploadinfo gefunden!\n");
		}
		return;
	}
	else
	{
		ERRLOG("Tauschupload in Bretter/Programmfach funktioniert nicht");
	}
}

