/* ta_stat.c: Statusnachrichtenhandling (incoming) */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <syslog.h>
#include <time.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "routing.h"
#include "offline/mdb_glob.h"
#include "msg.h" /* write_msg fr #LOG */

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"

#define LINEEND "\r\n"

#define ERRLOG(x) do { \
	ta_msg_log("  ?%s\n",x); \
	ta_do_log_error(x); \
	} while(0)


/*
 * BX-Statusnachrichten verarbeiten.
 * Features:
 * - in Anbetracht der leichten Flschbarkeit von Mail im Internet 
 *   werden BX-Nachrichten von Gates und Boxen nicht akzeptiert.
 * - ansonsten wird generell die From-Zeile zum Vergleich herangezogen, und
 *   nicht der Sender (siehe son-of-1036).
 * Auch News kann man im Usenet extrem leicht faken, aber Cancels von
 * Cancelmoose sind einfach zu wertvoll. Daher wird kontrolliert, was
 * kontrolliert werden kann, und nicht mehr:
 * Absender lokal? -> Nur er selbst!
 * sonst wenn Absender ~ from-Zeile -> OK.
 * Level 9 darf beliebige News canceln, GCs in ihrem Brett ebenso.
 */
static void
ta_status_cancel(message_t *head)
{
	char orig_empfaenger[HEADLEN]="";
	char orig_absender[HEADLEN]="";
	char gruppe[HEADLEN]="";
	BOOLEAN darf;
	USERNR unr;

	if (!msgid_find_infos(head->msg_id,gruppe,orig_empfaenger, orig_absender)
		|| (!*gruppe && !*orig_empfaenger))
	{
		if (user.typ==UTYP_USER)
		{
			ta_do_log_error("nicht zuzuordnen");
			ta_msg_log("  ?Cancel nicht zuzuordnen\n");
		}
		else
		{
			ta_do_log(":!nicht zuzuordnen\r\n");
			ta_msg_log("  !Cancel nicht zuzuordnen\n");
		}
		return;
	}
	darf=FALSE;

	if (*orig_empfaenger)
	{
		/*
		 * Cancel auf eine Mail.
		 */
		char *p;

		if (user.typ!=UTYP_USER)
		{
			/* remote Mailcancels glaube ich nicht. Zu leicht zu flschen,
			 * und jeder Idiot kann "telnet any.site smtp" machen. 
			 */
			ta_do_log_error("don't try it again, sam");
			ta_msg_log("  ?Cancel: remote Mailcancel ersucht\n");
			return;
		}

		p=strchr(orig_empfaenger,'@');
		if (!p || r_is_me(p+1))
		{
			if (p)
				*p=0;
			if (ub_find_username(&unr,orig_empfaenger) && unr==user.nummer)
				darf=TRUE;
			if (p)
				*p='@';
		}

		if (!darf)
		{
			p=strchr(orig_absender,'@');
			if (!p || r_is_me(p+1))
			{
				if (p)
					*p=0;
				if (ub_find_username(&unr,orig_absender) && unr==user.nummer)
					darf=TRUE;
				if (p)
					*p='@';
			}
		}

		if (!darf)
		{
			ta_do_log_error("Weder von Ihnen noch an Sie!");
			ta_msg_log("  ?Cancel: User ist weder Absender noch Empfnger\n");
			ta_msg_log("  !Absender: %s\n",orig_absender);
			ta_msg_log("  !Empfnger: %s\n",orig_empfaenger);
			return;
		}

		if (mdb_kill_article(head->msg_id))
		{
			ta_do_log(":gelscht"LINEEND);
			ta_msg_log("  !Cancel: ok\n");
		}
		else
		{
			ta_do_log_error("Nachricht kann nicht gelscht werden (TSNH)");
			ta_msg_log("  ?Cancel: nicht lschbar (TSNH)\n");
		}
		return;
	}

	/* 
	 * Ok, wir haben es hier mit einem Cancel auf eine ffentliche
	 * Nachricht zu tun.
	 * Die Zugriffsprfungen an sich sind relativ simpel.
	 */
	if (level()==9 && user.typ==UTYP_USER) /* ja, ich bin paranoid */
		darf=TRUE;
	else
	{
		char *p;
		BOOLEAN is_me=FALSE;
		p=strchr(orig_absender,'@');
		if (!p || r_is_me(p+1))
		{
			is_me=TRUE;
			if (p)
				*p=0;
			if (ub_find_username(&unr,orig_absender) && unr==user.nummer)
				darf=TRUE;
			if (p)
				*p='@';
		}

		if (!darf && !is_me)
		{
			/* also kein lokaler Absender */
			char zwi1[HEADLEN];
			char zwi2[HEADLEN];
			char dom1[HEADLEN*2];
			char dom2[HEADLEN*2];

			strcpy(zwi1,head->absender);
			p=strchr(zwi1,'@');
			if (p)
			{
				*p++=0;
				strcpy(dom1,p);
			}
			else
				*dom1=0;

			strcpy(zwi2,orig_absender);
			p=strchr(zwi2,'@');
			if (p)
			{
				*p++=0;
				strcpy(dom2,p);
			}
			else
				*dom2=0;

			if (*dom1 && *dom2 && !r_hostnamecmp(dom1,dom2))
			{
				char lok1[HEADLEN*2];
				char lok2[HEADLEN*2];
				ub_canonical(zwi1,lok1);
				ub_canonical(zwi2,lok2);
				if (!stricmp(lok1,lok2))
					darf=TRUE;
			}
		}
	}

	if (!darf)
	{
		ta_do_log_error("nicht erlaubt");
		ta_msg_log("  !Cancel: Zugriffsberechtigung verweigert\n");
		return;
	}

	if (mdb_kill_article(head->msg_id))
	{
		ta_do_log(":gelscht"LINEEND);
		ta_msg_log("  !Cancel: ok\n");
	}
	else
	{
		ta_do_log_error("Nachricht kann nicht gelscht werden (TSNH)");
		ta_msg_log("  ?Cancel: nicht lschbar (TSNH)\n");
	}
	return;
}

static void
ta_status_kopieren(message_t *head, char modus)
{
	char orig_empfaenger[HEADLEN]="";
	char orig_absender[HEADLEN]="";
	char gruppe[HEADLEN]="";
	char exportfname[PATH_MAX];
	char ihavefname[PATH_MAX];
	char readbuf[16384]; /* Endloszeilen! */
	FILE *f;
	char *p;

	if (!msgid_find_infos(head->msg_id,gruppe,orig_empfaenger, orig_absender)
		|| (!*gruppe && !*orig_empfaenger))
	{
		ERRLOG("nicht zuzuordnen");
		return;
	}
	if (*gruppe)
	{
		BRETTNR bnr;
		bnr=brettnr(gruppe);
		if (!bnr)
		{
			ERRLOG("unauffindbare Gruppe (This Should Not Happen)");
			return;
		}
		if (!(br_ind[bnr].brett->flags & B_READ))
		{
			ERRLOG("unlesbare Gruppe (netter Versuch)");
			return;
		}
	}
	else
	{
		USERNR unr;
		BOOLEAN darf=FALSE;

		p=strchr(orig_empfaenger,'@');
		if (!p || r_is_me(p+1))
		{
			if (p)
				*p=0;
			if (ub_find_username(&unr,orig_empfaenger) && unr==user.nummer)
				darf=TRUE;
			if (p)
				*p='@';
		}

		if (!darf)
		{
			p=strchr(orig_absender,'@');
			if (!p || r_is_me(p+1))
			{
				if (p)
					*p=0;
				if (ub_find_username(&unr,orig_absender) && unr==user.nummer)
					darf=TRUE;
				if (p)
					*p='@';
			}
		}
		if (!darf)
		{
			ERRLOG("Sie sind nicht der Empfnger!");
			return;
		}
	}

	/* Nachricht exportieren */
	sprintf(exportfname,"%s%ld.exp",tom_tmp_dir,(long)getpid());
	sprintf(ihavefname,"%s%ld.ihave",tom_tmp_dir,(long)getpid());
	f=xfopen(ihavefname,BINWRITE);
	fprintf(f,"%s\n",head->msg_id);
	fclose(f);

	if (!mdb_get_article(ihavefname,exportfname,FALSE))
	{
		ERRLOG("Nachricht unlesbar");
		unlink(ihavefname);
		return;
	}
	unlink(ihavefname);

	{
		FILE *in;
		FILE *out;
		char neufname[PATH_MAX];

		char *absender;
		char *empfaenger=strdup(head->to->data);
		char *wegen=NULL;
		if (modus=='K')
			absender=concat(user.name," @ ", MYDOMAINPART,NULL);
		else
			absender=strdup(orig_absender);

		if (!absender || !empfaenger)
		{
			if (absender)
				free(absender);
			if (empfaenger)
				free(empfaenger);
			ERRLOG("Out of memory");
			return;
		}
		in=fopen(exportfname,BINREAD);
		if (!in)
		{
			free(empfaenger);
			free(absender);
			ERRLOG("Out of Files");
			return;
		}
		sprintf(neufname,"%skopie.txt",tom_tmp_dir);
		out=fopen(neufname,BINWRITE);
		if (!out)
		{
			free(empfaenger);
			free(absender);
			fclose(in);
			ERRLOG("Out of Files");
			return;
		}
		while (fgets(readbuf,sizeof(readbuf)-1,in))
		{
			p=strpbrk(readbuf,"\r\n");
			if (p)
				*p=0;
			switch(*readbuf)
			{
			case ':': 
				fprintf(out,"%s\r\n",readbuf+1);
				break;
			case 'W': 
				wegen=concat(readbuf+1,
					(modus=='K') ? " (Kopie)" : " (Fwd)" , NULL);
				break;
			}
		}
		fclose(in);
		fclose(out);
		einfachnachricht(TRUE,level(),user.bezahlt,neufname,
			'A', empfaenger,
			'V', absender,
			'W', wegen ? wegen : 
				((modus=='K') ? 
					"kopierte Nachricht" : "weitergeleitete Nachricht"),
			0);
		if (wegen)
			free(wegen);
		unlink(exportfname);
		unlink(neufname);

		if (modus=='W')
		{
			/* ### Originalnachricht lschen ? */
		}
		{
			char buf[2];
			buf[0]=modus;
			buf[1]=0;
			r_st_set_status(head->msg_id,
				absender,
				empfaenger,
				buf,
				time(NULL));
		}
		free(absender);
		free(empfaenger);
	}	
}

void
ta_work_status(message_t *head)
{
	char c;
	time_t statuszeit;

	c=*head->bearbeitungsstatus;
	ta_msg_log("  B%c\n",c);

	if (user.typ==UTYP_USER)
	{
		BOOLEAN ok=FALSE;
		if (!strchr("XZGWK",c))
			ERRLOG("Illegaler/unbekannter Status");
		else if (head->bearbeitungsstatus[1])
			ERRLOG("Illegale B-Zeile");
		else if ((c=='W' || c=='K') && !head->to)
			ERRLOG("K-Zeile fehlt");
		else 	
		{	
			ok=TRUE;
		}
		if (!ok)
			return;
	}
	if (user.typ==UTYP_GATE)
	{
		if (!strchr("XZGYTAM",c))
		{
			ERRLOG("Illegaler Status");
			return;
		}
	}

	/*
	 * BW/BK ohne K-Zeile bedeutet, da uns die MAUS sagt, die Nachricht
	 * sei kopiert/weitergeleitet
	 */
	if ((c=='K' || c=='W') && head->to)
	{
		ta_status_kopieren(head, c);
		return;
	}
	if (c=='X')
	{
		ta_status_cancel(head);
		return;
	}

	/* MAUS sendet
	 * EEingabezeitOriginalnachricht
	 * B?nderungszeit
	 */
	statuszeit=-1;
	if (head->bearbeitungsstatus[1])
	{
		if (!mtzeit_to_time_t(head->bearbeitungsstatus+1,&statuszeit))
			statuszeit=-1;
	}
	if (head->eingabezeit_tt  && statuszeit==-1)
	{
		statuszeit=head->eingabezeit_tt;
	}
	if (statuszeit==-1)
		statuszeit=time(NULL);

	/* das bernimmt komplett der Router */
	if (!r_st_set_status(
		head->msg_id,
		*head->absender? head->absender : user.name,
		NULL, /* Empfnger des Status -> selbst heraussuchen */
		head->bearbeitungsstatus, /* Status */
		statuszeit /* nderungszeit */
		))
	{
		ta_msg_log("  ?nicht zuzuordnen\n");
	}
}

