#ifndef TOM_SYSTEM_H
#define TOM_SYSTEM_H

	/* Autoconf-Resultate einbinden */
#ifndef TOM_CONFIG_H
#include "config.h"
#endif

#include "portab.h"

#if defined HAVE_UNISTD_H
# include <unistd.h>
#endif

/* The following is from pathmax.h.  */
/* Non-POSIX BSD systems might have gcc's limits.h, which doesn't define
   PATH_MAX but might cause redefinition warnings when sys/param.h is
   later included (as on MORE/BSD 4.3).  */
#if defined(_POSIX_VERSION) || (defined(HAVE_LIMITS_H) && !defined(__GNUC__))
# include <limits.h>
#endif

#ifndef _POSIX_PATH_MAX
# define _POSIX_PATH_MAX 255
#endif

#if !defined(PATH_MAX) && defined(_PC_PATH_MAX)
# define PATH_MAX (pathconf ("/", _PC_PATH_MAX) < 1 ? 1024 : pathconf ("/", _PC_PATH_MAX))
#endif

/* Don't include sys/param.h if it already has been.  */
#if defined(HAVE_SYS_PARAM_H) && !defined(PATH_MAX) && !defined(MAXPATHLEN)
# include <sys/param.h>
#endif

#if !defined(PATH_MAX) && defined(MAXPATHLEN)
# define PATH_MAX MAXPATHLEN
#endif

#if !defined(LONG_MAX) && defined(HAVE_LIMITS_H)
# include <limits.h>
#endif

#ifndef PATH_MAX
# define PATH_MAX _POSIX_PATH_MAX
#endif

/* Don't use bcopy!  Use memmove if source and destination may overlap,
   memcpy otherwise.  */

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
#  ifdef HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#include <stddef.h> /* size_t */

#ifndef HAVE_STRICMP
/* strcasecmp wird in libquark.k definiert */
#define stricmp strcasecmp
#endif

#ifndef HAVE_STRNICMP
/* strncasecmp wird in libquark.k definiert */
#define strnicmp strncasecmp
#define stricmpn strncasecmp
#endif

/*
 * open-Modi fr binr-streams
 * ANSI verlagt, da "b" in jedem Fall funktioniert bzw. ignoriert wird.
 */
#define BINREAD         "rb"
#define BINWRITE            "wb"
#define BINAPPEND           "ab"
#define BIN_RW_OPEN     "rb+"
#define BIN_RW_CREATE   "wb+"
#define BIN_R_APPEND        "ab+"

#ifdef HAVE_DOSDRIVES
void get_drives(char *drives);
long bytes_free(int nr);
#else
#if SIZEOF_UNSIGNED_LONG_LONG > 0 
#define PATH_BYTES_FREE_TYPE unsigned long long
#else
#define PATH_BYTES_FREE_TYPE unsigned long
#endif
PATH_BYTES_FREE_TYPE path_bytes_free(const char *);
#endif

/*
 * Dies ist *kein* Hack und auch nicht berflssig. Damit sind einige
 * falsch printf's aufgefallen.
 */
#ifdef __GNUC__
#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
# define TOM_ATTRIB_CONST  __attribute__((__const__))
#endif
	/* gcc.info sagt, noreturn wre ab 2.5 verfgbar. HPUX-gcc 2.5.8
	 * kann es noch nicht - what's this?
	 */
#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 5)
# define TOM_ATTRIB_NORET  __attribute__((__noreturn__))
#endif
	/* 
	 * checked Formatstring (Argument Nr. "formatnr"). Der erste 
	 * Parameter des Formatstrings ist Argument Nr. "firstargnr".
	 * fr vprintf und co, wo das nicht mglich ist, ist firstargnr
	 * auf 0 zu setzen -> nur Formatstring wird geprft.
	 */
#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 5)
# define TOM_ATTRIB_PRINTF(formatnr,firstargnr)  \
	__attribute__((__format__ (printf,formatnr,firstargnr)))
#endif

#define TOM_ATTRIB_INLINE __inline__

#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 6)
#define TOM_ATTRIB_UNUSED __attribute__((__unused__))
#endif

#endif

#ifndef TOM_ATTRIB_CONST
#define TOM_ATTRIB_CONST
#endif
#ifndef TOM_ATTRIB_NORET
#define TOM_ATTRIB_NORET
#endif
#ifndef TOM_ATTRIB_PRINTF
#define TOM_ATTRIB_PRINTF(x,y)
#endif
#ifndef TOM_ATTRIB_INLINE
#define TOM_ATTRIB_INLINE
#endif
#ifndef TOM_ATTRIB_UNUSED
#define TOM_ATTRIB_UNUSED
#endif

#ifdef __MINT__
/* broken MiNT: ioctl(int,int,void *) */
#define IOCTL_THIRD_PARA ,0
#else
#define IOCTL_THIRD_PARA
#endif

#ifdef HAVE_LIBREADLINE
#  if defined(HAVE_READLINE_HISTORY_H) && defined(HAVE_READLINE_READLINE_H)
#    define WANT_READLINE
#  elif defined(HAVE_READLINE_H) && defined(HAVE_HISTORY_H)
#    define WANT_READLINE
#  endif
#endif

/* Take care of NLS matters.  */

#if HAVE_LOCALE_H
# include <locale.h>
#endif
#if !HAVE_SETLOCALE
# define setlocale(Category, Locale) /* empty */
#endif

#if ENABLE_NLS
# include <libintl.h>
# define _(Text) gettext (Text)
#else
# define bindtextdomain(Domain, Directory) /* empty */
# define textdomain(Domain) /* empty */
# define _(Text) Text
#endif

#define HACK_MPORT
#ifdef HACK_MPORT
#define lock_offline(x) 1
#define LOCKOFFLINE_RET(sleeptime,returncode) \
	do { if (!lock_offline(sleeptime)) return(returncode);} while(0)
#define LOCKOFFLINE_SET(sleeptime,var,value_ok,value_nok) \
	do { var=value_ok; if (!lock_offline(sleeptime)) var=value_nok;} while(0)
#define LOCKOFFLINE(sleeptime,var,value_ok,value_nok) \
	(lock_offline(sleeptime))
#define UNLOCKOFFLINE() unlock_offline()
#endif

extern const char *vstr_compileversion;
extern const char *vstr_date;
extern const char *vstr_time;

#endif
