/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <portab.h>
#include <errno.h>
#include <ctype.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h> /* off_t */
#include <syslog.h> 
#include <limits.h> 
#include <fcntl.h> 

#include "prototyp.h"
#include "userbase.h"
#include "ub_int.h"
#include "filelock.h"

#define MAGIC	0x0369CF01L
extern USER user;

/*
	User <nr> einlesen und ggf. fr weitere Zugriffe sperren

	->			nr		:	Usernummer
			<-	new	:	initialisierte Userstruktur
				lock	:	Zugriffssperre (bei TRUE)

	<-			TRUE	:	Userstruktur konnte initialisiert werden
				FALSE	:	Fehler (z.B. ungltige Usernummer)
*/
BOOLEAN 
ub_read_user(USERNR nr,USER *new,BOOLEAN lock)
{
	char path[PATH_MAX];

	ub_fd_control_t *search;

	int fd;
	time_t jetzt;
	int lock_done;
	int tries=3;

	if (!new) {
		fprintf(stderr,"ub_read_user: NULL ptr for user # %lu\n",nr);
		syslog(LOG_ERR,"ub_read_user: NULL ptr for user # %lu",nr);
		return FALSE;
	}
	/* der router liest den aktuellen User mglicherweise!
	 * Vielleicht sollte man das ndern, aber es kann auch nicht
	 * schaden, das generell abzufangen */
	if (nr==user.nummer && !lock) {
		*new=user;
		return TRUE;
	}
	/* prevent hangs */
	for (search=ub_fd_anker;search;search=search->next) {
		if (search->unr == nr) {
			fprintf(stderr,"ub_read_user: user # %lu already locked\n",nr);
			syslog(LOG_ERR,"ub_read_user: user # %lu already locked",nr);
			return FALSE;
		}
	}

	sprintf(path,"%s/users/%lu",ub_basepath,nr);

	if (lock)
		fd = open(path,O_RDWR);
	else
		fd = open(path,O_RDONLY);
	if (fd<0) {
		int e=errno;
		fprintf(stderr,"ub_read_user: cannot open %s: %s\n",path,strerror(e));
		syslog(LOG_ERR,"ub_read_user: cannot open %s: %s",path,strerror(e));
		return FALSE;
	}
	memset(new,0,sizeof(USER));
retry:
	if (lock) 
		lock_done=fd_write_lock(fd,0,SEEK_SET,1);
	else 
		lock_done=fd_read_lock(fd,0,SEEK_SET,1);
	if (lock_done<0 && lock) {
		int e=errno;
		if (tries--)  {
			fprintf(stderr,"ub_read_user: cannot lock %s (retrying): %s\n",path,strerror(e));
			syslog(LOG_DEBUG,"ub_read_user: cannot lock %s (retrying): %s",path,strerror(e));
			sleep(1);
			goto retry;
		}
		fprintf(stderr,"ub_read_user: cannot lock %s: %s\n",path,strerror(e));
		syslog(LOG_ERR,"ub_read_user: cannot lock %s: %s",path,strerror(e));
		close(fd);
	} else if (lock_done < 0) {
		/* dumme sache. Es gibt einige Stellen im Code wo die Userdaten
		 * unbedingt benoetigt werden auch wenn ein anderer Prozesse einen
		 * Lock hlt. Z.B. muss der Router nachsehen ob ein Vertreter
		 * angegeben ist auch wenn der User gerade eingeloggt ist.
		 */
		USER u1,u2;
		BOOLEAN first=TRUE;
		int read_tries=20; /* macht circa 4 Sekunden */
		do {
			ssize_t got;
			if (read_tries--==0) {
					fprintf(stderr,
							"ub_read_user: cannot read from %s: can't get consistent data\n",path);
					syslog(LOG_ERR,
						   "ub_read_user: cannot read from %s: can't get consistent data",path);
					close(fd);
					return FALSE;
			}
				
			if (first) {
				memset(&u1,0,sizeof(USER));
				got=xread(fd,&u1,sizeof(USER));
				if (got==-1) {
					int e=errno;
					fprintf(stderr,"ub_read_user: cannot read from %s: %s\n",path,strerror(e));
					syslog(LOG_ERR,"ub_read_user: cannot read from %s: %s",path,strerror(e));
					close(fd);
					return FALSE;
				}
				if (got!=(ssize_t) sizeof(USER)) {
					/* magische Grenze, aber 600 ist tauglich */
					if (got<600) {
						fprintf(stderr,"ub_read_user: cannot read from %s: Datei zu kurz\n",path);
						syslog(LOG_ERR,"ub_read_user: cannot read from %s: Datei zu kurz",path);
						close(fd);
						return FALSE;
					} else if (got > (ssize_t) (sizeof(USER))) {
						/* mal warnen wenn die Struktur sich verkleinert haben mte. */
						fprintf(stderr,"ub_read_user: read from %s: got %lu, expected %lu\n",
								path, (ULONG) got, (ULONG) sizeof(USER));
						syslog(LOG_INFO,"ub_read_user: read from %s: got %lu, expected %lu",
							   path, (ULONG) got, (ULONG) sizeof(USER));
					}
				}
				first=FALSE;
			} else
				u1=u2;
			/* 200 millisekunden schlafen */
			msleep(200);
			memset(&u2,0,sizeof(USER));
			if (-1==lseek(fd,0,SEEK_SET)) {
				int e=errno;
				fprintf(stderr,"ub_read_user: cannot lseek in %s: %s\n",path,strerror(e));
				syslog(LOG_ERR,"ub_read_user: cannot lseek in %s: %s",path,strerror(e));
				close(fd);
				return FALSE;
			}
			got=xread(fd,&u2,sizeof(USER));
			if (got==-1) {
				int e=errno;
				fprintf(stderr,"ub_read_user: cannot read from %s: %s\n",path,strerror(e));
				syslog(LOG_ERR,"ub_read_user: cannot read from %s: %s",path,strerror(e));
				close(fd);
				return FALSE;
			}
			if (got!=(ssize_t) sizeof(USER)) {
				/* magische Grenze, aber 600 ist tauglich */
				if (got<600) {
					fprintf(stderr,"ub_read_user: cannot read from %s: Datei zu kurz\n",path);
					syslog(LOG_ERR,"ub_read_user: cannot read from %s: Datei zu kurz",path);
					close(fd);
					return FALSE;
				} else if (got > (ssize_t) (sizeof(USER))) {
					/* mal warnen */
					fprintf(stderr,"ub_read_user: read from %s: got %lu, expected %lu\n",
							path, (ULONG) got, (ULONG) sizeof(USER));
					syslog(LOG_INFO,"ub_read_user: read from %s: got %lu, expected %lu",
						   path, (ULONG) got, (ULONG) sizeof(USER));
				}
			}
			/* wenn wir zweimal die gleichen Daten gelesen haben mssen
			 * sie wohl konsistent sein.
			 */
		} while(memcmp(&u1,&u2,sizeof(USER))!=0);
		*new=u1;
	} else {
		/* nicht auf != sizeof(USER) testen - das geht bei Aufnahme neuer Felder
		 * in die Struktru schief 
		 */
		ssize_t got;
		got=xread(fd,new,sizeof(USER));
		if (got==-1) {
			int e=errno;
			fprintf(stderr,"ub_read_user: cannot read from %s: %s\n",path,strerror(e));
			syslog(LOG_ERR,"ub_read_user: cannot read from %s: %s",path,strerror(e));
			close(fd);
			return FALSE;
		} 
		if (got!=(ssize_t) sizeof(USER)) {
			/* magische Grenze, aber 600 ist tauglich */
			if (got<600) {
				fprintf(stderr,"ub_read_user: cannot read from %s: Datei zu kurz\n",path);
				syslog(LOG_ERR,"ub_read_user: cannot read from %s: Datei zu kurz",path);
				close(fd);
				return FALSE;
			} else if (got > (ssize_t) (sizeof(USER))) {
				/* mal warnen */
				fprintf(stderr,"ub_read_user: read from %s: got %lu, expected %lu\n",
						path, (ULONG) got, (ULONG) sizeof(USER));
				syslog(LOG_INFO,"ub_read_user: read from %s: got %lu, expected %lu",
					   path, (ULONG) got, (ULONG) sizeof(USER));
			}
		}
	}

	/* paranoide Abfrage? */
	if (new->geloescht) {
		fprintf(stderr,"ub_read_user: user # %lu already deleted\n", nr);
		syslog(LOG_WARNING,"ub_read_user: user # %lu already deleted", nr);
		close(fd);
		return FALSE;
	}

	if (lock) {
	/* in die Liste geffneter Dateien hngen */
		ub_fd_control_t *n;
		n=XMALLOC(sizeof(ub_fd_control_t));
		n->fd=fd;
		n->unr=nr;
		n->next=ub_fd_anker;
		ub_fd_anker=n;
	}
	else
		close(fd);

	if (new->magic!=MAGIC) {
		fprintf(stderr,"ub_read_user:  wrong magic %lx != %lx for user # %lu\n",
			(ULONG) new->magic, (ULONG) MAGIC, nr);
		syslog(LOG_WARNING,"ub_read_user: wrong magic %lx != %lx for user # %lu",
			(ULONG) new->magic, (ULONG) MAGIC, nr);
	}


	jetzt=time(NULL);

	/* Bezahltflag updaten */
	if (new->bezahlt_bis > jetzt)
		new->bezahlt=TRUE;
	else
		new->bezahlt=FALSE;

	return TRUE;

} /* ub_read_user */

