/* vt.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"
#include <portab.h>
#include "config.h"

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#include "tom_termcap.h"

#include "prototyp.h"
#include "modem.h"
#include "io.h"
#include "terminal.h"
#include "box_conf.h"

extern char *ansiS[][4];

extern WORD
  anz_lineW,
  anz_spalten,
  count_crW,
  vtmodeW;

#define GETONE(name,resultstr,ret) do { \
	if (!vt_cache.name.getestet) tv_get_cap(&vt_cache.name); \
	if (!vt_cache.name.str) return ret; \
	resultstr=vt_cache.name.str; \
	} while (0)

struct tc_cap
{
	char *str;
	const char *cap;
	const char *vt100;
	const char *vt52;
	const char *vtascii;
	const char *vtansi;
};

int col_ix[] =
{
	4,5,6,7,8,9,10,11
};

	/*  VT100       VT52    ASCII ANSI */
static struct tc_cap vt_array[] =
{
	/* 0/ATTR_OFF: Attribute ausschalten 
	 * underline damit auszuschalten klappt nicht fr alle Terminals! */
	{ NULL, "me", "\033[0m",	"\033q",	"",	"\033[0m"},
	/* ATTR_FETT: fett - "double bright" */
	{ NULL, "md", "\033[1m",	"",			"",	"\033[1m"},
	/* ATTR_INVERS: invers (tc: reverse mode) */
	{ NULL, "mr", "\033[7m",	"\033p",	"",	"\033[7m"},
	/* 3/ATTR_BLINK: blinkend (tc: blinking mode) */
	{ NULL, "mb", "\033[5m",	"",			"",	"\033[5m"},

	/* 8 Schriftfarben -generell kein Termcapeintrge dafr vorhanden ... */
	/* 4/C_SCHWARZ */
	{ NULL, NULL, "\033[30m", "",		"",	"\033[30m"},
	/* C_ROT */
	{ NULL, NULL, "\033[31m", "",		"",	"\033[31m"},
	/* C_GRUEN */
	{ NULL, NULL, "\033[32m", "",		"",	"\033[32m"},
	/* C_GELB */
	{ NULL, NULL, "\033[33m", "",		"",	"\033[33m"},
	/* C_BLAU */
	{ NULL, NULL, "\033[34m", "",		"",	"\033[34m"},
	/* C_PINK */
	{ NULL, NULL, "\033[35m", "",		"",	"\033[35m"},
	/* C_TUERKIS */
	{ NULL, NULL, "\033[36m", "",		"",	"\033[36m"},
	/* 11/C_WEISS */
	{ NULL, NULL, "\033[37m", "",		"",	"\033[37m"},

	/* 12/ATTR_SCROLL: Scrollregion setzen */
	{ NULL, "cs", "\033[%i%d;%dr", "",   "", "\033[%i%d;%dr" },
	/* 13/ATTR_CURSOR: cursorposition setzen */
	{ NULL, "cm", "\033[%i%d;%dH", "",   "", "\033[%i%d;%dH" },

	/* 14/CUR_UP: Cursor Up */
	{ NULL, "ku", "\033[1A",	"\033A",	"",	"\033[A"},
	/* CUR_DOWN: Cursor Down */
	{ NULL, "kd", "\033[1B",	"\033B",	"",	"\033[B"},
	/* CUR_RIGHT: Cursor rechts */
	{ NULL, "kr", "\033[1C",	"\033C",	"",	"\033[C"},
	/* 17/CUR_LEFT: Cursor links */
	{ NULL, "kl", "\033[1D",	"\033D",	"\b",	"\033[D"},
	/* STORE_CUR */
	{ NULL, "sc", "\0337",	"\033j",	"",	"\033[s"},
	/* 19/RESTORE_CUR */
	{ NULL, "rc", "\0338",	"\033k",	"",	"\033[u"},

	/* 20/DEL_RIGHT */
	{ NULL, "ce", "\033[K",	"\033K",	"",	"\033[K"},
	/* DEL_SCR_REST */
	{ NULL, "cd", "\033[J",	"\033J",	"",	""},
	/* 22/DEL_SCR */
	{ NULL, "cl", "\033[2J",	"\033E",	"",	"\033[J\033[1J"},

};

static struct vt_cache
{
	int termtyp;
	int lines;
	int columns;
	char *LINES;
	char *COLUMNS;
}
vt_cache = {
	-1,
	0,
	0,
	NULL,
	NULL
};

static void
tv_get_cap (struct tc_cap *cap, const char *ersatz)
{
	char buf[1024];
	char *dummy=buf;
	char *q;
#ifndef HAVE_TERMCAP
	if (!*ersatz)
	{
		cap->str=NULL;
		return;
	}
#endif
	if (!*ersatz 
#ifdef HAVE_TERMCAP
		 && cap->cap && (q=tgetstr(cap->cap,&dummy))!=NULL && *q
#endif
		)
	{
		char *p=q;

		/* padding loswerden - das knnen und wollen wir nicht 
		 * verarbeiten (knnen: Die Ausgaben geschehen Highlevel 
		 * ohne tputs. wollen: Das soll auch so sein, tputs gibt
		 * zeichenweise aus :-/)
		 */
		while (isdigit(*p))
			p++;
		if (*p=='.')
		{
			p++;
			while (isdigit(*p))
				p++;
		}
		if (p!=q && *p=='*')
			p++;
		if (*p)
			cap->str=XSTRDUP(p);
		else
			cap->str=NULL;
	}
	else
		cap->str=XSTRDUP(ersatz);
}

void
sync_terminal (void)
{
#define FREEONE(x) \
	free(vt_cache.x.str); vt_cache.x.str=NULL; vt_cache.x.getestet=FALSE

	if (vtmodeW != vt_cache.termtyp)
	{
		const char *name=NULL;
		size_t i;

		/* Alte Werte lschen */
		for (i=0;i < (sizeof(vt_array)/sizeof(struct tc_cap));i++)
		{
			free(vt_array[i].str);
			vt_array[i].str=NULL;

		}

		set_vtmode(vtmodeW); /* $TERM setzen */
		switch(vtmodeW)
		{
		case T_ASCII: name=box_conf.ascii_termcap; break;
		case T_VT100: name=box_conf.vt100_termcap; break;
		case T_ANSI:  name=box_conf.ansi_termcap; break;
		case T_VT52:  name=box_conf.vt52_termcap; break;
		}

#ifdef HAVE_TERMCAP
#ifdef TGETENT_NULL
	    tgetent(0,name);
#else
		{
			void *ptr=XMALLOC(2048);
			tgetent(ptr,name);
		}
#endif
#endif
		/* Alte Werte lschen */
		for (i=0;i < (sizeof(vt_array)/sizeof(struct tc_cap));i++)
		{
			switch(vtmodeW)
			{
			case T_ASCII: tv_get_cap(&vt_array[i],vt_array[i].vtascii); break;
			case T_VT100: tv_get_cap(&vt_array[i],vt_array[i].vt100); break;
			case T_VT52: tv_get_cap(&vt_array[i],vt_array[i].vt52); break;
			case T_ANSI: tv_get_cap(&vt_array[i],vt_array[i].vtansi); break;
		}
		}

		vt_cache.termtyp = vtmodeW;
	}

	if (anz_lineW != vt_cache.lines || anz_spalten != vt_cache.columns)
	{
		io_change_term_size (user_in_fd, anz_spalten, anz_lineW);
		io_get_term_size (user_in_fd, &vt_cache.columns, &vt_cache.lines);
		if (anz_lineW != vt_cache.lines)
		{
			char buf[20];
			char *alt,*neu;
			sprintf(buf,"LINES=%d",(int) anz_lineW);
			neu=XSTRDUP(buf);
			alt=vt_cache.LINES;
			putenv(neu);
			free(alt);
			vt_cache.LINES=neu;
			vt_cache.lines=anz_lineW;
		}
		if (anz_spalten != vt_cache.columns)
		{
			char buf[20];
			char *alt,*neu;
			sprintf(buf,"COLUMNS=%d",(int) anz_spalten);
			neu=XSTRDUP(buf);
			alt=vt_cache.COLUMNS;
			putenv(neu);
			free(alt);
			vt_cache.COLUMNS=neu;
			vt_cache.columns=anz_spalten;
		}
	}
}

const char *
vt_S (int nr)
{
	const char *capstr;

	capstr=vt_array[nr].str;
	if (!capstr)
		return "";
	else
		return capstr;
}								/* vt_S */

/*
 * 08.02.93 Frank, neu
 * 
 * Zur Vermeidung dynamischer Strings und trotzdem mehrfachen Aufrufs wird 
 * ein statischer Ringpuffer verwendet, der max. 5 Verschachtelungen zult
 */
const char *
vt_pos_S (int y, int x)
{
#define PLATZ_CURSOR_MOTION 20
	static char puffer[5][PLATZ_CURSOR_MOTION];
	static int nr = 0;
	const char *capstr;
	char *tmp;

	/* nur unter VT100 Cursorroutinen benutzen - der bliche Gebrauch von
	 * ANSI in der Box ist dafr *ungeeignet*
	 */
	if (T_VT100 != vt_cache.termtyp)
		return "";

	capstr=vt_array[ATTR_CURSOR].str;
	if (!capstr)
		return "";

    count_crW = (y >= 5) ? y-5 : 0;
	nr++;
	if (nr == 5)
		nr = 0;
	while (isdigit(*capstr))
		capstr++;
	if (*capstr=='.')
	{
		capstr++;
		while (isdigit(*capstr))
			capstr++;
	}
	if (*capstr=='*')
		capstr++;
	tmp=tgoto(capstr,x,y);
	strncpy(puffer[nr],tmp,PLATZ_CURSOR_MOTION);
	*(puffer[nr]+PLATZ_CURSOR_MOTION-1)=0;

	return puffer[nr];

} /* vt_pos_S */

/*
 * Scrollfenster setzen:
 * 0: Ausschalten. 0..n-1 benutzbar.
 * 1: Normaleinstellung. 4..n-2 benutzbar.
 * 2:                    6..n-2 benutzbar.
 * sonst:                mode..n-2 benutzbar.
 */

int 
set_scroll_area (int mode)
{
	static int last_mode=0;
	int ret_mode;
	const char *capstr;

	ret_mode=last_mode;
	last_mode=mode;
	capstr=vt_array[ATTR_SCROLL].str;
	if (!capstr || vtmodeW!=T_VT100)
		return ret_mode;

	while (isdigit(*capstr))
		capstr++;
	if (*capstr=='.')
	{
		capstr++;
		while (isdigit(*capstr))
			capstr++;
	}
	if (*capstr=='*')
		capstr++;

	switch (mode)
	{
		char tbuf[256];
	case 0:
	   /* Ausgabefenster ausschalten */
		tparam(capstr,tbuf,sizeof(tbuf)-1,0,anz_lineW-1);
		mboutput(tbuf);
		mboutput (vt_pos_S (1, 1));
		break;
	case 1:
	   /* Normaleinstellung. */
		tparam(capstr,tbuf,sizeof(tbuf)-1,4-1,anz_lineW-1);
		mboutput(tbuf);
		mboutput (vt_pos_S (5, 1));
		break;
	case 2:
		tparam(capstr,tbuf,sizeof(tbuf)-1,6-1,anz_lineW-1);
		mboutput(tbuf);
		mboutput (vt_pos_S (7, 1));
		break;

	default:
		mboutput (vt_pos_S (1, 1));
		tparam(capstr,tbuf,sizeof(tbuf)-1,mode-1,(int) anz_lineW-1);
		mboutput(tbuf);
		mboutput (vt_pos_S (mode+1,1));
	}
	return ret_mode;
}								/* set_scroll_area */


