/* xwrite_tty.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "config.h"

#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include "portab.h"
#include "libquark.h"

#include "io.h"

#if defined HAVE_TERMIOS_H
#  include <termios.h>
#  define USE_TERMIO
#elif defined(HAVE_SYS_TERMIOS_H)
#  include <sys/termios.h>
#  define USE_TERMIO
#elif defined(HAVE_TERMIO_H)
#  include <termio.h>
#  define USE_TERMIO
#elif defined(HAVE_SYS_TERMIO_H)
#  include <sys/termio.h>
#  define USE_TERMIO
#endif

size_t
xwrite_tty(int fd, const char *data, size_t count)
{
	size_t written;
	BOOLEAN first;
	size_t total;

	total=0;
	/* 0 Zeichen zu schreiben akzeptieren wir beim ersten Mal, aber nicht
	 * beim zweiten und folgenden Versuch. Dann ist der Puffer garantiert
	 * leer, und da nichts geschrieben werden konnte *mu* ein Fehler
	 * sein.
	 */
	first=TRUE;

	while (total<count)
	{
		written=write(fd,data+total,count-total);
		if (written==0 && !first) 
			return (size_t) -1;
		if (written ==(size_t) -1 && errno!=EINTR)
			return (size_t) -1;
		if (written+total!=count)
		{
			/* Das OS hat nicht alles angenommen. Ok, that's life. Sagen wir
			 * ihm also, da wir warten, bis es den Puffer wieder leer hat
			 */
#ifdef USE_TERMIO
			tcdrain(fd);
#else
			sleep(1); /* ### das ist Mist - wie ersetzt man tcdrain? */
#endif
			/* Ok, der Puffer ist leer und das OS spielt uns in nchster
			 * Zeit keinen solchen Streich mehr. Falls allerdings die
			 * auszugebende Zeichenfolge sehr lang ist (oder der OS-Puffer
			 * fr das TTY recht klein ist), kann es passieren, d wir
			 * schon wieder nicht alles ausgeben knnen, weil die Bytefolge
			 * grer ist als der Puffer. Daher die Schleife.
			 */
		} 
		first=FALSE;
		total+=written;
	}
	return total;
}
