/* zeit.c : misc. time functions */
/*
    Copyright (C) 1994-1996, 1999 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdlib.h>
#include <string.h>

#include "modem.h"
#include "prototyp.h"

char *
datum_zeit_string_y2k(time_t *zeit)
{
	static char buf[20];
	struct tm *tm=localtime(zeit);
	/* no longer strftime since gcc warns about %y ... */
	sprintf(buf,"%02d.%02d.%4.4d %0d:%02d",
		tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900,
		tm->tm_hour, tm->tm_min);
	/* strftime(buf,sizeof(buf)-1,"%d.%m.%y %H:%M",localtime(zeit)); */
	return buf;
}

char *
datum_zeit_string(time_t *zeit)
{
	static char buf[20];
	struct tm *tm=localtime(zeit);
	/* no longer strftime since gcc warns about %y ... */
	sprintf(buf,"%02d.%02d.%2.2d %0d:%02d",
		tm->tm_mday, tm->tm_mon+1, tm->tm_year,
		tm->tm_hour, tm->tm_min);
	/* strftime(buf,sizeof(buf)-1,"%d.%m.%y %H:%M",localtime(zeit)); */
	return buf;
}

/*
	Tagesnamen liefern

	16.11.92 Frank
*/
const char *
wochentagS(void)
{
	static const char *tag[] =
	{
		"Sonntag",
		"Montag",
		"Dienstag",
		"Mittwoch",
		"Donnerstag",
		"Freitag",
		"Samstag",
	};

	time_t sec;
	struct tm *zeit;

	time(&sec);
	zeit = localtime(&sec);
	return tag[zeit->tm_wday];

} /* wochentagS */



/*
	liefert:
	"dd.mm.jj hh:mm:ss"
*/
char *
zeige_datum_S(time_t zeit)
{
	static char tmp[20];
	time_t tz = zeit;
	struct tm *tm=localtime(&tz);

	/* no longer strftime since gcc warns about %y ... */
	sprintf(tmp,"%02d.%02d.%2.2d %0d:%02d:%02d",
		tm->tm_mday, tm->tm_mon+1, tm->tm_year,
		tm->tm_hour, tm->tm_min, tm->tm_sec);

	return tmp;
}



/*
	Uwe Ohse, 9.7.92. 
	Zerlegt ein vom User eingegebenes Datum in Einzelteile, kommt auch mit 
	unvollstndigen Eingaben klar. 
	Bei ungltigen Eingaben wird FALSE zurckgegeben, die Werte in tW, mW 
	und jW sind dann undefiniert.
	Feature, nicht Bug: einzelne Zahlen knnen weggelassen werden, 
	so wird z.B. am 9.7.92 aus folgenden Eingaben: 
	"1.1."      -> 1.1.92 
	".1.1992"   -> 9.1.92 
	"..89"      -> 9.7.89 
	"1..89"     -> 1.7.89 
	".1"        -> 9.1.92 
	".."        -> 9.7.92 
	"-1.7.92"   -> Fehler 
	"31.2.92"   -> 31.2.92 
	"31.13.92"  -> Fehler 
*/
BOOLEAN 
zerlege_datum(const char *datum,int *t,int *m,int *j)
{
	time_t		tim;		/* fr Defaults */
	struct tm	*tm;		/* dito */
	long			value;
	char			*next;

	tim = time(NULL);

	value=strtol(datum,&next,10);
	if ((!next || !*next) && value < 0)
	{
		/* die Form "-10" - 10 Tage vor heute */
		tim += value * 86400L;
		tm = localtime(&tim);
		*t = tm->tm_mday;
		*m = tm->tm_mon+1;	/* +1 weil tm_mon bescheuerterweise von 0..11 */
		*j = tm->tm_year;
		return TRUE;
	}

	tm = localtime(&tim);
	*t = tm->tm_mday;
	*m = tm->tm_mon+1;	/* +1 weil tm_mon bescheuerterweise von 0..11 */
	*j = tm->tm_year;

	/* Am Anfang mu entweder eine Zahl oder ein Punkt stehen. 
		Letzteres ist gleichbedeutend damit, da das erste Zeichen
		nach der Zahl '.' sein mu
	*/
	value = strtol(datum,&next,10);
	if (value<0 || value>31 || (*next!='\0' && *next!='.'))
 	{
 		mbprintf(" Abbruch wegen falscher Tagesangabe (%s)\r\n\n",datum);
		return FALSE;
	}
	/* der Tag ist jetzt also abgehandelt, er bleibt unverndert,
		wenn value gleich Null, sonst ist Tag=Value.
	*/
	if (value)
		*t = (int) value;
	/* Weiteres Zeichen? *next kann jetzt nur noch Stringende oder
		'.' sein
	*/
	if (!*next)
		return TRUE;
	/* next einen nach vorne */
	next++;
	if (!*next)
		/* Stringende */
		return TRUE;

	/* Nchste Zahl auswerten                   -> Monat bestimmen */
	value = strtol(next,&next,10);
	/* und dasselbe Procedere noch einmal */
	if (value<0 || value>12 || (*next!='\0' && *next!='.'))
	{
		mbprintf(" Abbruch wegen falscher Monatsangabe (%s)\r\n\n",datum);
		return FALSE;
	}
	/* der Monat ist jetzt also abgehandelt, er bleibt unverndert,
		wenn value gleich Null, sonst ist Monat=Value.
	*/
	if (value)
		*m = (int) value;
	/* Weiteres Zeichen? *next kann jetzt nur noch Stringende oder
		'.' sein
	*/
	if (!*next)
		return TRUE;
	/* next einen nach vorne */
	next++;
	if (!*next)
		/* Stringende */
		return TRUE;

	/* Nchste Zahl auswerten                     -> Jahr bestimmen */
	value = strtol(next,&next,10);
	/* und dasselbe Procedere noch einmal */
	if (value > 1900 && value < 2100)
		value -= 1900;
	else if (value < 100) {
		/* y2k */
		if (tm->tm_year >= 100) 
			value+=100;
	}
	if (value<0 || *next!='\0' )
	{
		mbprintf(" Abbruch wegen falscher Jahresangabe (%s)\r\n\n",datum);
		return FALSE;
	}
 	if (value>0)
		*j = (int) value;

	return TRUE;

} /* zerlege_datum */



/*
	Zerlegt das als String einkommende Datum, ergnzt fehlende Teile
	aus dem aktuellen Tagesdatum.
	Das Ergebnis wird als time_t zurckgeliefert, dabei wird auch
	geprft, ob das Datum korrekt in time_t berfhrbar ist.
	Achtung: Die Routine prft ansonsten nicht, ob das Datum in der 
	Zukunft liegt!

	11.02.93 Uwe
*/
BOOLEAN 
zerlege_datum_2_time_t(const char *datum,time_t *tim)
{
	int t,m,j;

	if (zerlege_datum(datum,&t,&m,&j))
	{
	/* time_t ist erst ab 1970 definiert. Das wird abgefangen. */
		struct tm tm;

		if (j<70) 
			return FALSE;

		tm.tm_year = j;
		tm.tm_mon  = m-1;
		tm.tm_mday = t;
		tm.tm_hour = 0;
		tm.tm_min  = 0;
		tm.tm_sec  = 0;
		tm.tm_yday = 0;
		tm.tm_wday = 0;
		tm.tm_isdst= 0;
		*tim = mktime(&tm);

		return TRUE;
	}
	else
 		return FALSE;

} /* zerlege_datum_2_time_t */



/*
	11.02.93 Uwe

	Format des Strings mu ok sein
*/
LONG 
sek_time(const char *zeit)
{
	char *next;
	LONG value;

	value =3600*strtol(zeit,&next,10);
	value+=  60*strtol(next+1,&next,10);
	value+=     strtol(next+1,NULL,10);

	return value;

} /* sek_time */

